! $Id: necopt.prg,v 1.6 2001/07/15 14:15:25 amodigli Exp $
!------------------------------------------------------------------------------
!
! .IDENTIFICATION
!
! ECHELLE PROCEDURE : ECHOPT.PRG
!  Michele Peron	version 1.0   sep91
!  Sebastian Wolf	version 2.0   apr99
!
! .PURPOSE
!
! execute the command : level 1
! EXTR/OPT in out slit,ord1,ord2 ron,g,sigma table coeffs calcw
!
! .RETURN	-1 on error, 0 on successful return.
!
! .HISTORY
! 1999.04.08-SW	Compute the initial variance frame from
!		input frame, RON and GAIN. Here GAIN means the
!		inverse gain-factor, i.e. CONAD
! 1999.05.17-SW	Do an optimal extraction using the weights of an previous
!		call to EXTRACT/OPTIMAL. This may be done by setting P7=NO
!		This is necessary for flatfielding optimal extracted science
!		spectra.
! 1999.06.11-SW	Average the weights for each order and store in wbin.bdf
! 1999.09.08-SW	The variance of the input frame and the variance of the
!		extracted data will be determined in any case of P7
!		P7 may accept the name of the weight map (e.g. Y,name).
! 1999.12.08-SW	Take account for the flatfielding when calculating the
!		the variance. The FFOPT keyword controls this !!
!
! COMMENT:	It would be much better if the flatfielding could be read
!		from the input frame (but this has to be done in the frame
!		of the ECHELLE context)
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.6 $
! $Log: necopt.prg,v $
! Revision 1.6  2001/07/15 14:15:25  amodigli
! Extended In/Out file names to 80 chars
!
! Revision 1.5  2001/07/12 15:31:49  amodigli
! Changed ff_fct from 0.001 to 0.0001
!
! Revision 1.4  2001/01/04 12:20:51  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!-----------------------------------------------------------------------------
! Who  when      Where         Why
! AM   10/07/01  correction    changed fct from 0.001 to 0.0001
!-----------------------------------------------------------------------------
DEFINE/PARAM P1 ? IMAGE "Enter input image:"
DEFINE/PARAM P2 ? IMAGE "Enter output extracted orders:"
DEFINE/PARAM P3 ? NUMBER "Enter extraction pars(SLIT,ORD1,ORD2):"
DEFINE/PARAM P4 {RON},{GAIN},{EXTSIGMA}  NUMBER   "Enter extraction pars (RON,G,SIGMA):"
DEFINE/PARAM P5 {ORDTAB} TABLE "Enter auxiliary table:"
DEFINE/PARAM P6 COEFF CHAR "Enter coeffs with order position:"
DEFINE/PARAM P7 YES CHAR "Enter YES determine weights, NO apply weights:"
!
WRITE/KEYW IN_A/C/1/80   'P1'
WRITE/KEYW OUT_A/C/1/80  'P2'
WRITE/KEYW OUT_B/C/1/80  " " ALL
WRITE/KEYW OUTPUTC/C/1/80  " " ALL
WRITE/KEYW INPUTI/I/2/3  0,0
WRITE/KEYW INPUTI/I/1/3  'P3'
WRITE/KEYW INPUTR/R/1/3  'P4'
WRITE/KEYW IN_B/C/1/80   'P5'
WRITE/KEYW INPUTC/C/1/8  'P6'
 
DEFINE/LOCAL RON/R/1/1 0.0
!
! compute the variance frame. The input frame is assumed to be only background
! subtracted (de-biased)
! The variance frame is in units of [ADU^2]:
!   V_ij = (RON / CONAD)^2 + |IN_ij|/CONAD ;
!  [RON] = e-, [CONAD] = e-/ADU, [GAIN] = ADU/e-
!
if INPUTR(2) .gt. 0.0 then
  OUTPUTC = "variance.bdf"
  RON = INPUTR(1) / INPUTR(2)	!RON in ADU
  RON = RON * RON			!RON^2 in ADU
  WRITE/OUT "Compute the variance of input frame ..."
  if FFOPT(1:1) .ne. "N" then
    WRITE/OUT "... correct variance for flatfield."
    STATISTIC/IMA {correct} ? ? ? SN		!do the normalization
    OUTPUTR(3) = OUTPUTR(3) / INPUTR(2)
    COMPUTE/IMAGE {OUTPUTC} = {RON} + MAX(0.0,{P1}) * {OUTPUTR(3)} / {CORRECT}
  else
!   was before    COMPUTE/IMAGE {OUTPUTC} = {RON} + MAX(0.0,{P1}) / {INPUTR(2)}
!   added an extra term to take in count of small contribute from 
!   error on detector flat field determination
 
    define/local ff_fct/r/1/1 0.0001
    COMPUTE/IMAGE {OUTPUTC} = {RON} + MAX(0.0,{P1}) / {INPUTR(2)} + ({ff_fct}*{P1}/{INPUTR(2)})**2
  endif
else
  WRITE/OUT "Error bad CONAD to determine variance frame!"
  RETURN -1
endif


if P7(1:1) .eq. "N" then
  INPUTI(4) = 0
else				!determine the weights, we need flatfield
  INPUTI(4) = 1
endif

INPUTI(5) = M$INDEX(p7,",") + 1	!Check for the name of the weight map
IF INPUTI(5) .GT. 1 THEN
  OUT_B = P7({INPUTI(5)}:)
ELSE
  OUT_B = "weight.bdf"
ENDIF
				!Perform an optimal extraction with the
				!weights of a previous call to EXTRACT/OPTIMAL
				!(used for flatfields) if INPUTI(4) == 0
				!otherwise determine the weights.


WRITE/KEYW IN_B/C/1/80   {P5}
WRITE/KEYW INPUTC/C/1/8  {P6}
WRITE/KEYW IN_A/C/1/80   {P1}
WRITE/KEYW OUT_A/C/1/80  {P2}
define/local BINX/I/1/1  {{p1},ESO.DET.WIN1.BINX}
define/local BINY/I/1/1  {{p1},ESO.DET.WIN1.BINY}

!RUN STD_EXE:NECOPT
RUN PIPE_EXE:NECOPT
write/out "Optimal extraction done."

if INPUTI(4) .eq. 1 then
  DEFINE/LOCAL ISTP/i/1/1 1
  ISTP = M$VALUE({out_b},NPIX(2)) / M$VALUE({p2},NPIX(2))
  COPY/DD {p2} *,3 {out_b}
  REBIN/LINEAR {out_b} wbin 1,{ISTP} 0,0
  COPY/DD {p2} START wbin START
  COPY/DD {p2} STEP wbin STEP
endif
!
RETURN 0

