C $Id: necexor2d.for,v 1.4 2001/01/04 12:31:27 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR: D.PONZ
C
C.IDENTIFICATION
C  program  ECHEXOR  version 1.2 840114
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C
C  EXTRACT ECHELLE ORDERS IN CASPEC IMAGES. ORDER POSITION ARE DEFINED
C  BY COEFFICIENTS OF A REGRESSION. SLIT WIDTH, ANGLE AND OFFSET
C  ARE DEFINED BY PARAMETERS
C            WIDTH,ANGLE,OFFSET
C
C.ALGORITHM
C
C   ORDERS ARE EXTRACTED BY PASSING A NUMERICAL SLIT OF GIVEN WIDTH 
C  AND ANGLE IN POSITIONS DEFINED BY THE REGRESSION COEFFS. THE SLIT
C  WILL SAMPLE AT INCREMENTS OF ONE STEP IN X.
C
C  THERE ARE THREE METHODS :
C   LINEAR   - PIXEL VALUES ARE ADDED, LINEAR INTERPOLATION
C   AVERAGE  - PIXEL VALUES ARE AVERAGED, LINEAR INTERPOLATION
C   WEIGHTED - PIXEL VALUES ARE WEIGHTD PROPORTIONALLY TO THE
C              PROFI PERPENDICULAR TO THE DISPERSION DIRECTION
C.INPUT/OUTPUT
C
C   EXTRACT/ORDER output = input width,angle,offset 
C				 table coeffs [ord1,ord2]
C  the connection file contains
C  IN_A   input FRM
C  IN_B   input table with coeffs
C  OUT_A  output FRM
C  INPUTR  width,angle
C  INPUTC  coeffs
C  INPUTI  ord1,ord2
C  ACTION               T - table option
C                       B - batch order by order
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.4 $
C $Log: necexor2d.for,v $
C Revision 1.4  2001/01/04 12:31:27  amodigli
C Added Id, Name, Revision, Log conf. control info
C
C------------------------------------------------------------------------------
      PROGRAM ECHEXO
      IMPLICIT NONE
      INTEGER NDIM
      PARAMETER (NDIM=2)
      INTEGER MADRID, NAC, NAR, IMNO
      INTEGER I,IAV,INDX,IORD,IORD1
      INTEGER IORD2,ISTAT,NA,NAXISA,NAXISB
      INTEGER NCOL,NPAR,NROW,NSORT
      INTEGER*8 PNTRA
      INTEGER*8 PNTRB
      INTEGER*8 PNTRC
      INTEGER   STATUS,TID,INO,TMPID
      INTEGER NPIXA(3),NPIXB(3),IPAR(10),KUN,KNUL
      INTEGER NPIX_OFF_SHIFT
      
      INTEGER HS !high of slit
      INTEGER J  !current index going from 1 to HS to scan each order along y
C
      DOUBLE PRECISION STEPA(3),STEPB(3), STARTA(3),STARTB(3)
      REAL  RPAR(10)
      REAL  CUTS(6)
      REAL XMIN,XMAX,XMIN1,XMAX1
      REAL PARAMS(12)
C
      DOUBLE PRECISION DPAR(100)
C
      CHARACTER*72   IDENA,IDENB
      CHARACTER*64   CUNITA,CUNITB,DESNAM
      CHARACTER*1    METH
      CHARACTER*60   FRMIN,FRMOUT,INA1,INA2
      CHARACTER*60   TABLE,NAME,TYPE
C
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'
      COMMON /VMR/MADRID(1)
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'
C
      DATA CUNITB/
     +     'FLUX            PIXEL                           ORDER'/
C
C  initialize system
C
      CALL STSPRO('ECHEXO')
      CALL STKRDC('IN_A',1,1,60,I,INA1,KUN,KNUL,STATUS)
      CALL STKRDC('IN_B',1,1,60,I,TABLE,KUN,KNUL,STATUS)
      CALL STKRDC('IN_C',1,1,1,I,METH,KUN,KNUL,STATUS)
      CALL STKRDC('OUT_A',1,1,60,I,INA2,KUN,KNUL,STATUS)
      CALL STKRDC('INPUTC',1,1,60,I,NAME,KUN,KNUL,STATUS)
      CALL STKRDR('INPUTR',1,3,I,RPAR,KUN,KNUL,STATUS)
      CALL STKRDI('INPUTI',1,2,I,IPAR,KUN,KNUL,STATUS)
      IORD1  = IPAR(1)
      IORD2  = IPAR(2)
      FRMIN = INA1
      FRMOUT = INA2
      IF (METH(1:1) .EQ. 'l') METH = 'L' !Linear  extraction method
      IF (METH(1:1) .EQ. 'a') METH = 'A' !Average extraction method
      IF (METH(1:1) .EQ. 'm') METH = 'M' !Median  extraction method
      IF (METH(1:1) .EQ. 's') METH = 'S' !Spatial extraction method
    
C
C  input parameters in table
C
      CALL TBTOPN(TABLE,F_U_MODE,TID,STATUS)
      CALL TBIGET(TID,NCOL,NROW,NSORT,NAC,NAR,STATUS)
C
C  map input FRM
C
      CALL STIGET(FRMIN,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            NDIM,NAXISA,NPIXA,STARTA,STEPA,IDENA,
     +            CUNITA,PNTRA,IMNO,STATUS)
      PARAMS(1) = RPAR(1)/ABS(STEPA(2))
      PARAMS(2) = RPAR(2)
      PARAMS(3) = RPAR(3)/ABS(STEPA(2))
C      print *,'slit=',PARAMS(1),'angle=',PARAMS(2),'offset=',PARAMS(3)

      IF (METH(1:1) .EQ. 'M') THEN
         NWDTH = PARAMS(1) + 2
         CALL STFCRE('column',D_R4_FORMAT,F_X_MODE,1,NWDTH,TMPID,STATUS)
         CALL STFMAP(TMPID,F_X_MODE,1,NWDTH,IAV,PNTRC,STATUS)
      ENDIF
C
C  read coeffs
C
      NAME   = NAME(1:59)//' '
      INDX   = INDEX(NAME,' ') - 1
      DESNAM = NAME(1:INDX)//'C'
      CALL STDRDC(TID,DESNAM,1,17,4,IAV,TYPE,KUN,KNUL,ISTAT)
      TYPE   = 'MULT'
      IF (TYPE.EQ.'MULT') THEN
         DESNAM = NAME(1:INDX)//'I'
         CALL STDRDI(TID,DESNAM,4,4,IAV,IPAR,KUN,KNUL,ISTAT)
         NA     = (IPAR(3)+1)* (IPAR(4)+1)
         DESNAM = NAME(1:INDX)//'R'
         CALL STDRDR(TID,DESNAM,1,4,IAV,PARAMS(4),KUN,KNUL,ISTAT)
         DESNAM = NAME(1:INDX)//'D'
         CALL STDRDD(TID,DESNAM,1,NA,IAV,DPAR,KUN,KNUL,ISTAT)
         CALL TBTCLO(TID,STATUS)
C
C ... map output FRM
C
         IF (IORD1.EQ.IORD2 .AND. IORD1.EQ.0) THEN
            IORD1  = PARAMS(6)
            IORD2  = PARAMS(7)
         END IF
         STARTB(1) = STARTA(1)
         STARTB(2) = IORD1
         STEPB(1)  = STEPA(1)
         STEPB(2)  = 1.
         IDENB     = IDENA
         NAXISB    = 2
         NPIXB(1)  = NPIXA(1)
         JMIN=1
         IF (METH(1:1) .EQ. 'S') THEN
                  HS=PARAMS(1)      !Copy the slit size on the HS Aux variable
                  JMAX=HS           !Set the maximum value of the inter-order x-loop to it
                  PARAMS(1)=1       !Force the extraction slit to be 1 (inter-order extraction slit)
         ELSE
                  HS=1              !Normal case: width of the extracted order=1
                  JMAX=JMIN         !Normal case: no loop inter-order        
         ENDIF
         NPIXB(2)  = (IORD2 - IORD1 + 1)*HS  !Now the size of the output image is Nord*HS
C         print *,'Ima Y size=',NPIXB(2)
C         print *,'Extraction Method=',METH(1:1)
         
         CALL STIPUT(FRMOUT,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     +        NAXISB,NPIXB,STARTB,
     +        STEPB,IDENB,CUNITB,PNTRB,INO,STATUS) 
C     
C  extract order
C
         NPAR      = 10
         PARAMS(8) = IPAR(3)
         PARAMS(9) = IPAR(4)
         XMIN      = 1.E30
         XMAX      = -XMIN
         DO 10 I = IORD1,IORD2
            
            PARAMS(10) = I
            
            
            DO J=JMIN,JMAX
               IF (METH(1:1) .EQ. 'S') THEN
                  NPIX_OFF_SHIFT=HS/2
                  PARAMS(3)=J - NPIX_OFF_SHIFT              !It was PARAMS(1)=J
                  IORD       = (I - IORD1)*HS+J
               ELSE
                  IORD       = I - IORD1 + 1               
               ENDIF
               IF (METH(1:1) .EQ. 'S') THEN           !Spatial Extraction
                  CALL EXTR1MS(MADRID(PNTRA),NPIXA(1),NPIXA(2),
     +              MADRID(PNTRB),NPIXB(1),NPIXB(2),
     +              IORD,NPAR,PARAMS,NA,DPAR,XMIN1,XMAX1,METH,
     +              STARTA,STEPA)               
               ELSE IF (METH(1:1) .EQ. 'M') THEN       !Median extraction
                  CALL EXTR2M(MADRID(PNTRA),NPIXA(1),NPIXA(2),
     +              MADRID(PNTRB),NPIXB(1),NPIXB(2),MADRID(PNTRC),NWDTH,
     +              IORD,NPAR,PARAMS,NA,DPAR,XMIN1,XMAX1,METH,
     +              STARTA,STEPA)
               ELSE                                   !Default extraction
                  CALL EXTR1M(MADRID(PNTRA),NPIXA(1),NPIXA(2),
     +              MADRID(PNTRB),NPIXB(1),NPIXB(2),
     +              IORD,NPAR,PARAMS,NA,DPAR,XMIN1,XMAX1,METH,
     +              STARTA,STEPA)
               ENDIF

               XMIN   = AMIN1(XMIN,XMIN1)
               XMAX   = AMAX1(XMAX,XMAX1)
            ENDDO
 10      CONTINUE
      ELSE
         CALL STTPUT(' Error in dispersion coefficients ',ISTAT)
         GO TO 20
      END IF
C
C  descriptor handling
C
      CUTS(1) = XMIN
      CUTS(2) = XMAX
      CUTS(3) = XMIN
      CUTS(4) = XMAX
      CUTS(5) = 0.
      CUTS(6) = 0.
      CALL DSCUPT(IMNO,INO,' ',STATUS)
      CALL STDWRR(INO,'LHCUTS',CUTS,1,6,KUN,STATUS)
      CALL STDWRI(INO,'HS',HS,1,1,KUN,STATUS)
C
C  free data
C
C      CALL STFCLO(INO,STATUS)
 20   CALL STSEPI
      END

      SUBROUTINE EXTR1M(IN,NPIXA1,NPIXA2,OUT,NB1,NB2,INDX,NPAR,PAR,ND,
     +     DPAR,XMIN,XMAX,METH,STARTA,STEPA)
C
C       EXTRACTION ROUTINE - BILINEAR INTERPOLATION
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C XMIN   REAL*4  MINIMUM VALUE
C XMAX   REAL*4  MAXIMUMVALUE
C
      IMPLICIT NONE
      INTEGER  NB1, NB2, NPAR, NPIXA1, NPIXA2, ND
      REAL OUT(NB1,NB2),PAR(NPAR)
      REAL IN(NPIXA1,NPIXA2)
      REAL XMIN,XMAX
      DOUBLE PRECISION DPAR(ND),STARTA(2),STEPA(2)
      DOUBLE PRECISION XVAL,YVAL,AX
      DOUBLE PRECISION WIDTH,SKEW,HWIDTH,OFFSET,DC
      DOUBLE PRECISION YV,WW(50)
      DOUBLE PRECISION SKWDX,SKWDY,SKWOF,SUM
      DOUBLE PRECISION DEGREE,DD,DSIN,DCOS
      INTEGER          NWW, IWIDTH, IYLIM, NX, N1, N2, K, L, IX, INDX
      INTEGER          IX1, IP, L1, K1, IW
      LOGICAL IF1D
      INTEGER          LOSLIT, HISLIT
      DOUBLE PRECISION YLOW, YHIGH, WSUM
      CHARACTER*1      METH
      DOUBLE PRECISION IX_R8R8, XI_R8R8
C
C     DEGREE = RAD / DEGREE
C
      DATA DEGREE/0.017453292519943295769237D+0/
C

      NWW    = 50
C
C ... define extraction constants
C
      WIDTH  = PAR(1)
      IWIDTH = WIDTH
      HWIDTH = 0.5*WIDTH
      SKEW   = PAR(2)
      SKWDX  = 0.0
      SKWDY  = 1.0
      SKWOF  = 0.0
      XMIN   = 0.0
      XMAX   = XMIN
C      OFFSET = PAR(3) - HWIDTH + 0.5
      OFFSET = PAR(3)
      IF (SKEW.NE.0.0) THEN       
         DD      = SKEW*DEGREE
         SKWDX  = -DSIN(DD)     !   DX PER PIXEL
         SKWDY  = DCOS(DD)      !   DY PER PIXEL
         SKWOF  = -SKWDX*HWIDTH !   2D INTERPOLATION
         IF1D   = .FALSE.
      END IF
      AX     = 1.D0 + SKWOF
      IYLIM  = NPIXA2 - IWIDTH
      NX     = SKWOF
      N1     = NX
      N2     = NPIXA1 - NX
C
C ... get ordinate values
C
      K      = PAR(8)
      L      = PAR(9)
      YV     = PAR(10)
C
C ... loop over the samples
C
      DO 10 IX = 1,N1
         OUT(IX,INDX) = 0.
 10   CONTINUE

      DO 100 IX = N2 + 1,NPIXA1
         OUT(IX,INDX) = 0.
 100  CONTINUE

      DO 50 IX1 = N1 + 1,N2

         IP     = 0
         DC     = 1.D0
         YVAL   = 0.D0
         XVAL   = IX_R8R8(AX,STARTA(1),STEPA(1))

C Computes position of the center of the slit from the coefficients of
C the bivariate polynomial stored in table order.

         DO 30 L1 = 0,L
            IP     = IP + 1
            WW(IP) = DC
            YVAL   = YVAL + WW(IP)*DPAR(IP)
            DO 20 K1 = 1,K
               IP     = IP + 1
               WW(IP) = WW(IP-1)*XVAL
               YVAL   = YVAL + WW(IP)*DPAR(IP)
 20         CONTINUE
            DC     = DC*YV
 30      CONTINUE
C
C Corrects the central position for the offset
C OFFSET does not take into account HWIDTH 
C
         YVAL   = XI_R8R8(YVAL,STARTA(2),STEPA(2)) + OFFSET
C     
         YLOW   = YVAL - HWIDTH
         YHIGH  = YVAL + HWIDTH
         LOSLIT = INT(YLOW)
         HISLIT = INT(YHIGH)

         IF (LOSLIT.LT.1)      LOSLIT = 1
         IF (HISLIT.GT.NPIXA2) HISLIT = NPIXA2

         SUM    = 0.D0
         WSUM   = 0.D0
         IX     = INT(AX)
C
         IF (LOSLIT.EQ.HISLIT) THEN
            SUM = (YHIGH-YLOW)*IN(IX,LOSLIT)
C
         ELSE
C
            DO 40 IW = LOSLIT,HISLIT
               
               IF (IW.EQ.LOSLIT) THEN
                  SUM = SUM + IN(IX,IW)*(LOSLIT+1.-YLOW)
                  
               ELSE IF (IW.EQ.HISLIT) THEN
                  SUM = SUM + IN(IX,IW)*(YHIGH-HISLIT)

               ELSE
                  SUM = SUM + IN(IX,IW)

               ENDIF


 40         CONTINUE
C
         ENDIF
C
         IF (METH(1:1) .EQ. 'L') THEN ! Linear extraction
            OUT(IX1,INDX) = SUM
         ELSE                   ! Average extraction
            OUT(IX1,INDX) = SUM/WIDTH
         ENDIF
         XMIN   = AMIN1(XMIN,OUT(IX1,INDX))
         XMAX   = AMAX1(XMAX,OUT(IX1,INDX))
         AX     = AX + 1.D0
 50   CONTINUE

      RETURN

      END



      SUBROUTINE EXTR2M(IN,NPIXA1,NPIXA2,OUT,NB1,NB2,COL,NWDTH,INDX,NPAR
     $     ,PAR,ND,DPAR,XMIN,XMAX,METH,STARTA,STEPA)
C
C       EXTRACTION ROUTINE - MEDIAN
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C XMIN   REAL*4  MINIMUM VALUE
C XMAX   REAL*4  MAXIMUMVALUE
C
      IMPLICIT NONE
      INTEGER  NB1, NB2, NPAR, NPIXA1, NPIXA2, NWDTH, NWDTH2, ND
      REAL OUT(NB1,NB2),PAR(NPAR)
      REAL IN(NPIXA1,NPIXA2),COL(NWDTH)
      REAL XMIN,XMAX,LIMITS(2)
      DOUBLE PRECISION DPAR(ND),STARTA(2),STEPA(2)
      DOUBLE PRECISION XVAL,YVAL,AX
      DOUBLE PRECISION WIDTH,SKEW,HWIDTH,OFFSET,DC
      DOUBLE PRECISION YV,WW(50)
      DOUBLE PRECISION SKWDX,SKWDY,SKWOF,SUM
      DOUBLE PRECISION DEGREE,DD,DSIN,DCOS
      INTEGER          NWW, IWIDTH, IYLIM, NX, N1, N2, K, L, IX, INDX
      INTEGER          IX1, IP, L1, K1, IW
      LOGICAL IF1D
      INTEGER          LOSLIT, HISLIT
      DOUBLE PRECISION YLOW, YHIGH, WSUM
      CHARACTER*1      METH
      DOUBLE PRECISION IX_R8R8, XI_R8R8
C
C     DEGREE = RAD / DEGREE
C
      DATA DEGREE/0.017453292519943295769237D+0/
      DATA LIMITS/0.0,0.0/
C     

      NWW    = 50
C
C ... define extraction constants
C
      WIDTH  = PAR(1)
      IWIDTH = WIDTH
      HWIDTH = 0.5*WIDTH
      NWDTH2 = HWIDTH
      SKEW   = PAR(2)
      SKWDX  = 0.0
      SKWDY  = 1.0
      SKWOF  = 0.0
      XMIN   = 0.0
      XMAX   = XMIN
C      OFFSET = PAR(3) - HWIDTH + 0.5
      OFFSET = PAR(3)
      IF (SKEW.NE.0.0) THEN       
          DD      = SKEW*DEGREE
          SKWDX  = -DSIN(DD)      !   DX PER PIXEL
          SKWDY  = DCOS(DD)       !   DY PER PIXEL
          SKWOF  = -SKWDX*HWIDTH  !   2D INTERPOLATION
          IF1D   = .FALSE.
      END IF
      AX     = 1.D0 + SKWOF
      IYLIM  = NPIXA2 - IWIDTH
      NX     = SKWOF
      N1     = NX
      N2     = NPIXA1 - NX
C
C ... get ordinate values
C
      K      = PAR(8)
      L      = PAR(9)
      YV     = PAR(10)
C
C ... loop over the samples
C
      DO 10 IX = 1,N1
         OUT(IX,INDX) = 0.
 10   CONTINUE

      DO 100 IX = N2 + 1,NPIXA1
         OUT(IX,INDX) = 0.
 100  CONTINUE

      DO 50 IX1 = N1 + 1,N2

         IP     = 0
         DC     = 1.D0
         YVAL   = 0.D0
         XVAL   = IX_R8R8(AX,STARTA(1),STEPA(1))

C Computes position of the center of the slit from the coefficients of
C the bivariate polynomial stored in table order.

         DO 30 L1 = 0,L
            IP     = IP + 1
            WW(IP) = DC
            YVAL   = YVAL + WW(IP)*DPAR(IP)
            DO 20 K1 = 1,K
               IP     = IP + 1
               WW(IP) = WW(IP-1)*XVAL
               YVAL   = YVAL + WW(IP)*DPAR(IP)
 20         CONTINUE
            DC     = DC*YV
 30      CONTINUE
C
C Corrects the central position for the offset
C OFFSET does not take into account HWIDTH 
C
         YVAL   = XI_R8R8(YVAL,STARTA(2),STEPA(2)) + OFFSET
C     
         YLOW   = YVAL - HWIDTH
         YHIGH  = YVAL + HWIDTH
         LOSLIT = INT(YLOW)
         HISLIT = INT(YHIGH)

         IF (LOSLIT.LT.1)      LOSLIT = 1
         IF (HISLIT.GT.NPIXA2) HISLIT = NPIXA2

         SUM    = 0.D0
         WSUM   = 0.D0
         IX     = INT(AX)
C
         IF (LOSLIT.EQ.HISLIT) THEN
            OUT(IX1,INDX) = IN(IX,LOSLIT)
         ELSE
            K1 = 0
            DO 40 IW = LOSLIT,HISLIT
               K1 = K1 + 1
               IF (IW.EQ.LOSLIT) THEN
                  COL(K1) = IN(IX,IW)!*(LOSLIT+1.-YLOW)
               ELSE IF (IW.EQ.HISLIT) THEN
                  COL(K1) = IN(IX,IW)!*MIN(1.0,YHIGH-HISLIT)
               ELSE
                  COL(K1) = IN(IX,IW)
               ENDIF
 40         CONTINUE            !determine MEDIAN:
            L1 = K1 / 2
            IF (K1.GT.0) THEN
               CALL GET_MEDIAN(COL,NWDTH,K1,L1,OUT(IX1,INDX),IW)
               IF (IW.LT.0) print *,'ERROR at ',IX1,INDX,' occured.'
            ELSE
               OUT(IX1,INDX) = -1.0
            ENDIF
         ENDIF
C
         XMIN   = AMIN1(XMIN,OUT(IX1,INDX))
         XMAX   = AMAX1(XMAX,OUT(IX1,INDX))
         AX     = AX + 1.D0
 50   CONTINUE

      RETURN

      END


      SUBROUTINE GET_MEDIAN(RA,N,NUSE,LL,VALU,STATUS)

      IMPLICIT NONE
C 
      INTEGER   N,LL,STATUS
      INTEGER   NUSE
C 
      REAL      RA(N),VALU
C 
      VALU = 0.0
      STATUS = 0
C 
      IF (NUSE.LE.3) THEN
         IF (NUSE.LT.1) THEN
            STATUS = -1
            RETURN
         ELSE IF (NUSE.LE.2) THEN
            STATUS = 2
            VALU = RA(1)
c            VALU = (RA(1) + RA(2))/2.0               !for ndim = 1,2, median is 1st elem.
            RETURN
         ELSE 
            STATUS = 3
            VALU = RA(2)               !for ndim = 3, median is 2nd elem.
            RETURN
         ENDIF
      ENDIF
C 
      CALL NSORT(RA,NUSE,LL,VALU)
      RETURN
C 
      END




C ==============================================================================
C To Extract Keeping Spatial Resolution
C ==============================================================================

      SUBROUTINE EXTR1MS(IN,NPIXA1,NPIXA2,OUT,NB1,NB2,INDX,NPAR,PAR,ND,
     +     DPAR,XMIN,XMAX,METH,STARTA,STEPA)
C
C       EXTRACTION ROUTINE - BILINEAR INTERPOLATION
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C XMIN   REAL*4  MINIMUM VALUE
C XMAX   REAL*4  MAXIMUM VALUE
C
      IMPLICIT NONE

      INTEGER  NB1               !No of Samples
      INTEGER  NB2               !No of Orders
      INTEGER  NPAR              !No of Extraction Parameters
      INTEGER  NPIXA1            !No of Samples
      INTEGER  NPIXA2            !No of orders
      INTEGER  ND                !No of coefficients

      REAL OUT(NB1,NB2)          !Extracted Orders
      REAL PAR(NPAR)             !Extraction Parameters
      REAL IN(NPIXA1,NPIXA2)     !Input image
      REAL XMIN                  !Imin
      REAL XMAX                  !Imax

      DOUBLE PRECISION DPAR(ND)  !Coeffs to define order position
      DOUBLE PRECISION STARTA(2) !Starting x,y pixel value
      DOUBLE PRECISION STEPA(2)  !Step x,y pixel value

      DOUBLE PRECISION XVAL      !X position in World Coordinates
      DOUBLE PRECISION YVAL      !Y position in world coordinates
      DOUBLE PRECISION AX        !1.D0 + SKWOF

      DOUBLE PRECISION WIDTH     !Extraction Slit width
      DOUBLE PRECISION SKEW      !Skewness: order inclination angle
      DOUBLE PRECISION HWIDTH    !Half width Extraction Slit
      DOUBLE PRECISION OFFSET    !Offset
      DOUBLE PRECISION DC        !DC = DC*YV (YV=PAR(10)=I=current extraction order

      DOUBLE PRECISION YV        !YV=PAR(10)=I=current extraction order
      DOUBLE PRECISION WW(50)    !Stores DC values in a loop from 0 to PAR(9)=IPAR(4)=??

      DOUBLE PRECISION SKWDX     !DX per pixel
      DOUBLE PRECISION SKWDY     !DY per Pixel
      DOUBLE PRECISION SKWOF     !2D INTERPOLATION
      DOUBLE PRECISION SUM       !Total extracted counts

      DOUBLE PRECISION DEGREE    !Coefficient to pass from Degree to Radians
      DOUBLE PRECISION DD        !Aux Var: Skewnes (order inclination angle) in radians
      DOUBLE PRECISION DSIN      !Double Precision Sin
      DOUBLE PRECISION DCOS      !Double Precision Cos

      INTEGER  NWW               !Parameter fixed to 50...but never used!
      INTEGER  IWIDTH            !Integer width of the extraction slit
      INTEGER  IYLIM             !IYLIM=NPIXA2 - IWIDTH...but never used!
      INTEGER  NX                !NX=SKWOF
      INTEGER  N1                !N1=NX
      INTEGER  N2                !N2=NPIXA1-NX
      INTEGER  K                 !K=PAR(8)
      INTEGER  L                 !L=PAR(9)
      INTEGER  IX                !Auxilliary variable for first index of output image
      INTEGER  INDX              !Curr. order ind. diff. from the min. detected order
      
      INTEGER  IX1               !Auxilliary variable for first index of output image
      INTEGER  IP                !Auxilliary index to compute the position of the center 
                                 !of the slit from the coefficients of the bivariate 
                                 !polynomial stored in table order.
      INTEGER  L1                !Auxilliary index 
      INTEGER  K1                !Auxilliary index 
      INTEGER  IW                !Auxilliary index running from LOWSLIT TO HIGHSLIT
      
      LOGICAL  IF1D             !Logical value = FALSE if skewnes.NE.0
      
      INTEGER  LOSLIT           !LOSLIT = INT(YLOW)
      INTEGER  HISLIT           !HISLIT = INT(YHIGH)
      
      DOUBLE PRECISION YLOW     !YLOW    = YVAL - HWIDTH
      DOUBLE PRECISION YHIGH    !YHIGH   = YVAL + HWIDTH
      DOUBLE PRECISION WSUM     !Never used!!
      
      CHARACTER*1      METH     !Extraction Method
      
      DOUBLE PRECISION IX_R8R8  !Function to pass from pixel to world coordinates
      DOUBLE PRECISION XI_R8R8  !Function to pass from world to pixel coordinates
C
C     DEGREE = RAD / DEGREE
C
      DATA DEGREE/0.017453292519943295769237D+0/
C

      NWW    = 50
C
C ... define extraction constants
C
      WIDTH  = PAR(1)
      IWIDTH = WIDTH
      HWIDTH = 0.5*WIDTH
      SKEW   = PAR(2)
      SKWDX  = 0.0
      SKWDY  = 1.0
      SKWOF  = 0.0
      XMIN   = 0.0
      XMAX   = XMIN
C      OFFSET = PAR(3) - HWIDTH + 0.5
      OFFSET = PAR(3)
      IF (SKEW.NE.0.0) THEN       
         DD      = SKEW*DEGREE
         SKWDX  = -DSIN(DD)     !   DX PER PIXEL
         SKWDY  = DCOS(DD)      !   DY PER PIXEL
         SKWOF  = -SKWDX*HWIDTH !   2D INTERPOLATION
         IF1D   = .FALSE.
      END IF
      AX     = 1.D0 + SKWOF
      IYLIM  = NPIXA2 - IWIDTH
      NX     = SKWOF
      N1     = NX
      N2     = NPIXA1 - NX
C
C ... get ordinate values
C
      K      = PAR(8)
      L      = PAR(9)
      YV     = PAR(10)
C
C ... loop over the samples
C
      DO 10 IX = 1,N1
         OUT(IX,INDX) = 0.
 10   CONTINUE

      DO 100 IX = N2 + 1,NPIXA1
         OUT(IX,INDX) = 0.
 100  CONTINUE

      DO 50 IX1 = N1 + 1,N2

         IP     = 0
         DC     = 1.D0
         YVAL   = 0.D0
         XVAL   = IX_R8R8(AX,STARTA(1),STEPA(1))

C Computes position of the center of the slit from the coefficients of
C the bivariate polynomial stored in table order.

         DO 30 L1 = 0,L
            IP     = IP + 1
            WW(IP) = DC
            YVAL   = YVAL + WW(IP)*DPAR(IP)
            DO 20 K1 = 1,K
               IP     = IP + 1
               WW(IP) = WW(IP-1)*XVAL
               YVAL   = YVAL + WW(IP)*DPAR(IP)
 20         CONTINUE
            DC     = DC*YV
 30      CONTINUE
C
C Corrects the central position for the offset
C OFFSET does not take into account HWIDTH 
C
         YVAL   = XI_R8R8(YVAL,STARTA(2),STEPA(2)) + OFFSET
C     
         YLOW   = YVAL - HWIDTH
         YHIGH  = YVAL + HWIDTH
         LOSLIT = INT(YLOW)
         HISLIT = INT(YHIGH)

         IF (LOSLIT.LT.1)      LOSLIT = 1
         IF (HISLIT.GT.NPIXA2) HISLIT = NPIXA2

         SUM    = 0.D0
         WSUM   = 0.D0
         IX     = INT(AX)
C
         IF (LOSLIT.EQ.HISLIT) THEN
            SUM = (YHIGH-YLOW)*IN(IX,LOSLIT)
C
         ELSE
C
            DO 40 IW = LOSLIT,HISLIT
               
               IF (IW.EQ.LOSLIT) THEN
                  SUM = SUM + IN(IX,IW)*(LOSLIT+1.-YLOW)
                  
               ELSE IF (IW.EQ.HISLIT) THEN
                  SUM = SUM + IN(IX,IW)*(YHIGH-HISLIT)

               ELSE
                  SUM = SUM + IN(IX,IW)

               ENDIF


 40         CONTINUE
C
         ENDIF
C
         IF (METH(1:1) .EQ. 'L') THEN ! Linear extraction
            OUT(IX1,INDX) = SUM
         ELSE                   ! Average extraction
            OUT(IX1,INDX) = SUM/WIDTH
         ENDIF
         XMIN   = AMIN1(XMIN,OUT(IX1,INDX))
         XMAX   = AMAX1(XMAX,OUT(IX1,INDX))
         AX     = AX + 1.D0
 50   CONTINUE

      RETURN

      END
