C $Id: necmerge2d.for,v 1.10 2001/07/23 09:12:42 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.10 $
C $Log: necmerge2d.for,v $
C Revision 1.10  2001/07/23 09:12:42  amodigli
C Eliminated some warinngs indicated by ftncheck
C
C Revision 1.9  2001/06/29 12:34:50  amodigli
C AMIN-AMAX always REAL, STARTA, STEPA,WI
C always DOUBLE, also to not have extra
C warnings on alpha
C
C Revision 1.8  2001/01/11 08:29:24  amodigli
C Some bug fixes to allow compilation on
C Linux+g77. Also shortening of subroutine
C names to less than 6 chars
C
C Revision 1.7  2001/01/04 12:31:26  amodigli
C Added Id, Name, Revision, Log conf. control info
C
C------------------------------------------------------------------------------
      PROGRAM ECHMRG
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  22:26 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR: D.PONZ
C
C
C.IDENTIFICATION
C
C  program  ECHMERG 
C
C.MODIFICATIONS
C
C  910128   P. Ballester   Define and set variables for mmake -i
C  991005   S.Wolf         ECM2O added: does an optimal merging.
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, ORDER MERGING
C
C.PURPOSE
C
C  OBTAIN A 1D FRM CALIBRATED IN WAVELENGTHS FROM A 2D FRM
C  SAMPLED IN THE SPACE WAVELENGTH-ORDER.
C
C.ALGORITHM
C
C  OVERLAPPING REGIONS OF CONSECUTIVE ORDERS ARE PROCESSED IN TWO WAYS:
C  - CONCATENATION OF CONSECUTIVE ORDERS, USING THE MIDDLE POINT AS
C    CONCATENATING POSITION (METHOD='CONCATENATE', DEFAULT)
C  - AVERAGE OF OVERLAPPING REGION (METHOD='AVERAGE')
C  - WEIGHTED AVERAGE OF OVERLAPPING REGION (METHOD='OPTIMAL')
C    spmerged = (w1*sp1 + w2*sp2)/(w1+w2)
C  - NO MERGING. INDIVIDUAL ORDERS ARE WRITTEN IN DIFFERENT FILES
C    (METHOD = 'NOMERGE')
C  - WEIGHTED AVERAGE. WEIGHTS PROPORTIONAL TO SINC**2 (METHOD='SINC')
C  THE METHOD 'NOMERGE' WILL PRODUCE AS MANY FILES AS DEFINED ORDERS
C   WITH NAMES xxxyyyy, xxxx IS THE OUTPUT FILE NAME, yyyy IS THE 
C   ORDER NUMBER
C
C.INPUT/OUTPUT
C
C  MERGE/ECHELLE INPUT OUTPUT  [CONC]
C  MERGE/ECHELLE INPUT OUTPUT  ORD1,ORD2 NOCONCAT
C  MERGE/ECHELLE INPUT OUTPUT  DELTA     AVERAGE
C  MERGE/ECHELLE INPUT OUTPUT  DELTA     OPTIMAL WEIGHTS
C  OR
C  MERGE/ECHELLE INPUT OUTPUT  TABLE SINC
C
C-
      IMPLICIT  NONE
      INTEGER HS_MAX                        !max size of slit
      INTEGER NMAX_ORDER                    !max forseen orders
      INTEGER NMAX_Y_OUT                    !max forseeen out ima Y size
      PARAMETER(NMAX_ORDER=100,HS_MAX=100)  !HS probably should be read 
                                            !from input frame
      PARAMETER(NMAX_Y_OUT=NMAX_ORDER*HS_MAX)                                            
      INTEGER   MADRID                      !Using this array the addresss 
                                            !of in/out data 
                                            !are passed to subsequent routines
                                            
      INTEGER   I
      INTEGER   II1                         !Auxilliary index used to count chars in name
      INTEGER   II2                         !Auxilliary index used in loop over the orders
      INTEGER   IORD1                       !Minimum val of orders of input frame
      
      INTEGER   IORD2                       !No of orders of input frame
      
      INTEGER*8 IPNTRA                      !Pointer to mapped data for input frame
      INTEGER*8 IPNTRB                      !Pointer to mapped data for output frame
      INTEGER*8 IPNTRV                      !Pointer to mapped data for variance frame
      INTEGER*8 IPNTRW                      !Pointer to mapped data for weight frame
      
      INTEGER   ISTAT                       !Status parameter indicating success of op.
      
      INTEGER   NAXISA                      !No of axis of input frame
      INTEGER   NAXISB                      !No of axis of output frame
      INTEGER   NAXISW                      !No of axis of weight frame
      

C
      INTEGER NPIXA(2)                      !No of pix per axis of input frame
      INTEGER NPIXB(2)                      !No of pix per axis of output frame
      INTEGER NPIXW(2)                      !No of pix per axis of weight frame
      INTEGER NPTOT(NMAX_Y_OUT)             !Buffer for data containing nptot
      INTEGER ICOL(3)                       !Buffer for data containing table column values
      INTEGER KUN                           !Units
      INTEGER KNUL                          !No of null values
      
      INTEGER INIMA                         !File id of frame: input ima
      INTEGER OUTIMA                        !File id of frame: output ima
      INTEGER VARIMA                        !File id of frame: variance ima
      INTEGER WGTIMA                        !File id of frame: weight ima
      INTEGER TID                           !File id of table
      INTEGER HS                            !Actual size of the extraction slit      

      REAL CUT(4)                           !Cut values
      REAL XMIN                             !Xmin
      REAL XMAX                             !Xmax
      REAL WRANGE(2)                        !Buffer for data values for INPUTR
      
      REAL VAL(3)                           !Buffer for data values from a table
      REAL K(100)                           !Array to hold data from one col of a table
      REAL A(100)                           !Array to hold data from one col of a table
      REAL RO(100)                          !Array to hold data from one col of a table
      
      DOUBLE PRECISION    DEL               !Auxilliary variable for DELTA wavelength (range)
      DOUBLE PRECISION    WEND              !wav end
      DOUBLE PRECISION    WINIT             !wav init
      DOUBLE PRECISION    WSTEP             !wav step
            
      DOUBLE PRECISION  WSTART(NMAX_Y_OUT)  !Buffer for data containing wstart
      DOUBLE PRECISION  STARTA(2)           !Start values for input image
      DOUBLE PRECISION  STEPA(2)            !Step values for input image
      DOUBLE PRECISION  STARTB(2)           !Start values for output image
      DOUBLE PRECISION  STEPB(2)            !Step values for output image
      DOUBLE PRECISION  STARTW(2)           !Start values for weight image
      DOUBLE PRECISION  STEPW(2)            !Step values for weight image
      

      CHARACTER*60 INFRM                    !Char array for input frame name
      CHARACTER*60 OUTFRM                   !Char array for output frame name
      CHARACTER*60 VARFRM                   !Char array for variance frame name
      CHARACTER*60 WGTFRM                   !Char array for weight frame name
      
      CHARACTER OUTFIL*12                   !Char array for output frame name
      CHARACTER TABLE*12                    !Char array for table frame name

      CHARACTER METHOD*1                    !Merging method
      CHARACTER WS*5                        !Temporary string name
      
      CHARACTER IDENTA*72                   !ASCII identification for input frame
      CHARACTER IDENTB*72                   !ASCII identification for output frame
      CHARACTER CUNITA*64                   !Unit of each axis + data for input frame
      CHARACTER CUNITB*64                   !Unit of each axis + data for output frame
      
      
      
      
      
      
      CHARACTER IDENTW*72                   !ASCII identification for weight frame
      CHARACTER CUNITW*64                   !Unit of each axis + data for weight frame
      CHARACTER*16 LABEL(3)                 !Char array for label data
      
      LOGICAL NULL(3)
C
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'
      COMMON /VMR/MADRID(1)
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'
C
      DATA CUNITB/'FLUX            WAVELENGTH'/
      DATA LABEL(1)/'ORDER'/,LABEL(2)/'KFIT'/,LABEL(3)/'AFIT'/
C
C ... INITIALIZE SYSTEM
C
      CALL STSPRO('ECHMRG')
      INFRM = ' '
      OUTFRM = ' '
      VARFRM = ' '
      WGTFRM = ' '
      CALL STKRDC('P1',1,1,60,I,INFRM,KUN,KNUL,ISTAT)
      CALL STKRDC('P2',1,1,60,I,OUTFRM,KUN,KNUL,ISTAT)
      CALL STKRDC('P6',1,1,60,I,VARFRM,KUN,KNUL,ISTAT)
      CALL STKRDC('P5',1,1,60,I,WGTFRM,KUN,KNUL,ISTAT)
      CALL CLNFRA(INFRM,INFRM,0)
      CALL CLNFRA(OUTFRM,OUTFRM,0)
      CALL CLNFRA(VARFRM,VARFRM,0)
      CALL CLNFRA(WGTFRM,WGTFRM,0)
      CALL STKRDC('P4',1,1,1,I,METHOD,KUN,KNUL,ISTAT)
      CALL FORUPC(METHOD,METHOD)
      IF (METHOD.EQ.'S') 
     .    CALL STKRDC('P3',1,1,8,I,TABLE,KUN,KNUL,ISTAT)
      CALL STKRDR('INPUTR',1,2,I,WRANGE,KUN,KNUL,ISTAT)
      IF (METHOD.EQ.'A' .OR. METHOD.EQ.'O') THEN
          WINIT  = 0.D0
          WEND   = WINIT
          DEL    = WRANGE(1)
      ELSE
          WINIT  = WRANGE(1)
          WEND   = WRANGE(2)
      END IF
C
C ... MAP INPUT FRM
C
      CALL STIGET(INFRM,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            3,NAXISA,NPIXA,STARTA,STEPA,IDENTA,CUNITA,
     .            IPNTRA,INIMA,ISTAT)
      CALL STDRDD(INIMA,'WSTART',1,NPIXA(2),I,
     .            WSTART,KUN,KNUL,ISTAT)
      CALL STDRDI(INIMA,'NPTOT',1,NPIXA(2),I,NPTOT,
     .            KUN,KNUL,ISTAT)
      CALL STKRDI('HS',1,1,I,HS,KUN,KNUL,ISTAT)
C ... CHECK IF NPTOT(NPIXA(2)) IS 0 
c     the following may happen if the order hits the detector's border
5     IF (NPTOT(NPIXA(2)).EQ.0) THEN  
          NPIXA(2) = NPIXA(2) - 1
          GOTO 5
      ELSE
         CONTINUE          
      END IF
      WSTEP  = STEPA(1)
C      print *,'WSTEP=',WSTEP
C
C ... MAP WEIGHT FRM
C
      IF (METHOD.EQ.'O') THEN
         CALL STIGET(WGTFRM,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .        3,NAXISW,NPIXW,STARTW,STEPW,IDENTW,CUNITW,
     .        IPNTRW,WGTIMA,ISTAT)
         IF (NAXISW.NE.NAXISA.OR.
     .        STARTW(1).NE.STARTA(1).OR.STARTW(2).NE.STARTA(2).OR.
     .        STEPW(1).NE.STEPA(1).OR.STEPW(1).NE.STEPA(1)) THEN
            GOTO 9998
         END IF
      END IF
      IF (METHOD.EQ.'N') THEN
C
C ... NO MERGE - PRODUCE ONE FILE PER ORDER
C
C          print *,'Method=N'
          IF (WINIT.EQ.WEND .AND. WINIT.LT.0.5) THEN
              IORD1  = 1
              IORD2  = NPIXA(2)
          ELSE
              IORD1  = MAX(NINT(WINIT),1)
              IORD2  = MIN(NINT(WEND),NPIXA(2))
          END IF
C          print *,'IORD1=',IORD1,'IORD2=',IORD2          
          OUTFRM = OUTFRM(1:59)//' '
          II1    = INDEX(OUTFRM,' ') - 1
          II1    = MIN(II1,4)
          OUTFIL = OUTFRM(1:II1)
          DO 10 I = IORD1,IORD2
              II2    = 10000 + I
              WRITE (WS,9000) II2
              OUTFIL(II1+1:II1+4) = WS(2:5)
              IF (HS.EQ.1) THEN 
                  NAXISB    = 1
                  NPIXB(1)  = NPTOT(I)
                  NPIXB(2)  = 1
                  STARTB(1) = WSTART(I)
                  STARTB(2) = 1.D0
                  STEPB(1)  = WSTEP
                  STEPB(2)  = 0.D0
              ELSE
                  NAXISB    = 2
                  NPIXB(1)  = NPTOT(I)
                  NPIXB(2)  = HS
                  STARTB(1) = WSTART(I*HS)
                  STARTB(2) = 1.D0
                  STEPB(1)  = WSTEP
                  STEPB(2)  = 0.D0              
              ENDIF

C              print *,'NAXISB=',NAXISB
C              print *,'NPIXB(1)=',NPIXB(1)
C              print *,'NPIXB(2)=',NPIXB(2)
C              print *,'STARTB(1)=',STARTB(2)
C              print *,'STEPB(1)=',STEPB(1)
C              print *,'STEPB(2)=',STEPB(2)
                        
              WRITE (IDENTB,9010) I
              IDENTB(11:72) = IDENTA(1:62)
              CALL STIPUT(OUTFIL,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     .                    NAXISB,NPIXB,STARTB,STEPB,
     .                     IDENTB,CUNITB,IPNTRB,OUTIMA,ISTAT)
     
              IF (HS.EQ.1) THEN      
                 CALL COPY(MADRID(IPNTRA),NPIXA(1),NPIXA(2),
     +                     MADRID(IPNTRB),NPIXB(1),I,XMIN,XMAX)
              ELSE
                  CALL SCOPY(MADRID(IPNTRA),NPIXA(1),NPIXA(2),
     +                       MADRID(IPNTRB),NPIXB(1),I,XMIN,XMAX,HS)            
              ENDIF
              CUT(1) = XMIN
              CUT(2) = XMAX
              CUT(3) = XMIN
              CUT(4) = XMAX
              CALL DSCUPT(INIMA,OUTIMA,' ',ISTAT)
              CALL STDWRR(OUTIMA,'LHCUTS',CUT,1,4,KUN,ISTAT)
C              CALL SXDFND(OUTFIL,'WSTART',TYPE,NOELEM,IBYTEL,STATUS)
C              DESNAM = 'WSTART'
C              IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFIL,DESNAM,STATUS)
C              CALL SXDFND(OUTFIL,'NPTOT',TYPE,NOELEM,IBYTEL,STATUS)
C              DESNAM = 'NPTOT'
C              IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFIL,DESNAM,STATUS)
C              CALL SXDFND(OUTFIL,'NORDER',TYPE,NOELEM,IBYTEL,STATUS)
C              DESNAM = 'NORDER'
C              IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFIL,DESNAM,STATUS)
C
              CALL STFCLO(OUTIMA,ISTAT)
              CALL STTPUT('File '//OUTFIL//' created ...',ISTAT)
   10     CONTINUE

      ELSE
C
C ... MAP OUTPUT FRAME
C
C          print *,'Method=C,A,O'
          IF (WINIT.EQ.WEND) THEN
              WINIT  = WSTART(1)
c             to take in count of the actual format of the image              
              WEND   = WSTART((NPIXA(2)-1)*HS+1) + 
     !                       (NPTOT((NPIXA(2)-1)*HS+1)-1)*WSTEP
c             wat was originally
c              WEND   = WSTART(NPIXA(2)) + (NPTOT(NPIXA(2))-1)*WSTEP
c             the following may happen if the order hits the detector's border              
              if (WEND .LT. 0.) THEN
                 NPIXA(2)=NPIXA(2)-1
                 WINIT  = WSTART(1)
                 WEND   = WSTART(NPIXA(2)) + (NPTOT(NPIXA(2))-1)*WSTEP
              endif
          END IF
C           print *,'NPIXA(2)=',NPIXA(2),'WINIT=',WINIT,'WEND=',WEND
C          print *,'WSTART=',WSTART
C          print *,'WEND=',WEND 
          IF (HS.EQ.1) THEN         
              NAXISB    = 1
              NPIXB(1)  = (WEND-WINIT)/WSTEP + 1
              NPIXB(2)  = 1
              STARTB(1) = WINIT
              STARTB(2) = 1.D0
              STEPB(1)  = WSTEP
              STEPB(2)  = 0.D0
          ELSE
              NAXISB    = 2
              NPIXB(1)  = (WEND-WINIT)/WSTEP + 1
              NPIXB(2)  = HS
              STARTB(1) = WINIT
              STARTB(2) = 1.D0
              STEPB(1)  = WSTEP
              STEPB(2)  = 1.D0
          
          END IF
          
C          print *,'NAXISB=',NAXISB
C          print *,'NPIXB(1)=',NPIXB(1)
C          print *,'NPIXB(2)=',NPIXB(2)
C          print *,'STARTB(1)=',STARTB(1)
C          print *,'STARTB(2)=',STARTB(2)
C          print *,'STEPB(1)=',STEPB(1)
C          print *,'STEPB(2)=',STEPB(2)
                                                  
          CALL STIPUT(OUTFRM,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     .                    NAXISB,NPIXB,STARTB,STEPB,
     .                    IDENTA,CUNITB,IPNTRB,OUTIMA,ISTAT)
C
C ... METHODS OF OVERLAPPING
C
          IF (METHOD.EQ.'C') CALL ECM1(MADRID(IPNTRA),
     +                            NPIXA(1),NPIXA(2),STARTA,
     +                            STEPA,WSTART,NPTOT,MADRID(IPNTRB),
     +                            NPIXB,STARTB,XMIN,XMAX)
          IF (METHOD.EQ.'A') THEN
             IF (HS.EQ.1) THEN
                             CALL ECM2(MADRID(IPNTRA),
     +                            NPIXA(1),NPIXA(2),STARTA,
     +                            STEPA,WSTART,NPTOT,MADRID(IPNTRB),
     +                            NPIXB,STARTB,XMIN,XMAX,DEL)
             ELSE
     
                             CALL ECM2S(MADRID(IPNTRA),
     +                            NPIXA(1),NPIXA(2),STARTA,
     +                            STEPA,WSTART,NPTOT,MADRID(IPNTRB),
     +                            NPIXB,STARTB,XMIN,XMAX,DEL,HS)
     
             END IF
          END IF
     
          IF (METHOD.EQ.'O') THEN
                       
             CALL STIPUT(VARFRM,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     .            NAXISB,NPIXB,STARTB,STEPB,
     .            IDENTA,CUNITB,IPNTRV,VARIMA,ISTAT)
             IF (HS.EQ.1) THEN
                 CALL ECM2O(MADRID(IPNTRA),MADRID(IPNTRW),
     +                NPIXA(1),NPIXA(2),STARTA,
     +                STEPA,WSTART,NPTOT,
     +                MADRID(IPNTRB),MADRID(IPNTRV),
     +                NPIXB,STARTB,XMIN,XMAX,DEL)
             ELSE
                 CALL ECM2SO(MADRID(IPNTRA),MADRID(IPNTRW),
     +                NPIXA(1),NPIXA(2),STARTA,
     +                STEPA,WSTART,NPTOT,
     +                MADRID(IPNTRB),MADRID(IPNTRV),
     +                NPIXB,STARTB,XMIN,XMAX,DEL,HS)
             ENDIF
             CALL DSCUPT(INIMA,VARIMA,' ',ISTAT)

                  
          ENDIF
          IF (METHOD.EQ.'S') THEN
C
C ... READ RIPPLE TABLE
C
              CALL TBTOPN(TABLE,F_IO_MODE,TID,ISTAT)
              CALL TBLSER(TID,LABEL(1),ICOL(1),ISTAT)
              CALL TBLSER(TID,LABEL(2),ICOL(2),ISTAT)
              CALL TBLSER(TID,LABEL(3),ICOL(3),ISTAT)
              DO 20 I = 1,NPIXA(2)
                  CALL TBRRDR(TID,I,3,ICOL,VAL,NULL,ISTAT)
                  RO(I)  = VAL(1)
                  K(I)   = VAL(2)
                  A(I)   = VAL(3)
   20         CONTINUE
              CALL ECM3(MADRID(IPNTRA),
     +                      NPIXA(1),NPIXA(2),STARTA,STEPA,WSTART,
     +                      NPTOT,MADRID(IPNTRB),NPIXB,STARTB,XMIN,XMAX,
     +                      RO,K,A)
          END IF
C
C ... WRITE PROCESS DESCRIPTORS
C
          CUT(1) = XMIN
          CUT(2) = XMAX
          CUT(3) = XMIN
          CUT(4) = XMAX
          CALL DSCUPT(INIMA,OUTIMA,' ',ISTAT)
          CALL STDWRR(OUTIMA,'LHCUTS',CUT,1,4,KUN,ISTAT)
C          CALL STTPUT(' Warning: modified code ECHMERG',STATUS)
C          CALL SXDFND(OUTFRM,'WSTART',TYPE,NOELEM,IBYTEL,STATUS)
C          IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFRM,'WSTART',STATUS)
C          CALL SXDFND(OUTFRM,'NPTOT',TYPE,NOELEM,IBYTEL,STATUS)
C          IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFRM,'NPTOT',STATUS)
C          CALL SXDFND(OUTFRM,'NORDER',TYPE,NOELEM,IBYTEL,STATUS)
C          IF (TYPE(1:1).NE.' ') CALL SXDDEL(OUTFRM,'NORDER',STATUS)
      END IF
      GOTO 9999
 9998 CALL STTPUT
     $     ('Error: Bad weight map! '/
     $     /'(START, STEP and NAXIS same as in INPUT frame !?)',ISTAT)
 9999 CALL STSEPI
 9000 FORMAT (I5)
 9010 FORMAT ('ORDER:',I3)
      END

      SUBROUTINE ECM1(X,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,NY,YSTR,
     +                  XMIN,XMAX)
C
C MERGE THE ORDERS, USING SIMPLE CONCATENATION IN THE MIDDLE POINT
      IMPLICIT NONE
C
      INTEGER  NY                      !No of pix per axis of output frame
      INTEGER  J2                      !Auxilliary variable: No of points to the order's end 
      INTEGER  I                       !Auxilliary variable for loop
      INTEGER  J1                      !Auxilliary variable: No of points from the order's start
      INTEGER  JOFF                    !Auxilliary variable: offset in points
      INTEGER  J                       !Auxilliary variable for loop
      INTEGER  JJ                      !Auxilliary variable for loop
      
      INTEGER  NPIXA1                  !No of pix per axis of input frame: X axis
      INTEGER  NPIXA2                  !No of pix per axis of input frame: Y axis
      INTEGER  NP(NPIXA2)              !Buffer for data containing nptot
      
      REAL     XMIN                    !Xmin
      REAL     XMAX                    !Xmax
      REAL     WEN1                    !Auxilliary variable with wav end
      REAL     WST2                    !Auxilliary variable with wav start
      REAL     WSTART                  !Auxilliary variable with starting wav
      REAL     X(NPIXA1,NPIXA2)        !Input frame
      REAL     Y(NY)                   !Output image
      REAL     AMAX1                   !Function to the the maximum from 2 values
      REAL     AMIN1                   !Function to the the minimum from 2 values
C 
      
      DOUBLE PRECISION STARTA(2)       !Start value
      DOUBLE PRECISION STEPA(2)        !Step value
      DOUBLE PRECISION WI(NPIXA2)      !Starting wavelength
      DOUBLE PRECISION YSTR            !Starting point for output image
C 

      DOUBLE PRECISION WEND            !Auxilliary variable to hold wav end    
      DOUBLE PRECISION YSTEP           !Auxilliary variable to hold wav step
      DOUBLE PRECISION YEND            !Auxilliary variable to hold wav end
      DOUBLE PRECISION WSTR            !Auxilliary variable to hold wav start
C
      XMIN   = 0.
      XMAX   = 0.
      DO 10 I = 1,NY
          Y(I)   = 0.
   10 CONTINUE
      YSTEP  = STEPA(1)
      YEND   = YSTR + (NY-1)*YSTEP
C
C ... ITERATION ON ORDERS
C
      WEND   = 0.D0
      DO 30 I = 1,NPIXA2
          WSTR   = DMAX1(WI(I),WEND+YSTEP)
          IF (I.EQ.NPIXA2) THEN
              WEND   = WI(I) + (NP(I)-1)*YSTEP
          ELSE
              WEN1   = WI(I) + (NP(I)-1)*YSTEP
              WST2   = WI(I+1)
              IF (WST2.LT.WEN1) THEN
                  WEND   = 0.5D0* (WEN1+WST2)
              ELSE
                  WEND   = WEN1
              END IF
          END IF

          IF (WSTR.GE.YEND) RETURN
          IF (WEND.GT.YSTR) THEN
C
C ... ITERATION ON WAVELENGTHS
C
              WSTART = DMAX1(YSTR,WSTR)
              WEND   = DMIN1(WEND,YEND)
              J1     = NINT((WSTART-WI(I))/YSTEP) + 1
              J2     = NINT((WEND-WI(I))/YSTEP) + 1
              JOFF   = NINT((WI(I)-YSTR)/YSTEP)
              DO 20 J = J1,J2
                  JJ     = J + JOFF
                  IF (JJ.GT.0) THEN
                      Y(JJ)  = X(J,I)
                      XMAX   = AMAX1(XMAX,Y(JJ))
                      XMIN   = AMIN1(XMIN,Y(JJ))
                  END IF

   20         CONTINUE
          END IF

   30 CONTINUE
      RETURN

      END

      SUBROUTINE ECM2(X,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,NY,YSTR,
     +                  XMIN,XMAX,DEL)
C
C MERGE THE ORDERS, AVERAGE ON THE OVERLAPPING REGION
C
      IMPLICIT NONE
C
      INTEGER NPIXA1               !No of pix per axis of input frame: X axis
      INTEGER NPIXA2               !No of pix per axis of input frame: Y axis
      INTEGER NY                   !No of pix per axis of output frame
      
      INTEGER NP(NPIXA2)           !Buffer for data containing nptot
      INTEGER IORD1                !Auxiliary variable for minimum order
      INTEGER IORD2                !Auxilliary variaable for maximum order
      INTEGER IPIX                 !Auxilliary variable for a loop
      INTEGER IPIX1                !Auxilliary variable indicating number of pixels
      INTEGER IPIX2                !Auxilliary variable indicating number of pixels
      
      REAL X(NPIXA1,NPIXA2)        !Input frame
      REAL Y(NY)                   !Output image
      REAL XMIN                    !Xmin
      REAL XMAX                    !Xmax
      
      DOUBLE PRECISION STARTA(2)   !Start value
      DOUBLE PRECISION STEPA(2)    !Step value
      DOUBLE PRECISION YSTR        !Starting point for output image
      DOUBLE PRECISION DEL         !Auxilliary variable for DELTA wavelength
      DOUBLE PRECISION WI(NPIXA2)  !Starting wavelength
      

C

      
      DOUBLE PRECISION YSTEP       !Auxilliary variable for wav step
      DOUBLE PRECISION W0          !Auxilliary variable
      DOUBLE PRECISION W1          !Auxilliary variable
      DOUBLE PRECISION WL          !Auxilliary variable
      DOUBLE PRECISION P1          !Auxilliary variable
      DOUBLE PRECISION P2          !Auxilliary variable
      REAL   AMAX1                 !Maximum of a function
      REAL   AMIN1                 !Minimum of to values
C
      XMIN   = 0.
      XMAX   = 0.
      YSTEP  = STEPA(1)
      IORD1  = 1
      IORD2  = 2
      W0     = WI(IORD2) + DEL
      W1     = WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL
C
C ... ITERATION ON ORDERS
C
      DO 20 IPIX = 1,NY
          Y(IPIX) = 0.0
          WL     = YSTR + (IPIX-1)*YSTEP
          IF (WL.LT.W0) THEN
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              Y(IPIX) = X(IPIX1,IORD1)
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          ELSE IF (WL.LT.W1) THEN
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              IPIX2  = NINT((WL-WI(IORD2))/YSTEP) + 1
              P2     = (WL-W0)/ (W1-W0)
              P1     = 1.D0 - P2
              IF (X(IPIX1,IORD1).LE.0.0) THEN
                  P2     = 1.D0
                  P1     = 0.D0
              END IF
              IF (X(IPIX2,IORD2).LE.0.0) THEN
                  P2     = 0.D0
                  P1     = 1.D0
              END IF
              Y(IPIX) = X(IPIX1,IORD1)*P1 + X(IPIX2,IORD2)*P2
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          ELSE
              IORD1  = IORD1 + 1
              IF (IORD1.GT.NPIXA2) RETURN
              IORD2  = IORD2 + 1
              IF (IORD2.GT.NPIXA2) THEN
                  W0     = 1.D20
              ELSE
                  W0     = WI(IORD2) + DEL
              END IF
              W1= WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              Y(IPIX) = X(IPIX1,IORD1)
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          END IF
   20 CONTINUE
      RETURN
      END



C    ====================================================================
C    ====================================================================
C    ====================================================================




      SUBROUTINE ECM2S(X,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,NY,YSTR,
     +                  XMIN,XMAX,DEL,HS)
C
C MERGE THE ORDERS, AVERAGE ON THE OVERLAPPING REGION
C
      IMPLICIT NONE
C
      INTEGER NPIXA1               !No of pix per axis of input frame: X axis
      INTEGER NPIXA2               !No of pix per axis of input frame: Y axis
      INTEGER NY                   !No of pix per axis of output frame
      
      INTEGER NP(NPIXA2)           !Buffer for data containing nptot
      INTEGER IORD1                !Auxiliary variable for minimum order
      INTEGER IORD2                !Auxilliary variaable for maximum order
      INTEGER IPIX                 !Auxilliary variable for a loop
      INTEGER IPIX1                !Auxilliary variable indicating number of pixels
      INTEGER IPIX2                !Auxilliary variable indicating number of pixels
      INTEGER HS                   !slit dimension
      INTEGER J                    !loop variable
      
      REAL X(NPIXA1,NPIXA2)        !Input frame
      REAL Y(NY,HS)                   !Output image
      REAL XMIN                    !Xmin
      REAL XMAX                    !Xmax
      
      DOUBLE PRECISION STARTA(2)   !Start value
      DOUBLE PRECISION STEPA(2)    !Step value
      DOUBLE PRECISION YSTR        !Starting point for output image
      DOUBLE PRECISION DEL         !Auxilliary variable for DELTA wavelength
      DOUBLE PRECISION WI(NPIXA2)  !Starting wavelength
      

C

      
      DOUBLE PRECISION YSTEP       !Auxilliary variable for wav step
      DOUBLE PRECISION W0          !Auxilliary variable
      DOUBLE PRECISION W1          !Auxilliary variable
      DOUBLE PRECISION WL          !Auxilliary variable
      DOUBLE PRECISION P1          !Auxilliary variable
      DOUBLE PRECISION P2          !Auxilliary variable
      REAL   AMAX1                 !Maximum of a function
      REAL   AMIN1                 !Minimum of to values
C


C     ITERATION WITHIN AN ORDER
C      print *,'HS=',HS
      DO J=1,HS
      XMIN   = 0.
      XMAX   = 0.
      YSTEP  = STEPA(1)
      IORD1  = 1
      IORD2  = 2
      W0     = WI((IORD2-1)*HS+J) + DEL
      W1     = WI((IORD1-1)*HS+J) + 
     +         (NP((IORD1-1)*HS+J)-1)*YSTEP - DEL
c      print *,'W0=',W0,'W1=',W1


C
C ... ITERATION ON ORDERS
C
      DO IPIX = 1,NY            !Here iterates within an order on X points
          Y(IPIX,J) = 0.0       !Initializes result image point to zero
          WL = YSTR+(IPIX-1)*YSTEP !Finds wav at the actual X pix point
          IF (WL.LT.W0) THEN
              IPIX1  = NINT((WL-WI((IORD1-1)*HS+J))/YSTEP) + 1
              Y(IPIX,J) = X(IPIX1,(IORD1-1)*HS+J)     !Define Y(IPIX)
              XMAX   = AMAX1(Y(IPIX,J),XMAX)            !Define max of intensity
              XMIN   = AMIN1(Y(IPIX,J),XMIN)            !Define min of intensity
          ELSE IF (WL.LT.W1) THEN                     !In this case there is overlapping
              IPIX1  = NINT((WL-WI((IORD1-1)*HS+J))/YSTEP) + 1 !Actual point on lower order
              IPIX2  = NINT((WL-WI((IORD2-1)*HS+J))/YSTEP) + 1 !Actual point on upper order
              P2     = (WL-W0)/ (W1-W0)               !Weight of one term in merging
              P1     = 1.D0 - P2                      !Weight of the other in merging
              IF (X(IPIX1,(IORD1-1)*HS+J).LE.0.0) THEN         !Lower Order intensity=0
                  P2     = 1.D0                       !Weight of one term in merging
                  P1     = 0.D0                       !Weight of the other in merging
              END IF
              IF (X(IPIX2,(IORD2-1)*HS+J).LE.0.0) THEN         !Upper order intensity=0
                  P2     = 0.D0                       !Weight of one term in merging
                  P1     = 1.D0                       !Weight of the other in merging
              END IF
              Y(IPIX,J) = X(IPIX1,(IORD1-1)*HS+J)*P1 + 
     !                  X(IPIX2,(IORD2-1)*HS+J)*P2 !Weighted mean
              XMAX   = AMAX1(Y(IPIX,J),XMAX)            !Define max of intensity
              XMIN   = AMIN1(Y(IPIX,J),XMIN)            !Define min of intensity
          ELSE
              IORD1  = IORD1 + 1                      !Increase the order
              IF (IORD1.GT.NPIXA2) RETURN             !Exit if actual order > max
              IORD2  = IORD2 + 1                      !Increase the order
              IF (IORD2.GT.NPIXA2) THEN               !if actual order > max
                  W0     = 1.D20                      !Spec condition
              ELSE
                  W0     = WI((IORD2-1)*HS+J) + DEL    !Redefine W0
              END IF
              W1     = WI((IORD1-1)*HS+J) + 
     +                (NP((IORD1-1)*HS+J)-1)*YSTEP - DEL !Redefine W1
              IPIX1  = NINT((WL-WI((IORD1-1)*HS+J))/YSTEP) + 1        !Redefine IPIX1
              Y(IPIX,J) = X(IPIX1,(IORD1-1)*HS+J)                       !Define Y(IPIX)
              XMAX   = AMAX1(Y(IPIX,J),XMAX)                   !Define Imax
              XMIN   = AMIN1(Y(IPIX,J),XMIN)                   !Define Imin
          END IF
c          print *,'W0=',W0,'W1=',W1
c           print *,'IORD1=',IORD1,'IORD2=',IORD2
      ENDDO
      ENDDO
      RETURN
      END

C    ====================================================================
C    ====================================================================
C    ====================================================================




      SUBROUTINE ECM2SO(X,WGT,NPIXA1,NPIXA2,STARTA,STEPA,
     +                    WI,NP,Y,VAR,NY,YSTR,XMIN,XMAX,DEL,HS)
C
C MERGE THE ORDERS, AVERAGE ON THE OVERLAPPING REGION USING WEIGHTS
C
      IMPLICIT NONE
C
      INTEGER NPIXA1                !No of pix per axis of input frame: X axis
      INTEGER NPIXA2                !No of pix per axis of input frame: Y axis
      INTEGER NY                    !No of pix per axis of output frame
      
      INTEGER NP(NPIXA2)            !Buffer for data containing nptot
      INTEGER IORD1                 !Auxiliary variable for minimum order
      INTEGER IORD2                 !Auxiliary variable for maximum order
      INTEGER IPIX                  !Auxilliary variable for a loop
      INTEGER IPIX1                 !Auxilliary variable indicating number of pixels
      INTEGER IPIX2                 !Auxilliary variable indicating number of pixels
      INTEGER IBAD                  !Counter for undefined pixels
      INTEGER HS                    !slit dimension
      INTEGER J                     !loop variable
      REAL X(NPIXA1,NPIXA2)         !Input frame
      REAL  WGT(NPIXA1,NPIXA2)      !Weight frame
      REAL Y(NY,HS)                 !Output frame
      REAL VAR(NY,HS)               !Variance output frame
      REAL XMIN                     !Xmin
      REAL XMAX                     !Xmax
      
      DOUBLE PRECISION STARTA(2)    !Start value
      DOUBLE PRECISION STEPA(2)     !Step value
      DOUBLE PRECISION YSTR         !Starting point for output image
      DOUBLE PRECISION DEL          !Wav range
      DOUBLE PRECISION WI(NPIXA2)   !Starting wavelength
      

C

      DOUBLE PRECISION YSTEP        !Wav step
      DOUBLE PRECISION WG1          !Auxilliary variable for a weight
      DOUBLE PRECISION WG2          !Auxilliary variable for a weight
      DOUBLE PRECISION W0           !Auxilliary variable
      DOUBLE PRECISION W1           !Auxilliary variable
      DOUBLE PRECISION WL           !Auxilliary variable
      REAL   AMAX1                  !Function to return the max of 2 values
      REAL   AMIN1                  !Function to return the min of 2 values
      
      CHARACTER*80 MES              !Char array for messages
C

C     ITERATION WITHIN AN ORDER
C      print *,'HS=',HS
      DO J=1,HS
      IBAD   = 0
      XMIN   = 0.
      XMAX   = 0.
      YSTEP  = STEPA(1)
      IORD1  = 1
      IORD2  = 2
      W0     = WI((IORD2-1)*HS+J) + DEL
      W1     = WI((IORD1-1)*HS+J) + 
     +         (NP((IORD1-1)*HS+J)-1)*YSTEP - DEL
c      print *,'W0=',W0,'W1=',W1
C
C ... ITERATION ON ORDERS
C
      DO IPIX = 1,NY
          Y(IPIX,J) = 0.0                 !Here iterates within an order on X points
          VAR(IPIX,J) = 0.0               !Here iterates within an order on X points
          WL     = YSTR + (IPIX-1)*YSTEP  !Finds wav at the actual X pix point
          IF (WL.LT.W0) THEN
              IPIX1  = NINT((WL-WI((IORD1-1)*HS+J))/YSTEP) + 1
              Y(IPIX,J) = X(IPIX1,(IORD1-1)*HS+J)     !Define Y(IPIX)
              VAR(IPIX,J) = WGT(IPIX1,(IORD1-1)*HS+J) !Define W(IPIX)
              IF (VAR(IPIX,J).NE.0.0) VAR(IPIX,J) = 1.0/VAR(IPIX,J)
              XMAX   = AMAX1(Y(IPIX,J),XMAX)            !Define max of intensity
              XMIN   = AMIN1(Y(IPIX,J),XMIN)            !Define min of intensity
          ELSE IF (WL.LT.W1) THEN                       !In this case there is overlapping
              IPIX1  = NINT((WL-WI((IORD1-1)*HS+J))/YSTEP) + 1 !Actual point on lower order
              IPIX2  = NINT((WL-WI((IORD2-1)*HS+J))/YSTEP) + 1 !Actual point on upper order
              WG1 = WGT(IPIX1,(IORD1-1)*HS+J)
              WG2 = WGT(IPIX2,(IORD2-1)*HS+J)

              IF (WG1.LT.1e-10 .AND. WG2.LT.1e-10) THEN
                 Y(IPIX,J) = 0.0
                 VAR(IPIX,J) = 0.0
                 IBAD = IBAD + 1
              ELSE
                 VAR(IPIX,J) = 1.0/(WG1 + WG2)
                 Y(IPIX,J) = X(IPIX1,(IORD1-1)*HS+J)*WG1 + 
     +                       X(IPIX2,(IORD2-1)*HS+J)*WG2
                 Y(IPIX,J) = Y(IPIX,J) * VAR(IPIX,J)
                 XMAX   = AMAX1(Y(IPIX,J),XMAX)       !Define max of intensity
                 XMIN   = AMIN1(Y(IPIX,J),XMIN)       !Define min of intensity
              END IF
          ELSE
              IORD1  = IORD1 + 1                     !Increase the order
              IF (IORD1.GT.NPIXA2) GOTO 99           !Exit if actual order > max
              IORD2  = IORD2 + 1                     !Increase the order
              IF (IORD2.GT.NPIXA2) THEN              !if actual order > max
                  W0     = 1.D20                     !Spec condition
              ELSE
                  W0     = WI((IORD2-1)*HS+J) + DEL  !Redefine W0
              END IF
              W1     = WI((IORD1-1)*HS+J) + 
     +                 (NP((IORD1-1)*HS+J)-1)*YSTEP - DEL  !Redefine W1
              IPIX1  = NINT((WL-WI((IORD1-1)*HS+J))/YSTEP) + 1 !Redefine IPIX1
              Y(IPIX,J) = X(IPIX1,(IORD1-1)*HS+J)              !Define Y(IPIX)
              VAR(IPIX,J) = WGT(IPIX1,(IORD1-1)*HS+J)          !Define Var
              IF (VAR(IPIX,J).NE.0.0) VAR(IPIX,J) = 1.0/VAR(IPIX,J)
              XMAX   = AMAX1(Y(IPIX,J),XMAX)                   !Define Imax
              XMIN   = AMIN1(Y(IPIX,J),XMIN)                   !Define Imin
          END IF
      ENDDO
 99   IF (IBAD.GT.0) THEN
         WRITE(MES,*) IBAD,' undefined pixels ... set to 0.0!'
         CALL STTPUT(MES,IBAD)
      END IF
      END DO
      RETURN
         
      END





      SUBROUTINE ECM2O(X,WGT,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,VAR,NY
     $     ,YSTR,XMIN,XMAX,DEL)
C
C MERGE THE ORDERS, AVERAGE ON THE OVERLAPPING REGION USING WEIGHTS
C
      IMPLICIT NONE
C
      INTEGER NPIXA1                !No of pix per axis of input frame: X axis
      INTEGER NPIXA2                !No of pix per axis of input frame: Y axis
      INTEGER NY                    !No of pix per axis of output frame
      
      INTEGER NP(NPIXA2)            !Buffer for data containing nptot
      
      REAL X(NPIXA1,NPIXA2)         !Input frame
      REAL  WGT(NPIXA1,NPIXA2)      !Weight frame
     
      DOUBLE PRECISION STARTA(2)    !Start value
      DOUBLE PRECISION STEPA(2)     !Step value
      DOUBLE PRECISION YSTR         !Starting point for output image
      DOUBLE PRECISION DEL          !Wav range
      DOUBLE PRECISION WI(NPIXA2)   !Starting wavelength
      
      REAL Y(NY)                    !Output frame
      REAL VAR(NY)                  !Variance output frame
      REAL XMIN                     !Xmin
      REAL XMAX                     !Xmax
C
      INTEGER IORD1                 !Auxiliary variable for minimum order
      INTEGER IORD2                 !Auxiliary variable for maximum order
      INTEGER IPIX                  !Auxilliary variable for a loop
      INTEGER IPIX1                 !Auxilliary variable indicating number of pixels
      INTEGER IPIX2                 !Auxilliary variable indicating number of pixels
      INTEGER IBAD                  !Counter for undefined pixels
      
      DOUBLE PRECISION YSTEP        !Wav step
      DOUBLE PRECISION WG1          !Auxilliary variable for a weight
      DOUBLE PRECISION WG2          !Auxilliary variable for a weight
      DOUBLE PRECISION W0           !Auxilliary variable
      DOUBLE PRECISION W1           !Auxilliary variable
      DOUBLE PRECISION WL           !Auxilliary variable
      REAL   AMAX1                  !Function to return the max of 2 values
      REAL   AMIN1                  !Function to return the min of 2 values
      
      CHARACTER*80 MES              !Char array for messages
C
      IBAD   = 0
      XMIN   = 0.
      XMAX   = 0.
      YSTEP  = STEPA(1)
      IORD1  = 1
      IORD2  = 2
      W0     = WI(IORD2) + DEL
      W1     = WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL
C
C ... ITERATION ON ORDERS
C
      DO 20 IPIX = 1,NY
          Y(IPIX) = 0.0
          VAR(IPIX) = 0.0
          WL     = YSTR + (IPIX-1)*YSTEP
          IF (WL.LT.W0) THEN
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              Y(IPIX) = X(IPIX1,IORD1)
              VAR(IPIX) = WGT(IPIX1,IORD1)
              IF (VAR(IPIX).NE.0.0) VAR(IPIX) = 1.0/VAR(IPIX)
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          ELSE IF (WL.LT.W1) THEN
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              IPIX2  = NINT((WL-WI(IORD2))/YSTEP) + 1
              WG1 = WGT(IPIX1,IORD1)
              WG2 = WGT(IPIX2,IORD2)

              IF (WG1.LT.1e-10 .AND. WG2.LT.1e-10) THEN
                 Y(IPIX) = 0.0
                 VAR(IPIX) = 0.0
                 IBAD = IBAD + 1
              ELSE
                 VAR(IPIX) = 1.0/(WG1 + WG2)
                 Y(IPIX) = X(IPIX1,IORD1)*WG1 + X(IPIX2,IORD2)*WG2
                 Y(IPIX) = Y(IPIX) * VAR(IPIX)
                 XMAX   = AMAX1(Y(IPIX),XMAX)
                 XMIN   = AMIN1(Y(IPIX),XMIN)
              ENDIF
          ELSE
              IORD1  = IORD1 + 1
              IF (IORD1.GT.NPIXA2) GOTO 99
              IORD2  = IORD2 + 1
              IF (IORD2.GT.NPIXA2) THEN
                  W0     = 1.E20
              ELSE
                  W0     = WI(IORD2) + DEL
              END IF
              W1     = WI(IORD1) + (NP(IORD1)-1)*YSTEP - DEL
              IPIX1  = NINT((WL-WI(IORD1))/YSTEP) + 1
              Y(IPIX) = X(IPIX1,IORD1)
              VAR(IPIX) = WGT(IPIX1,IORD1)
              IF (VAR(IPIX).NE.0.0) VAR(IPIX) = 1.0/VAR(IPIX)
              XMAX   = AMAX1(Y(IPIX),XMAX)
              XMIN   = AMIN1(Y(IPIX),XMIN)
          END IF
 20   CONTINUE
 99   IF (IBAD.GT.0) THEN
         WRITE(MES,*) IBAD,' undefined pixels ... set to 0.0!'
         CALL STTPUT(MES,IBAD)
      END IF
      RETURN
         
      END

      SUBROUTINE COPY(A,NA1,NA2,B,NB,I,XMIN,XMAX)
C
C COPY THE ORDER NUMBER I
C
      IMPLICIT NONE
      INTEGER NA1     !No of point on X axis
      INTEGER NA2     !No of point on Y axis
      INTEGER NB      !No of points on output image
      INTEGER I       !Auxilliary variable
      INTEGER J       !Auxilliary variable
      
      REAL A(NA1,NA2) !Input frame
      REAL B(NB)      !Output frame
      
      REAL XMIN       !Xmin
      REAL XMAX       !Xmax
C
      XMIN = 0.
      XMAX = 0.
C
      DO 10 J = 1,NB
          B(J)   = A(J,I)
          XMIN   = AMIN1(B(J),XMIN)
          XMAX   = AMAX1(B(J),XMAX)
   10 CONTINUE
      RETURN
      END




      SUBROUTINE SCOPY(A,NA1,NA2,B,NB,I,XMIN,XMAX,HS)
C
C 2D COPY OF THE ORDER NUMBER I
C
      IMPLICIT NONE
      INTEGER NA1     !No of point on X axis
      INTEGER NA2     !No of point on Y axis
      INTEGER NB      !No of points on output image
      INTEGER I       !Auxilliary variable
      INTEGER J       !Auxilliary variable
      INTEGER K       !Auxilliary variable
      INTEGER HS      !Size of the slit
      
      REAL A(NA1,NA2) !Input frame
      REAL B(NB,HS)   !Output frame
      
      REAL XMIN       !Xmin
      REAL XMAX       !Xmax
C
      XMIN = 0.
      XMAX = 0.
C
      DO K = 1,HS
      DO J = 1,NB
          B(J,K)   = A(J,I*HS+K)
          XMIN   = AMIN1(B(J,K),XMIN)
          XMAX   = AMAX1(B(J,K),XMAX)
      ENDDO
      ENDDO
      
      RETURN
      END
      
      



      SUBROUTINE ECM3(X,NPIXA1,NPIXA2,STARTA,STEPA,WI,NP,Y,NY,YSTR,
     +                    XMIN,XMAX,RORDER,K,A)
C
C      IMPLICIT NONE
C MERGE THE ORDERS, AVERAGE ON THE OVERLAPPING REGION WITH
C WEIGHTS PROPORTIONAL TO THE SINC**2
C
      IMPLICIT NONE

      INTEGER NY                 !No of pix per axis of output frame
      INTEGER IORD1              !Auxiliary variable for minimum order
      INTEGER IPIX               !Auxilliary variable to iterate on ouput frame wav
      INTEGER NF                 !Pixel in order range
      INTEGER IORD               !Auxilliary variable to do an iteration on orders
      INTEGER IPIX1              !Auxilliary variable containing pix position
      INTEGER I                  !Auxilliary variable for a loop
      INTEGER IFL                !Auxilliary variable to average over fluxes
      INTEGER IORD2              !Auxilliary variable for order value
      
      DOUBLE PRECISION  YSTR     !Starting wav for output frame
      REAL    XMIN               !Xmin
      REAL    XMAX               !Xmax
      REAL    YSTEP              !Step wav for output frame
      REAL    PI                 !Auxilliary variable
      REAL    DW                 !Auxilliary variable
      REAL    WL                 !Auxilliary variable for wav pos
      
      INTEGER NPIXA1             !No of pix per axis of input frame: X axis
      INTEGER NPIXA2             !No of pix per axis of input frame: Y axis

      INTEGER NP(NPIXA2)         !Buffer for data containing nptot

      REAL    X(NPIXA1,NPIXA2)   !Input image
      DOUBLE PRECISION STARTA(2) !Start value
      DOUBLE PRECISION STEPA(2)  !Step value
      
      REAL    Y(NY)              !Output image
      DOUBLE PRECISION WI(NPIXA2)!Starting wavelength
      REAL    RORDER(NPIXA2)     !Reference order table column value
      
      REAL    K(NPIXA2)          !Table column value
      REAL    A(NPIXA2)          !Table column value
      
      DOUBLE PRECISION DK        !Auxilliary variable
      DOUBLE PRECISION DA        !Auxilliary variable
      DOUBLE PRECISION PA        !Auxilliary variable
      DOUBLE PRECISION DM        !Auxilliary variable
      DOUBLE PRECISION DC        !Auxilliary variable
      DOUBLE PRECISION DX        !Auxilliary variable
      DOUBLE PRECISION WEIGHT(3) !Weight factor
      DOUBLE PRECISION FLUX(3)   !Flux value
      DOUBLE PRECISION SW        !Auxilliary variable for weights
      DOUBLE PRECISION SF        !Auxilliary variable for fluxes
C
      XMIN   = 0.
      XMAX   = 0.
      PI     = 0.     !  To be updated (ULTRIX installation)
      DW     = 0.     !  To be updated (ULTRIX installation)
      YSTEP  = STEPA(1)
      IORD1  = 1
      IORD2  = MIN(IORD1+2,NPIXA2)
C
C ... ITERATION ON OUTPUT WAVELENGTHS
C
      DO 30 IPIX = 1,NY
          Y(IPIX) = 0.
          FLUX(1) = 0.D0
          FLUX(2) = 0.D0
          FLUX(3) = 0.D0
          WEIGHT(1) = 0.D0
          WEIGHT(2) = 0.D0
          WEIGHT(3) = 0.D0
          NF     = 0
          WL     = YSTR + (IPIX-1)*YSTEP
C
C ... ITERATION ON ORDERS
C
          DO 10 IORD = IORD1,IORD2
              IPIX1  = (WL-WI(IORD))/YSTEP + 1
              IF (IPIX1.GT.5 .AND. IPIX1.LT. (NP(IORD)-5)) THEN
C
C ... PIXEL IN THE ORDER RANGE
C
                  NF     = NF + 1
                  FLUX(NF) = X(IPIX1,IORD)
                  DK     = K(IORD)
                  DA     = A(IORD)
                  PA     = PI*DA
                  DM     = RORDER(IORD)
                  DC     = DM/DK
                  DX     = (PA*DM*DC)* (DW-1.D0/DC)
                  IF (DABS(DX).LT.1.D-10) THEN
                      WEIGHT(NF) = 1.D0

                  ELSE
                      WEIGHT(NF) = 1.D0/ (DSIN(DX)/DX)**4
                  END IF

              END IF

   10     CONTINUE
C
C ... AVERAGE OVER FLUXES
C
          IF (NF.GE.1) THEN
              SW     = 0.D0
              SF     = 0.D0
              DO 20 IFL = 1,NF
                  SW     = SW + WEIGHT(IFL)
                  SF     = SF + WEIGHT(IFL)*FLUX(IFL)
   20         CONTINUE
              Y(IPIX) = SF/SW
              XMIN   = MIN(XMIN,Y(IPIX))
              XMAX   = MAX(XMAX,Y(IPIX))
          END IF
C
C ... CHECK ON ORDER LIMITS
C
          IPIX1  = (WL-WI(IORD1))/YSTEP + 1
          IF (IPIX1.GE.NP(IORD1)-5) THEN
              IORD1  = IORD1 + 1
              IORD2  = MIN(IORD1+2,NPIXA2)
          END IF

          IF (IORD1.GT.NPIXA2) RETURN
   30 CONTINUE
      DO 40 I = IPIX + 1,NY
          Y(I)   = 0.
   40 CONTINUE
      RETURN

      END
