C $Id: necopt.for,v 1.48 2001/08/08 13:16:44 amodigli Exp $
C===========================================================================
C Copyright (C) 1995 European Southern Observatory (ESO)
C
C This program is free software; you can redistribute it and/or 
C modify it under the terms of the GNU General Public License as 
C published by the Free Software Foundation; either version 2 of 
C the License, or (at your option) any later version.
C
C This program is distributed in the hope that it will be useful,
C but WITHOUT ANY WARRANTY; without even the implied warranty of
C MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
C GNU General Public License for more details.
C
C You should have received a copy of the GNU General Public 
C License along with this program; if not, write to the Free 
C Software Foundation, Inc., 675 Massachusetss Ave, Cambridge, 
C MA 02139, USA.
C
C Corresponding concerning ESO-MIDAS should be addressed as follows:
C	Internet e-mail: midas@eso.org
C	Postal address: European Southern Observatory
C			Data Management Division 
C			Karl-Schwarzschild-Strasse 2
C			D 85748 Garching bei Muenchen 
C			GERMANY
C===========================================================================
C
C @(#)echopt.for	2.4 (ESO-IPG) 12/6/91 12:26:19
C+
C.COPYRIGHT: Copyright (c) 1987 European Southern Observatory,
C                                         all rights reserved
C
C.VERSION: 1.0  ESO-FORTRAN Conversion, AA  21:13 - 3 DEC 1987
C
C.LANGUAGE: F77+ESOext
C
C.AUTHOR : M Peron, S. Wolf, A. Modigliani
C
C.IDENTIFICATION
C  program  ECHOPT  version 1.0 910120 
C
C.KEYWORDS
C
C  ECHELLE, CASPEC, DATA EXTRACTION
C
C.PURPOSE
C 
C Extract orders from an echelle spectrum, the order position is defined by 
C the coefficients of a regression. The slit width and the orders to be 
C extracted are defined by the keyword INPUTI
C The flux is calculated by assigning to each pixel a weighting factor 
C proportional to the amount of signal in it.
C.ALGORITHM
C The algorithm is based on a paper of Koji Mukai ( 1990, Optimal Extraction of
C Cross-Dispersed Spectra, Pub. of Astr.Soc. of the Pacific,102:183-189)
C AND a paper of Keith Horne:
C (1986, Pub. of Astr.Soc. of the Pacific, 98:609-617)
C
C.SYNTAX
C   EXTRACT/ORDER input output  width,ord1,ord2 ron,g,sigma table coeffs
C
C  the connection file contains
C  IN_A      input FRM (has to be de-biased BEFORE)
C  OUTPUTC   variance image of input data (same dimension as input frame)
C  OUT_A     output FRM
C  OUT_B     weight map FRM
C  IN_B      table with positions of the orders
C  INPUTC    coeffs
C  INPUTI    slit,ord1,ord2, weight_determination_flag
C  INPUTR    ron,g,sigma (ron: readout-noise, g: inverse gain,
C                         sigma: threshold for cosmic ray rejection)
C
C.HISTORY:
C  1999.04.08-SW save weight map, make use of variances,
C                iterative cosmic ray rejection (negative sigma switches off)
C                calculates the variance of the extracted spectrum
C  1999.09.08-SW the variance of the extracted spectrum is calculated in
C                  any case of INPUTI(4).
C                bug fix: the variance has to be devided by the squared
C                  number of extracted lines (represented usually by the
C                  keyword SLIT) in order to be consistent with the
C                  extracted orders which are averaged data! The number of
C                  lines is specified by INPUTI(1).
C  1999.10.28-SW Do a gaussian fit to the profils (CAPROF(),
C                  G_PROF()).
C                Save the cosmic ray detections in a table if EXTSIGMA>0
C  1999.11.09-SW Prevent division by zero in CALCULx() and CALC_x().
C  1999.11.11-SW Profil normalization removed in VSAMPL() so that the
C                variances may be easily derived for the cosmic ray
C                rejection.
C  1999.12.07-SW Gaussian profil model implemented together with
C                S. Cristiani (scristia@eso.org)
C  1999.12.08-SW Virtual resampling of the variance frame. This enables
C                a better access to the variances in CAPROF() and G_PROF()
C                and they have not to be re-calculated everytime.
C                (necopt.prg should take account for the flatfielding
C                when the variance frame is determined!!)
C  1999.12.16-SC Made the gauss fit more robust, get initial position of the
C                the object reading MIDAS keyword OBJSET.
C  1999.12.19-SW removed the pause statement (not allowed within pipeline)
C  1999.12.20-SW do a parabola fit to the CHI2 to get error on amplitude
C                of the gaussian (= optimal extracted spectrum).
C  2000.02.14-AM Put a check on Imax=Imin condition (which verifies when
C                the center of one order has a distance from the detector's
C                y border less than half order's size) to skip to process
C                such an order
C  2000.03.03-AM-SC Fixed crash problem in optimal extraction; the variance
C                calculation was wrong. Now we calculate DELTA with a possibly
C                variable step, and calculate OUT and VOUT only if 
C                (DELTA !=0) .AND. (DELTA .LE. 0.9). It result a better
C                variance value and anyway we check it is .GT. 0 and .LE. e8
C                
C  2000.03.13-AM Cleaned
C  2000.03.17-AM Suppressed unnecessary Output
C  2000.04.17-AM patch to improove optimal extraction for high S/N (THRESH depends from S/N)
C  2000.05/18-AM S/N now calculated order per order and within an order column by column
C  2000.06.06-AM Added several comments to clarify
C  2000.06.27-AM Addec QC1 parameters MID_S_N_CENT, OBJ_POS_CENT
C  2000.06.30-AM Release to DFO
C  2000.08.04-AM Uniformed QC parameter naming conventions
C  2000.11.10-AM POLY_FITX to improove optimal extraction
C  2000.11.30-AM Added k-s clipping
C  2000.12.04-AM Added order_trace table to check optimaql extraction positions and POLFIT
C             (copy of LFIT from Numerical Recipes). LSFIT was wrong!!!
C  2000.12.15-AM Found stable set of numbers to have good extraction quality.
C  2000.12.18-AM Release to DFO
C
C***************************************************************************
C------------------------------------------------------------------------------
C $Name:  $
C $Revision: 1.48 $
C $Log: necopt.for,v $
C Revision 1.48  2001/08/08 13:16:44  amodigli
C Added again ESO prefix to QC key
C
C Revision 1.47  2001/07/15 14:14:48  amodigli
C Cleaned usinf GNU ftncheck
C
C Revision 1.46  2001/07/10 08:00:36  amodigli
C error-factor 2 in AMPERR was wrong   G_PROF-variance calculation
C
C Revision 1.45  2001/07/03 11:55:06  amodigli
C D1NAME,D2NAME char buffer now of dimension
C which is a multiple of 8 (portability to
C alpha). Added a few comments
C
C Revision 1.44  2001/06/29 12:36:45  amodigli
C Added BKG (sky) val,err
C Changed QC key name.
C An integer pointer was not *8
C
C Revision 1.43  2001/06/15 08:44:33  amodigli
C Changed QC naming conventions to be DICB
C compliant
C
C Revision 1.42  2001/06/07 08:52:03  amodigli
C Corrected some typo and some not proper
C initializations to fix portability problems
C to PC Linux
C
C Revision 1.41  2001/05/29 06:22:46  amodigli
C Corrected typo in IKIND info
C
C Revision 1.40  2001/05/28 16:24:26  amodigli
C Cleaned some more comments.
C
C Revision 1.39  2001/05/24 08:44:43  amodigli
C BINX and BINY must not be initialized to 0
C more than once!
C
C Revision 1.38  2001/05/23 14:58:56  amodigli
C Check error status in call to MIDAS env
C
C Revision 1.37  2001/05/22 17:48:32  amodigli
C Initialized variables to 0 or null string
C
C Revision 1.36  2001/05/22 17:46:38  amodigli
C Added some pseudocode
C
C Revision 1.35  2001/05/19 16:34:11  amodigli
C Further cleaning for DFS code rev step1
C
C Revision 1.34  2001/05/14 13:45:58  amodigli
C Initialized to blank string the buffers
C to fix a portability error to pc-linux of SB
C
C Revision 1.33  2001/05/10 13:10:52  amodigli
C Added comments on variables
C
C Revision 1.32  2001/03/16 09:32:58  amodigli
C Increased char size of LINE from 72 to 99
C to support long file names for input raw FITS
C data.
C
C Revision 1.31  2001/03/12 14:34:56  amodigli
C From MAX(arg1,0) to MAX(arg1,0.0001)
C to fix a portability error problem
C to DEC XP1000
C
C Revision 1.30  2001/02/05 07:41:33  amodigli
C Fixed some portability errors to ALPHA Linux
C due to:
C a) FWHM(N_CURR_ORD) non initialized to 0
C b) In case one order is 1 point in CALC_I
C a check more was necessary.
C
C Revision 1.29  2001/01/26 12:37:44  amodigli
C Added a dummy filling of displayed info
C (fwhm,S/N,obj pos) in case the order has
C only one point, for robustness.
C
C Revision 1.28  2001/01/24 08:29:39  amodigli
C IXWIND from 16 to 32 to speed-up and have
C less outliers
C Added switch to average fit in case even the
C linear is very strange
C threshold CHI of fit from 1.0 to 0.5
C thresholds on slope and curvature relaxed
C below, more strict above chi threshold.
C
C Revision 1.27  2001/01/18 14:18:37  amodigli
C As delivered to DFO.
C
C Revision 1.26  2001/01/17 15:58:06  amodigli
C Good Configuration with chunk=32
C and clean points t 2sigma in pos and fwhm
C and their errors
C
C Revision 1.25  2001/01/16 09:28:11  amodigli
C Displayed FWHM now right as average at order center
C Added removal of points outside 2 sigma from av pos
C Set checks on FWHM slope and curvature
C Fit uses errors equal to 1 (not weighted fit)
C
C Revision 1.24  2001/01/12 10:23:29  amodigli
C First version with also fwhm a parabola
C function of x (before was a constant)
C
C Revision 1.23  2001/01/11 15:07:12  amodigli
C Used TBRWRD and TBRWRR in place of TBEPUT
C to not to have problem with Linux and g77
C
C Revision 1.22  2001/01/09 08:01:34  amodigli
C Several small bug adjustment to allow
C compilation with Linux and g77
C debug table renamed order_trace
C debug2 table renamed xorder_trace
C
C Revision 1.21  2001/01/04 12:31:32  amodigli
C Added Id, Name, Revision, Log conf. control info
C
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     15/07/01       Cleaned usinf GNU ftncheck        all
C  AM     29/06/01       QC-added BKGVAL,BKGERR col        main,CAPROF
C                        to monitor Sky level
C  AM     15/06/01       QC-changed QC naming to be DICB   main
C                        compliant
C  AM     07/06/01       Corrected some typo and some not   main
C                        proper initializations to fix 
C                        portability problems to PC Linux
C  AM     28/05/01       Cleaned some more comments.        all
C  AM     23/05/01       Check error status in call to      all
C                        MIDAS env
C  AM     22/05/01       Initialized variables to 0 or      all
C                        null string
C  AM     22/05/01       added some pseudocode              all
C  AM     19/05/01       Further cleaning for DFS           all
C                        code rev step1
C  AM     14/05/01       Initialized to blank string the    main
C                        buffers to fix a portability error 
C                        to pc-linux of SB
C
C  AM     10/05/01       Added comments on variables        all
C  AM     08/05/01       clean-comment                      main


C------------------------------------------------------------------------------
C  Role: Initialize the Midas environment. Do In/Out. for each order do OBJ
C        or FF extraction. Close the environment.
C------------------------------------------------------------------------------
C  Pseudocode:
C    Initialize the system: 
C       define the program name,reads MIDAS descriptors
C       open input table and gets parameters
C       map variance frame
C       map input frame
C       get other data from input frame descriptors
C       crea ut variance
C       maps weight frame
C       init 'crm_' frame
C       maps output tables
C    display some info
C    DO I=ord_min,ord_max
C       if OBJ ima then
C              call EXTOBJ
C              display : Ord No., S/N, FWHM, Obj Pos, info
C       else then
C              call EXTFF
C       endif
C    ENDDO
C    DO I=ord_min,ord_max
C       store S/N, FWHM, Obj Pos info in out frm descriptors
C    ENDDO
C------------------------------------------------------------------------------




      PROGRAM ECHOPT

      IMPLICIT NONE

      INTEGER NDIM
      INTEGER N_MAX_ORD               !maximum expected order number 
      INTEGER NIRPAR                  !No of elements of int array IPAR
      INTEGER NPARAM                  !No of elements of array PARAMS
      INTEGER NDPAR                   !No of elements of array DPAR
      INTEGER NSIZIM                  !No of elements for ima descriptors
      INTEGER NSCAN                   !No of elements of SCAN array
      INTEGER LENFIL                  !Max len ofc File Name
      PARAMETER (LENFIL=80)
      PARAMETER (NDIM=2)
      PARAMETER (N_MAX_ORD=50)        !maximum expected order number 
                                      !To be changed accordingly in many places
                                      !(valid for UVES)
      PARAMETER (NIRPAR=10)           !No of elements of int array IPAR
      PARAMETER (NPARAM=12)           !No of elements of array PARAMS
      PARAMETER (NDPAR=40)            !No of elements of array DPAR
      PARAMETER (NSIZIM=3)            !No of elements for ima descriptors 
      PARAMETER (NSCAN=2)             !No of elements of SCAN array
      INTEGER FIDNO                   !File Identification Number
      INTEGER I_FILE                  !Aux. variable to do string manipulation 
      INTEGER I_SIZE                  !Aux. variable to do string manipulation 
      INTEGER N_CURR_ORD              !Variable to store current order number
      INTEGER MADRID                  !Memory Addres Identification
      INTEGER NAC                     !No. of words alloc. per raw 
                                      !(MIDAS env. int.)

      INTEGER NAR                     !No. of raw allocated (MIDAS env int.)
      INTEGER IMNO                    !File id of frame (MIDAS env int.)
      INTEGER SCAN(NSCAN)             !Scan image descriptor
      INTEGER BINSIZE                 !bin size of virtual resampling
      
      INTEGER I                       !Auxilliary variable
      INTEGER IAV                     !Actual No. of vals read (MIDAS env int.)
      INTEGER INDX                    !Aux. variable for string manipulation
      INTEGER IORD                    !Auxilliary variable as order counter
      INTEGER IORD1                   !Minimum detected order value 
      INTEGER IORD2                   !Maximum detected order value
      INTEGER IORDER                  !Aux. variable as order counter=IORD
       
      INTEGER ICOL1                   !Column No.(MIDAS env int.)
      INTEGER ICOL2                   !Column No.(MIDAS env int.)
      
      INTEGER ISTAT                   !Status of operation (MIDAS env int.)
      INTEGER NA                      !(IPAR(3)+1)* (IPAR(4)+1)
      INTEGER NAXISA                  !No. of axis of Image (MIDAS env int.)
      INTEGER NAXISB                  !No. of axis of Image (MIDAS env int.)
      
      INTEGER NCOL                    !No. of def. cols of tab (MIDAS env int.)
      INTEGER NPAR                    !No. of parameters
      INTEGER NROW                    !No. of def. rows of tab (MIDAS env int.)
      INTEGER NSORT                   !Sorted columns of table (MIDAS env int.)
      
      INTEGER STATUS                  !Status of operation (MIDAS env int.)
      INTEGER TID                     !Table Id.(MIDAS env int.)
      INTEGER MTID                    !Table Id.(MIDAS env int.)
      INTEGER INO                     !File Id of frame (MIDAS env int.) 
                                      !(out ima)

      INTEGER INVO                    !File Id of frame (MIDAS env int.) 
                                      !(out variance)
      
      INTEGER*8 PNTRA                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTRB                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTRC                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTRV                 !Pointer to mapped data (MIDAS env int.)
      INTEGER*8 PNTVO                 !Pointer to mapped data (MIDAS env int.)
      
      INTEGER NPIXA(NSIZIM)           !No. of pix in ima (MIDAS env int.)
      INTEGER NPIXB(NSIZIM)           !No. of pix in ima (MIDAS env int.)
      INTEGER NPIXC(NSIZIM)           !No. of pix in ima (MIDAS env int.)
      
      INTEGER IPAR(NIRPAR)            !array for int. params (MIDAS env int.)
      INTEGER KUN                     !unit (MIDAS env int.)
      INTEGER KNUL                    !nul (MIDAS env int.)
      
      INTEGER IKIND                   !switch for even/odd pix No. slit width
      INTEGER IWIDTH                  !slit width in pix
      INTEGER DIMP                    !dimention of weight profile
      
      INTEGER NWY                     !NWY=DIMP*(IORD2 - IORD1 + 1)

      INTEGER WID                     !File id of frame (MIDAS env int.) 
                                      !for weight ima

      INTEGER VID                     !File id of frame (MIDAS env int.) 
                                      !for variance ima
      
      INTEGER STOCH                   !tmp variable to store a value
      INTEGER STOCH2                  !tmp variable to store a value
      INTEGER DTID1                   !Table id (MIDAS env int.)
      INTEGER DTID2                   !Table id (MIDAS env int.)
      INTEGER BINX                    !No. of pix per bin along X
      INTEGER BINY                    !No. of pix per bin along Y
      INTEGER DUN                     !Units (MIDAS env int.)
      INTEGER VARLEN                  !variable length of VARNAME (1 or 2)  
      INTEGER DETWGT                  !Switch to determine weights 
                                      !(1:determine 0: use weight map)

      INTEGER NAXISC                  !Number of axis of weight map
      REAL MID_S_N_CENT(N_MAX_ORD)    !Average S/N     at order center
      REAL OBJ_POS_CENT(N_MAX_ORD)    !Average OBJ pos at order center
      REAL FWHM(N_MAX_ORD)            !Average FWHM    at order center
  
      REAL  RPAR(NIRPAR)              !array for real params descriptor 
                                      !(MIDAS env int.)
      REAL PARAMS(NPARAM)             !real extraction parameters
      REAL OFFSET                     !offset
      REAL OBJSET                     !object offset


      DOUBLE PRECISION STEPA(NSIZIM)  !Step of input image
      DOUBLE PRECISION STEPB(NSIZIM)  !Step of output image
      DOUBLE PRECISION STARTA(NSIZIM) !Start of input image
      DOUBLE PRECISION STARTB(NSIZIM) !Start of output image
      DOUBLE PRECISION DPAR(NDPAR)    !array containing order polynomial coeff.

      CHARACTER*72 IDENA              !ASCII identification for frame
      CHARACTER*72 IDENB              !ASCII identification for frame
      CHARACTER*96 LINE               !Buffer to display informations
      
      CHARACTER*64 CUNITA             !Unit of each axis + data 
                                      !(MIDAS env int.)

      CHARACTER*64 CUNITB             !Unit of each axis + data 
                                      !(MIDAS env int.)

      CHARACTER*64 DESNAM             !char descriptor
      
C     NB LENFIL=80

      CHARACTER*80 FRMIN              !Input frame
      CHARACTER*80 FRMOUT             !Output frame
      CHARACTER*80 FRMVOUT            !output variance frame
      CHARACTER*80 FRMWGT             !weight image frame
      CHARACTER*80 FRMVAR             !input variance frame
      
      CHARACTER*80 INA1           !buffer for data values (MIDAS env int.)
      CHARACTER*80 INA2           !buffer for data values (MIDAS env int.)
      CHARACTER*80 INA3           !buffer for data values (MIDAS env int.)
      CHARACTER*80 INA4           !buffer for data values (MIDAS env int.)
      CHARACTER*80 TABLE          !buffer for data values (MIDAS env int.)
      
      CHARACTER*80 INFO_BUFFER        !buffer for information
      CHARACTER*8  NAME               !buffer for data values (MIDAS env int.)
      CHARACTER*8  TYPE               !buffer for data values (MIDAS env int.)
      
      CHARACTER*2  VARNAME            !buffer for information
      
      CHARACTER*80 DESQCNM            !FITS key for information
      CHARACTER*80 DES_QC             !FITS base key for information


      COMMON /FIT/BINX,BINY              !Necessary to set tresholds
                                         !over polynomial fit slope and 
                                         !curvature BINX,BINY dependent

      COMMON /DBG/IORDER                 !Used for debugging purposes?

      COMMON /CRMASK/MTID                !Cosmic Ray Mask?Used?

      INCLUDE 'MID_INCLUDE:ST_DEF.INC'   !to get the definitions 
                                         !on the ST interfaces
      COMMON /VMR/MADRID(1)              !COMMON block for the 
                                         !pointer emulation in F77
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'   !to get data values for 
                                         !definitions above




      DATA CUNITB/
     +     'FLUX            PIXEL                           ORDER'/

C     Initializes all int variables to 0 to be sure...

      FIDNO       =0
      I_FILE      =0
      I_SIZE      =0 
      N_CURR_ORD  =0

C      MADRID      =0

      DO I=1,NSCAN
         SCAN(I)=0
      ENDDO

      DO I=1,NSIZIM
         NPIXA(I)=0
         NPIXB(I)=0
         NPIXC(I)=0
         SCAN(I)=0
      ENDDO

      DO I=1,NIRPAR 
         IPAR(I)=0
         RPAR(I)=0.0
      ENDDO

      NAC         =0 

      NAR         =0
      IMNO        =0

      BINSIZE     =0
      
      I           =0
      IAV         =0
      INDX        =0
      IORD        =0
      IORD1       =0 
      IORD2       =0
      IORDER      =0
       
      ICOL1       =0
      ICOL2       =0
      
      ISTAT       =0
      NA          =0
      NAXISA      =0
      NAXISB      =0
      
      NCOL        =0
      NPAR        =0
      NROW        =0
      NSORT       =0
      
      STATUS      =0
      TID         =0
      MTID        =0
      INO         =0
                           
      INVO        =0
      VARLEN      =0                   
C      To not have portability problems to PC-Linux,
C      better not initialize pointers PNTR A,B,C,V,O                                 
 
      
      KUN        =0
      KNUL       =0 
      
      IKIND      =0
      IWIDTH     =0
      DIMP       =0
      
      NWY        =0

      WID        =0 
                           

      VID        =0 
                                 
      STOCH      =0
      STOCH2     =0
      DTID1      =0
      DTID2      =0
      BINX       =0
      BINY       =0
      DUN        =0
      DETWGT     =0
      NAXISC     =0
C     Initializes all real variables to 0 to be sure...
  
      DO I=1,N_MAX_ORD
         MID_S_N_CENT(I) =0.0    
         OBJ_POS_CENT(I) =0.0  
         FWHM(I)         =0.0
      ENDDO

      DO I=1,NPARAM
         PARAMS(I)=0.0
      ENDDO 
      OFFSET           =0.0    
      OBJSET           =0.0    


C     Initializes all double precision variables to 0 to be sure...

       DO I=1,NDPAR
         DPAR(I)=0.0D0
       ENDDO

       DO I=1,NSIZIM
          STEPA(I)=0.0D0
          STEPB(I)=0.0D0
          STARTA(I)=0.0D0
          STARTB(I)=0.0D0 
       ENDDO


C     Initializes all char variables to null string to be sure...

      IDENA(1:) = ' '
      IDENB(1:) = ' '
      LINE(1:) = ' '
      CUNITA(1:) = ' '
C      CUNITB(1:) = ' '    !CUNITB is initialized by a DATA statement
      DESNAM(1:) = ' '
      FRMIN(1:) = ' '
      FRMOUT(1:) = ' '
      FRMVOUT(1:)= ' '
      FRMWGT(1:) = ' '
      FRMVAR(1:) = ' '
      INA1(1:)   = ' '
      INA2(1:)   = ' '
      INA3(1:)   = ' '
      TABLE(1:)  = ' '
      INFO_BUFFER(1:) = ' '
      NAME(1:) = ' '
      TYPE(1:) = ' '
      VARNAME(1:) = ' '
      DESQCNM(1:) = ' '
      DES_QC(1:) = ' '




C
C  initialize system
C

C STSPRO = ST (F77) system: get into MIDAS environment
C          read key DB, init file control table and open logfile

C STKRDC = STK (F77) module: ReaD Character key
C STKRDI = STK (F77) module: ReaD Integer key
C STKRDR = STK (F77) module: ReaD Real key

C IN_A is the input frame
C IN_B is the table with the positions of the orders
C OUT_A is the output frame
C OUT_B is the weight map frame
C OUTPUTC is the variance image of input data
C INPUTC are the coefficients
C INPUTR are ron,gain,sigma
C INPUTI is slit,ord1,ord2


      CALL STSPRO('ECHOPT')              !set up MIDAS environment
                                         !+enamble automatic error abort
                                         
      CALL STKRDC('IN_A',1,1,LENFIL,I,INA1,KUN,KNUL,STATUS)  !get in frm name
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT Error reading IN_A')

      CALL STKRDC('IN_B',1,1,LENFIL,I,TABLE,KUN,KNUL,STATUS) !get in tab name
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading IN_B')

      CALL STKRDC('OUT_A',1,1,LENFIL,I,INA2,KUN,KNUL,STATUS) !set out frm name
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading OUT_A')

      CALL STKRDC('OUT_B',1,1,LENFIL,I,INA3,KUN,KNUL,STATUS) !set out weight map 
                                                         !frm
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading OUT_B')

      CALL STKRDC('OUTPUTC',1,1,LENFIL,I,INA4,KUN,KNUL,STATUS) !set out var im frm
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading OUTPUTC')

      CALL STKRDC('INPUTC',1,1,8,I,NAME,KUN,KNUL,STATUS) !get in coeff
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading INPUTC')

      CALL STKRDR('INPUTR',1,3,I,RPAR,KUN,KNUL,STATUS)   !get in ron,gain,sigma
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading INPUTR')

      CALL STKRDI('INPUTI',1,4,I,IPAR,KUN,KNUL,STATUS)   !get in slit,ord1,ord2
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading INPUTI')

      CALL STKRDI('SCAN',1,2,I,SCAN,KUN,KNUL,STATUS)     !get in scan
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading SCAN')

      CALL STKRDR('OFFSET',1,1,I,OFFSET,KUN,KNUL,STATUS) !get in offset
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading OFFSET')

      CALL STKRDR('OBJSET',1,1,I,OBJSET,KUN,KNUL,STATUS) !get in objset
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading OBJSET')
      CALL STKRDI('BINX',1,1,I,BINX,KUN,KNUL,STATUS)     !get in binx (used to
                                                         !set curvature limits)
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading BINX')

      CALL STKRDI('BINY',1,1,I,BINY,KUN,KNUL,STATUS)     !get in biny (used to
                                                         !set curvature limits)
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error reading BINY')


C      CALL STKRDI('BINSIZE',1,1,I,BINSIZE,N1,N1,N1)

      BINSIZE =  10              !binsize for virtual resampling VSAMPL().
                                 !now 10 but when used is re-set to 1 !

      IORD1  = IPAR(2)           !starting detected order number
      IORD2  = IPAR(3)           !ending detected order number
      DETWGT = IPAR(4)           !1: determine 0: use weight map
      
C CLNFRA stand this for Call Link Frame?
      
      CALL CLNFRA(INA1,FRMIN,0)  !input frame (pixel-pixel-space)
      CALL CLNFRA(INA2,FRMOUT,0) !output frame (extracted orders)
      CALL CLNFRA(INA3,FRMWGT,0) !weight map (pixel-full_order-space)
      CALL CLNFRA(INA4,FRMVAR,0) !variance frame (pixel-pixel-space)
      CALL CLNTAB(TABLE,TABLE,0) !order table (order positions)

C
C  input parameters in table
C

      CALL TBTOPN(TABLE,F_U_MODE,TID,STATUS)          !(F77) open table
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error opening TABLE')

      CALL TBIGET(TID,NCOL,NROW,NSORT,NAC,NAR,STATUS) !(F77) read size params
      IF (STATUS.NE.0) THEN
          CALL STETER(11,'ECHOPT: Error getting TBL size parms')
      ENDIF
      

C
C  map variance frame:
C  map the complete data frame for reading or updating and 
C  read the standard ard descriptors
C
      CALL STIGET(FRMVAR,D_R4_FORMAT,F_IO_MODE,F_IMA_TYPE,
     .     NDIM,NAXISA,NPIXA,STARTA,STEPA,IDENA,
     +     CUNITA,PNTRV,VID,STATUS)
      IF (STATUS.NE.0) THEN
         CALL STETER(11,'ECHOPT: Error mapping IN VAR FRM')
      ENDIF
     

     
C  map input FRM
C  map the complete data frame for reading or updating and 
C  read the standard ard descriptors
C
C This was to check how looks like the input frame

      CALL STIGET(FRMIN,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            NDIM,NAXISA,NPIXA,STARTA,STEPA,IDENA,
     +            CUNITA,PNTRA,IMNO,STATUS)
      IF (STATUS.NE.0) CALL STETER(11,'ECHOPT: Error mapping FRMIN')


C   FRMIN:       input frame
C   D_R4_FORMAT: data type
C   F_I_MODE:    opening mode, F_I_MODE or F_IO_MODE
C   F_I_TYPE:    file type
C   NDIM:        max no. of dimensions
C   NAXISA:          no. of dimensions
C   NPIXA:       no. of pixels per axis
C   STARTA:      start values
C   STEPA:       step sizes
C   IDENA:       ASCII id for frame
C   CUNITA:      unit of each axis + data
C   PNTRA:       pointer to mapped data
C   IMNO:        file id of frame
C   STATUS:      status (0= ok)


      PARAMS(1) = IPAR(1)       !slitwidth (in spatial direction)
      PARAMS(2) = RPAR(3)       !threshold factor for CR-rejection
      PARAMS(3) = RPAR(2)       !invers gain factor [e-/ADU]
      PARAMS(4) = RPAR(1)       !readout-noise [e-]
      
      
C
C  read coeffs
C
      INDX   = INDEX(NAME//' ',' ') - 1
      DESNAM = NAME(1:INDX)//'C'
      
C     STD (f77) module: ReaD char descriptor
      CALL STDRDC(TID,DESNAM,1,17,4,IAV,TYPE,KUN,KNUL,ISTAT)  
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'ECHOPT: Error reading char descriptor')
      ENDIF

      TYPE   = 'MULT'
      IF (TYPE.EQ.'MULT') THEN
          DESNAM = NAME(1:INDX)//'I'
          
C     STD (f77) module: ReaD integer descriptor          
          CALL STDRDI(TID,DESNAM,4,4,IAV,IPAR,KUN,KNUL,ISTAT)
          IF (ISTAT.NE.0) THEN
             CALL STETER(11,'ECHOPT: Error reading int descriptor')
          ENDIF
          NA     = (IPAR(3)+1)* (IPAR(4)+1)
          DESNAM = NAME(1:INDX)//'R'
          
C     STD (f77) module: ReaD real descriptor           
          CALL STDRDR(TID,DESNAM,1,4,IAV,PARAMS(5),KUN,KNUL,ISTAT)
          IF (ISTAT.NE.0) THEN
             CALL STETER(11,'ECHOPT: Error reading real descriptor')
          ENDIF
          DESNAM = NAME(1:INDX)//'D'
          
C     STD (f77) module: ReaD double descriptor           
          CALL STDRDD(TID,DESNAM,1,NA,IAV,DPAR,KUN,KNUL,ISTAT)
          IF (ISTAT.NE.0) THEN
             CALL STETER(11,'ECHOPT: Error reading dbl descriptor')
          ENDIF
C
C ... map output FRM
C

          IF (IORD1.EQ.IORD2 .AND. IORD1.EQ.0) THEN
              IORD1  = PARAMS(7)
              IORD2  = PARAMS(8)
          END IF

         
          STARTB(1) = STARTA(1)
          STARTB(2) = IORD1                 !ORD1 is the starting order ID No
          STEPB(1)  = STEPA(1)
          STEPB(2)  = 1.D0
          IDENB     = IDENA
          NAXISB    = 2
          NPIXB(1)  = NPIXA(1)
          NPIXB(2)  = IORD2 - IORD1 + 1     !No of extracted orders
          FRMVOUT = 'var_'//FRMOUT
          I = MAX(INDEX(FRMVOUT,' ')-1,1)

          CALL STIPUT(FRMOUT,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     +         NAXISB,NPIXB,STARTB,STEPB,IDENB,CUNITB,PNTRB,INO,STATUS)
          IF (STATUS.NE.0) THEN
             CALL STETER(11,'ECHOPT: Error mapping out frm')
          ENDIF

          WRITE(LINE,*) 'OUTPUT:',FRMOUT(1:I),' extracted spectrum'

          CALL STTPUT(LINE,STATUS)
          
                    
C
C ... variance of extracted orders
C

          IDENB = 'variance'
C         creates a new image frame, map it and store standard descriptors    
          CALL STIPUT(FRMVOUT,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     +         NAXISB,NPIXB,STARTB,STEPB,IDENB,CUNITB,PNTVO,INVO,STATUS)
          IF (STATUS.NE.0) THEN
             CALL STETER(11,'ECHOPT: Error mapping out var frm')
          ENDIF

          WRITE(LINE,*) '       ',FRMVOUT(1:I)
     +         ,' variance of extr. spectrum'
     
C         module STST (f77): display text and store it in MIDAS log file
C         IN=LINE:           text string
C         OUT=STATUS:        return status (0=ok)      
          CALL STTPUT(LINE,STATUS)


                    
C
C  ... map weight FRM
C

                                                   !slit width in pix
          IWIDTH = PARAMS(1)                       !number of pix=slit width

          IF (MOD(IWIDTH,2).EQ.0)THEN          
             IKIND=1            !even slit width
             DIMP = IWIDTH+1    !dimension of the weight profil
          ELSE
             IKIND=0            !odd slit width
             DIMP = IWIDTH+2    !dimension of the weight profil
          ENDIF

          NWY  = DIMP*(IORD2 - IORD1 + 1)

          STARTB(1) = STARTA(1)
          STARTB(2) = 1.D0
          STEPB(1)  = STEPA(1)
          STEPB(2)  = 1.D0
          IDENB     = 'weight map'
          NAXISC    = 2
          NPIXC(1)  = NPIXA(1)
          NPIXC(2)  = NWY

 

          IF (DETWGT .EQ. 1) THEN
C          
C         creates a new image frame, map it and store standard descriptors    
C
             CALL STIPUT(FRMWGT,D_R4_FORMAT,F_O_MODE,F_IMA_TYPE,
     +            NAXISC,NPIXC,STARTB,STEPB,
     +            IDENB,CUNITB,PNTRC,WID,STATUS)
             IF (STATUS.NE.0) THEN
                CALL STETER(11,'ECHOPT: Error mapping wght frm')
             ENDIF

      
     
C
C cosmic ray rejection ONLY if weights should be determined
C
             IF (PARAMS(2).LT.0) THEN
                WRITE(LINE,*) '       NO cosmic ray rejection'
                CALL STTPUT(LINE,STATUS) !display text and store it 
                                         !in MIDAS log file
                MTID = -1
             ELSE

C  create CR-mask table

                FRMOUT = 'crm_'//FRMIN
                I = MAX(INDEX(FRMOUT,' ')-4,1)
                FRMOUT = FRMOUT(1:I)//'tbl'
                I = I + 3
                
C               initialize and open table file                
                CALL TBTINI(FRMOUT,F_TRANS,F_O_MODE,2,100,MTID
     $               ,ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'ECHOPT: Error init out tab frm')
                ENDIF

C               initialize a column in a table     
                CALL TBCINI(MTID,D_I4_FORMAT,1,'I','PIXEL',':X',ICOL1
     $               ,ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'ECHOPT: Error init col in tab frm')
                ENDIF

C               initialize a column in a table     
                CALL TBCINI(MTID,D_I4_FORMAT,1,'I','PIXEL',':Y',ICOL2
     $               ,ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'ECHOPT: Error init col in tab frm')
                ENDIF

              
                
                WRITE(LINE,*) '       Cosmic ray rejection (>',PARAMS(2)
     .               ,'*sigma)'
     
                CALL STTPUT(LINE,STATUS)  !display text and store it
                                          !in MIDAS log file   
                                                       
                WRITE(LINE,*) '         save detections on: '
     $               ,FRMOUT(1:I)
     
                CALL STTPUT(LINE,STATUS)  !display text and store it
                                          !in MIDAS log file                  
             ENDIF
          ELSE
          
C            map complete data frame for reading or updating 
C            and read the standard ard descriptors          
             CALL STIGET(FRMWGT,D_R4_FORMAT,F_I_MODE,F_IMA_TYPE,
     .            NDIM,NAXISC,NPIXC,STARTB,STEPB,IDENB,
     +            CUNITB,PNTRC,WID,STATUS)
             IF (STATUS.NE.0) THEN
                CALL STETER(11,'ECHOPT: Error mapping wgth frm')
             ENDIF

          ENDIF

          I = MAX(1,INDEX(FRMWGT,' ')-1)
          WRITE(LINE,*) '       ',FRMWGT(1:I),' weight map'
          
          CALL STTPUT(LINE,STATUS)  !display text and store it 
                                         !in MIDAS log file
                                                   
          I = MAX(1,INDEX(TABLE,' ')-1)
          WRITE(LINE,*) '       ',TABLE(1:I),' order table'
          
          CALL STTPUT(LINE,STATUS)  !display text and store it
                                    !in MIDAS log file     
                                         
          WRITE(LINE,*) '       OFFSET to order positions:',OFFSET
          
          CALL STTPUT(LINE,STATUS)  !display text and store it
                                    !in MIDAS log file 
                                              
          WRITE(LINE,*) '       OFFSET to object positions:',OBJSET
          
          CALL STTPUT(LINE,STATUS)  !display text and store it
                                    !in MIDAS log file          

C  !!!!!!! BEWARE here set binsize to 1 before (at the beginning) was set to 10
          BINSIZE = 1

          WRITE(LINE,*) '       BINSIZE of virtually resampled data:'
     $         ,BINSIZE
          
          CALL STTPUT(LINE,STATUS)  !display text and store it
                                    !in MIDAS log file               
        
   
C
C  extract order
C
          NPAR       = 11
          PARAMS(9)  = IPAR(3)
          PARAMS(10) = IPAR(4)
          IF (SCAN(1).EQ.0) THEN
             SCAN(1) = 1
             SCAN(2) = NPIXA(2)
          ENDIF


          IF (DETWGT .EQ. 1) THEN
C
C crea a table to monitor some results
C      

              CALL TBTINI('order_trace',F_TRANS,F_O_MODE,16,
     +                    10000,DTID1,ISTAT)  
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init tab frm')
              ENDIF


              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F5.0','none',
     +                     ':ORDER',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :ORDER')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F3.0','bin',
     +                    ':I_CH',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :I_CH')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F6.0','bin',
     +                    ':X_CH_0',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :X_CH_0')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'G8.2','bin',
     +                    ':P_ERR0',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :P_ERR0')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'G10.4','bin',
     +                    ':S_CH_0',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :S_CH_0')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'G10.4','bin',
     +                    ':S_CH_1',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :S_CH_1')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','bin',
     +                    ':P_FWHM',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col "P_FWHM')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'G10.4','bin',
     +                    ':P_FWHM_ERR',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :P_FWHM_ERR')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','bin',
     +                    ':P_CH',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :P_CH')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','bin',
     +                    ':P_F0',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :P_F0')
              ENDIF
              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','bin',
     +                    ':P_F1',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :P_F1')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F2.0','none',
     +                    ':M0',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init tab frm')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F2.0','none',
     +                    ':M1',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :M1')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F3.0','none',
     +                    ':DEG',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :DEG')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'G10.4','none',
     +                    ':CHI0',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :CHI0')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F3.0','none',
     +                    ':KS',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :KS')
              ENDIF

              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','counts',
     +                    ':BKGVAL',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :BKGVAL')
              ENDIF
              
              CALL TBCINI(DTID1,D_R8_FORMAT,1,'F10.4','counts',
     +                    ':BKGERR',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :BKGERR')
              ENDIF


              CALL STDWRI(DTID1,"ESO.QC.OPTEX.XSIZE",
     +                    NPIXB(1),1,1,DUN,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error writing descriptor')
              ENDIF

              CALL STDWRI(DTID1,"ESO.QC.OPTEX.YSIZE",
     +                    DIMP,1,1,DUN,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error writing descriptor')
              ENDIF

              CALL STDWRI(DTID1,"ESO.QC.OPTEX.NORD",
     +                    NPIXB(2),1,1,DUN,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error writing descriptor')
              ENDIF



              CALL TBTINI('xorder_trace',F_TRANS,F_O_MODE,4,200000,
     +                    DTID2,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init xorder tab')
              ENDIF

              CALL TBCINI(DTID2,D_R8_FORMAT,1,'F5.0','none',
     +                    ':ORDER',ICOL1,ISTAT)  
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :ORDER')
              ENDIF

              CALL TBCINI(DTID2,D_R8_FORMAT,1,'F5.0','none',
     +                    ':I_CH',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :I_CH')
              ENDIF

              CALL TBCINI(DTID2,D_R8_FORMAT,1,'F5.0','bin',
     +                    ':Y_CH',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :Y_CH')
              ENDIF

              CALL TBCINI(DTID2,D_R8_FORMAT,1,'F5.0','ADU',
     +                    ':FLUX',ICOL1,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error init col :FLUX')
              ENDIF


              CALL STDWRI(DTID1,"ESO.QC.OPTEX.XSIZE",
     +                    NPIXB(1),1,1,DUN,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error writing descriptor')
              ENDIF

              CALL STDWRI(DTID1,"ESO.QC.OPTEX.YSIZE",
     +                    DIMP,1,1,DUN,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error writing descriptor')
              ENDIF

              CALL STDWRI(DTID2,"ESO.QC.OPTEX.NORD",
     +                    NPIXB(2),1,1,DUN,ISTAT)
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error writing descriptor')
              ENDIF
              
          ENDIF
 
C     Now print out the QC parameters as different descriptor names       
      DES_QC='ESO.QC.OBJ'

 
    
      STOCH=0
      STOCH2=0

          DO 10 I = IORD1,IORD2

 
              N_CURR_ORD=I
C
C here write out infos only for OBJ not for FF extraction
C
            
              PARAMS(11) = I
              IORD       = I - IORD1 + 1
              IORDER = IORD
              IF (DETWGT .EQ. 1) THEN                   
                  CALL STDWRI(DTID2,"ESO.QC.OPTEX.NORD",
     +                        NPIXB(2),1,1,DUN,STATUS)
              IF (STATUS.NE.0) THEN
                 CALL STETER(11,'ECHOPT: Error writing descriptor')
              ENDIF

                  
C The optimal extraction is performed only on the Object (not on FF)
C do an optimal extraction, save weights and variances on disk.


                 CALL EXTOBJ(MADRID(PNTRA),MADRID(PNTRV),NPIXA(1)
     $                ,NPIXA(2),MADRID(PNTRB),MADRID(PNTVO),NPIXB(1)
     $                ,NPIXB(2),MADRID(PNTRC),NPIXC(1),NPIXC(2),IORD
     $                ,NPAR,PARAMS,NA,DPAR,SCAN,OFFSET,BINSIZE,STARTA
     $                ,STEPA,IKIND,IWIDTH,DIMP,OBJSET,FWHM
     $                ,MID_S_N_CENT,OBJ_POS_CENT,N_CURR_ORD
     $                ,STOCH,STOCH2,DTID1,DTID2)

                 WRITE(LINE,100) 'Order number : ',I
     $                          ,' S/N : ',MID_S_N_CENT(N_CURR_ORD)
     $                          ,' FWHM : ',FWHM(N_CURR_ORD)     
     +                          ,' Obj Pos : ',OBJ_POS_CENT(N_CURR_ORD)
     
                 CALL STTPUT(LINE,STATUS)  !display text and store it
                                           !in MIDAS log file  
                                        

                                        

              ELSE

C              
C the immediate extraction is performed either for OBJ and Flat Field
C do an immediate extraction using the weights of a previous optimal extract.
C
                 CALL EXTFF(MADRID(PNTRA),MADRID(PNTRV),NPIXA(1)
     $                ,NPIXA(2),MADRID(PNTRB),MADRID(PNTVO),NPIXB(1)
     $                ,NPIXB(2),MADRID(PNTRC),NPIXC(1),NPIXC(2),IORD
     $                ,NPAR,PARAMS,NA,DPAR,SCAN,OFFSET,STARTA,STEPA
     $                ,IKIND,IWIDTH,DIMP)
              ENDIF

10        CONTINUE

      ELSE
      
C         display text and store it in MIDAS log file       
          CALL STTPUT('Error in Dispersion coefficients',ISTAT)
          GOTO 20
      ENDIF
      
      
      
      
      

            
      IF (DETWGT .EQ. 1) THEN     
          I_FILE = MAX(INDEX(FRMVOUT,' ')-1,1)
          FRMOUT = 'xb_'//FRMIN
          DUN=0

         
          DO 50 N_CURR_ORD=IORD1,IORD2

                 CALL STFOPN(FRMOUT(1:I_FILE),D_R4_FORMAT,0,
     +                       F_IMA_TYPE,FIDNO,STATUS)
                 IF (STATUS.NE.0) THEN
                    CALL STETER(11,'ECHOPT: Error opening out frm')
                 ENDIF

                 IF (N_CURR_ORD .LE. 9) THEN
                    WRITE(VARNAME,1000) N_CURR_ORD
                    VARLEN=1
                 ELSE
                    WRITE(VARNAME,2000) N_CURR_ORD
                    VARLEN=2
                 ENDIF
                 
                 
                 
                 I_SIZE = MAX(INDEX(DES_QC,' ')-1,1)
                 DESQCNM=DES_QC(1:I_SIZE)//VARNAME(1:VARLEN)//'.SN'
                
                 CALL STDWRR(FIDNO,DESQCNM,
     +                       MID_S_N_CENT(N_CURR_ORD),
     +                       1,1,DUN,STATUS)
                 IF (STATUS.NE.0) THEN
                    CALL STETER(11,'ECHOPT: Error writing descriptor')
                 ENDIF



                 INFO_BUFFER='Av. S/N at order center'
                 CALL STDWRH(FIDNO,DESQCNM,INFO_BUFFER,
     +                       1,23,STATUS)
                 IF (STATUS.NE.0) THEN
                    CALL STETER(11,'ECHOPT: Error writing header')
                 ENDIF
     
                 I_SIZE = MAX(INDEX(DES_QC,' ')-1,1)
                 DESQCNM=DES_QC(1:I_SIZE)//VARNAME(1:VARLEN)//'.FWHM'



                 CALL STDWRR(FIDNO,DESQCNM,
     +                       FWHM(N_CURR_ORD),
     +                       1,1,DUN,STATUS)
                 IF (STATUS.NE.0) THEN
                    CALL STETER(11,'ECHOPT: Error writing descriptor')
                 ENDIF


                 INFO_BUFFER='Av. FWHM on order'
                 CALL STDWRH(FIDNO,DESQCNM,INFO_BUFFER,
     +                       1,23,STATUS)
                 IF (STATUS.NE.0) THEN
                    CALL STETER(11,'ECHOPT: Error writing header')
                 ENDIF
          


                 I_SIZE = MAX(INDEX(DES_QC,' ')-1,1)
                 DESQCNM=DES_QC(1:I_SIZE)//VARNAME(1:VARLEN)//'.POS'

                 INFO_BUFFER='Av. OBJ POS at order center'    
                 CALL STDWRR(FIDNO,DESQCNM,
     +                       OBJ_POS_CENT(N_CURR_ORD),
     +                       1,1,DUN,STATUS)
                 IF (STATUS.NE.0) THEN
                    CALL STETER(11,'ECHOPT: Error writing descriptor')
                 ENDIF



                 CALL STDWRH(FIDNO,DESQCNM,INFO_BUFFER,
     +                       1,27,STATUS)
                 IF (STATUS.NE.0) THEN
                    CALL STETER(11,'ECHOPT: Error writing header')
                 ENDIF
     
     
                 CALL STFCLO(FIDNO,STATUS)  


   50     CONTINUE
                                                          
      ENDIF
      
      IF (MTID.GT.0) CALL TBTCLO(MTID,ISTAT) !close CR-mask table


   20 CALL STSEPI !Module STST (f77) close down the MIDAS env.

  100 FORMAT(A,I2,A, F8.2, A, F6.2,A, F6.2)
  
 1000 FORMAT(I1)
 2000 FORMAT(I2)


      END

C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
          
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     EXTOBJ
C------------------------------------------------------------------------------
C  Role: Extract the Object
C        creates and maps vdummx, vdummy, mask, vari, pmsk, pbuf, ebuf, column
C                         sigma, xpos
C        call CENT to get the centres of the orders at each wavelength
C        call VSAMPLE to do a virtual sampling of the obj and variance
C        call CAPROF to determine the object and FWHMs distributions along 
C                    dispersion direction
C-----------------------------------------------------------------------------
C Pseudocode:
C        some initial setting for the extraction constants
C        crea/maps frame vdummx
C        call CENT to get order centers
C        crea/maps vdummy,vari,mask,pbuf,ebuf,pmsk,colum,sigma,xpos
C        initializes out ima to 0
C        call VSAMPL for variance and input ima
C        call CAPROF to calculate x-order prof, weights, and extract the obj
C        close every open frame
C        RETURN
C-----------------------------------------------------------------------------
      SUBROUTINE EXTOBJ(IN,VARI,NPIXA1,NPIXA2,OUT,VOUT,NB1,NB2,WGT
     $     ,NC1,NC2,INDX,NPAR,PAR,ND,DPAR,SCAN,OFFSET,BINSIZE,STARTA
     $     ,STEPA,IKIND,IWIDTH,DIMP,OBJSET,FWHM,MID_S_N_CENT
     +     ,OBJ_POS_CENT,N_CURR_ORD,STOCH,STOCH2,DTID1,DTID2)
C
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE
C VARI   REAL*4  VARIANCE IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C VOUT   REAL*4  VARIANCE OF EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C WGT    REAL*4  WEIGHT MAP
C NC1    INTG*4  NO. OF SAMPLES
C NC2    INTG*4  NO. OF ORDERS * THICKNESS OF ORDERS (profil dimension)
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C XMIN   REAL*4  MINIMUM VALUE
C XMAX   REAL*4  MAXIMUMVALUE
C IKIND  INTG*4  EVEN/ODD FLAG: 1: EVEN, 0: ODD SLIT WIDTH SIZE
C IWIDTH INTG*4  SLIT WIDTH  (size perpendicular to dispersion in pixel)
C DIMP   INTG*4  DIMENSION OF THE PROFILE OF THE ORDERS
C
      IMPLICIT NONE
      INTEGER  NAXIMA
      INTEGER  N_MAX_ORD
      PARAMETER (NAXIMA=2)         !Max No. of elements in STEP START
      PARAMETER (N_MAX_ORD=50)     !Max No. of Orders to be changed 
                                   !accordingly in many places
C
C Interface variable (defined and initialized elsewere)
C
      INTEGER  NPIXA1              !1st dimension (X) of input ima
      INTEGER  NPIXA2              !2nd dimension (Y) of input ima
      INTEGER  NB1                 !1st dimension (X) of output ima
      INTEGER  NB2                 !2nd dimension (Y) of output ima
      INTEGER  NC1                 !1st dimension (X) of weight ima
      INTEGER  NC2                 !2nd dimension (Y) of weight ima
      INTEGER  NPAR                !Number of parameters
      INTEGER  ND                  !Dimension of array with order polyn coeff
      INTEGER  IKIND               !switch for even/odd pix No. slit width
      INTEGER  IWIDTH              !dimention slit width in pix
      INTEGER  DIMP                !Dimension (No of points) of x-order profile
      INTEGER N_CURR_ORD           !current order position    
      INTEGER DTID1                !Debug table Id (MIDAS env)
      INTEGER DTID2                !Debug table Id (MIDAS env)
      INTEGER INDX                 !current chunk?
      INTEGER BINSIZE              !Size of bin (for virtual resampling)
      INTEGER SCAN(NAXIMA)         !Scan ima parameter (MIDAS)
      INTEGER STOCH                !current pos in debug table 1
      INTEGER STOCH2               !current pos in debug table 2
       
      REAL WGT(NC1,NC2)            !Array for weight ima
      REAL OUT(NB1,NB2)            !Array for out ima
      REAL VOUT(NB1,NB2)           !Array for varance
      REAL IN(NPIXA1,NPIXA2)       !Array for in ima
      REAL VARI(NPIXA1,NPIXA2)     !Array for variance
      REAL PAR(NPAR)               !Array parameter with Thresh,CONAD,RON
      REAL OFFSET                  !Offset of object (initial value constant)
      REAL OBJSET                  !Offset of object (variable)
      REAL MID_S_N_CENT(N_MAX_ORD) !Av S/N at the central 100 pix in the order
      REAL OBJ_POS_CENT(N_MAX_ORD) !Av obj pos at order center (100 pix)
      REAL FWHM(N_MAX_ORD)         !Av FWHM at the central 100 pix in the order

      DOUBLE PRECISION DPAR(ND)    !Parameters for polynomial coefficients
      DOUBLE PRECISION STARTA(NAXIMA)!Start point of image (X,Y)
      DOUBLE PRECISION STEPA(NAXIMA) !Step of image        (X,Y)
      


C
C     Local variables
C

     
           
      INTEGER          K           !K=PAR(9)   !Dimension of the coefficient
      INTEGER          L           !L=PAR(10) !matrix to get order pos.
      INTEGER          IX          !Loop variable
      INTEGER          NBY1        !Ysize of out ima
      INTEGER          ISTAT       !Status of operation (MIDAS env)
      INTEGER          MADRID      !Memory Adders Id (MIDAS env)
      INTEGER          DIM         !Size of profile
      
      INTEGER          NBY2        !Ysize of out ima
      INTEGER          IAC         !Actual number of mapped pix (MIDAS env)
      INTEGER          NBY4        !Size of the buffer (MIDAS env)
      
      INTEGER          FID1        !file Id (MIDAS env)
      INTEGER          FID2        !file Id (MIDAS env)
      INTEGER          FID2V       !file Id (MIDAS env)
      INTEGER          FID3        !file Id (MIDAS env)
      INTEGER          FID4        !file Id (MIDAS env)
      INTEGER          FID4E       !file Id (MIDAS env)
      INTEGER          FID4M       !file Id (MIDAS env)
      
      INTEGER          FID5        !file Id (MIDAS env)
      INTEGER          FID6        !file Id (MIDAS env)
      INTEGER          FID7        !file Id (MIDAS env)
      
      INTEGER          IR(NAXIMA) 
      INTEGER          IXMIN       !pix id correxpondent to XMIN
      INTEGER          IXMAX       !pix id correxpondent to XMAX
      
      INTEGER          IXWIND      !size of order chunk to get obj pos 
                                   !and FHHM ditrib
      INTEGER          IOFF        !offset pixel point
      INTEGER          I           !Loop variable
 
      INTEGER*8        PT1         !Pointer id (MIDAS env)
      INTEGER*8        PT2         !Pointer id (MIDAS env)
      INTEGER*8        PT2V        !Pointer id (MIDAS env)
      INTEGER*8        PT3         !Pointer id (MIDAS env)
      INTEGER*8        PT4         !Pointer id (MIDAS env)
      INTEGER*8        PT4E        !Pointer id (MIDAS env)
      INTEGER*8        PT4M        !Pointer id (MIDAS env)
      INTEGER*8        PT5         !Pointer id (MIDAS env)
      INTEGER*8        PT6         !Pointer id (MIDAS env)
      INTEGER*8        PT7         !Pointer id (MIDAS env)
      

      REAL RON                     !Read Out Noise
      REAL CONAD                   !Conversion Analog to Digital
      REAL THRESH                  !Treshold for cosmic ray rejection
      
      REAL XMIN                    !Minimum X of order


      DOUBLE PRECISION START(NAXIMA) !Start point of image (X,Y)
      DOUBLE PRECISION STEP(NAXIMA)     !Step of image        (X,Y)
      DOUBLE PRECISION WIDTH       !Slit width in pix   

      DOUBLE PRECISION YV          !YV=PAR(11)!Current order
      DOUBLE PRECISION SKWOF       !Extraction parameter?



      INCLUDE 'MID_INCLUDE:ST_DEF.INC'  !to get the definitions
                                        !on the ST interfaces
                                        
      COMMON /VMR/MADRID(1)             !COMMON block for the 
                                        !pointer emulation in F77
                                        
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'  !to get data values for 
                                        !definitions above
                                        


C
C     Local variables initialization. 
C     NB DATA variables initialized with DATA statement
C

C     Integers


      K    =0
      L    =0
      IX   =0
      NBY1 =0
      ISTAT=0 
C      MADRID     !Memory Adders Id (MIDAS env)
      DIM  =0
      
      NBY2 =0
      IAC  =0
      NBY4 =0
      
      FID1 =0
      FID2 =0
      FID2V=0
      FID3 =0
      FID4 =0
      FID4E=0
      FID4M=0
      
      FID5 =0
      FID6 =0
      FID7 =0
      
      DO I=1,NAXIMA
         IR(I)=0 
      ENDDO
      IXMIN=0  
      IXMAX=0
      
      IXWIND=0 
      IOFF=0
C      Better to not initialize pointers to not have portability
C      problems on PC-Linux (this applies PT1-7)    

      

C     Reals
      RON=0.0   
      CONAD=0.0 
      THRESH=0.0       
      XMIN=0.0   


C     Doubles
      DO I=1,NAXIMA
         START(I)=0.0D0
         STEP(I) =0.0D0
      ENDDO
      WIDTH =0.0D0      

      YV    =0.0D0   
      SKWOF =0.0D0   

C     Characters
C      CUNIT(1:) = ' '   !CUNIT is initialized from a DATA statement

     
      
      

     
C
C ... define extraction constants
C
      WIDTH  = IWIDTH                          ! slit width in pix units
      SKWOF  = 0.0D0
      XMIN   = 0.0
      NX     = SKWOF
            
          
                
C
C ... get RON and CONAD
C
      THRESH = PAR(2)           !cosmic ray rejection threshold
      CONAD  = PAR(3)           !inverse GAIN: CONAD [e-/ADU]
      RON    = PAR(4)           !readout-noise [e-]
      RON    = (RON / CONAD)**2 !-> converted to ADU^2 (variance!)
      
      
      

C
C ... get ordinate values
C
      K      = PAR(9)           !Dimension of the coefficient
      L      = PAR(10)          !matrix to get order pos.
      YV     = PAR(11)          !Current order
      
      
      
                
C
C ... get offset within weight buffer for corresponding order
C
      IOFF   = (INDX - 1) * DIMP + 1
             

C
C ... loop over the samples
C

      NBY1 = NPIXA1                !number of samples
      
C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.      
      CALL STFCRE('vdummx',D_R8_FORMAT,F_X_MODE,1,NBY1,FID1,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error opening vdummx')
      ENDIF      

C     Module STFMAP (f77): Map a new data frame into memory 
C                         (just get virtual memory (iomode=F_X_MODE))
      CALL STFMAP(FID1,F_X_MODE,1,NBY1,IAC,PT1,ISTAT) 
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping vdummx')
      ENDIF
      




C     Gets centers of the orders at each wavelength     
      CALL CENT(MADRID(PT1),NPIXA1,DPAR,ND,K,L,YV,IWIDTH,SCAN,OFFSET,
     1          IXMIN,IXMAX,STARTA,STEPA)
     

           

      DIM = DIMP                !same size as the profil, why not !?
                                !DIM = ((IWIDTH/2)*2+2)*BINSIZE+1
      START(1) = 0
      START(2) = 0
      STEP(1) = 1.D0
      STEP(2) = 1.D0/BINSIZE 
      IR(1) = NPIXA1
      IR(2) = DIM
      NBY2  = NPIXA1 * DIM
      

C     creates and maps new frames to store output results from this routine
C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))                
      CALL STFCRE('vdummy',D_R4_FORMAT,F_X_MODE,1,NBY2,FID2,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating vdummy')
      ENDIF

      CALL STFMAP(FID2,F_X_MODE,1,NBY2,IAC,PT2,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping vdummy')
      ENDIF

      CALL STFCRE('vari',D_R4_FORMAT,F_X_MODE,1,NBY2,FID2V,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating vari')
      ENDIF

      CALL STFMAP(FID2V,F_X_MODE,1,NBY2,IAC,PT2V,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping vari')
      ENDIF

C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))       
      CALL STFCRE('mask',D_I2_FORMAT,F_X_MODE,1,NBY2,FID3,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating mask frm')
      ENDIF

      CALL STFMAP(FID3,F_X_MODE,1,NBY2,IAC,PT3,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping mask frm')
      ENDIF

      IR(2) = 5                 !parameter, error and mask buffer
      NBY4 = IR(1) * IR(2)      !size of the buffer



C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))  
      CALL STFCRE('pbuf',D_R4_FORMAT,F_X_MODE,1,NBY4,FID4,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating pbuf frm')
      ENDIF

      CALL STFMAP(FID4,F_X_MODE,1,NBY4,IAC,PT4,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping pbuf frm')
      ENDIF

      CALL STFCRE('ebuf',D_R4_FORMAT,F_X_MODE,1,NBY4,FID4E,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating ebuf frm')
      ENDIF

      CALL STFMAP(FID4E,F_X_MODE,1,NBY4,IAC,PT4E,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping ebuf frm')
      ENDIF

      CALL STFCRE('pmsk',D_I2_FORMAT,F_X_MODE,1,NBY4,FID4M,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating pmsk frm')
      ENDIF

      CALL STFMAP(FID4M,F_X_MODE,1,NBY4,IAC,PT4M,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping pmsk frm')
      ENDIF

      IR(1) = DIM
      IR(2) = 1                 !parameter buffer
      NBY4 = IR(1) * IR(2)      !size of the buffer
      
 
C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))        
      CALL STFCRE('colum',D_R4_FORMAT,F_X_MODE,1,NBY4,FID5,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating colum frm')
      ENDIF

      CALL STFMAP(FID5,F_X_MODE,1,NBY4,IAC,PT5,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping colum frm')
      ENDIF

      CALL STFCRE('sigma',D_R4_FORMAT,F_X_MODE,1,NBY4,FID6,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating sigma frm')
      ENDIF

      CALL STFMAP(FID6,F_X_MODE,1,NBY4,IAC,PT6,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping sigma frm')
      ENDIF

      CALL STFCRE('xpos',D_R4_FORMAT,F_X_MODE,1,NBY4,FID7,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error creating xpos frm')
      ENDIF

      CALL STFMAP(FID7,F_X_MODE,1,NBY4,IAC,PT7,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTOBJ: Error mapping xpos frm')
      ENDIF

      DO IX=1,NPIXA1
         OUT(IX,INDX) = 0
      ENDDO
      
                 
      CALL VSAMPL(VARI,WGT(1,IOFF),MADRID(PT1),MADRID(PT2V),
     1     MADRID(PT3),DIMP,IWIDTH,BINSIZE,NPIXA1,IXMIN,
     2     IXMAX,NPIXA2,DIM,IKIND)
     


    
      CALL VSAMPL(IN,WGT(1,IOFF),MADRID(PT1),MADRID(PT2),
     1     MADRID(PT3),DIMP,IWIDTH,BINSIZE,NPIXA1,IXMIN,
     2     IXMAX,NPIXA2,DIM,IKIND)
     
     

          

      IXWIND = 32! 16               !window size of the order chunks

C     Get profile pos,FWHM distributions and fits x-order profile
C     determining output image and weights
      CALL CAPROF(WGT(1,IOFF),MADRID(PT2V),MADRID(PT2),MADRID(PT3)
     $     ,MADRID(PT4),MADRID(PT4E),MADRID(PT4M),MADRID(PT5)
     $     ,MADRID(PT6),MADRID(PT7),OUT,VOUT,NB1,NB2,NPIXA1,IXWIND,IXMIN
     $     ,IXMAX,DIM,DIMP,5,RON,CONAD,THRESH,BINSIZE,WIDTH,INDX,OBJSET
     $     ,FWHM,MID_S_N_CENT,OBJ_POS_CENT,N_CURR_ORD,DPAR,ND
     $     ,STOCH,STOCH2,DTID1,DTID2)
     
     
     
C     Module STFCLO (f77): close a frame 

 99   CALL STFCLO(FID1,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID1')

      CALL STFCLO(FID2,ISTAT) 
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID2')

      CALL STFCLO(FID2V,ISTAT) 
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID2V')

      CALL STFCLO(FID3,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID3')

      CALL STFCLO(FID4,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID4')

      CALL STFCLO(FID4E,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID4E')

      CALL STFCLO(FID4M,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID4M')

      CALL STFCLO(FID5,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID5')

      CALL STFCLO(FID6,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID6')

      CALL STFCLO(FID7,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTOBJ: Error closing FID7')
    
      RETURN

      END

C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where


C  AM     07/06/01       Corrected some typo and some not  EXTFF
C                        proper initializations to fix 
C                        portability problems to PC Linux
C  AM     29/05/01       Corrected typo in IKIND info      EXTFF
C  AM     08/05/01       clean-comment                     EXTFF
C------------------------------------------------------------------------------
C  Role: Extract the FF
C        creates and maps vdummx
C        call CENT to get the centres of the orders at each wavelength
C        call CALC_I/CALC_P to to a weigted mean of the FF signal
C------------------------------------------------------------------------------
C Pseudocode:
C        some initial setting for the extraction constants
C        crea/maps frame vdummx
C        call CENT to get order centers
C        initializes out ima to 0
C        call CALC_I or CALC_P if IKIND is 0 or 1
C        close every open frame
C        RETURN
C-----------------------------------------------------------------------------
      SUBROUTINE EXTFF(IN,VARI,NPIXA1,NPIXA2
     $     ,OUT,VOUT,NB1,NB2,WGT,NC1,NC2,INDX,NPAR,PAR
     $     ,ND,DPAR,SCAN,OFFSET,STARTA,STEPA,IKIND,IWIDTH,DIMP)
C
C  Do an immediate extract using the weights of a previous optimal extraction
C
C   ARGUMENTS
C IN     REAL*4  INPUT IMAGE

C VARI   REAL*4  VARIANCE IMAGE
C NPIXA1 INTG*4  NO. OF SAMPLES
C NPIXA2 INTG*4  NO. OF LINES
C OUT    REAL*4  EXTRACTED ORDERS
C VOUT   REAL*4  VARIANCE OF EXTRACTED ORDERS
C NB1    INTG*4  NO. OF SAMPLES
C NB2    INTG*4  NO. OF ORDERS
C WGT    REAL*4  WEIGHT MAP
C NC1    INTG*4  NO. OF SAMPLES
C NC2    INTG*4  NO. OF ORDERS * THICKNESS OF ORDERS (profil dimension)
C INDX   INTG*4  SEQUENTIAL NO. OF THE ORDER
C NPAR   INTG*4  NO. OF EXTRACTION PARAMETERS
C PAR    REAL*4  EXTRACTION PARAMETERS
C ND     INTG*4  NO. OF COEFFS.
C DPAR   REAL*8  COEFFS. TO DEFINE ORDER POSITION
C IKIND  INTG*4  EVEN/ODD FLAG: 1: EVEN, 0: ODD SLIT WIDTH SIZE
C IWIDTH INTG*4  SLIT WIDTH  (size perpendicular to dispersion in pixel)
C DIMP   INTG*4  DIMENSION OF THE PROFILE OF THE ORDERS
C
      IMPLICIT NONE
C
C     Variable from interface (defined and initialized elsewere)
C
      INTEGER  NAXIMA
      PARAMETER (NAXIMA=2)              !Number of ima axes
      INTEGER  NPIXA1                   !Number of X pix of ima
      INTEGER  NPIXA2                   !Number of Y pix of ima
      INTEGER  NB1                      !Number of X pix of ima
      INTEGER  NB2                      !Number of Y pix of ima
      INTEGER  NC1                      !Number of X pix of ima
      INTEGER  NC2                      !Number of Y pix of ima
      INTEGER  NPAR                     !Number of parameters
      INTEGER  ND                       !Number of polynomial coefficients
      INTEGER  IKIND                    !switch for even/odd pix No. slit width
      INTEGER  IWIDTH                   !Dimension of order
      INTEGER  DIMP                     !Dim (No of points) of x-order profile
      INTEGER  INDX
      INTEGER  SCAN(NAXIMA)
      
      REAL WGT(NC1,NC2)                 !Weight ima
      REAL OUT(NB1,NB2)                 !Output ima
      REAL VOUT(NB1,NB2)                !output variance ima
      
      REAL IN(NPIXA1,NPIXA2)            !Input ima
      REAL VARI(NPIXA1,NPIXA2)          !input variance ima
      REAL OFFSET                       !offset
      REAL PAR(NPAR)                    !array holding extraction par
      
      DOUBLE PRECISION DPAR(ND)         !Array with polynomial coefficients
      DOUBLE PRECISION STARTA(NAXIMA)   !Ima starting point (X,Y)
      DOUBLE PRECISION STEPA(NAXIMA)    !Ima steps (X,Y)

C
C     Local variables
C

      INTEGER          K
      INTEGER          L
      INTEGER          IX
      INTEGER          NBY1             !Number of X points of ima
      INTEGER*8        PT1              !Pointer id (MIDAS env)
      INTEGER          ISTAT            !Status of operation (MIDAS env)
      INTEGER          MADRID           !Memory Address Id (MIDAS env)
      INTEGER          IAC              !Actual No of mapped points (MIDAS env)
      INTEGER          FID1             !File Id (MIDAS env)
      INTEGER          IXMIN            !Pixel id corresponting to XMIN
      INTEGER          IXMAX            !Pixel Id corresponding to XMAX
      INTEGER          IOFF             !offset of pixel point      


      DOUBLE PRECISION YV               !Current order

 
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'  !to get the definitions
                                        !on the ST interfaces
                                        
      COMMON /VMR/MADRID(1)             !COMMON block for the 
                                        !pointer emulation in F77
                                        
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'  !to get data values for 
                                        !definitions above
      
C
C     Initialization of variables
C               

C     Integers
      K  =0
      L  =0
      IX =0
      NBY1=0  
      PT1 =0  
      ISTAT=0 
C      MADRID           !Memory Address Id (MIDAS env)
      IAC=0   
      FID1=0 
      IXMIN=0
      IXMAX=0
      IOFF=0
C     Reals

C     Doubles

      YV=0.0D0       
C     Characters


C
C ... get ordinate values
C
      K      = PAR(9)           !Dimension of the coefficient
      L      = PAR(10)          !matrix to get order pos.
      YV     = PAR(11)          !Current order
C
C ... get offset within weight buffer for corresponding order
C
      IOFF   = (INDX - 1) * DIMP + 1
C
C ... loop over the samples
C
      NBY1 = NPIXA1 
      
      
C     Module STFCRE (f77): create a frame with data of given type 
C                          and return its internal file no.
C     Module STFMAP (f77): Map a new data frame into memory 
C                          (just get virtual memory (iomode=F_X_MODE))            
      CALL STFCRE('vdummx',D_R8_FORMAT,F_X_MODE,1,NBY1,FID1,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTFF: Error creating vdummx frm')
      ENDIF

      CALL STFMAP(FID1,F_X_MODE,1,NBY1,IAC,PT1,ISTAT) 
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'EXTFF: Error mapping vdummx frm')
      ENDIF
c
c get the centers of the orders: stored in yval[] == madrid(pt1)
c
      CALL CENT(MADRID(PT1),NPIXA1,DPAR,ND,K,L,YV,IWIDTH,SCAN,OFFSET,
     1          IXMIN,IXMAX,STARTA,STEPA)

          
      DO IX=1,NPIXA1
         OUT(IX,INDX) = 0
      ENDDO


      IF (IKIND.EQ.0) THEN !switch for even/odd pix No. slit width
                
         CALL CALC_I(IN,VARI,WGT(1,IOFF),MADRID(PT1),OUT,VOUT
     $        ,NPIXA1,NPIXA2,NB1,NB2,DIMP,IXMIN,IXMAX,IWIDTH,INDX)
     
     
      ELSE                 
                       
         CALL CALC_P(IN,VARI,WGT(1,IOFF),MADRID(PT1),OUT,VOUT
     $        ,NPIXA1,NPIXA2,NB1,NB2,DIMP,IXMIN,IXMAX,IWIDTH,INDX)


      ENDIF
             
C     Module STFCLO (f77): close a frame           
      CALL STFCLO(FID1,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'EXTFF: Error closing FID1')
     

      END



C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C
C  AM     03/07/01       portability-D1NAME,D2NAME are     CAPROF
C                        now char*N with N multiple of 8
C
C  AM     24/05/01       BINX and BINY must not be         CAPROF
C                        initialized to 0 more than once!
C
C  AM     08/05/01       clean-comment                     CAPROF
C------------------------------------------------------------------------------
C  Role: determines the order and FWHM distribution along dispersion coordinate
C        First divide the order in subsections, collapse the signal in each 
C        subsection over dispersion direction obtaining a series of x-order
C        light distribution profiles. To these are done gaussian fits so that
C        one gets a series of order center positions and FWHMs. These two
C        distribution are interpolated with a parabola (or if the parabola
C        coefficients are not proper a linear or uniform) approximation.
C------------------------------------------------------------------------------
C Pseudocode:
C        Prepare tables to log info on obj pos/fwhm and x-order chunk profile
C        Check:
C        if an order has one point only
C           Output Variance=Flux =1
C           PROF=1
C           PBUF=VBUG=1
C           FWHM=S/N=OBJ_POS=0
C           RETURN
C        endif
C        DO K=1,NUM  :Loops over all chunks
C           DO J=1,DIM : Loop over x-order direction
C              DO=start_pix(chunk),end_pix(chunk)
C                if not masked collapse intensity (COLUMN=sum(FBVAL)*MASK)
C                if not masked collapse variance  (SIGMA=sum(VBVAL)*MASK)
C              ENDDO
C              get max of x-order profile and its pos
C              fill x-order table with x-order profile values found
C           ENDDO
C           call GAUSS_FITX to fit the x-order profile with a Gaussian+linear
C           save Gauss parameters in array PBUF,VBUF,PMSK
C           check: counts the masked points
C
C           copy PBUF,VBUF,PMSK into POSVAL,SIGVAL,POSMSK and set POSWAV
C           for later fit along order
C        ENDDO
C        if the number of masked point is less than 10 issue a warning
C        do a pre k-sigma clip on POS,FWHM values found and mask out bad points
C        do a polynomial fit of POS and FWHM unmasked point distributions
C        save results in order_table
C        calculates chi2 between fit and actual POS,FWHM distributions
C        if both the chi2(POS & FWHM) .lt. 2e-2 => good fit =>stop (goto 96)
C        if chi2(POS).gt.0.5 check coeff polyn and if bad lower degree to 1
C        else (diff tresh)   check coeff polyn and if bad lower degree to 1
C
C        if chi2(FWHM).gt.1  check coeff polyn and if bad lower degree to 1
C        else (diff tresh)   check coeff polyn and if bad lower degree to 1
C        do a polynomial fit of POS and FWHM unmasked point distributions
C        if pol degree=1 check coeff polyn(POS)  and if bad lower degree to 0
C        if pol degree=1 check coeff polyn(FWHM) and if bad lower degree to 0
C        logs the polynomial deg in the order_table
C        do a k-sigma clipping to remove bad point from POS & FWHM distribution
C           do a polynomial fit (POS)
C           do a polynomial fit (FWHM)
C           get the points which scatter the most from POS & FWHM distribution
C           mask it out
C        enddo (k-sigma)
C
C 96:    The found distributions are now Ok (end):
C
C        DO =IXMIN,IXMAX (loop along the order)
C           PBUF(2,I)=position from polynomial fit of POS  distribution
C           PBUF(3,I)=position from polynomial fit of FWHM distribution
C        ENDDO
C        Stores results in order_table
C        RETURN
C
C------------------------------------------------------------------------------
      SUBROUTINE CAPROF(PROF,VBVAL,FBVAL,MASK,PBUF,VBUF,PMSK
     $     ,COLUM,SIGMA,XPOS,OUT,VOUT,NB1,NB2,NPIXA1,IXWIND,IXMIN,IXMAX
     $     ,DIM,DIMP,PDIM,RON,CONAD,THRESH,BINSIZE,WIDTH,INDX,OBJSET
     $     ,FWHM,MID_S_N_CENT,OBJ_POS_CENT,N_CURR_ORD,DPAR,ND,STOCH
     $     ,STOCH2,DTID1,DTID2) 

      IMPLICIT NONE
      INTEGER MINPOI
      INTEGER POLORD                     !Order of polynomial
      INTEGER NGPAR                      !No of par of Gauss+linear
      INTEGER ORDFIT                     !Order of polynomial fit
      INTEGER N_MAX_ORD                  !Expected max number of orders
      INTEGER MAX_DIM                    !Expected max X ima size
      INTEGER ND1COL
      INTEGER ND2COL
      PARAMETER (N_MAX_ORD=50)           !Max No. of Orders to be changed 
                                         !accordingly in many places
      PARAMETER (MAX_DIM=4096,ND1COL=18,ND2COL=4)
      REAL KAPPA                         !constant for k-sigma clipping
      PARAMETER (MINPOI=15,KAPPA=6.0,POLORD=3,ORDFIT=5,NGPAR=5)
C
C     Interface variables  
C   
      INTEGER DIM                        !Y dimension of mask
      INTEGER DIMP                       !Dimention of profile
      INTEGER PDIM                       !X dimention of mask
      INTEGER NPIXA1                     !Y dimention of mask
      INTEGER NB1                        !X dim of ima
      INTEGER NB2                        !Y dim of ima
      INTEGER INDX
      INTEGER*2 MASK(NPIXA1,DIM)         !Mask array
      INTEGER*2 PMSK(PDIM,NPIXA1)        !profile mask
      INTEGER IXMIN                      !pix position of XMIN
      INTEGER IXMAX                      !pix position of XMAX
      INTEGER IXWIND                     !size of window (chunk size)
      INTEGER BINSIZE                    !Detector Bin Size
      INTEGER DTID1                      !Debug table Id.
      INTEGER DTID2                      !Id of debug table  (MIDAS env)
      INTEGER STOCH                      !current raw position in tab 1
      INTEGER STOCH2                     !current raw position in tab 2
      INTEGER ND                         !No of coeffs in polynomial order

      REAL MID_S_N_CENT(N_MAX_ORD)       !av. S/N at the center of the order
      REAL OBJ_POS_CENT(N_MAX_ORD)       !av. obj pos at order center
      REAL FWHM(N_MAX_ORD)               !average FWHM at the center of order

      REAL VBVAL(NPIXA1,DIM)             !variance ima
      REAL FBVAL(NPIXA1,DIM)             !raw data
      REAL PROF(NPIXA1,DIMP)             !profile
      REAL PBUF(PDIM,NPIXA1)             !Profile buffer
      REAL VBUF(PDIM,NPIXA1)             !Variance buffer
      REAL OUT(NB1,NB2)                  !Output image
      REAL VOUT(NB1,NB2)                 !Output variance
      REAL RON                           !Read Out Noise
      REAL CONAD                         !Analog To Digital conversion factor
      REAL THRESH                        !threshold??
      
      REAL XPOS(DIM)
      REAL COLUM(DIM)
      REAL SIGMA(DIM)                    
      REAL OBJSET                        !position of object

      DOUBLE PRECISION DPAR(ND)          !Polynomial coefficients of order
      DOUBLE PRECISION WIDTH             !Slit width

      
C
C     Local variables
C

      INTEGER ILOOP                      !loop variable
      INTEGER IPOI                       !counter variable
      INTEGER I                          !loop variable
      INTEGER J                          !loop variable
      INTEGER IXWMAX                     !IXWMAX=IXMIN+(IXMAX-IXMIN)/IXWIND-1  
      INTEGER*2 IMASK                    !mask: 1=valid pixel, 0:invalid
      INTEGER ISTAT                      !Status of operation (MIDAS env)
      INTEGER JWIN                       !JWIN=2*BINSIZE
      INTEGER LP(NGPAR)                  !data structure to swich on/off fit 
                                         !coeff variation
      INTEGER MFIT                       !No. of fit coeff
      INTEGER IMASK_COUNTER              !counts how many chunks survives.
      INTEGER STATUS                     !status of operation (MIDAS env)
      INTEGER IAPOS(ORDFIT)              !auxilliary array to dis/activate fit 
                                         !of coeff in position
      INTEGER IAFWHM(ORDFIT)             !auxilliary array to dis/activate 
                                         !fit of coeff in FWHM
      INTEGER NCLEAN                     !counter
      INTEGER IDPCH                      !Id of table column (MIDAS env)
      INTEGER IDPF0                      !Id of table column (MIDAS env)
      INTEGER IDPF1                      !Id of table column (MIDAS env)
      INTEGER IDPER0                     !Id of table column (MIDAS env)
      INTEGER IDS0                       !Id of table column (MIDAS env)
      INTEGER IDS1                       !Id of table column (MIDAS env)
      INTEGER IDICH                      !Id of table column (MIDAS env)
      INTEGER IDMSK0                     !Id of table column (MIDAS env)
      INTEGER IDXCH0                     !Id of table column (MIDAS env)
      INTEGER IDORD                      !Id of table column (MIDAS env)
      INTEGER IDFWHM                     !Id of table column (MIDAS env)
      INTEGER IDFWER                     !Id of table column (MIDAS env)
      INTEGER ICURP                      !current raw position in table
      INTEGER NPOINT                     !actual unmasked point for average
      INTEGER IT                         !IT=ORDFIT=fit degree
      INTEGER KFIT                       !number of fit params to be modified
      INTEGER KP(ORDFIT)                 !array to spec parms to be modified 
      INTEGER IDDEG                      !Id of table column (MIDAS env)
      INTEGER IDCHI0                     !Id of table column (MIDAS env)
      INTEGER IDKS                       !Id of table column (MIDAS env)
      INTEGER BINX                       !Bin X size
      INTEGER BINY                       !Bin Y size
      INTEGER NFIT                       !counter to get chi2red
      INTEGER D1COL(ND1COL)              !Descriptor id (MIDAS env)
      INTEGER D2COL(ND2COL)              !Descriptor id (MIDAS env)
      INTEGER D1ROW                      !current raw
      INTEGER D2ROW                      !current raw
      INTEGER PDEG                       !Polynomial degree
      INTEGER SDEG                       !Polynomial degree
      INTEGER N_CURR_ORD                 !current order position




      INTEGER I1                         !Aux var
      INTEGER I2                         !aux var
      INTEGER ICURP2                     !aux var
      INTEGER IDBKGE                     !column no.
      INTEGER IDBKGV                     !column no.
      INTEGER IDMSK1                     !column no.
      INTEGER IMAX                       !aux var
      INTEGER J1                         !aux var
      INTEGER JMAX                       !aux var
      INTEGER NUM                        !aux var
      INTEGER IORDER                     !current order position
      INTEGER K                          !Loop variable
      INTEGER KK                         !Loop variable
      INTEGER kkmax                      !aux variable

 

      REAL CHISQ                         !chi square
      REAL EPOS                          !error on pos
      REAL AVERR                         !average error
      REAL PARAM(NGPAR)                  !parameters of Gaussian profile
      REAL VPARAM(NGPAR)                 !variance parameters
      REAL CHI2(NGPAR)                   !chi square
      REAL AMIN1                         !funct to get the min of 2 real values
      REAL LOW                           !min thresh on FWHM to mask out points
      REAL HGH                           !max thresh on FWHM to mask out points
      REAL PCOF1(POLORD)                 !profile coefficients
      REAL PCOF2(POLORD)                 !profile coefficients
      REAL POLYN3                        !polynomial of degree 3
      REAL X1                            !tmp holder for X position
      REAL XSTP                          !X step= size of chunk
      REAL KAPPAMAX                      !maximum value of k in k-sigma clip

      REAL X1MAX                         !max discrepancy of pos
      REAL TCOEF(ORDFIT)                 !true coefficient
      REAL COVAR(ORDFIT,ORDFIT)          !covariance array
      REAL POSWAV(MAX_DIM)               !wavelength of pos
      REAL POSVAL(MAX_DIM)               !value of position
      REAL POSERR(MAX_DIM)               !error on position
      REAL BKGVAL(MAX_DIM)               !value of position
      REAL BKGERR(MAX_DIM)               !error on position
      REAL POSMSK(MAX_DIM)               !mask on position
      REAL SIGVAL(MAX_DIM)               !sigma of value
      REAL SIGERR(MAX_DIM)               !sigma of error
      REAL SIGMSK(MAX_DIM)               !sigma of mask
      REAL CI                            !current index along the order
      REAL CHIFIT                        !chi-square value of fit to chunk pos
      REAL AFWHM                         !Average FWHM
      REAL EFWHM                         !Error on FWHM  
      REAL AVPOS                         !Average position
      REAL AVFWHM                        !Average FWHM

      REAL SIFWHM                        !Sigma FWHM
      REAL SIPOS                         !Sigma FWHM
      REAL SUM                           !Aux var to store sum of chi2
      REAL XFIT                          !value for chi of fit
      REAL YMAX                          !Aux variable
      REAL YMIN                          !aux variable

      DOUBLE PRECISION D2VAL(ND2COL)     !Descriptor value
       
      CHARACTER*72 LINE                  !Buffer to display info
      CHARACTER*64 D1NAME(ND1COL)        !Descriptor name
      CHARACTER*64 D2NAME(ND2COL)        !Descriptor name

      EXTERNAL POLYN                     !Polynomial
      EXTERNAL POLYN3                    !Polynomial of degree 3
      EXTERNAL GAUSS_FITX                !Gaussian Fit function
      EXTERNAL POLY_FITX                 !Polynomial Fit function
      EXTERNAL POLFIT                    !Polynomial Fit function
      EXTERNAL LSFIT                     !Least Square Fit function
      EXTERNAL MYAVRG                    !Function to get an average
      EXTERNAL MYSIGM                    !Function to get a sigma

      COMMON /FIT/BINX,BINY              !Necessary to set tresholds
                                         !over polynomial fit slope and 
                                         !curvature BINX,BINY dependent

      COMMON /DBG/IORDER                 !Used for debugging purposes?
      
      INCLUDE 'MID_INCLUDE:ST_DEF.INC'   !to get the definitions 
                                         !on the ST interfaces
      COMMON /VMR/MADRID(1)              !COMMON block for the 
                                         !pointer emulation in F77
      INCLUDE 'MID_INCLUDE:ST_DAT.INC'   !to get data values for 
                                         !definitions above
                                         
                                         
      DATA LP/1,2,3,4,0/                 !To dis/activate parameter variation
      DATA MFIT/4/                       !Fit degree
      
      DATA KP/1,2,3,0,0/                 !To dis/activate parameter variation
      DATA KFIT/3/                       !Fit degree
   
C
C     Initialization of local variables: 
C     DATA variables are initialized with DATA statement
C

C     Integers
      ILOOP=0     
      IPOI =0  
      I    =0  
      J    =0  
      IXWMAX=0   
      IMASK =0 
      ISTAT =0 
      JWIN  =0


      IMASK_COUNTER=0
      STATUS=0

      DO I=1,ORDFIT
         IAPOS(I)=0
         IAFWHM(I)=0 
      ENDDO

      NCLEAN=0
      IDPCH =0
      IDPF0 =0
      IDPF1 =0
      IDPER0=0
      IDS0  =0
      IDS1  =0
      IDICH =0
      IDMSK0=0
      IDXCH0=0
      IDORD =0
      IDFWHM=0
      IDFWER=0
      ICURP =0            
      NPOINT=0
      IT    =0
      KFIT  =0

      IDDEG =0
      IDCHI0=0
      IDKS  =0
      IDBKGV=0
      IDBKGE=0
      NFIT  =0
      K=0
      KK=0
      kkmax=0

      DO I=1,ND1COL           
         D1COL(I)=0
      ENDDO

      DO I=1,ND1COL           
         D2COL(I)=0
      ENDDO

      D1ROW =0
      D2ROW =0
      PDEG  =0
      SDEG  =0


      I1=0             
      I2=0             
      ICURP2=0         
      IDBKGE=0         
      IDBKGV=0         
      IDMSK1=0         
      IMAX=0           
      J1=0             
      JMAX=0          
      NUM=0           



C     Reals

      CHISQ=0.0
      EPOS =0.0
      AVERR=0.0
      DO I=1,NGPAR
         PARAM(I)=0.0 
         VPARAM(I)=0.0
         CHI2(I)=0.0  
      ENDDO

      LOW  =0.0
      HGH  =0.0
      DO I=1,POLORD
         PCOF1(I)=0.0
         PCOF2(I)=0.0
      ENDDO

C      POLYN3               !this is a function
      X1    =0.0  
      XSTP  =0.0  
      KAPPAMAX=0.0
     
      X1MAX   =0.0
      
      DO I=1,ORDFIT
      TCOEF(I)=0.0           
         DO J=1,ORDFIT
            COVAR(I,J)=0.0   
         ENDDO
      ENDDO

      DO I=1,MAX_DIM
         POSWAV(I)=0.0
         POSVAL(I)=0.0
         POSERR(I)=0.0
         POSMSK(I)=0.0
         SIGVAL(I)=0.0
         SIGERR(I)=0.0
         SIGMSK(I)=0.0
         BKGVAL(I)=0.0
         BKGERR(I)=0.0
      ENDDO
      CI=0.0
      CHIFIT=0.0
      AFWHM=0.0 
      EFWHM=0.0 
  
      AVPOS=0.0 
      AVFWHM=0.0

      SIFWHM=0.0   
      SIPOS=0.0    
      SUM=0.0     
      XFIT=0.0     
      YMAX=0.0     
      YMIN=0.0   

C     Doubles

      DO I=1,ND2COL
         D2VAL(I)=0.0D0 
      ENDDO
C     Chars
      LINE(1:)= ' ' 


   
c ================================================================
c >>>>>>>> OPTIMAL EXTRACTION DEBUG TABLES COLUMN ID SEARCH  <<<<<
c ================================================================
 
 
 
      D1NAME(1)=':ORDER'
      D1NAME(2)=':I_CH'
      D1NAME(3)=':X_CH_0'
      D1NAME(4)=':P_ERR0'
      D1NAME(5)=':S_CH_0'
      D1NAME(6)=':S_CH_1'
      D1NAME(7)=':P_FWHM'
      D1NAME(8)=':P_FWHM_ERR'
      D1NAME(9)=':P_CH'
      D1NAME(10)=':P_F0'
      D1NAME(11)=':P_F1'
      D1NAME(12)=':M0'
      D1NAME(13)=':M1'
      D1NAME(14)=':DEG'
      D1NAME(15)=':CHI0'
      D1NAME(16)=':KS'
      D1NAME(17)=':BKGVAL'
      D1NAME(18)=':BKGERR'
      
      DO ILOOP=1,ND1COL
         CALL TBCSER(DTID1,D1NAME(ILOOP),D1COL(ILOOP),ISTAT)
         IF (D1COL(ILOOP).EQ.-1) THEN
            print *,'I=',ILOOP
            CALL STETER(11,'Column not present')
         ENDIF
      ENDDO
    
      CALL TBCSER(DTID1,':ORDER',IDORD,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :ORDER')

      CALL TBCSER(DTID1,':I_CH',IDICH,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :I_CH')

      CALL TBCSER(DTID1,':X_CH_0',IDXCH0,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :X_CH_0')
         

      CALL TBCSER(DTID1,':P_ERR0',IDPER0,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :P_ERR0')

      CALL TBCSER(DTID1,':S_CH_0',IDS0,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :S_CH_0')

      CALL TBCSER(DTID1,':S_CH_1',IDS1,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :S_CH_1')
    


      CALL TBCSER(DTID1,':P_FWHM',IDFWHM,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :P_FWHM')

      CALL TBCSER(DTID1,':P_FWHM_ERR',IDFWER,ISTAT)
      IF (ISTAT.NE.0) THEN
         CALL STETER(11,'CAPROF: Error searching :P_FWHM_ERR')
      ENDIF



      CALL TBCSER(DTID1,':P_CH',IDPCH,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :P_CH')

      CALL TBCSER(DTID1,':P_F0',IDPF0,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :P_F0')

      CALL TBCSER(DTID1,':P_F1',IDPF1,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :P_F1')
     


      CALL TBCSER(DTID1,':M0',IDMSK0,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :M0')

      CALL TBCSER(DTID1,':M1',IDMSK1,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :M1')
      


      CALL TBCSER(DTID1,':DEG',IDDEG,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :DEG')

      CALL TBCSER(DTID1,':CHI0',IDCHI0,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :CHI0')

      CALL TBCSER(DTID1,':KS',IDKS,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :KS')
    
    
      CALL TBCSER(DTID1,':BKGVAL',IDBKGV,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :BKGVAL')
   
      CALL TBCSER(DTID1,':BKGERR',IDBKGE,ISTAT)
      IF (ISTAT.NE.0) CALL STETER(11,'CAPROF: Error searching :BKGERR')
     
      D2NAME(1)=':ORDER'
      D2NAME(2)=':I_CH'
      D2NAME(3)=':Y_CH'
      D2NAME(4)=':FLUX'
      DO ILOOP=1,4
         CALL TBCSER(DTID2,D2NAME(ILOOP),D2COL(ILOOP),ISTAT)
         IF (D2COL(ILOOP).EQ.-1) CALL STETER(11,'Column not present')
      ENDDO


c ==============================================================

      ICURP=STOCH    
      ICURP2=STOCH2



c
c     Chop the order (FBVAL) into equaly spaced windows (IXWIND) in
c     wavelength space.
c     Do an averaging in the chunks to get averaged order profiles
c     Fit a gaussian to each of these averaged profiles.
c     Save the parameters of the gaussian in the first four rows
c     of FBVAL.
c     These parameters will finally be fitted in wavelength space,
c     this will be done by G_PROF().
c


      IMASK = 1                 !mask: 1=valid pixel, 0:invalid

      LOW = 1.1                 !these are empirical data !!!
      HGH = BINSIZE*WIDTH/3.0D0 !.. and should be replaced by measurements

      NUM = (IXMAX - IXMIN) / IXWIND    !dimension in pix of each chunk
      STOCH=STOCH+NUM
      STOCH2=STOCH2+NUM*DIM

C      if NUM is 0 means that it has identified one order containing only
C      one point. So we have to fill basic arrays with something and skip
C      the order
      IF (NUM .EQ. 0) THEN 
               
               
         DO I=1,NPIXA1
            DO J=1,DIM
               VBVAL(I,J)=1.0
               FBVAL(I,J)=1.0
            ENDDO
            DO J=1,DIMP
               PROF(I,J)=1.0
            ENDDO
            DO J=1,PDIM
               PBUF(J,I)=1.0
               VBUF(J,I)=1.0
            ENDDO   
         ENDDO  
         FWHM(N_CURR_ORD)=0
         MID_S_N_CENT(N_CURR_ORD)=0
         OBJ_POS_CENT(N_CURR_ORD)=0
         GOTO 100
      ENDIF     
             
      I1 = IXMIN
      I2 = I1 + IXWIND - 1
      CI = I1-IXWIND/2

      kk=1
      
      IMASK_COUNTER = 0
      

      DO K=1,NUM
         YMIN = 1E33
         YMAX = -YMIN
         J1 = 1

         DO J = 1,DIM           !Do the averaging within one window
            XPOS(J) = -1.0
            COLUM(J1) = 0.0
            SIGMA(J1) = 0.0
            IPOI  = 0
C
C           collapses the chunk            
C
            DO I=I1,I2

               COLUM(J1) = COLUM(J1) + FBVAL(I,J)*MASK(I,J)
               SIGMA(J1) = SIGMA(J1) + VBVAL(I,J)*MASK(I,J)
               IF(MASK(I,J).GT.0) IPOI = IPOI+1
            ENDDO

C
C ??? is this check ok!? we have to prevent that sigma is getting .le.0.0 !!
C
            IF (SIGMA(J1) .GT. 1.E-9) THEN  
               COLUM(J1) = COLUM(J1) / IPOI
               SIGMA(J1) = SQRT( SIGMA(J1) / IPOI / IPOI )
c               SIGMA(J1) = 1.0

c               SIGMA(J) = (RON + MAX(0.0,COLUM(J))/CONAD) / IXWIND 
               YMIN = AMIN1(COLUM(J1),YMIN) !get the minimum
               IF (COLUM(J1) .GT. YMAX) THEN !get maximum
                  JMAX = J1
                  YMAX = COLUM(JMAX)
               ENDIF
               XPOS(J1) = FLOAT(J)  !Here set the x pos to its real value
                                    !taking in count of the masking
 
           
C ==================================================================           
                D2VAL(1) = N_CURR_ORD
                D2ROW=ICURP2+(K-1)*DIM+J1
                D2VAL(2)=K 
                D2VAL(3)=XPOS(J1)
                D2VAL(4)=COLUM(J1)
                CALL TBRWRD(DTID2,D2ROW,4,D2COL,D2VAL,ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl value')
                ENDIF
C =================================================================
               J1 = J1+1
            ENDIF
         ENDDO
c        so finds Ymax and Ymin


C     Determine the initial parameters for gaussfit
         JWIN = 2 * BINSIZE
         PARAM(4) = (YMAX - YMIN) * 0.01    !base gaussian: constant of linear 
         PARAM(1) = YMAX - PARAM(4)         !gaussian amplitude
         PARAM(2) = DIM/2.+OBJSET           !gaussian center
C
C         IF (JMAX.LT.JWIN .OR. DIM-JMAX.LT.JWIN) GOTO 99
C
         PARAM(3) = 2.                      !gaussian sigma
         PARAM(5) = 0.                      !slope of linear term
         LP(5) = 0
         I = 5




         CALL GAUSS_FITX(XPOS,COLUM,SIGMA,J1-1,PARAM,I,LP,MFIT,VPARAM
     $        ,CHI2(1),ISTAT)

C  Check whether the output parameters are reasonable
C if not put IMASK = 0 

         IF (ISTAT.LT.0 .OR.
     $        PARAM(3).LT.LOW .OR. PARAM(3).GT.HGH .OR.
     $        PARAM(2).LT.1. .OR. PARAM(2).GT. FLOAT(DIM)) THEN 
            IMASK = 0
         ENDIF

         I = IXMIN+K-1


c     Save parameters for later fit of the parameters -> G_PROF()!

         PBUF(1,I) = 1.0
        
         DO J=2,5
            PBUF(J,I) = PARAM(J)
            VBUF(J,I) = VPARAM(J)
            PMSK(J,I) = IMASK   !mask out if parameters are BAD
                                !(PMSK may be a 1dim array, future changes)
         ENDDO
         
         IF(IMASK .GT. 0) THEN
            IMASK_COUNTER=IMASK_COUNTER+1
         ENDIF
         
         
         
         IMASK = 1
         I1 = I2 + 1
         I2 = I1 + IXWIND - 1          !in this way we go to the next chunk
         CI = CI + IXWIND



         IF (PBUF(2,I) .LT. 0 .OR. PBUF(2,I) .GT. DIM .OR.
     +       VBUF(2,I) .LT. 0 .OR. VBUF(2,I) .GT. DIM .OR.
     +       PMSK(2,I) .EQ. 0 ) THEN 
            POSVAL(kk)=DIM/2.+OBJSET
            SIGVAL(kk)=2.0
            POSERR(kk) = 1.0e6
            SIGERR(kk) = 1.0e6
            BKGVAL(kk)=PBUF(4,I)
            BKGERR(kk)=VBUF(4,I)
            POSMSK(kk)=PMSK(2,I)
            SIGMSK(kk)=PMSK(2,I)
            POSWAV(kk)=CI
            kk=kk+1 

         ELSE 

            POSVAL(kk)=PBUF(2,I) 
            POSERR(kk)=VBUF(2,I)
            SIGVAL(kk)=PBUF(3,I)
            SIGERR(kk)=VBUF(3,I)
            BKGVAL(kk)=PBUF(4,I)
            BKGERR(kk)=VBUF(4,I)
            POSMSK(kk)=PMSK(2,I)
            SIGMSK(kk)=PMSK(2,I)
            POSWAV(kk)=CI
            kk=kk+1 

         ENDIF

      ENDDO

 

      
C     ====================================================================  
      kkmax=kk-1

      IF ( IMASK_COUNTER .LT. 10 ) THEN
            print *,'Warning: IMASK_COUNTER LESS THAN 10'
      ENDIF
      
      TCOEF(1)=DIM/2.+OBJSET
      TCOEF(2)=.01
      TCOEF(3)=.0
      TCOEF(4)=.0
      TCOEF(5)=.0
      

        
      IT = ORDFIT
c       KP(2) = 0
c       KP(3) = 0
        KP(4) = 0
        KP(5) = 0
        
        

 
C     
C     calculates mean and std dev of errors on position   
C
      NCLEAN=0
      
70    NPOINT=0
      AVPOS=0
      AVFWHM=0
      NCLEAN=NCLEAN+1   


c      CALL MYAVRG(PBUF,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,2,AVPOS)
c      CALL MYSIGM(PBUF,AVPOS,PMSK,NPIXA1,IXMIN,IXMIN+NUM-1,3,AVFWHM)
c      print *,'AV=',AVPOS,AVFWHM

      AVPOS=0
      AVFWHM=0
      DO I = IXMIN,IXMIN+NUM-1
         IF (PMSK(2,I).GT.0) THEN
            AVPOS = AVPOS+PBUF(2,I) 
            AVFWHM = AVFWHM+PBUF(3,I)     
            NPOINT=NPOINT+1
         ENDIF      
      ENDDO
      AVPOS=AVPOS/NPOINT
      AVFWHM=AVFWHM/NPOINT
      
      NPOINT=0
      SIPOS=0.
      SIFWHM=0.
      DO I = IXMIN,IXMIN+NUM-1
         IF (PMSK(2,I).GT.0) THEN
            SIPOS = SIPOS+(PBUF(2,I)-AVPOS)*(PBUF(2,I)-AVPOS) 
            SIFWHM = SIFWHM+(PBUF(3,I)-AVFWHM)*(PBUF(3,I)-AVFWHM) 
            NPOINT=NPOINT+1
         ENDIF      
      ENDDO
      SIPOS=SQRT(SIPOS/(NPOINT-1))
      SIFWHM=SQRT(SIFWHM/(NPOINT-1))
      
      NPOINT=0
      AVERR=0
      AFWHM=0
      DO I = IXMIN,IXMIN+NUM-1
         IF (PMSK(2,I).GT.0) THEN
            AVERR = AVERR+VBUF(2,I) 
            AFWHM = AFWHM+VBUF(3,I)     
            NPOINT=NPOINT+1
         ENDIF      
      ENDDO
      AVERR=AVERR/NPOINT
      AFWHM=AFWHM/NPOINT
     
        
      NPOINT=0
      EPOS=0
      EFWHM=0
      DO I = IXMIN,IXMIN+NUM-1
         IF (PMSK(2,I).GT.0) THEN
            EPOS = EPOS+(VBUF(2,I)-AVERR)*(VBUF(2,I)-AVERR)   
            EFWHM = EFWHM+(VBUF(3,I)-AFWHM)*(VBUF(3,I)-AFWHM)   
            NPOINT=NPOINT+1
         ENDIF      
      ENDDO
      EPOS=SQRT(EPOS/(NPOINT-1))
      EFWHM=SQRT(EFWHM/(NPOINT-1))


c     mask out the point which have too big error  
c     these points are generated from GAUSS_FITX
c     the idea is that if a point has associated a big error
c     has not to be trusted    

      DO I = IXMIN,IXMIN+NUM-1
         IF ( 
     +        (VBUF(2,I)-AVERR).GT.(2*EPOS).OR.
     +        (VBUF(3,I)-AFWHM).GT.(2*EFWHM).OR.
     +    ABS((PBUF(2,I)-AVPOS)).GT.(2*SIPOS).OR.
     +    ABS((PBUF(3,I)-AVFWHM)).GT.(2*SIFWHM).OR.
     +        PBUF(3,I).LE.0.) THEN
            PMSK(2,I)=0
            POSMSK(I)=0
            SIGMSK(I)=0  
            POSERR(I)=1e6
            SIGERR(I)=1e6   
         ENDIF      
      ENDDO
      IF (NCLEAN.LE.1) GOTO 70  !Repeat if necessary 
                                !to have a better estimate of AVERR and EPOS
      
         

c =========================================================
      IXWMAX = IXMIN + (IXMAX - IXMIN) / IXWIND - 1

c =========================================================
c     Monitoring the values after LSFIT
      I1 = IXMIN+IXWIND/2            !central point in each chunk
      XSTP = IXWIND
      X1 = I1



      
      PCOF1(1)=AVPOS
      PCOF1(2)=.01
      PCOF1(3)=.0
      
      PCOF2(1)=AVFWHM
      PCOF2(2)=.01
      PCOF2(3)=.0




     
      IAPOS(1)=1
      IAPOS(2)=1
      IAPOS(3)=1

      IAFWHM(1)=1
      IAFWHM(2)=1
      IAFWHM(3)=1


      

      CALL POLFIT(POSWAV,POSVAL,POSERR,POSMSK,kkmax,PCOF1
     $               ,IAPOS,POLORD,COVAR,POLORD,CHISQ,POLYN)


      CALL POLFIT(POSWAV,SIGVAL,SIGERR,SIGMSK,kkmax,PCOF2
     $               ,IAFWHM,POLORD,COVAR,POLORD,CHISQ,POLYN)
  


      X1 = I1 - XSTP
      kk=1

      IF(ISTAT.LT.1) THEN
          DO I = IXMIN,IXMIN+NUM-1
             X1 = X1 + XSTP


    
             IF(PBUF(2,I).GT.0.AND.PBUF(2,I).LT.DIM) THEN       
c >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
                D1ROW=kk+ICURP  
                CALL TBRWRR(DTID1,D1ROW,1,IDORD,
     +                    FLOAT(N_CURR_ORD),ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDPCH,
     +                    PBUF(2,I),ISTAT)       !POSVAL(I-IXMIN+1) in fit
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDICH,
     +                    FLOAT(kk),ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDPER0,
     +                    VBUF(2,I),ISTAT)       !POSERR(I-IXMIN+1) in fit
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDFWHM,
     +                    PBUF(3,I),ISTAT)       !SIGVAL(I-IXMIN+1) in fit
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDFWER,
     +                    VBUF(3,I),ISTAT)       !SIGERR(I-IXMIN+1) in fit
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF


                CALL TBRWRR(DTID1,D1ROW,1,IDBKGV,
     +                    BKGVAL(kk),ISTAT)      
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF
                
                CALL TBRWRR(DTID1,D1ROW,1,IDBKGE,
     +                    BKGERR(kk),ISTAT)      
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDMSK0,
     +                    FLOAT(PMSK(2,I)),ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDXCH0,
     +                    X1,ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDPF0,
     +                    POLYN3(X1,PCOF1),ISTAT)
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF

                CALL TBRWRR(DTID1,D1ROW,1,IDS0,
     +                    POLYN3(X1,PCOF2),ISTAT)      
                IF (ISTAT.NE.0) THEN
                   CALL STETER(11,'CAPROF: Error writing tbl val')
                ENDIF
             ENDIF
c >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>     
             kk=kk+1             


          ENDDO
          ISTAT=1
      ENDIF

      NPOINT=0
      AVPOS=0
      AVFWHM=0
      DO I = IXMIN,IXMIN+NUM-1
         IF (PMSK(2,I).GT.0) THEN
            AVPOS = AVPOS+PBUF(2,I) 
            AVFWHM = AVFWHM+PBUF(3,I)     
            NPOINT=NPOINT+1
         ENDIF      
      ENDDO
      AVPOS=AVPOS/NPOINT
      AVFWHM=AVFWHM/NPOINT

c     I chech also the chi-square deviation of fit from points
      NFIT=0
      CHIFIT=0.0
      XFIT=0.0
      X1 = I1 - XSTP
      DO I = IXMIN,IXMIN+NUM-1
         X1 = X1 + XSTP
         IF (PMSK(2,I).GT.0) THEN
            CHIFIT = CHIFIT + (PBUF(2,I)-POLYN3(X1,PCOF1))**2     !mean squares
            XFIT   = XFIT   + (PBUF(3,I)-POLYN3(X1,PCOF2))**2     !mean squares
            NFIT=NFIT+1
         ENDIF
      ENDDO
      CHIFIT=CHIFIT/(NFIT-3.0)
      XFIT  =XFIT/(NFIT-3.0)

     
c     check if first fit on "good" points give too strange coefficients
c     the curvature and the inclination cannot be too strange
c     If yes, it change the polinomial fit to a linear one
      
c     max allowed error:  0.5 pix incl err over 1024 pix in X dir for obj pos
c                         0.5 pix curvature over 1024 pix in X dir for obj pos
c     max allowed error:  3 pix incl err over 1024 pix in X dir for obj pos
c                         3 pix curvature over 1024 pix in X dir for obj pos
c     If chi-square < 1
c     max allowed error:  0.5 pix incl err over 1024 pix in X dir for obj fwhm
c                         0.5 pix curvature over 1024 pix in X dir for obj fwhm
c     max allowed error:  2 pix incl err over 1024 pix in X dir for obj fwhm
c                         2 pix curvature over 1024 pix in X dir for obj fwhm
c     remember that BINX and BINY are swapped on the detector
c     if very good fit no k-sigma clipping


      PDEG=2
      SDEG=2
      IF (CHIFIT.LT.2.0e-2.AND.XFIT.LT.2.0e-2) THEN
       D1ROW=ICURP+1
          CALL TBRWRR(DTID1,D1ROW,1,IDDEG,
     +                FLOAT(PDEG),ISTAT)
          CALL TBRWRR(DTID1,D1ROW,1,IDCHI0,
     +                CHIFIT,ISTAT)
          CALL TBRWRR(DTID1,D1ROW,1,IDKS,
     +            FLOAT(0),ISTAT)
          GOTO 96
      ENDIF


      IF (CHIFIT.GT.0.5) THEN
          IF ( ABS(PCOF1(2)).GT.(0.5/1024*BINY/BINX).OR.    
     +         ABS(PCOF1(3)).GT.(0.5/1024/1024*BINY*BINY/BINX/BINX) 
     +        ) THEN
              IAPOS(3)=0
              PDEG=1
              PCOF1(1)=AVPOS
              PCOF1(2)=0.0
              PCOF1(3)=0.0
          ENDIF
      ELSE
          IF ( ABS(PCOF1(2)).GT.(3.0/1024*BINY/BINX).OR.    
     +    ABS(PCOF1(3)).GT.(3.0/1024/1024*BINY*BINY/BINX/BINX) 
     +        ) THEN
              IAPOS(3)=0
              PDEG=1
              PCOF1(1)=AVPOS
              PCOF1(2)=0.0
              PCOF1(3)=0.0
          ENDIF
      ENDIF
      
      IF (XFIT.GT.1) THEN
          IF ( ABS(PCOF2(2)).GT.(0.5/1024*BINY/BINX).OR.    
     +         ABS(PCOF2(3)).GT.(0.5/1024/1024*BINY*BINY/BINX/BINX) 
     +        ) THEN
              IAFWHM(3)=0
              SDEG=1
              PCOF2(1)=AVFWHM
              PCOF2(2)=0.0
              PCOF2(3)=0.0
          ENDIF
      ELSE
          IF ( ABS(PCOF2(2)).GT.(2.0/1024*BINY/BINX).OR.    
     +    ABS(PCOF2(3)).GT.(2.0/1024/1024*BINY*BINY/BINX/BINX) 
     +        ) THEN
              IAFWHM(3)=0
              SDEG=1
              PCOF2(1)=AVFWHM
              PCOF2(2)=0.0
              PCOF2(3)=0.0
          ENDIF
      ENDIF
      
      
      
c     before starting real k-sigma clipping calculates again the
c     right coefficients
      

      CALL POLFIT(POSWAV,POSVAL,POSERR,POSMSK,kkmax,PCOF1
     $               ,IAPOS,PDEG+1,COVAR,PDEG+1,CHISQ,POLYN)
      CALL POLFIT(POSWAV,SIGVAL,SIGERR,SIGMSK,kkmax,PCOF2
     $               ,IAFWHM,SDEG+1,COVAR,SDEG+1,CHISQ,POLYN)
     
     
      IF (PCOF1(3).EQ.0) THEN
          IF ( ABS(PCOF1(2)).GT.(0.5/1024*BINY/BINX)) THEN
              IAPOS(3)=0
              IAPOS(2)=0
              PDEG=0
              PCOF1(1)=AVPOS
              PCOF1(2)=0.0
              PCOF1(3)=0.0
          ENDIF
      ENDIF

      
      IF (PCOF2(3).EQ.0) THEN
          IF ( ABS(PCOF2(2)).GT.(0.25/1024*BINY/BINX)) THEN
              IAFWHM(3)=0
              IAFWHM(2)=0
              SDEG=0
              PCOF2(1)=AVFWHM
              PCOF2(2)=0.0
              PCOF2(3)=0.0
          ENDIF
      ENDIF


      D1ROW=ICURP+1
      CALL TBRWRR(DTID1,D1ROW,1,IDDEG,
     +            FLOAT(PDEG),ISTAT)
      CALL TBRWRR(DTID1,D1ROW,1,IDCHI0,
     +            CHIFIT,ISTAT)
      CALL TBRWRR(DTID1,D1ROW,1,IDKS,
     +            FLOAT(1),ISTAT)
        


     
     
c  ==============================================================

c     Do a linear least squares fit on the position and the sigma
c     parameter of the gaussians

 95   I1 = IXMIN+IXWIND/2            !central point in each chunk
      XSTP = IXWIND
      X1 = I1
       

     

      IF(PDEG.GT.0) THEN
      CALL POLFIT(POSWAV,POSVAL,POSERR,POSMSK,kkmax,PCOF1
     $               ,IAPOS,PDEG+1,COVAR,PDEG+1,CHISQ,POLYN)
      ENDIF
      IF(SDEG.GT.0) THEN
      CALL POLFIT(POSWAV,SIGVAL,SIGERR,SIGMSK,kkmax,PCOF2
     $               ,IAFWHM,SDEG+1,COVAR,SDEG+1,CHISQ,POLYN)
      ENDIF

        
         
c     PCOF(1) is the y constant coeff position of the spectrum within the order
c     PCOF(2) is the y slope    coeff position of the spectrum within the order

     
C     It is safer to assume that the sigma is constant along the order      
c      WRITE(*,*) 'FITPOS',PCOF1

      
c =========================================================================
c     Now do a kappa sigma clipping on the profiles to get
c     best positions and sigma of the gaussians
c =========================================================================
      KAPPAMAX = 2*KAPPA**2
      IMAX = 0
      IPOI = 0
      X1 = I1 - XSTP
      DO I = IXMIN,IXMIN+NUM-1
         X1 = X1 + XSTP
         IF (PMSK(2,I).GT.0) THEN
            IPOI = IPOI+1
            CHI2(2) = (PBUF(2,I)-POLYN3(X1,PCOF1))**2/VBUF(2,I)
            CHI2(3) = (PBUF(3,I)-POLYN3(X1,PCOF2))**2/VBUF(3,I)
            SUM = CHI2(2) + CHI2(3)
            IF (SUM.GT.KAPPAMAX) THEN
               KAPPAMAX = SUM         !determines maximum discrepancy 
               IMAX = I               !of position and FWHM and its index
               x1max = x1
            ENDIF
         ENDIF
      ENDDO
      
      

      IF (IMAX.GT.0) THEN
         DO J=1,4
            PMSK(J,IMAX) = 0
         ENDDO
         POSMSK(IMAX) = 0
         POSERR(IMAX) = 1e6
         SIGMSK(IMAX) = 0
         SIGERR(IMAX) = 1e6

C        
C        if less than some (15) chunks the k-sigma clipping is repeated 
C        else goes on
C         
         IF (IPOI.LE.MINPOI) GOTO 96
         GOTO 95                !re-do the least squares fit
      ENDIF

c     Now determine the position and sigma of the gaussian at
c     each column ( = each wavelength).



      
 96   DO I =  IXMIN,IXMAX
         PBUF(2,I) = POLYN3(FLOAT(I),PCOF1)    !position of maximum
c         PBUF(2,I) = POLYN5(FLOAT(I),TCOEF)    !position of maximum
         

         PBUF(3,I) = POLYN3(FLOAT(I),PCOF2)   !sigma of gaussian

      ENDDO

c  ============================================================
c   Monitoring the values after LSFIT
      ISTAT=0
      X1 = I1 - XSTP
     
      kk=1
      IF(ISTAT.LT.1) THEN
          DO I = IXMIN,IXMIN+NUM-1
             X1 = X1 + XSTP

c >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>> 
              D1ROW=kk+ICURP
              CALL TBRWRR(DTID1,D1ROW,1,IDPF1,
     +                    POLYN3(X1,PCOF1),ISTAT)  
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'CAPROF: Error writing tbl val')
              ENDIF
              CALL TBRWRR(DTID1,D1ROW,1,IDMSK1,
     +                    FLOAT(PMSK(2,I)),ISTAT) 
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'CAPROF: Error writing tbl val')
              ENDIF
              CALL TBRWRR(DTID1,D1ROW,1,IDS1,
     +                    POLYN3(X1,PCOF2),ISTAT) 
              IF (ISTAT.NE.0) THEN
                 CALL STETER(11,'CAPROF: Error writing tbl val')
              ENDIF
c >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>     
              kk=kk+1
          ENDDO
          ISTAT=1
      ENDIF
c ==============================================================




c
c     Now determine background and amplitude for fixed 
c     sigma and position of maximum (result from least squares fit).
c     Determine the gaussian profils at each wavelength. At the beginning
c     the gaussians have a zero background and an amplitude of 1.0.
c

      CALL G_PROF(PROF,VBVAL,FBVAL,MASK,PBUF,VBUF,PMSK,COLUM,SIGMA,OUT
     $     ,VOUT,NB1,NB2,NPIXA1,IXWMAX,IXWIND,IXMIN,IXMAX,DIM,DIMP,PDIM
     $     ,RON,CONAD,THRESH,BINSIZE,INDX,OBJSET      
     $     ,FWHM,MID_S_N_CENT,OBJ_POS_CENT,N_CURR_ORD)  
         
100   CONTINUE     
  

      RETURN
      END



C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                                  where

C  AM     10/07/01       error-factor 2 in AMPERR was wrong   G_PROF-variance calculation
C  AM     08/05/01       clean-comment                        G_PROF
C------------------------------------------------------------------------------
C  Role: Once it has been determined the distribution of positions and FWHMs
C        from CAPROF keeping them fixed to the fitting function we are now ready 
C        to determine amplitude and background of a gaussian profile wich fits
C        the cross order profile. This is done through a chi-square process.
C        To remove cosmic ray it is done a k-sigma clipping with threshold value
C        adjusted as a polynomial (parabola) function of the S/N to relax the
C        rejection in case of high S/N data.
C------------------------------------------------------------------------------
C Pseudocode:  
C            define SIGNAL,VARIANCE,OBJ_POS,FWHM,S/N at order center
C            do I= pix_start,pix_end
C               set to 0 gauss amplitude, bkg, linear slope
C               check that Gauss POS,FWHM are within tresholds
C               do J =1, chunk_dimension
C                  call FGAUSS_C to get x-order prof intensity  ?
C                  set error: SIGMA(J)=VBVAL(I,J)
C                  store prof val (COLUM) in array PROF: PROF(I,J)=COLUM(J)
C                  Find the best Gauss Amplitude & bkg with given FWHM & POS
C               enddo
C               do k-sigma clip to remove cosmic ray:
C                  adjust threshold (k) according to S/N in order center
C                  do J=1,x-order_dimension
C                     if not a masked point
C                        call FGAUSS_C to get the model intensity
C                        evaluate the quadratic difference to raw intensity
C                        if .GT. treshold get the position and mask it out
C                     endif
C                  enddo
C               end of k-sigma clipping
C
C               with a parabolic approximation of chi2 near min
C               get the error associated to each amplitude (look for the small
C               change in aplitude which give the corresponding unitary change
C               in chi2)
C
C               define image intensity and its variance at X,Y position 
C            enddo (end scan along order)
C          RETURN
C
C------------------------------------------------------------------------------
      SUBROUTINE G_PROF(PROF,VBVAL,FBVAL,MASK,PBUF,VBUF,PMSK,COLUM,SIGMA
     $     ,OUT,VOUT,NB1,NB2,NPIXA1,IXWMAX,IXWIND,IXMIN,IXMAX,DIM,DIMP
     $     ,PDIM,RON,CONAD,THRESH,BINSIZE,INDX,OBJSET
     $     ,FWHM,MID_S_N_CENT,OBJ_POS_CENT,N_CURR_ORD) 
     
      IMPLICIT NONE
      INTEGER N_MAX_ORD
      INTEGER NGPAR
      INTEGER NPOINT
      PARAMETER(N_MAX_ORD=50,NGPAR=5,NPOINT=101)

C
C     Interface variables
C
      INTEGER DIM                        !Y dimension of mask
      INTEGER DIMP                       !Y dimension (No. of points) of prof
      INTEGER PDIM                       !X dimension of mask
      INTEGER N_CURR_ORD                  !Current order No.
      INTEGER NPIXA1                     !X size of mask 
      INTEGER NB1                        !X size of out ima
      INTEGER NB2                        !Y size of out ima
      INTEGER INDX                       !current x-order pos
      INTEGER IXWMAX                     !????used???  
      INTEGER IXWIND                     !size in pix of extraction window
      INTEGER IXMIN                      !pix point correxponding to XMIN
      INTEGER IXMAX                      !pix point correxponding to XMAX
      INTEGER BINSIZE                    !Bin size
      INTEGER*2 MASK(NPIXA1,DIM)         !mask ima
      INTEGER*2 PMSK(PDIM,NPIXA1)        !masked points
      
      REAL VBVAL(NPIXA1,DIM)             !variance of ima
      REAL FBVAL(NPIXA1,DIM)             !raw ima
      REAL PROF(NPIXA1,DIMP)             !profile
      REAL PBUF(PDIM,NPIXA1)             !buffer for coefficients of profile
      REAL VBUF(PDIM,NPIXA1)             !buffer for coefficients of raw values
      REAL COLUM(DIM)                    !output weights
      REAL SIGMA(DIM)                    !errors associated to points?
      REAL RON                           !Read Out Noise
      REAL CONAD                         !Analog To Digital conversion factor
      REAL THRESH                        !Threshold factor
      REAL OUT(NB1,NB2)                  !output ima
      REAL VOUT(NB1,NB2)                 !output variance ima
      REAL OBJSET                        !object offset
      REAL MID_S_N_CENT(N_MAX_ORD)       !tmp S/N value
      REAL OBJ_POS_CENT(N_MAX_ORD)       !tmp obj position
      REAL FWHM(N_MAX_ORD)               !tmp array to hold average FWHM
       
C
C     Local variables
C
      INTEGER I                          !loop variable
      INTEGER J                          !loop variable
      INTEGER NVAL                       !counter for un masked point of 
                                         !Gaussian profile

      INTEGER IXCENTRAL                  !central pix position of order
      INTEGER I_SIZ_CENT                 !size of central order region to get 
                                         !average values
      INTEGER INDEX                      !tmp variable to hold temporary X pos

      INTEGER JMAX                       !Aux var
      INTEGER k                          !loop variable
      INTEGER l                          !loop variable

      REAL PVAL                          !x-profile value
      REAL CHI2MIN                       !minimum of chi-square
      REAL CHI2DEL                       !delta of chi-square
      REAL A(NGPAR)                      !coefficient of profile
      REAL B(NGPAR)                      !weight holder in chi2 of prof. coeff.
      REAL X                             !X position
      REAL DY(NGPAR)                     !deriv of Gaus prof. respect to coeff.
      REAL THRESH2                       !Square of threshold
      REAL DMAX                          !Holder for max diff of Y pos     
      REAL CHI2G                         !Gaussian chi-quare value
      REAL DCHI1                         !function to get chi-square difference
      REAL X1                           !X1=(1-ALPHA*SQRT(1.0/DELTA))*PBUF(1,I)
      REAL ZBRENT                        !function to hold zero of polynomial
      REAL AMPERR                        !error amplitude
      REAL AVERSIG                       !Average of sigma
      REAL ERF                           !Error function
      REAL MID_SIG_CENT                  !midium S/N at center of order
      REAL MID_VAR_CENT                  !medium var at center of order
      REAL MID_SDEV_CENT(N_MAX_ORD)      !medium sdev at center of order
      REAL SDEV_CENT                     !standard dev at center of order
      REAL SIGNAL(NPOINT)                !tmp array holder of signal at 
                                         !center of order
      REAL AV_SIGNAL                     !average of signal
      REAL EPOS                          !error in position determination
      REAL S_N_THRESH                    !thresh contribute proportional to S/N
      REAL S_N_WEIGHT                    !weight for linear term proportional 
                                         !to S/N in thresh
      REAL BASIC_THRESH2                 !basic threshold squared
      REAL S_N_WEIGHT2                   !weight for quadratic term 
                                         !proportional to S/N in thresh

      REAL ALPHA                         !Amplitude variation in percent
      REAL DELTA                         !CHI-CHIMIN

      CHARACTER*72 LINE                  !Buffer to display messanges


      EXTERNAL FGAUSS_C                  !Function to get Y of gauusian+linear
                                         !prof. + deriv.
      EXTERNAL CHI2G                     !Function to get Gaussian chi-square
      EXTERNAL DCHI1                     !function to get chi-square difference
      EXTERNAL ZBRENT                    !Function to get zero points of polyn
      EXTERNAL ERF                       !ERF function

C
C     Initialization of local variables
C

C     Integers
      I=0    
      J=0
      k=0
      l=0
      NVAL=0 
      IXCENTRAL=0  
      I_SIZ_CENT=0  
      INDEX=0 
      JMAX=0

C     Reals
      PVAL=0.0   
      CHI2MIN=0.0 
      CHI2DEL=0.0 

      DO I=1,NGPAR
         A(I)=0.0    
         B(I)=0.0
         DY(I)=0.0  
      ENDDO
 
      X=0.0
      THRESH2=0.0
      DMAX   =0.0     
C      CHI2G  =0.0    !external function
C      DCHI1  =0.0    !external function
      X1     =0.0
C      ZBRENT =0.0    !external function
      AMPERR =0.0
      AVERSIG=0.0
C      ERF    =0.0    !external function
      MID_SIG_CENT=0.0
      MID_VAR_CENT=0.0

      DO I=1,N_MAX_ORD
         MID_SDEV_CENT(I)=0.0
      ENDDO

      SDEV_CENT=0.0

      DO I=1,NPOINT
         SIGNAL(I)=0.0
      ENDDO 
                   
      AV_SIGNAL=0.0
      EPOS=0.0          
      S_N_THRESH=0.0
      S_N_WEIGHT=0.0 
      BASIC_THRESH2=0.0
      S_N_WEIGHT2=0.0 

      ALPHA=0
      DELTA=0

C     Doubles

C     Chars
      LINE(1:)= ' '


c    
c     Fit the parameters of all the gaussians fitted to the averaged
c     order profils. As a result you get the parameters of a gaussian
c     at each wavelength, stored in PBUF[]. The gaussians itself are
c     saved in FBVAL[].
c
     
      THRESH2 = THRESH**2
      BASIC_THRESH2=THRESH2


C QC check:
C Calculates medium S/N in central part (50 pixs) of the order 
C Calculates the medium object position in central part (50 pixs) of the order
C This data is stored as descriptor in the output frame to monitor
C optimal extraction quality
      IXCENTRAL = INT((IXMAX+IXMIN)/2)
      I_SIZ_CENT=25    !half size of the box 
                       !over which calculates medium S/N
                       
      IF ( ((IXCENTRAL-I_SIZ_CENT) .LE. IXMIN) .OR.
     +     ((IXCENTRAL+I_SIZ_CENT) .GE. IXMAX) )  THEN 
           I_SIZ_CENT = MAX(MIN((IXCENTRAL-IXMIN),
     +                          (IXMAX-IXCENTRAL))-1,0)
      ENDIF
      MID_SIG_CENT=0
      MID_VAR_CENT=0
  
       
      OBJ_POS_CENT(N_CURR_ORD)=0       !initialize to zero
      MID_SDEV_CENT(N_CURR_ORD)=0       !initialize to zero
      FWHM(N_CURR_ORD)=0               !initialize to zero
      DO k =  IXCENTRAL-I_SIZ_CENT,IXCENTRAL+I_SIZ_CENT
         OBJ_POS_CENT(N_CURR_ORD)=OBJ_POS_CENT(N_CURR_ORD)+PBUF(2,k)
         FWHM(N_CURR_ORD)=FWHM(N_CURR_ORD)+PBUF(3,k) 
      ENDDO  

      
      OBJ_POS_CENT(N_CURR_ORD)=OBJ_POS_CENT(N_CURR_ORD)/(2*I_SIZ_CENT+1)
      FWHM(N_CURR_ORD)=FWHM(N_CURR_ORD)/(2*I_SIZ_CENT+1)

      
      AV_SIGNAL=0
      DO k =  IXCENTRAL-I_SIZ_CENT,IXCENTRAL+I_SIZ_CENT
         index=k-IXCENTRAL+I_SIZ_CENT+1
         SIGNAL(index)=0
         DO l = 1,DIM
            SIGNAL(index)=SIGNAL(index)+FBVAL(k,l)
         ENDDO
         AV_SIGNAL=AV_SIGNAL+SIGNAL(index)
      ENDDO

      AV_SIGNAL=MAX(AV_SIGNAL,0.001) !In case bk too high
      AV_SIGNAL=AV_SIGNAL/(2*I_SIZ_CENT+1)
      EPOS=0
      DO k =  1,2*I_SIZ_CENT+1
         EPOS=EPOS+(SIGNAL(k)-AV_SIGNAL)*(SIGNAL(k)-AV_SIGNAL)
      ENDDO
      EPOS=sqrt(EPOS/(2*I_SIZ_CENT))
      MID_S_N_CENT(N_CURR_ORD)=
     +  sqrt(AV_SIGNAL*FWHM(N_CURR_ORD)*CONAD*2.5066283)


      S_N_WEIGHT =  0.05         !Linear term
      S_N_WEIGHT2 = 0.00500      !quadratic term

      S_N_THRESH=S_N_WEIGHT*MID_S_N_CENT(N_CURR_ORD)+
     +           S_N_WEIGHT2*MID_S_N_CENT(N_CURR_ORD)*
     +                       MID_S_N_CENT(N_CURR_ORD)
            
      THRESH2 = BASIC_THRESH2+S_N_THRESH**2

     
c     The parameters of the gaussians at each wavelength are determined
c     Now calculate the gaussian profils and save them in FBVAL[].  
   
      DO I =  IXMIN,IXMAX

 101     PBUF(1,I) = 1.0        !amplitude of gaussian
         PBUF(4,I) = 0.0        !background (additional constant)
         PBUF(5,I) = 0.0

C if the order is very weak you may get a strange fit
C and have the position (=PBUF(2,I)) < 1 or > DIM
C the following fitting procedure may give crazy results
C in this case it is advisable to set artificially the position
C in the middle of the slit

         IF(PBUF(2,I).LT.1.OR.PBUF(2,I).GT. FLOAT(DIM)) THEN
            PBUF(2,I) = DIM/2.+OBJSET
            PBUF(3,I) = 2.
         ENDIF

C if the sigma (=PBUF(3,I)) is < 1 one may get again a strange fit
C below. It happens only in orders without signal.
C in this case it is advisable to set artificially to 1

         IF(PBUF(3,I).LT.1.)  PBUF(3,I)=1.
c     The amplitude and the background of the gaussian
c     may be determined analytically

         DO J = 1,5
            A(J) = 0.0
         ENDDO

         AVERSIG = 0.0
c        for each chunk .....         
         DO J = 1,DIM
            IF (VBVAL(I,J).LT.1.E-9) MASK(I,J) = 0
            IF (MASK(I,J).GT.0) THEN
               X = J
               CALL FGAUSS_C(X,PBUF(1,I),COLUM(J),DY)
 
c COLUMN is a gaussian with amplitude 1.0 and 0.0 background.
c               SIGMA(J) = RON + MAX(0.,FBVAL(I,J))/CONAD

               SIGMA(J) = VBVAL(I,J)
               PROF(I,J) = COLUM(J) !save the profile == weights!

C FBVAL is the value of the observed signal
C Column is the output of FGAUSS_C so the fit profile
C Found the optimal gaussian with fixed FWHM and POSITION
C calculates only the optimal amplitude + BKG(constant)
 
               
               AVERSIG = AVERSIG+SIGMA(J)    !sums
               B(1) = COLUM(J)/SIGMA(J)      !weighted mean
               B(2) = 1.0/SIGMA(J)           !weights
               A(1) = A(1) + B(1) * COLUM(J)
               A(2) = A(2) + B(1)
               A(3) = A(3) + B(1) * FBVAL(I,J)
               A(4) = A(4) + B(2)
               A(5) = A(5) + B(2) * FBVAL(I,J)
            ELSE
               PROF(I,J) = 0.0
            ENDIF
         ENDDO
         PBUF(4,I) = A(1)*A(4) - A(2)*A(2)
         IF (PBUF(4,I) .NE. 0) THEN
            PBUF(4,I) = (A(1)*A(5) - A(2)*A(3)) / PBUF(4,I)
         ELSE
         PBUF(4,I) = 1E33
            print *,'(4) BAD profil at X=',I,A(1),A(2),A(3),A(4),A(5)
         ENDIF
         IF (A(1) .NE. 0) THEN
            PBUF(1,I) = (A(3) - A(2)*PBUF(4,I)) / A(1)
         ELSE
            PBUF(1,I) = 1E33
            print *,'(1) BAD profil at X=',I,A(1),A(2),A(3),PBUF(4,I)
         ENDIF

c     Now the amplitude and the background are known
c     Determine the final gaussian profil

c     Do the kappa sigma clipping if enabled (THRESH>0) to remove
c     cosmic ray hits.
         
         IF (THRESH.GT.0) THEN
            JMAX = -1
            NVAL = 0
            DMAX = 0.0


              
            DO J = 1,DIM
            
c 13/4/2000 am:                  
c to monitor points which may be chopped of as out of threshold
c put before check on mask(I,J) to have all the points anyway                
C It would be nice to put values in a table to be used to do QC
c                  X = J
c                  CALL FGAUSS_C(X,PBUF(1,I),PVAL,DY)
c                  NVAL = NVAL + 1
c                  DY(1) = FBVAL(I,J) - PVAL
c                  DY(2) = DY(1)*DY(1)/SIGMA(J)
c                  IF ((I.GE.(IXMIN+500)).AND.(I.LT.(IXMIN+600))) THEN 
c                      WRITE(LINE,*), X,FBVAL(I,J),PVAL,SIGMA(J),DY(1),DY(2)
c                      CALL STTPUT(LINE,STATUS)
c                  ENDIF               
c end of check            
            
        
        
            
               IF (MASK(I,J).GT.0) THEN
                  X = J
                  CALL FGAUSS_C(X,PBUF(1,I),PVAL,DY)
                  NVAL = NVAL + 1
                  DY(1) = FBVAL(I,J) - PVAL
                  DY(2) = DY(1)*DY(1)/SIGMA(J)
                  
                  
                  IF (DY(2).GT.THRESH2) THEN
                     IF (DY(2) .GT. DMAX) THEN
                        JMAX = J
                        DMAX = DY(2)
                     ENDIF
                  ENDIF
               ENDIF


            ENDDO

C It is necessary to have at least 3 pixels left to fit the gaussian + BKGD
c here does a check to eliminate the point over THRESH and goes on only if
c it has at least 3 points for the gaussian fit            
          IF (JMAX.GT.0.AND.NVAL.GT.3) THEN
               MASK(I,JMAX) = 0
               GOTO 101
            ENDIF
         ENDIF

c here we have the final profile including the amplitude
C we need to estimate the error on the amplitude
c first compute the minimum chi2 for which we need the 'original' profil
c with amplitude = 1.0
c was         ALPHA = 0.1
c 13/3/2000 am: was  ALPHA = 0.2
              ALPHA = 0.2
         
  150    CONTINUE
         CHI2MIN = CHI2G(PBUF(1,I),COLUM,FBVAL,SIGMA,MASK,PBUF(4,I)
     $        ,NPIXA1,DIM,I)



c NEW: to get the error on the amplitude we now do a parabola fit on
c the chi2 function. That provides us to use the root finding procedure
c which was instable due to a bad start interval.

C am: Here is the problem, now fixed !

         CHI2DEL = CHI2G(PBUF(1,I)*(1-ALPHA),COLUM,FBVAL,SIGMA,MASK
     $        ,PBUF(4,I),NPIXA1,DIM,I)



c 
c was From Pascal-sSebastian
c         DELTA = CHI2DEL - CHI2MIN + 1.E-6
c Stefano suggest:


         DELTA = CHI2DEL - CHI2MIN 
         



C AM: 13/03/2000 
C     Added check on DELTA .LT. e8 and corresponding
C     redefinition of DELTA=1e8 and AMPERR=1e-5
C     We noticed that with a Quasar @ R860 2x2
c     with IXWIND=64 DELTA=NAN and then GaussJ pauses
C     asking to type go to continue. In the end the program
c     was crashing when optimal extraction was done 
c     (Invalid cut values) 
C                
         IF((DELTA .GT. 0) .AND. (DELTA .LT. 1e8)) THEN
            X1 = (1 - ALPHA * SQRT( 1.0 / DELTA )) * PBUF(1,I)
            AMPERR = ALPHA * SQRT( 1.0 / DELTA ) * PBUF(1,I)
         ELSE IF ((DELTA .LE. 0) .AND. (ALPHA .LT. 0.9)) THEN
            ALPHA =ALPHA+0.1
            GOTO 150
         ELSE IF  (DELTA .GT. 1e8) THEN
            DELTA = 1e8
            X1=0
            AMPERR= 1e-5
         ELSE                     
            X1= 0
            AMPERR = 1e2            
         ENDIF

c Here calculates the variance
c OUT is the amplitude
c VOUT is its variance
C 2.5066283=sqrt(2*PI)
C VOUT= [D(OUT)/D(AMP)*ERR_AMP]^2

         OUT(I,INDX)  = 2.5066283*PBUF(3,I)*PBUF(1,I)
         VOUT(I,INDX) = (2.5066283*PBUF(3,I)*AMPERR)**2
         

         IF ((VOUT(I,INDX) .GT. 0) .AND. (VOUT(I,INDX) .LT. 1.e8)) THEN 
            CONTINUE
         ELSE            

             VOUT(I, INDX) = 1.e8
         ENDIF

      ENDDO        
      RETURN
      END




C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     FGAUSS_C
C------------------------------------------------------------------------------
C  Role: gives the gaussian + linera term at point X with given 
C        coefficients stored in the array A. Derivatives are stored in the
C        array DYDA 
C------------------------------------------------------------------------------

      SUBROUTINE FGAUSS_C(X,A,Y,DYDA)
      IMPLICIT NONE
      REAL X                     !X position
      REAL Y                     !Y value
      REAL A(5)                  !Gaussial+linear coefficients
      REAL DYDA(5)               !derivs of function respect to coefficients

C
C     Local variables
C
      REAL A3I                   !reciprocal of A(3)
      REAL ARG                   !argument of gaussian
      REAL EX                    !exponential contribute
      REAL FAC                   !variable holder to compact expression

C
C     Initialization and 1st Local variables setting
C            

      Y=0.
      A3I=1.0/A(3)
      ARG=(X-A(2))*A3I            !was  ARG=(X-A(2))/A(3)
      EX=EXP(-ARG**2)
      FAC=A(1)*EX*2.*ARG

      Y=A(1)*EX + A(4)*(1.+(X-A(2))*A(5))
      DYDA(1)=EX
      DYDA(2)=FAC*A3I-A(4)*A(5)   !was DYDA(2)=FAC/A(3)-A(4)*A(5)
      DYDA(3)=FAC*ARG*A3I         !was DYDA(3)=FAC*ARG/A(3)
      DYDA(4)=1.+(X-A(2))*A(5)
      DYDA(5)=A(4)*(X-A(2))
      END


C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     POLYN
C------------------------------------------------------------------------------
C  Role: gives the polinomial of order 3 (degree 2) at point X with given 
C        unitary coefficients  
C------------------------------------------------------------------------------

      SUBROUTINE POLYN(X,Y,NDIM)
      IMPLICIT NONE
      INTEGER NDIM
      REAL X                !X position
      REAL Y(3)             !polynomial derivatives respect to coefficients

      Y(1) = 1.0
      Y(2) = X
      Y(3) = X*X

      RETURN
      END


C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     POLYN3
C------------------------------------------------------------------------------
C  Role: gives the polinomial of order 3 (degree 2) at point X with given 
C        coefficients stored in the array A 
C------------------------------------------------------------------------------
      FUNCTION POLYN3(X,A)
      IMPLICIT NONE
      REAL X              !X position
      REAL A(3)           !polynomial coefficients
      REAL POLYN3         !result

      POLYN3 = A(1)+A(2)*X+A(3)*X*X
      RETURN
      END



C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     POLYN5
C------------------------------------------------------------------------------
C  Role: gives the polinomial of order 5 (degree 4) at point X with given 
C        coefficients stored in the array A 
C------------------------------------------------------------------------------
      FUNCTION POLYN5(X,A)
      IMPLICIT NONE
      REAL X                !X position
      REAL A(5)             !polynomial coefficients
      REAL POLYN5           !result

      POLYN5 = A(1)+
     &         A(2)*X+
     &         A(3)*X*X+
     &         A(4)*X*X*X+
     &         A(5)*X*X*X*X

     
      RETURN
      END


C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     MYAVRG
C------------------------------------------------------------------------------
C  Role: evaluates an average of points
C------------------------------------------------------------------------------
      SUBROUTINE MYAVRG(ARRAY,MASK,IX,IMIN,IMAX,ISLICE,RES)
      IMPLICIT NONE
      INTEGER IMIN        !min value of loop variable
      INTEGER IMAX        !max value of loop variable
      INTEGER ISLICE      !current X position along the order
      INTEGER IX          !X size
      INTEGER MASK(5,IX)  !array to mask profile points
      
      REAL ARRAY(5,IX)    !array to hold raw points

C
C     Local variables
C      
      INTEGER NP          !counter
      INTEGER I           !loop variable
      REAL RES            !result

C
C    Initialization of Local variables
C

      RES=0.
      NP=0
      I=0
      
      DO I = IMIN,IMAX
         IF (MASK(2,I).GT.0) THEN    !NB We use only 2nd component of mask
            RES = RES+ARRAY(ISLICE,I)      
            NP=NP+1
         ENDIF      
      ENDDO
      RES=RES/NP
      
      RETURN  
      END
      
      
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     MYSIGM
C------------------------------------------------------------------------------
C  Role: evaluates the standard deviation of point data respect to an average
C------------------------------------------------------------------------------
      SUBROUTINE MYSIGM(ARRAY,AVREF,MASK,IX,IMIN,IMAX,ISLICE,RES)
      IMPLICIT NONE
      INTEGER IMIN              !min value of loop variable
      INTEGER IMAX              !max value of loop variable
      INTEGER ISLICE            !current X position along the order
      INTEGER IX                !X size
      INTEGER MASK(5,IX)        !array to mask profile points
      REAL ARRAY(5,IX)          !array to hold raw points
      REAL AVREF                !average ref value

C
C     Local variables
C      
      INTEGER NP                !counter
      INTEGER I                 !loop variable
      REAL RES                  !result 

C
C     Initialization of local variables
C


      RES=0.
      NP=0
      I=0
      
      DO I = IMIN,IMAX
         IF (MASK(2,I).GT.0) THEN    !NB We use only 2nd component of mask
            RES = RES+(ARRAY(ISLICE,I)-AVREF)*(ARRAY(ISLICE,I)-AVREF)      
            NP=NP+1
         ENDIF      
      ENDDO
      RES=SQRT(RES/(NP-1))
      
      RETURN 
      END      
      
C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C*******************************************************************      
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     CENT
C------------------------------------------------------------------------------
C  Role: get the centres of the orders at each wavelength
C------------------------------------------------------------------------------
      SUBROUTINE CENT(YVAL,NPIXA1,DPAR,ND,K,L,YV,IWIDTH,
     +                SCAN,OFFSET,IXMIN,IXMAX,STARTA,STEPA)
C
C     Get the centres of the orders at each wavelength
C
      IMPLICIT NONE
      INTEGER NWAV                     !No of wavelength points
      INTEGER NAXIMA                   !No of axes of image
      PARAMETER (NWAV=50,NAXIMA=2)           

      INTEGER K                       !max value of loop variable
      INTEGER L                       !max value of loop variable
      INTEGER ND                      !number of polynomial coefficients
      INTEGER NPIXA1                  !X ima size
      INTEGER IWIDTH                  !size in pix of slit
      INTEGER IXMIN                   !pix position of X min
      INTEGER IXMAX                   !pix position of X max
      INTEGER SCAN(NAXIMA)            !Scan ima parameter (X, Y)

      REAL OFFSET                     !Offset of object

      DOUBLE PRECISION YVAL(NPIXA1)   !Y position in world
      DOUBLE PRECISION DPAR(ND)       !order polynomial coeff
      DOUBLE PRECISION YV             !actual order number
      DOUBLE PRECISION STARTA(NAXIMA) !Start value of ima (X,Y)
      DOUBLE PRECISION STEPA(NAXIMA)  !Step of ima (X,Y)



C
C     Local variables
C

      INTEGER IX1                   !loop variable
      INTEGER L1                    !loop variable
      INTEGER K1                    !loop variable
      INTEGER IP                    !tmp variable
      INTEGER ITEST                 !switch

      INTEGER I                     !loop variable
   
      DOUBLE PRECISION AX           !X position in pix
      DOUBLE PRECISION XVAL         !X position in world
      DOUBLE PRECISION DC           !tmp variable
      DOUBLE PRECISION WW(NWAV)       !wav position


      DOUBLE PRECISION XI_R8R8      !MIDAS function to convert from 
                                    !world to pix coordinate
      DOUBLE PRECISION IX_R8R8      !MIDAS function to convert from 
                                    !pix to world coordinate
C
C     Local variables initialization
C

C     Integers

      IX1=0
      L1=0
      K1=0   
      IP=0   
      ITEST=0
      I=0

C     Reals

C     Double precision
      AX=0.0D0
      XVAL=0.0D0  
      DC=0.0D0    
      DO I=1,NWAV
         WW(I)=0.0D0  
      ENDDO






          AX = 1
          IXMIN = 1
          IXMAX = NPIXA1
          ITEST = 0
          DO 50 IX1 = 1,NPIXA1
              IP     = 0
              DC     = 1.D0
              YVAL(IX1)  = 0.D0
              XVAL   = IX_R8R8(AX, STARTA(1), STEPA(1))
              DO 30 L1 = 0,L
                  IP     = IP + 1
                  WW(IP) = DC
                  YVAL(IX1)  = YVAL(IX1)+ WW(IP)*DPAR(IP)
                  DO 20 K1 = 1,K
                      IP     = IP + 1
                      WW(IP) = WW(IP-1)*XVAL
                      YVAL(IX1)  = YVAL(IX1) + WW(IP)*DPAR(IP)
   20             CONTINUE
                  DC     = DC*YV
   30         CONTINUE
              YVAL(IX1) = XI_R8R8(YVAL(IX1),STARTA(2),STEPA(2)) + OFFSET
              AX = AX+1
  
          IF ( INT(YVAL(IX1)-IWIDTH/2.0-1).LE.SCAN(1)) THEN
              IXMIN = IX1
          ELSE IF ( INT(YVAL(IX1)+IWIDTH/2.0+2).GE.SCAN(2).AND.
     +         ITEST.EQ.0) THEN
              IXMAX = IX1
              ITEST = 1
          ENDIF

   50     CONTINUE

      RETURN
      END
    

C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     VSAMPL
C  AM     29/05/01       Corrected typo in IKIND info      EXTFF
C------------------------------------------------------------------------------
C  Role: Does the sampling in the virtual space
C------------------------------------------------------------------------------
      

      SUBROUTINE VSAMPL(IN,PROF,YVAL,FBVAL,MASK,DIMP,
     1     IWIDTH,BINSIZE,NPIXA1,IXMIN,IXMAX,NPIXA2,DIM,IKIND)
C
C     Do the sampling in the virtual space.
C
      IMPLICIT NONE

      INTEGER IWIDTH                 !size of slit in pixel
      INTEGER DIM                    !Y size of ima
      INTEGER BINSIZE                !Bin size
      INTEGER DIMP                   !profile dimension
      INTEGER NPIXA1                 !X dim of ima
      INTEGER NPIXA2                 !Y dim of ima
      INTEGER IKIND                  !switch for even/odd pix No. slit width
      INTEGER IXMIN                  !index corresponding to XMIN
      INTEGER IXMAX                  !index corresponding to XMAX
      INTEGER*2 MASK(NPIXA1,DIM)     !Mask ima

      DOUBLE PRECISION YVAL(NPIXA1)  !current Y order position(X)

      REAL FBVAL(NPIXA1,DIM)         !Output Intensity at X,Y position
      REAL PROF(NPIXA1,DIMP)         !Profile at X,Y position
      REAL IN(NPIXA1,NPIXA2)         !Input raw frame

C
C     Local variables
C

      INTEGER I                      !loop variable
      INTEGER J                      !loop variable
      INTEGER IX                     !tmp val holding pix position along order
      INTEGER IND                    !tmp variable to hold values
      INTEGER IY                     !Actual order value
      INTEGER NPOINT                 !Point counter
      INTEGER K                      !loop variable
      
      REAL DX                        !Fractional pixel X discrepancy 
      REAL DY                        !Fractional pixel Y discrepancy
      REAL YDEC                      !Y position at pixel center
      REAL SUM                       !tmp sum of x-order intensity at pos X

      DOUBLE PRECISION YDVAL         !Y pos in double precision
                                     !YDVAL=YVAL(IND)-IWIDTH/2-1 + 
                                     !              (J-1.)/BINSIZE
      DOUBLE PRECISION XDVAL         !X pos in double precision
      

C
C     Initialization of local variables
C

C     Integers

      I =0
      J =0
      IX =0
      IND=0
      IY=0 
      NPOINT=0
      K=0     

C     Reals
      
      DX=0
      DY=0
      YDEC=0
      SUM=0

C     Doubles

      YDVAL=0.0D0
      XDVAL=0.0D0




      
      DO J=1,DIM
         DO I=IXMIN,IXMAX
            FBVAL(I,J) = 0
            MASK(I,J) = 0
         ENDDO
      ENDDO

      DO J=1,DIMP
         DO I=IXMIN,IXMAX
            PROF(I,J) = 0
         ENDDO
      ENDDO
      
       
      
      IF (IKIND.EQ.0) THEN
      
         DO I = IXMIN,IXMAX
            SUM = 0
c change?

            K = INT(YVAL(I)-IWIDTH/2.0-1) 
            
c     am: Found the error: 
c         if K<0 (as when for order 1 at the botton of the detector=> 
c         YVAL(I)<IWIDTH+1) the array IN(I,K) explodes
c         so we try to skip this case with the following IF  
c         do the same as before but starting from k=0 (bottom of detector)   
            IF (K .LT. 1) THEN
               DO  J = 1,DIMP
                  PROF(I,J) = IN(I,J)
               ENDDO
               GOTO 10 
            ENDIF  
                         
            PROF(I,1) = IN(I,K)         
            
            DO  J = K+1,K+DIMP-2
               PROF(I,J-K+1) = IN(I,J)
               SUM = SUM+IN(I,J)
            ENDDO
            
            IF ( NINT(YVAL(I)).EQ.INT(YVAL(I))) THEN
               SUM = SUM + IN(I,K)
            ELSE
               SUM = SUM + IN(I,K+DIMP-1)
            ENDIF  
                      
            PROF(I,DIMP) = IN(I,K+DIMP-1)
            
c            IF (SUM.EQ.0)  SUM=1
c            DO J = K,K+DIMP-1
c               PROF(I,J-K+1) = PROF(I,J-K+1)/SUM
c            ENDDO

c      am: added the following CONTINUE to escape from loop
c          in case K<0

   10    CONTINUE
         ENDDO
      ELSE
      
         DO I=IXMIN,IXMAX
         
            SUM = 0
c change?
            K = INT(YVAL(I)-IWIDTH/2.0)
            DO J = K,K+DIMP-1
               PROF(I,J-K+1) = IN(I,J)
               SUM = SUM+IN(I,J)
            ENDDO

c            IF (SUM.EQ.0) SUM = 1  
c            DO J = K,K+DIMP-1
c               PROF(I,J-K+1) = PROF(I,J-K+1)/SUM
c            ENDDO

         ENDDO
      ENDIF
      
      IF (IKIND.EQ.0) THEN



         DO J=1,DIM
            NPOINT =0
            DO I= IXMIN,IXMAX
               XDVAL = I
               IND = I
c change?
               K = INT(YVAL(I)-IWIDTH/2-1)
               



            IF (K .LT. 1) THEN
              GOTO 20 
            ENDIF  
            
                                  
               YDVAL = YVAL(IND)-IWIDTH/2.0-1 + (J-1.)/BINSIZE
               IX = XDVAL
               YDEC = INT(YDVAL)+0.5
               DX = XDVAL-IX
               DY = YDVAL-YDEC
               IY  = INT(YDVAL)
               IF (DY.GT.-0.5.AND.DY.LE.0) THEN
                  IF ((IY-K).GT.0) THEN
                     FBVAL(I,J) =  (1+DY)*PROF(IX,IY-K+1) -
     1                    DY*PROF(IX,IY-K)
                     MASK(I,J) = 1
                     NPOINT = NPOINT+1
                  ENDIF
               ELSE IF (DY.LT.0.5.AND.DY.GE.0) THEN
                  IF ((IY-K+1).LT.DIMP) THEN
                     FBVAL(I,J) =  (1-DY)*PROF(IX,IY-K+1) +
     1                    DY*PROF(IX,IY-K+2)
                     MASK(I,J) = 1
                     NPOINT = NPOINT+1
                  ENDIF
               ENDIF

c           am: added to allow to skip if K .LT. 1

20          CONTINUE   

            ENDDO
         ENDDO
      ELSE




         DO J=1,DIM
            NPOINT =0
            DO I= IXMIN,IXMAX
               XDVAL = I
               IND = I
c change?
               K = INT(YVAL(I)-IWIDTH/2.0)
               YDVAL = YVAL(IND)-IWIDTH/2-1 + (J-1.)/BINSIZE
               IX = XDVAL
               YDEC = INT(YDVAL)+0.5
               DX = XDVAL-IX
               DY = YDVAL-YDEC
               IY  = INT(YDVAL)
               IF (DY.GT.-0.5.AND.DY.LE.0.AND.
     1              (IY-K+1).LE.DIMP.AND.(IY-K).GE.0) THEN
                  IF ((IY-K).NE.0.AND.(IY-K).NE.DIMP) THEN
                     FBVAL(I,J) =  (1+DY)*PROF(IX,IY-K+1) -
     1                    DY*PROF(IX,IY-K)
                  ELSE IF((IY-K).EQ.0) THEN
                     FBVAL(I,J) = PROF(IX,IY-K+1) 
                  ELSE
                     FBVAL(I,J) = PROF(IX,IY-K)
                  ENDIF
                  MASK(I,J) = 1
                  NPOINT = NPOINT+1
               ELSE IF (DY.LT.0.5.AND.DY.GE.0.AND.
     1                 (IY-K+1).LE.DIMP.AND.(IY-K).GE.0) THEN
                  IF ((IY-K+1).NE.DIMP.AND.(IY-K+1).NE.0) THEN
                     FBVAL(I,J) =  (1-DY)*PROF(IX,IY-K+1) +
     1                    DY*PROF(IX,IY-K+2)
                  ELSE IF ((IY-K+1).EQ.DIMP) THEN
                     FBVAL(I,J) = PROF(IX,IY-K+1)
                  ELSE
                     FBVAL(I,J) = PROF(IX,IY-K+2)
                  ENDIF
                  MASK(I,J) = 1
                  NPOINT = NPOINT+1
               ENDIF
            ENDDO
         ENDDO
      ENDIF
      
      RETURN
      END


C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     CHI2G
C------------------------------------------------------------------------------
C  Role: evaluates chi-square between points and the found gaussian 
C        distribution
C------------------------------------------------------------------------------

      FUNCTION CHI2G(AMP, PR, FBVAL, VAR, MASK, BKG, NPIXA1, DIM, INDX)
      
      IMPLICIT NONE
      INTEGER NPIXA1               !X dimension of mask ima
      INTEGER DIM                  !Y dimension of mask ima
      INTEGER INDX                 !current X position of x-order profile
      INTEGER*2 MASK(NPIXA1,DIM)   !array with mask ima

      REAL AMP                     !Gaussian amplitide
      REAL PR(DIM)                 !Gaussian profile
      REAL FBVAL(NPIXA1,DIM)       !Raw ima points
      REAL VAR(DIM)                !Variance ima
      REAL BKG                     !constant term in Gaussian+linear

C
C     Local variables
C
      INTEGER I                    !index for loop
      REAL CHI2G                   !tmp value of Gaussian chi-square


      CHI2G = 0.
      DO I = 1, DIM
         IF (MASK(INDX,I).GT.0) THEN
            CHI2G = CHI2G + (AMP*PR(I)+BKG-FBVAL(INDX,I))**2/VAR(I)
         ENDIF
      END DO
      RETURN
      END


C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     DCHI1
C------------------------------------------------------------------------------
C  Role: evaluates CHI2G-CHI2MIN -1
C------------------------------------------------------------------------------
      FUNCTION DCHI1(AMP,PR,FBVAL,VAR,MASK,A,NPIXA1,DIM,CHI2MIN,INDX)

      IMPLICIT NONE

      INTEGER NPIXA1             !X dimension of mask
      INTEGER DIM                !Y dimension of mask
      INTEGER INDX               !current X position of x-order profile
      INTEGER*2 MASK(NPIXA1,DIM) !Array to hold mask image

      REAL AMP                   !Gaussian amplitude
      REAL PR(DIM)               !Gaussian profile points
      REAL FBVAL(NPIXA1,DIM)     !Ray ima points
      REAL VAR(DIM)              !variance image
      REAL BKG                   !constant value of gaussian + linear
      REAL A(10)                 !coeffs of x-order (Gaussian+linear) profile
      
C
C     Local variables
C
      REAL CHI2G                 !chi2 of gaussian relative to x-order profile
      REAL CHI2MIN               !chi-square minimum
      REAL DD                    !temporary value of chi-square
      REAL DCHI1                 !result

      EXTERNAL CHI2G             !chi2 for Gaussian profile

      BKG = (A(5)-A(2)*AMP)/A(4)
      DD    = CHI2G(AMP, PR, FBVAL, VAR, MASK, BKG, NPIXA1, DIM, INDX)
      DCHI1 = DD - CHI2MIN - 1.0


      RETURN
      END

C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     zbrent
C------------------------------------------------------------------------------
C  Role: finds roots of equations. ZBRENT uses a combination of methods to give
C        assured and relatively efficient convergence.
C------------------------------------------------------------------------------

c
c Some numerical recipe routines:
c     
      FUNCTION zbrent(func,x1,x2,tol,
     $     PR,FBVAL,VAR,MASK,GPAR,NPIXA1,DIM,CHI2MIN,INDX)

      IMPLICIT NONE
      INTEGER ITMAX

      REAL zbrent,tol,x1,x2,func,EPS

      EXTERNAL func

      PARAMETER (ITMAX=100,EPS=3.e-8)

      INTEGER iter

      REAL a,b,c,d,e,fa,fb,fc,p,q,r,s,tol1,xm

      INTEGER NPIXA1, DIM, INDX
      INTEGER*2 MASK(NPIXA1,DIM)

      REAL PR(DIM),FBVAL(NPIXA1,DIM),VAR(DIM),GPAR(10)
     $     ,CHI2MIN

      a=x1
      b=x2
      fa=func(a, PR,FBVAL,VAR,MASK,GPAR,NPIXA1,DIM,CHI2MIN,INDX)
      fb=func(b, PR,FBVAL,VAR,MASK,GPAR,NPIXA1,DIM,CHI2MIN,INDX)

      if((fa.gt.0..and.fb.gt.0.).or.(fa.lt.0..and.fb.lt.0.))pause
     *'root must be bracketed for zbrent'
      c=b
      fc=fb

      do 11 iter=1,ITMAX

        if((fb.gt.0..and.fc.gt.0.).or.(fb.lt.0..and.fc.lt.0.))then
          c=a
          fc=fa
          d=b-a
          e=d
        endif

        if(abs(fc).lt.abs(fb)) then
          a=b
          b=c
          c=a
          fa=fb
          fb=fc
          fc=fa
        endif

        tol1=2.*EPS*abs(b)+0.5*tol
        xm=.5*(c-b)

        if(abs(xm).le.tol1 .or. fb.eq.0.)then
          zbrent=b
          return
        endif

        if(abs(e).ge.tol1 .and. abs(fa).gt.abs(fb)) then
          s=fb/fa

          if(a.eq.c) then
            p=2.*xm*s
            q=1.-s
          else
            q=fa/fc
            r=fb/fc
            p=s*(2.*xm*q*(q-r)-(b-a)*(r-1.))
            q=(q-1.)*(r-1.)*(s-1.)
          endif

          if(p.gt.0.) q=-q

          p=abs(p)

          if(2.*p .lt. min(3.*xm*q-abs(tol1*q),abs(e*q))) then
            e=d
            d=p/q
          else
            d=xm
            e=d
          endif

        else
          d=xm
          e=d
        endif

        a=b
        fa=fb

        if(abs(d) .gt. tol1) then
          b=b+d
        else
          b=b+sign(tol1,xm)
        endif

        fb=func(b, PR,FBVAL,VAR,MASK,GPAR,NPIXA1,DIM,CHI2MIN,INDX)
11    continue

      pause 'zbrent exceeding maximum iterations'
      zbrent=b


      return
      END

C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     erf
C------------------------------------------------------------------------------
C  Role:
C------------------------------------------------------------------------------


C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.

      FUNCTION erf(x)

      IMPLICIT NONE
      REAL erf,x

CU    USES gammp

      REAL gammp

      if(x.lt.0.)then
        erf=-gammp(.5,x**2)
      else
        erf=gammp(.5,x**2)
      endif

      return
      END

C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     gammp
C------------------------------------------------------------------------------
C  Role:
C------------------------------------------------------------------------------

C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      FUNCTION gammp(a,x)

      IMPLICIT NONE
      REAL a,gammp,x

CU    USES gcf,gser

      REAL gammcf,gamser,gln

      if(x.lt.0..or.a.le.0.) pause 'bad arguments in gammp'

      if(x.lt.a+1.)then
        call gser(gamser,a,x,gln)
        gammp=gamser
      else
        call gcf(gammcf,a,x,gln)
        gammp=1.-gammcf
      endif


      return
      END


C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     gser
C------------------------------------------------------------------------------
C  Role:
C------------------------------------------------------------------------------

C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      SUBROUTINE gser(gamser,a,x,gln)

      IMPLICIT NONE
      INTEGER ITMAX

      REAL a,gamser,gln,x,EPS

      PARAMETER (ITMAX=100,EPS=3.e-7)

CU    USES gammln

      INTEGER n

      REAL ap,del,sum,gammln


      gln=gammln(a)


      if(x.le.0.)then
        if(x.lt.0.)pause 'x < 0 in gser'
        gamser=0.
        return
      endif

      ap=a
      sum=1./a
      del=sum

      do 11 n=1,ITMAX
        ap=ap+1.
        del=del*x/ap
        sum=sum+del
        if(abs(del).lt.abs(sum)*EPS) goto 1
11    continue
      pause 'a too large, ITMAX too small in gser'

1     gamser=sum*exp(-x+a*log(x)-gln)

      return
      END

C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     gcf
C------------------------------------------------------------------------------
C  Role:
C------------------------------------------------------------------------------

C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      SUBROUTINE gcf(gammcf,a,x,gln)

      IMPLICIT NONE
      INTEGER ITMAX

      REAL a,gammcf,gln,x,EPS,FPMIN

      PARAMETER (ITMAX=100,EPS=3.e-7,FPMIN=1.e-30)

CU    USES gammln

      INTEGER i

      REAL an,b,c,d,del,h,gammln


      gln=gammln(a)
      b=x+1.-a
      c=1./FPMIN
      d=1./b
      h=d

      do 11 i=1,ITMAX
        an=-i*(i-a)
        b=b+2.
        d=an*d+b

        if(abs(d).lt.FPMIN)d=FPMIN
        c=b+an/c

        if(abs(c).lt.FPMIN)c=FPMIN

        d=1./d
        del=d*c
        h=h*del

        if(abs(del-1.).lt.EPS)goto 1

11    continue

      pause 'a too large, ITMAX too small in gcf'
1     gammcf=exp(-x+a*log(x)-gln)*h

      return
      END

C*******************************************************************
C*********************FUNCTION**************************************
C*******************************************************************

C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     gammln
C------------------------------------------------------------------------------
C  Role: Natural logarithm of the gamma function for positive arguments
C------------------------------------------------------------------------------
C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      FUNCTION gammln(xx)
      IMPLICIT NONE

      REAL gammln,xx

      INTEGER j

      DOUBLE PRECISION ser,stp,tmp,x,y,cof(6)

      SAVE cof,stp

      DATA cof,stp/76.18009172947146d0,-86.50532032941677d0,
     *24.01409824083091d0,-1.231739572450155d0,.1208650973866179d-2,
     *-.5395239384953d-5,2.5066282746310005d0/


      x=xx
      y=x
      tmp=x+5.5d0
      tmp=(x+0.5d0)*log(tmp)-tmp
      ser=1.000000000190015d0

      do 11 j=1,6
        y=y+1.d0
        ser=ser+cof(j)/y
11    continue

      gammln=tmp+log(stp*ser/x)


      return
      END

C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     calc_i
C------------------------------------------------------------------------------
C  Role: to do a weighted mean of the FF signal using the weight image coming
C       from the object extraction
C------------------------------------------------------------------------------
c here extracts the profiles and the variances doing a weighted mean of the 
c profile and its variance
C  (C) Copr. 1986-92 Numerical Recipes Software 3#ks)!.
      SUBROUTINE CALC_I(IN,VARI,PROF,YVAL,OUT,VOUT
     1     ,NPIXA1,NPIXA2,NB1,NB2,DIMP,IXMIN,IXMAX,IWIDTH,INDX)
      
      IMPLICIT NONE

      INTEGER IXMIN                         !index corresponding to Xmin pos
      INTEGER IXMAX                         !index corresponding to Xmax pos
      INTEGER IWIDTH                        !Y size of order
      INTEGER INDX                          !current Y position
      INTEGER DIMP                          !x-order size of profile
      INTEGER NPIXA1                        !x size of input frm
      INTEGER NPIXA2                        !y size of input frm
      INTEGER NB1                           !x size of output frm
      INTEGER NB2                           !y size of output frm

      REAL IN(NPIXA1,NPIXA2)                !input raw ima 
      REAL VARI(NPIXA1,NPIXA2)              !input variance ima
      REAL OUT(NB1,NB2)                     !out ima
      REAL VOUT(NB1,NB2)                    !output variance ima
      REAL PROF(NPIXA1,DIMP)                !profile (weights)

      DOUBLE PRECISION YVAL(NPIXA1)         !Y order position(X) 

C
C     Local variables
C
      INTEGER I                             !Loop variable
      INTEGER J                             !Loop variable
      INTEGER K                             !current x-order pos index
      INTEGER KMAX                          !max value of x-order point
      INTEGER L                             !Aux var for Y point

      REAL PART                             !Fraction of pix holder

      DOUBLE PRECISION SUM1                 !Image sum along x-order dir
      DOUBLE PRECISION SUM2                 !weight sum along x-order dir
      DOUBLE PRECISION SUM3                 !var ima sum along x-order dir


 
          
      DO I = IXMIN,IXMAX
         SUM3 = 0
         SUM2 = 0               !do the extraction ...
         SUM1 = 0               !... sum over spatial coordinate
      IF (IXMIN.EQ.IXMAX) GOTO 10 !skip if the order is just 1 point

c change?
         K = INT(YVAL(I)-IWIDTH/2.0)
               
c     am added to skip problems if outside the detector        
          IF ( (K .LT. 1) .OR. (K .GT. 4096) )  GOTO 10 
          
         
         
                 
c     am: added to trace an error     
c         print *,'We are inside necopt.for, subroutine CALC_I 4b'                   
         KMAX = K+DIMP-1


C
C  Do the extraction
C

         IF (NINT(YVAL(I)).EQ.INT(YVAL(I))) THEN
            PART = NINT(YVAL(I)-0.5)-YVAL(I)+0.5
            SUM1 = PROF(I,1)*IN(I,K)*PART
            SUM2 = PROF(I,1)
            SUM3 = PROF(I,1)*PROF(I,1)*VARI(I,K)*PART
            DO J=K+1,KMAX-2
               L = J-K+1
               SUM1 = SUM1 + PROF(I,L)*IN(I,J)
               SUM2 = SUM2 + PROF(I,L)
               SUM3 = SUM3 + PROF(I,L)*PROF(I,L)*VARI(I,J)
            ENDDO
            J = KMAX-1
            PART = 0.5+YVAL(I)-NINT(YVAL(I)-0.5)
            SUM1 =  SUM1 + PROF(I,DIMP-1)*IN(I,J)*PART
            SUM2 =  SUM2 + PROF(I,DIMP-1)
            SUM3 =  SUM3 + PROF(I,DIMP-1)*PROF(I,DIMP-1)*VARI(I,J)*PART
         ELSE
            PART = 1-(YVAL(I)-0.5-NINT(YVAL(I)-0.5))
            SUM1 = PROF(I,2)*IN(I,K+1)*PART
            SUM2 = PROF(I,2)
            SUM3 = PROF(I,2)*PROF(I,2)*VARI(I,K+1)*PART
            DO J= K+2,KMAX-1
               L = J-K+1
               SUM1 = SUM1 + PROF(I,L)*IN(I,J)
               SUM2 = SUM2 + PROF(I,L)
               SUM3 = SUM3 + PROF(I,L)*PROF(I,L)*VARI(I,J)
            ENDDO
            J = KMAX
            PART = YVAL(I)-0.5-NINT(YVAL(I)-0.5)
            SUM1 = SUM1 + PROF(I,DIMP)*IN(I,J)*PART
            SUM2 = SUM2 + PROF(I,DIMP)
            SUM3 = SUM3 + PROF(I,DIMP)*PROF(I,DIMP)*VARI(I,J)*PART
         ENDIF
         



c     am added CONTINUE to be able to exit from the loop
c        when K<1 or K>4096
10    CONTINUE


         IF (SUM2.NE.0) THEN
c            OUT(I,INDX)  = SUM1/SUM2/(DIMP-2)               !extracted spect. 
c            VOUT(I,INDX) = SUM3/SUM2/SUM2/(DIMP-2)/(DIMP-2) !and its variance
            OUT(I,INDX)  = SUM1/SUM2                         !extracted spect. 
            VOUT(I,INDX) = SUM3/SUM2/SUM2                    !and its variance
         ELSE
            OUT(I,INDX)  = 1.0
            VOUT(I,INDX) = 1.0
         ENDIF            

c To be consistent the variance has to be devided by the square of the
c number of extracted lines as the extracted data are averged data!
c THIS HAS BEEN CHANGED NOW (07.12.99-SW): extracted data are NOT averaged!


      ENDDO
      RETURN
      END



C*******************************************************************
C********************SUBROUTINE*************************************
C*******************************************************************
C------------------------------------------------------------------------------
C  who    when           why                               where
C  AM     08/05/01       clean-comment                     calc_p
C------------------------------------------------------------------------------
C  Role: to do a weighted mean of the FF signal using the weight image coming
C       from the object extraction
C------------------------------------------------------------------------------
c here extracts the profiles and the variances doing a weighted mean of the 
c profile and its variance
      SUBROUTINE CALC_P(IN,VARI,PROF,YVAL,OUT,VOUT
     $     ,NPIXA1,NPIXA2,NB1,NB2,DIMP,IXMIN,IXMAX,IWIDTH,INDX)
      

      IMPLICIT NONE
      INTEGER IXMIN                       !index corresponding to Xmin pos
      INTEGER IXMAX                       !index corresponding to Xmax pos
      INTEGER IWIDTH                      !Y size of order
      INTEGER INDX                        !current Y position
      INTEGER DIMP                        !x-order size of profile
      INTEGER NPIXA1                      !x size of input frm
      INTEGER NPIXA2                      !y size of input frm
      INTEGER NB1                         !x size of output frm
      INTEGER NB2                         !y size of output frm

      REAL IN(NPIXA1,NPIXA2)              !input raw ima
      REAL VARI(NPIXA1,NPIXA2)            !input variance ima
      REAL OUT(NB1,NB2)                   !out ima
      REAL VOUT(NB1,NB2)                  !output variance ima
      REAL PROF(NPIXA1,DIMP)              !profile (weights)

      DOUBLE PRECISION YVAL(NPIXA1)       !Y order position(X) 

C
C     Local variables
C
      INTEGER I                           !Loop variable
      INTEGER J                           !Loop variable
      INTEGER K                           !current x-order pos index
      INTEGER KMAX                        !max value of x-order point
      INTEGER L                           !Aux var for Y point
      INTEGER IWIDTH2                     !index Y position of order center

      REAL PART                           !Fraction of pix holder

      DOUBLE PRECISION SUM1               !Image sum along x-order dir
      DOUBLE PRECISION SUM2               !weight sum along x-order dir
      DOUBLE PRECISION SUM3               !var ima sum along x-order dir

      IWIDTH2 = IWIDTH/2
      DO I = IXMIN,IXMAX
         SUM = 0
c change?
         K = INT(YVAL(I)-IWIDTH2)
         
c        am: added IF check to skip prtoblems if K<1 
        
         IF (K .LT. 1) GOTO 10
         KMAX = K+DIMP-1

C
C  Do the extraction
C
         PART = INT(YVAL(I))+1-YVAL(I)
         SUM1 = PROF(I,1)*IN(I,K)*PART
         SUM2 = PROF(I,1)
         SUM3 = PROF(I,1)*PROF(I,1)*VARI(I,K)*PART

         DO J=K+1,KMAX-1
            L = J-K+1
            SUM1 = SUM1 + PROF(I,L)*IN(I,J)
            SUM2 = SUM2 + PROF(I,L)
            SUM3 = SUM3 + PROF(I,L)*PROF(I,L)*VARI(I,J)
         ENDDO

         J = K+DIMP-1
         PART = YVAL(I)-INT(YVAL(I))
         SUM1 = SUM1 + PROF(I,DIMP)*IN(I,J)*PART
         SUM2 = SUM2 + PROF(I,DIMP)
         SUM3 = SUM3 + PROF(I,DIMP)*PROF(I,DIMP)*VARI(I,J)*PART

c        am: if K< 1 this CONTINUE allow to skip problems
10       CONTINUE

         IF (SUM2.NE.0) THEN
c            OUT(I,INDX)  = SUM1/SUM2/(DIMP-1)              !extracted spect. 
c            VOUT(I,INDX) = SUM3/SUM2/SUM2/(DIMP-1)/(DIMP-1)!and its variance
            OUT(I,INDX)  = SUM1/SUM2                        !extracted spect. 
            VOUT(I,INDX) = SUM3/SUM2/SUM2                   !and its variance
         ELSE
         
c     am: added to trace an error     
C          print *,'Warning: Filling OUT and VOUT with 0s !' 
                  
            OUT(I,INDX)  = 0.0
            VOUT(I,INDX) = 0.0
         ENDIF            

c To be consistent the variance has to be devided by the square of the
c number of extracted lines as the extracted data are averaged data!
c THIS HAS BEEN CHANGED NOW (07.12.99-SW): extracted data are NOT averaged!


      ENDDO
      RETURN
      END

C*******************************************************************
C*************************END OF PROGRAM****************************
C*******************************************************************
