C ===========================================================================
C $Id: lfit.for,v 1.4 2001/01/04 13:23:02 amodigli Exp $
C $Name:  $
C $Revision: 1.4 $
C $Log: lfit.for,v $
C Revision 1.4  2001/01/04 13:23:02  amodigli
C Added Id, Name, Revision, Log conf. control info
C
C ===========================================================================

C Taken from Numerical Recipes in Fortran 77 (86-92)
      SUBROUTINE polfit(x,y,sig,msk,ndat,a,ia,ma,covar,npc,chisq,funcs)
      IMPLICIT NONE
      INTEGER ma,ia(ma),npc,ndat,MMAX
      INTEGER msk(ndat)
      REAL chisq,a(ma),covar(npc,npc),sig(ndat),x(ndat),y(ndat)
      EXTERNAL funcs
      PARAMETER (MMAX=50)
C     USES covsrt,gaussj
C
C     Given a set of data points x(1:ndat), y(1:ndat) with individual standard 
C     deviations sig(1:ndat), use chi-2 minimization to fit for some of the 
C     coefficients a(1:ma) of a function that depends linearly on a, 
C     y = SUM_i[a(i)*aFunc_i(x). The input array ia(1:ma) indicates by non zero 
C     entries those components of a that should be fitted for, and by zero entries 
C     those components that should be held fixed at their input values. The program 
C     returns values for a(1:ma),chi-2=chisq, and the covariance C matrix 
C     covar(1:ma,1:ma). (Parameters held fixed will return zero covariances). 
C     npc is the physical dimension of covar(npc,npc) in the calling routine. 
C     The user supplies funcs(x,aFunc,ma) that returns the ma basis functions 
C     evaluated at x=X in the array aFunc.
 
      INTEGER i,j,k,l,m,mfit
      REAL sig2i,sum,wt,ym,afunc(MMAX),beta(MMAX)
      mfit=0
      
      do j=1,ma
          if(ia(j).ne.0) mfit=mfit+1
      enddo 
      if(mfit.eq.0) pause 'lfit: no parameters to be fitted'
      
     
      do  j=1,mfit         !initialize the (symmetric) matrix
          do  k=1,mfit
              covar(j,k)=0.
          enddo 
          beta(j)=0.
      enddo 
      
     
      do  i=1,ndat         !Loop over data to accumulate coefficients 
                             !of the normal equations               
          call funcs(x(i),afunc,ma)
          ym=y(i)
          if(mfit.lt.ma) then  !Subtract off dependencies on known pieces 
                               !of the fitting function
              do j=1,ma
                  if(ia(j).eq.0) ym=ym-a(j)*afunc(j)
              enddo
          endif
c          sig2i=1./sig(i)**2    !original line from NR
          sig2i=1./sig(i)        !we pass the variance=sig(i)**2
          j=0
          do  l=1,ma
             if(ia(l).ne.0) then
                 j=j+1
                 wt=afunc(l)*sig2i
                 k=0
                 do  m=1,l
                    if(ia(m).ne.0) then
                        k=k+1
                        covar(j,k)=covar(j,k)+wt*afunc(m)
                    endif
                 enddo 
                 beta(j)=beta(j)+ym*wt
             endif
          enddo 
      enddo
       
     
      do  j=2,mfit         !fill in above the diagonal for symmetry
          do k=1,j-1
              covar(k,j)=covar(j,k)
          enddo
      enddo 
      
      
      call f_gaussj(covar,mfit,npc,beta,1,1)  !Matrix Solution
      
      j=0
      do  l=1,ma
          if(ia(l).ne.0) then
              j=j+1
              a(l)=beta(j)   !Partition solution to appropriate coefficients a
          endif
      enddo 
      chisq=0.               !Evaluate chi-2 of the fit
     
      do  i=1,ndat
          call funcs(x(i),afunc,ma)
          sum=0.
          do  j=1,ma
              sum=sum+a(j)*afunc(j)
          enddo
          
c          chisq=chisq+((y(i)-sum)/sig(i))**2  !original line from NR
          chisq=chisq+((y(i)-sum))**2/sig(i)   !we pass the variance=sig(i)**2
      enddo 
c      call covsrt(covar,npc,ma,ia,mfit)    
      return
      END
C  (C) Copr. 1986-92 Numerical Recipes Software %5210G.31zR2.1"n26@1.
