/* ===========================================================================
 * $Id: physmod.c,v 1.7 2001/01/04 13:23:10 amodigli Exp $
 * $Name:  $
 * $Revision: 1.7 $
 * $Log: physmod.c,v $
 * Revision 1.7  2001/01/04 13:23:10  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */
/*+++ physmod.c ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		
 
 .KEYWORDS	uves physical model, spectroscopy, echelle,
 
 .USAGE		.
 
 .INPUT		.
		
 .OUTPUT	.
 
 .RETURN	Q1:  0: successful return
 		    -1: a successful return failed
 
 .PURPOSE	Library of UVES Physical Model Functions. 
 
 .ALGORITHM	UVES Physical Model.

 .ENVIRON	MIDAS, UVES context
    
 .LANGUAGE	C
 
 .AUTHOR	Pascal Ballester, Olivier BOITQUIN, ESO-DMD
 
 .VERSION	1.0 1999/07/01	  Creation
                    1999/09/30-SW Binning implemented.
		    1999/10/27-OB Bugs with n index fixed.

 .COMMENT       
----------------------------------------------------------------------------*/

#include <math.h>
#include <physmod.h>
#include <midas_def.h>
static double dsqrarg;
#define DSQR(a) ((dsqrarg=(a)) == 0.0 ? 0.0 : dsqrarg*dsqrarg)

#define     PROID "physmod.c"  

/* for messout function: no "static" (clear) -
   no "extern" (otherwise redeclaration in all files neccessary) */
int messlevel;

/* globals declared as extern in physmod.h */

float    bin[2]      = {1, 1}; /* binning of exposure in x and y */
double   pixsize[2]  = {15e-6, 15e-6}; /* meters */
double   delta[2]  = {75.04, 76.0};

double   cdgroov[4]  = {1000.0e-6, 660.0e-6, 600.0e-6, 312.5e-6};  /* grooves/nm */
double   echgroov[2] = {31.6e-6,  41.59e-6};  /* grooves/nm */
/*double   echblaz[2]  = {75.04, 75.9}; /* degrees */
/*double   echblaz[2]  = {74.74,  76.1};*/
double   echblaz[2]  = {74.57, 75.9}; /* degrees */
/*double   cdbeam[2]   = {45.336,  45.0}; /* degrees */
/*double   cdbeam[2]   = {45.336,  45.9};*/
/*double   cdbeam[2]   = {45.336,  45.0};  degrees */
double   cdbeam[2]   = {45.336,  46.0}; /* degrees */
/*double   ccdrot[2]   = {0.7, -0.45}; /* degrees */
/*double   ccdrot[2]   = {-1.0,  -1.50}; */
/*double   ccdrot[2]   = {0.2, -0.45};   degrees */
double   ccdrot[2]   = {0.3, -0.55}; /* degrees */
int      imsize[2]   = {4096, 3000}; /* pixels */
int      rowsiz[2]   = {2048, 2048}; /* pixels */

/* Configurations 
   1:Blue CD1,     2:Blue CD2, 
   3:Red CD 3 EEV, 4:Red CD 4 EEV, 
   5:Red CD3 MIT,  6: Red CD4 MIT 
*/
 
double   offsetx[6]  = {1391.0,1385.0,1852.0,1835.0,1852.0,1834.0};
double   offsety[6]  = {1030.0,1025.0,2098.0,2104.0,-52.0,-49.0};

double   flipx       = 1.; 
double   flipy       = -1.;
double   airt        = 25;  /* Celsius */
double   airp        = 760; /* mmHg */
double   airw        = 3;   /* mmHg */

int      cfgindx     = 0; 
int      XdispId     = 0;
int      echId       = 0;
double   alpha0cd, beta0cd;
char     armId       = 'x';

double   DEG2RAD     = M_PI/180.;

/* For history, please keep track HERE of the previous offset used: 
double   offsetx[6]  = {1470.0,1450.0,2130.0,2140.0,2130.0,2140.0}; Garching 1 
double   offsetx[6]  = {1515.0,1514.0,2010.0,2000.0,2010.0,2000.0}; Garching 1 
double   offsetx[6]  = {1474.0,1471.0,1960.0,1948.0,1961.0,1949.0}; Comissioning 1 
double   offsetx[6]  = {1390.0,1386.0,1849.0,1840.0,1854.0,1840.0}; Comissioning 2, Dec 2000 
double   offsetx[6]  = {1390.0,1384.0,1851.0,1840.0,1851.0,1839.0}; Paranal 2, Feb 2000 

double   offsety[6]  = {1030.0,1030.0,2020.0,2020.0,-125.0,-115.0}; Garching 1 
double   offsety[6]  = {1029.0,1025.0,2072.0,2080.0,-74.0,-65.0}; Garching 1 
double   offsety[6]  = {1027.0,1024.0,2069.0,2077.0,-74.0,-65.0}; Comissioning 1 
double   offsety[6]  = {1027.0,1025.0,2084.0,2094.0,-65.0,-54.0}; Comissioning 2, Dec 2000 
double   offsety[6]  = {1030.0,1025.0,2088.0,2094.0,-57.0,-54.0}; Paranal 2, Feb 2000

The gap between the EEV and MIT chips is estimated to 95 pixels */
/*
Mean pixel-scales for the 6 configurations: 
   pixscale : 0.252396, 0.246, 0.182, 0.175266, 0.182, 0.175266 */
 
/*---------------------------------------------------------------------*/

static void error()
{
  int  unit;

   SCKWRC("Q1",1,"1",1,60,&unit);
   SCSEPI();
}

/*--------------------------------------------------------------------*/

/************************************************************************* */
void setIncidence(echred, echblue, xred, xblue)
double echred, echblue, xred, xblue;
{
     echblaz[0] += echred;
     echblaz[1] += echblue;
     cdbeam[0]  += xred;
     cdbeam[1]  += xblue;
}

/************************************************************************* */
void setccdrot(ccdrotred, ccdrotblue)
double ccdrotred, ccdrotblue;
{
     ccdrot[0] += ccdrotred;
     ccdrot[1] += ccdrotblue;
}

/************************************************************************* */
void setOffset(xoff, yoff)
double xoff, yoff;
{
     offsetx[cfgindx-1] += xoff;
     offsety[cfgindx-1] += yoff;
}


/************************************************************************* */
void setBinning(binx, biny)
float binx, biny;
{
  bin[0] = binx;
  bin[1] = biny;
}


/************************************************************************* */
void airConfig(p, t, w)
double p, t, w;
{
/* 
  Initialises atmospheric conditions with  p in mmHg, t in Celsius, w in mmHg
  Typical values, in Garching: t=25, p=760, w=3; in Paranal: t=15, p=560, w=3
*/

    airt = t;
    airp = p;
    airw = w;

}

/************************************************************************* */
double airIndex(lambda)
double lambda;
{
   /* wavelength is expected in nanometers. */

   double t1, t2, t3, airdx;
 
   t1 = 1.0e-6/lambda/lambda;  /* 1e-6 for nm to um conversion, squared */

   t2 = (64.328+29498.1/(146.0-t1)+255.4/(41.0-t1))*airp*
			   (1.0+1.0e-6*(1.049-0.0157*airt)*airp)/
			   (720.883*(1.0+0.003661*airt));

   t3 = t2 - airw*(0.0624 - 0.000680*t1)/(1+0.003661*airt);

   airdx = 1.0+ t3*1.0e-6;
   /*   airdx = 1.0; */

   return(airdx);
}


/************************************************************************* */
void findAlphaBeta(lm, m, k, theta, alpha, beta)
int m;
double lm, k, theta, *alpha, *beta;
{
 /* Solves sin(alpha) + sin(beta) = m*k*lm, given beta - alpha = theta */
 char tmps[200];
 sprintf (&tmps[0],"lm, m, k, theta : %f %d %f %f",lm,m,k,theta);
 messout(messlevel,3,PROID,tmps);

 lm /= airIndex(lm);

 *alpha = 0.5* ( 2*asin( m*k*lm/2/cos(theta*DEG2RAD/2) ) + theta*DEG2RAD );
 *beta  = 0.5* ( 2*asin( m*k*lm/2/cos(theta*DEG2RAD/2) ) - theta*DEG2RAD );
 *alpha /= DEG2RAD;
 *beta  /= DEG2RAD;
}

/* ************************************************************************ */
/* ArmId = 'r'   for the red arm, ArmId = 'b' for the blue arm of Uves */
/* CcdId = 'e' for the EEV detector, CcdId = 'm' for the MIT detector*/
/* XdispId = 1 to 4 for cross-dispersers CD#1 to CD#4 */
/************************************************************************* */
int  uvesConfig(arm, CcdId, disp, waveCent, binx, biny)
char arm, CcdId;
int disp;
float binx, biny;
double waveCent;
{
  int cfg = 0;
  char tmps[200];
  int  unit;
  echId   = 2;
  XdispId = disp;
  armId   = arm;
  
/*  printf("Configuring: Arm %c CCD %c  Xdisp %d Wave %f\n",arm,CcdId,disp,waveCent); */
  sprintf(&tmps[0],"Configuring: Arm %c CCD %c  Xdisp %d Wave %f",arm,CcdId,disp,waveCent);
  messout(messlevel,3,PROID,tmps);

  if (arm == 'b' && disp == 1)  cfg = 1;
  if (arm == 'b' && disp == 2)  cfg = 2;

  if (arm == 'r' && disp == 3) {
          echId  = 1;
          if (CcdId == 'e') cfg = 3;
          if (CcdId == 'm') cfg = 5;
  }
  if (arm == 'r' && disp == 4) {
          echId  = 1;
          if (CcdId == 'e') cfg = 4;
          if (CcdId == 'm') cfg = 6;
  }

  if (cfg == 0) {
    messout(messlevel,1,PROID,"Wrong configuration!");
    error();
  }

  cfgindx = cfg;

  (void) setBinning(binx, biny);
  (void) findAlphaBeta(waveCent, 1, cdgroov[XdispId-1], 
                        cdbeam[echId-1], &alpha0cd, &beta0cd); 


  sprintf (&tmps[0],"alpha, beta for Xdisp: %f %f\nin config %d and CCD-ID %c",alpha0cd, beta0cd,cfg, CcdId);
  messout(messlevel,3,PROID,tmps);
  
  return(cfg);

}

/************************************************************************* */
double cameraFocal(lm)
double lm;
/*
** Blue Arm  
**
** computes the focal length of the blue camera (in mm), lm is in nm.
**
** nfk=5.3961886e-7*lm*lm*lm-0.00079597882e0*lm*lm+0.41122805e0*lm+287.89644e0;
**
** Red Arm
**
** nfk=6.0172051e-13*lm*lm*lm*lm*lm-2.5623231e-9*lm*lm*lm*lm+
**     4.3585543e-6*lm*lm*lm-0.0037286381e0*lm*lm+
**     1.6289971e0*lm+210.06767e0;
*/
{
   double nfk;
   /*printf("lm = %f ", lm);*/
   lm /= airIndex(lm);

   if (armId == 'b' ) { 
    nfk=5.3961886e-7*lm*lm*lm-0.00079597882*lm*lm+0.41122805*lm+287.89644;
   }

   if (armId == 'r' ) { 
    nfk=6.0172051e-13*lm*lm*lm*lm*lm-2.5623231e-9*lm*lm*lm*lm+
        4.3585543e-6*lm*lm*lm -0.0037286381*lm*lm+
        1.6289971*lm + 210.06767;

}

 /*printf("Nfk = %f, lm = %f\n",nfk/1000.,lm); */
return(nfk/1000.);

}


/************************************************************************* */
int findM(lm)
double lm;
{
  /* Solves sin(alpha) + sin(beta) = m*k*lm to find m */
  int order;
  double k, alpha, beta;

  lm /= airIndex(lm);

  k     = echgroov[echId-1];
  alpha = echblaz[echId-1];
  beta  = echblaz[echId-1];
  order = (int)((sin(alpha*DEG2RAD) + sin(beta*DEG2RAD))/k/lm + 0.5);
  return(order);
}

/************************************************************************* */
double findML(k, alpha, beta)
double k, alpha, beta;
{
  double ML;
  /* Solves sin(alpha*DEG2RAD) + sin(beta*DEG2RAD) = m*k*lm to find m*lm */
  ML =  ( ( sin(alpha*DEG2RAD) + sin(beta*DEG2RAD) )/k);
  ML *= airIndex(ML);
  return(ML);
}


/************************************************************************* */
double findL(k, alpha, beta)
double k, alpha, beta;
{
  double L;
  /* Solves sin(alpha) + sin(beta) = m*k*lm to find m*lm */
  L = (sin(alpha*DEG2RAD) + sin(beta*DEG2RAD) )/k ;
  L *= airIndex(L);
  return(L);
}

/************************************************************************* */
double findBeta(m, k, l, alpha)
int m;
double k, l, alpha;
{
  /* Solves sin(alpha) + sin(beta) = m*k*lm to find m*lm */
   /* mprintf ("m, k, l, alpha: %d %f %f %f\n",m, k, l, alpha); */
 
  l /= airIndex(l);
  
  /*  I check that the argument of asin is <=0 otherwhise return
      a dummy angle (~89 deg) which will produce (x,y) pos out
      of detector so the line will be discarted
   */
    
  if ( (m*k*l - sin(alpha*DEG2RAD)) <=1.0 )
  {
     return( (asin(m*k*l - sin(alpha*DEG2RAD)))/DEG2RAD );
  }
  else
  {
     return( (asin(0.999))/DEG2RAD );
  }
}

/************************************************************************* */
void lm2beta(lambda, m, pbetaech, pbetacd, pfc)
int m;
double lambda, *pbetaech, *pbetacd, *pfc;
{
    /* printf ("Disp: %d Ech: %d\n",XdispId,echId-1); */

/*   lambda /= airIndex(lambda);  bog fixed ! */
   
   *pfc       = cameraFocal(lambda);

    /* printf ("New Camera focal (m) : %f\n",fc); */

   *pbetaech = findBeta(m, echgroov[echId-1], lambda, echblaz[echId-1]);

   *pbetacd  = findBeta(1, cdgroov[XdispId-1], lambda, alpha0cd); 

}

/************************************************************************* */
void beta2xy(betacd, betaech, fc, px, py)
double betacd, betaech, fc, *px, *py;
{
   double  xd, yd, xr, yr, angle;   
    
   xd       = fc*tan( (betaech - echblaz[echId-1])*DEG2RAD )/pixsize[0]/bin[0];    
   yd       = fc*tan( (alpha0cd - betacd - cdbeam[echId-1])*DEG2RAD )/pixsize[1]/bin[1];

    /* printf ("beta(CD), yorg: %f %f\n", betacd, yd); */

   xd = xd*flipx;
   yd = yd*flipy;   
   /* printf ("Positions after flip: %f %f\n",xd,yd); */
    
   angle    = ccdrot[echId-1]*DEG2RAD;
   xr       =  xd*cos(angle) + yd*sin(angle);
   yr       = -xd*sin(angle) + yd*cos(angle);

    /* printf ("Rotated positions %f %f\n",xr,yr); */

   *px = offsetx[cfgindx-1] / bin[0] + xr;
   *py = offsety[cfgindx-1] / bin[1] + yr;
	

}

/************************************************************************* */
void photobeta(lambda, betaech, betacd, prech, prcd, pblz)
double  lambda, betaech, betacd, *prech, *prcd, *pblz;
{
   double gam;

   *prech = cos(echblaz[echId-1]*DEG2RAD)/cos(betaech*DEG2RAD);
   
   *prcd = cos(alpha0cd*DEG2RAD)/cos(betacd*DEG2RAD);

   /* Computes the blaze function */
   gam = M_PI / echgroov[echId-1] * cos(delta[echId-1]*DEG2RAD) * 
         (sin((betaech-delta[echId-1])*DEG2RAD)-sin((echblaz[echId-1]-delta[echId-1])*DEG2RAD))
	 / lambda/ airIndex(lambda);
   *pblz = DSQR(sin(gam)/gam);
   
   /*printf("gamma = %f, Blaze function = %f \n",gam,blz); */
       
}

/************************************************************************* */
void pixscl(wave, order, rech, rcd, binx, biny, fc, slitwidth, slitlength,
            pbinsize, ppixscale, ppixscalCD, 
	    plinewidpx, plinelenpx, plinewidth, presol)
double wave, rech, rcd, fc, slitwidth, slitlength;
int order;
float binx, biny;
double *pbinsize,*ppixscale,*ppixscalCD,*plinewidpx,*plinelenpx,*plinewidth,*presol;
{
   double binsize, pixscale, pixscalCD, linewidpx, linelenpx, linewidth, resol;
   static double scale;

/* Computation of the true pixel-scale along echelle dispersion */
/* scale = (K*FocalRatioNasmyth*PixelSize*ApertureCamera*1e-3)/FocalNasmyth
   K = 206265 : conversion radian to arcsec (1 radian = 206265 arcsec)*/
   if (!(scale)) scale = (206265.0*15.0*0.015*200*1e-3*binx)/120000;

/* Computes the width (in pixel and A) and resolution lines */
   binsize = waveBin(wave, order) * 1e4;  /* in mA */
   pixscale = scale/(fc*rech); /* in arcsec/pixel */
   pixscalCD = pixscale *(biny/binx) *rech/rcd; /* in arcsec/pixel */ 
   linewidpx = slitwidth / pixscale;  /* pixel */
   linelenpx = slitlength /pixscalCD;  /* pixel */	
   linewidth = binsize * linewidpx * 1e-3; /* in A (* pixel) */
   resol = wave * 10.0 / linewidth;  /* without unit */
	                          /* (10.0: conversion factor from nm to A)*/
   *pbinsize = binsize;
   *ppixscale = pixscale;
   *ppixscalCD = pixscalCD;
   *plinewidpx = linewidpx;
   *plinelenpx = linelenpx;
   *plinewidth = linewidth;
   *presol = resol;

}

/************************************************************************* */
void xyModel(lambda, m, px, py)
int m;
double lambda, *px, *py;
{
   double fc, betaech, betacd;

   lm2beta(lambda, m, &betaech, &betacd, &fc);
   beta2xy(betacd, betaech, fc, px, py);
   
/* exemple : 
   photobeta(lambda, betaech, betacd, prech, prcd, pblz); 
*/

}

/************************************************************************* */
void xy2beta(pbetaech, pbetacd, fc, x, y)
double *pbetaech, *pbetacd, fc, x, y;
{
   double xr, yr, xd, yd, angle;

   angle    = ccdrot[echId-1]*DEG2RAD;

   xr = (x - offsetx[cfgindx-1]/bin[0]);
   yr = (y - offsety[cfgindx-1]/bin[1]);

   xd = xr*cos(angle) - yr*sin(angle);
   yd = xr*sin(angle) + yr*cos(angle);

   xd  /= flipx;
   yd  /= flipy;

   *pbetaech = atan(xd*pixsize[0]*bin[0]/fc)/DEG2RAD + echblaz[echId-1];
   *pbetacd  =  alpha0cd - cdbeam[echId-1] - atan(yd*pixsize[1]*bin[1]/fc)/DEG2RAD; 

}

/************************************************************************* */
void beta2lamb(betaech, betacd, plambda, m)
int m;
double betaech, betacd, *plambda;
{
   *plambda = findL(m*echgroov[echId-1], echblaz[echId-1], betaech);

}

/************************************************************************* */
void lmfModel(plambda, pdm, fc, x, y)
double *plambda, *pdm, fc, x, y;
{
  /* Finds the couple lambda,m corresponding to a given position (x,y), */
  /* assuming the focal of the camera for this wavelength is fc. */

   double betaech, betacd;
   
   xy2beta(&betaech, &betacd, fc, x, y);
   
   *plambda = findML(cdgroov[XdispId-1],  alpha0cd, betacd);
   *pdm      = findML(echgroov[echId-1], echblaz[echId-1], betaech)/(*plambda);
/*
   printf("  m= %f, lambda= %f, position (x,y)= (%f , %f)\n",*pdm,*plambda, x,y);
*/
}

/************************************************************************* */
void lModel(plambda, m, fc, x, y)
int m;
double *plambda, fc, x, y;
{
  /*Finds the couple lambda,m corresponding to a given position (x,y), */
  /*assuming the focal of the camera for this wavelength is fc. */

   double betaech, betacd;

   xy2beta(&betaech, &betacd, fc, x, y);

/*   rech = cos(echblaz[echId-1]*DEG2RAD)/cos(betaech*DEG2RAD);
   rcd = cos(alpha0cd*DEG2RAD)/cos(betacd*DEG2RAD); */

   beta2lamb(betaech, betacd, plambda, m);

}

/************************************************************************* */
void lmBetaModel(plambda, pm, x, y, pbetaech, pbetacd)
int *pm;
double *plambda, x, y, *pbetaech, *pbetacd;
{
  /* Iteratively finds the couple lambda,m corresponding to a given position */
  /* (x,y), by starting with an initial guess and iteratively refining it. */
  /* assuming the focal of the camera for this wavelength is fc. */

double fcguess, wave, mdbl, xe, ye, xd, yd,wavep,dwave=0.;
int  i=0;

if (armId == 'b') fcguess = 0.360;/*0.35722;*/
if (armId == 'r') fcguess = 0.500;

lmfModel(&wave,&mdbl,fcguess,x,y);
/*
if (x == 1500.) {
  printf("m= %f, lambda= %f, position (x,y)= (%f , %f)\n",mdbl,wave, x,y);
    printf("focal guess= %f\n",fcguess);
}
*/
do {
   
   fcguess = cameraFocal(wave);
   /*
     if (x == 1500.) { 
       printf("i= %d, focal = %f, m= %f, lambda= %f\n", i,fcguess,mdbl,wave);
     }
   */
   if (*pm <= 0) *pm = (int)(mdbl+0.5);
/*   lModel(&wave,*pm,fcguess,x,y); */
   xy2beta(pbetaech, pbetacd, fcguess, x, y);
   beta2lamb(*pbetaech, *pbetacd, &wave, *pm);

   /*
     if (x == 1500.) { 
       printf("i= %d, focal = %f, m= %d, lambda= %f\n", i,fcguess,*pm,wave);
     }
   */
   
   xyModel(wave,(int)(mdbl+0.5), &xe, &ye);
   /*
      if (x == 1500.) { 
        printf("m= %f, lambda= %f, position (xe,ye)= (%f , %f)\n",mdbl,wave, xe,ye);
        printf("focal = %f\n",fcguess);
      }
   */
   i++;
   
   xd = fabs(x-xe);
   yd = fabs(y-ye);
   
   } while (!((xd < 1.) && (yd < 1.)) && (i <= 4));

*plambda = wave;

/*printf("i= %d", i);*/
/* photobeta(wave, *pbetaech, *pbetacd, prech, prcd, pblz); 
printf("lmBetaModel(%f, %d, %f, %f, %f, %f)\n",wave, *pm, x, y, *pbetaech, *pbetacd);
*/

}

/************************************************************************* */
void lmModel(plambda, pm, x, y)
int *pm;
double *plambda, x, y;
{
  /* Iteratively finds the couple lambda,m corresponding to a given position */
  /* (x,y), by starting with an initial guess and iteratively refining it. */
  /* assuming the focal of the camera for this wavelength is fc. */

  double betaech, betacd;
  
  lmBetaModel(plambda, pm, x, y, &betaech, &betacd);
  
/* exemple : 
  photobeta(*plambda, &betaech, &betacd, prech, prcd, pblz); 

printf("lmBetaModel(%f, %d, %f, %f, %f, %f)\n",*plambda, *pm, x, y, betaech, betacd);
*/
}

/************************************************************************* */
void findFSR(m, lambdaC, fsrStart, fsrEnd)
int m;
double *lambdaC, *fsrStart, *fsrEnd;
{
  /* Finds start and end of the free-spectral range */

  double delta;

  *lambdaC = 2*sin(echblaz[echId-1]*DEG2RAD)/m/echgroov[echId-1];

  delta = *lambdaC/m;

  *fsrStart = *lambdaC - delta/2.;
  *fsrEnd   = *lambdaC + delta/2.;

}

/************************************************************************* */
double waveBin(l, m)
int m;
double l;
{
  /* Finds start and end of the free-spectral range */

  double dl, x0,y0,x1,y1;

  dl = 20e-4; /* Wavelength increment in nm (nearly one pixel)*/

  xyModel( l,    m,&x0,&y0);
  xyModel((l+dl),m,&x1,&y1);

  return( dl/(x1-x0) );
}

/************************************************************************* */
void ccdSize(nx, ny)
int *nx, *ny;
{
  /* Finds x and y dimension of the selected CCD */

  *nx = imsize[echId-1] / bin[0];
  *ny = rowsiz[echId-1] / bin[1];
}

/************************************************************************* */
void xyRegres(x,y,px,py)
double x,y,*px,*py;
{
  double xdiff,ydiff;
  int  xnpix, ynpix;
  
  static double xcoef[6][9] = {
				{      /* cfgindx = 1 (346 nm) */
				 7.2297601440608, -.40735183147301e-02, -.64543515348780e-06,
				 -.29073467820897e-02,.18085528931278e-05, -.17369616853897e-09,
				 .12763951767020e-05, -.10449350438237e-08, .21237141926298e-12
				},{   /* cfgindx = 2 (437 nm) */ 
				8.2119114833688, -.46043784496767e-02, -.36550910624737e-06, 
				-.38246292584966e-02, .19573882351248e-05, -.39338288900862e-10, 
				.16343388544504e-05, -.57175182716971e-09, -.10447804314703e-12
				},{   /* cfgindx = 3 (580 nm EEV) */ 
				3.7413106732150, .46748733984490E-03, -.10373674021445E-05,
				.32160141767636E-02, -.65755677130179E-06, -.27920927856270E-09,
				.49645223156544E-06, -.91641530008746E-09, .27921343156861E-12
				},{   /* cfgindx = 4 (860 nm EEV)) */ 
				-4.0470785506542, .64505578848367E-02, -.24064115513399E-05,
				.19637136969759E-01, -.19167305771748E-04, .39358707499725E-08,
				-.66711813543599E-05, .74553635846620E-08, -.16592717469965E-11
				},{   /* cfgindx = 5 (580 nm MIT)) */ 
				9.3646848997663, -.14867315739333E-02, -.12266047696368E-05, 
				.54078567946324E-02, -.43142824717971E-05, .93930280215483E-09, 
				-.41372038318427E-05, .29456216968165E-08, -.40742415470899E-12
				},{   /* cfgindx = 6 (860 nm MIT)) */ 
				7.9771176628248, -.17362375615848E-02, -.10537220113702E-05,
				.21380033242417E-02, -.40464769916741E-05, .12285338312767E-08,
				-.23223584365342E-05, .29857529680670E-08, -.68273255838748E-12
				}
							  };
						
  static double ycoef[6][9] = {
				{      /* cfgindx = 1 (346 nm) */
				 6.8032499799802, -.12324016699119e-01, .39816858608275e-05,
				 .23093828587347e-02, -.19038127068031e-05, .35725581456459e-09, 
				 -.11785051251629e-05, .13882841668899e-08, -.33905448366714e-12
				},{   /* cfgindx = 2 (437 nm) */ 
				 7.6215427012097, -.11958584227548e-01, .36310659652957e-05, 
				 -.18753309641511e-02, .21045680891948e-05, -.62114786229517e-09, 
				 .11213802862204e-06, .35494025482824e-09, -.11753943514033e-12
				},{   /* cfgindx = 3 (580 nm EEV)) */
				4.9018771615702, -.56566790719636E-02, .12433005375662E-05,
				.61179933424244E-02, -.37321198731942E-05, .88406883344494E-09, 
				-.10834561613249E-05, .23588293619469E-09, -.47849714320988E-13
				},{   /* cfgindx = 4 (860 nm EEV)) */ 
				-3.7804247843098, .15769905019374E-03, -.15587296446293E-06,
				.59575197956945E-02, -.43208086633811E-05, .10613284931449E-08,
				-.88548392602879E-06, .38686643599148E-09, -.10445435196266E-12
				},{   /* cfgindx = 5 (580 nm MIT)) */
				13.307626649783, -.12606684238589E-01, .27932549598420E-05, 
				.26466346655048E-02, -.55284350232586E-05, .15120614089452E-08, 
				.25375473276787E-06, .12282380934765E-08, -.40956732092905E-12 
				},{   /* cfgindx = 6 (860 nm MIT)) */ 
				7.2073676301164, -.71110204718800E-02, .15299874197833E-05, 
				.50246814423568E-02, -.74811146667246E-05, .18222289686011E-08, 
				-.16943579329917E-05, .27524619002227E-08, -.68106490576341E-12
				}
							  };
  goto simplified;
  						  
  xdiff = xcoef[cfgindx-1][8]*DSQR(x*y) + xcoef[cfgindx-1][7]*x*DSQR(y) + xcoef[cfgindx-1][6]*DSQR(y)
		+ xcoef[cfgindx-1][5]*DSQR(x)*y + xcoef[cfgindx-1][4]*x*y       + xcoef[cfgindx-1][3]*y
		+ xcoef[cfgindx-1][2]*DSQR(x)   + xcoef[cfgindx-1][1]*x         + xcoef[cfgindx-1][0];

  ydiff = ycoef[cfgindx-1][8]*DSQR(x*y) + ycoef[cfgindx-1][7]*x*DSQR(y) + ycoef[cfgindx-1][6]*DSQR(y)
		+ ycoef[cfgindx-1][5]*DSQR(x)*y + ycoef[cfgindx-1][4]*x*y       + ycoef[cfgindx-1][3]*y
		+ ycoef[cfgindx-1][2]*DSQR(x)   + ycoef[cfgindx-1][1]*x         + ycoef[cfgindx-1][0];

  
  /* New, simplified correction */ 
  simplified: {
  ccdSize(&xnpix, &ynpix);

  xdiff = (-7.)*(x-(double)xnpix/2.)/((double)xnpix/2.);
  /* ydiff = (5.)*pow((x-(double)xnpix/2.)/((double)xnpix/2.),2.); */
  ydiff = (5.)*DSQR((x-(double)xnpix/2.)/((double)xnpix/2.));

  *px = x + xdiff;
  *py = y + ydiff;
  }
}
