/* ===========================================================================
 * $Id: surffit.c,v 1.2 2001/01/04 13:23:06 amodigli Exp $
 * $Name:  $
 * $Revision: 1.2 $
 * $Log: surffit.c,v $
 * Revision 1.2  2001/01/04 13:23:06  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "matrix.h"

int surf_fit(dat,lx,ly,ux,uy,degree,res,coef)
float **dat, **res, **coef;
int  lx, ly, ux, uy, degree;
/*
  dat    data matrix   (dimension nx x ny)
  res    result matrix (dimension nx x ny)
  coef   coefficients matrix (dimension degree+1 x degree+1)
  degree degree of 2-dim polynomial fit

  The memory for all matrices have to be allocated BEFORE calling !

  WARNING:
  Due to the powf() function this subroutine has to be compiled
  using the -Ae compiler-switch.

*/
{
  int i, j, ii, jj, i2, k, l, l1, l2, m, n;
  int nx, ny, nn, deg1, deg2;
  float  exi, exj;
  float  **x, **y;
  double **ut, **utt, **ut2, **b, **c;

  nx = ux - lx + 1;
  ny = uy - ly + 1;
  nn = nx*ny;
  deg1 = degree+1;
  deg2 = deg1*deg1;

  x = f_matrix(1,ny,1,nx);
  y = res;
  ut = d_matrix(1,nn,1,deg2);
  utt = d_matrix(1,deg2,1,nn);
  b = d_matrix(1,deg2,1,1);
  /*  c = d_matrix(1,deg1,1,deg1); */

  for (j=1,jj=ly;j<=ny;jj++,j++)
    for (i=1,ii=lx;i<=nx;ii++,i++){
      x[j][i] = j-1;
      y[jj][ii] = i-1;
    }

  for (l1=0;l1<=degree;l1++){
    exj = l1;
    for (l2=0;l2<=degree;l2++){
      exi = l2;
      k = 0;
      i2 = l1*deg1 + l2;
      for (j=1,jj=ly;j<=ny;jj++,j++)
	for (i=1,ii=lx;i<=nx;ii++,i++){
	  ut[k+1][i2+1] = powf(x[j][i],exi) * powf(y[jj][ii],exj);
	  utt[i2+1][k+1] = ut[k+1][i2+1];
	  k++;
	}
    }
  }

  /*  show_dmat(ut,1,nn,1,deg2); */

  ut2 = d_matmulcr(ut,utt,nn,deg2,deg2,nn,1);
  free_dmatrix(utt,1,deg2,1,nn);

  /*
  printf("ut2:\n");
  show_dmat(ut2,1,deg2,1,deg2);
  */

  n=dgaussj(ut2,deg2,b,1);
  if (n<0) return(-1);
  utt = d_matmulcr(ut2,ut,deg2,deg2,nn,deg2,1);

  m=1;
  n=1;
  k=1;
  for (n=1;n<=deg1;n++){
    for (m=1;m<=deg1;m++,k++){
      l=1;
      coef[n][m] = 0.0;
      for (j=ly;j<=uy;j++)
	for (i=lx;i<=ux;i++){
	  coef[n][m] += utt[l++][k] * dat[j][i];
	}
    }
  }
  /*
  printf("coef:\n");
  show_fmat(coef,1,deg1,1,deg1);
  */

  l=1;
  for (n=ly;n<=uy;n++){
    for (m=lx;m<=ux;m++,l++){
      k=1;
      res[n][m] = 0.0;
      for (j=1;j<=deg1;j++)
	for (i=1;i<=deg1;i++){
	  res[n][m] += coef[j][i] * ut[l][k++];
	}
    }
  }
  /*  free_dmatrix(utt,1,deg2,1,nn); */
  free_dmatrix(ut,1,nn,1,deg2);
  free_dmatrix(ut2,1,deg2,1,deg2);
  free_dmatrix(b,1,deg2,1,1);
  free_matrix(x,1,ny,1,nx);
  return(0);
}




/* surface fit on the entire matrix: dat */

int surffit(dat,nx,ny,degree,res,coef)
float **dat, **res, **coef;
int  nx, ny, degree;
/*
  dat    data matrix   (dimension nx x ny)
  res    result matrix (dimension nx x ny)
  coef   coefficients matrix (dimension degree+1 x degree+1)
  degree degree of 2-dim polynomial fit

  The memory for all matrices have to be allocated BEFORE calling !

  WARNING:
  Due to the powf() function this subroutine has to be compiled
  using the -Ae compiler-switch.

*/
{
  int i, j, ii, k, l, l1, l2, m, n;
  int nn, deg1, deg2;
  float  exi, exj;
  float  **x, **y;
  double **ut, **utt, **ut2, **b, **c;

  nn = nx*ny;
  deg1 = degree+1;
  deg2 = deg1*deg1;

  x = f_matrix(1,ny,1,nx);
  y = res;
  ut = d_matrix(1,nn,1,deg2);
  utt = d_matrix(1,deg2,1,nn);
  b = d_matrix(1,deg2,1,1);
  /*  c = d_matrix(1,deg1,1,deg1); */

  for (j=1;j<=ny;j++)
    for (i=1;i<=nx;i++){
      x[j][i] = j-1;
      y[j][i] = i-1;
    }

  for (l1=0;l1<=degree;l1++){
    exj = l1;
    for (l2=0;l2<=degree;l2++){
      exi = l2;
      k = 0;
      ii = l1*deg1 + l2;
      for (j=1;j<=ny;j++)
	for (i=1;i<=nx;i++){
	  ut[k+1][ii+1] = powf(x[j][i],exi) * powf(y[j][i],exj);
	  utt[ii+1][k+1] = ut[k+1][ii+1];
	  k++;
	}
    }
  }

  /*  show_dmat(ut,1,nn,1,deg2); */

  ut2 = d_matmulcr(ut,utt,nn,deg2,deg2,nn,1);
  free_dmatrix(utt,1,deg2,1,nn);

  /*
  printf("ut2:\n");
  show_dmat(ut2,1,deg2,1,deg2);
  */

  n=dgaussj(ut2,deg2,b,1);
  if (n<0) return(-1);
  utt = d_matmulcr(ut2,ut,deg2,deg2,nn,deg2,1);

  m=1;
  n=1;
  k=1;
  for (n=1;n<=deg1;n++){
    for (m=1;m<=deg1;m++,k++){
      l=1;
      coef[n][m] = 0.0;
      for (j=1;j<=ny;j++)
	for (i=1;i<=nx;i++){
	  coef[n][m] += utt[l++][k] * dat[j][i];
	}
    }
  }
  /*
  printf("coef:\n");
  show_fmat(coef,1,deg1,1,deg1);
  */

  l=1;
  for (n=1;n<=ny;n++){
    for (m=1;m<=nx;m++,l++){
      k=1;
      res[n][m] = 0.0;
      for (j=1;j<=deg1;j++)
	for (i=1;i<=deg1;i++){
	  res[n][m] += coef[j][i] * ut[l][k++];
	}
    }
  }
  return(0);
}
