! $Id: uves_append.prg,v 1.2 2001/01/04 10:39:41 amodigli Exp $
!+++ UVES_APPEND ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT	(C) 1998  European Southern Observatory
!
!.IDENT		uves_append.prg
!
!.KEYWORDS	Concatenate message text to be send to uves_message.prg
!
!.USAGE		@p (IDENT) text
!
!.INPUT		text  	(P1): line to be appended
!		crlf	(P2): append carriage return linefeed character after
!				each line: Yes or No.
!
!.OUTPUT	
!
!.RETURN	Q1:  0: successful return
!		    -1: a successful return failed
!
!.DEFAULT	P2	N
!
!.PURPOSE	Append text to already existing text buffer CBUF.
!		This text buffer may be put on the terminal by using
!		uves_messages:
!			@p uves_messages 3 prgname
!
!		Further information see uves_messages.
!
!.ALGORITHM	Append text to already existing text buffer CBUF.
!
!.ENVIRON	UVES, CCDRED context
!   
!.LANGUAGE	MIDAS 98NOV
!
!.AUTHOR	Sebastian Wolf, ESO-DMO
!
!.VERSION	0.5 1998/03/04	Creation.
!		1.0 1999/07/20	Option to append CRLF character added.
!
!.COMMENT	This procedure allows to handle characterbuffer with any size.
!		Usually, MIDAS allows only 100 character to be entered by one
!		command (WRITE/KEY).
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_append.prg,v $
! Revision 1.2  2001/01/04 10:39:41  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "text line: "
define/param p2 n c "Append carriage return character, yes/no:"

define/local CLEN/i/1/1 0
define/local LLEN/i/1/1 0
define/local LINE/c/1/100
define/local CRLF/c/1/2 " " ALL
define/local csz/i/1/1 0

!check carriage return linefeed character:
p2 = m$upper(p2)
if p2(1:1) .eq. "Y" CRLF = "\n"		!append carriage return line feed char.

!copy key P1 line
if m$strlen(CRLF) .ne. 0 then
  line = {P1} // "{CRLF}"
else
  line = {P1}
endif
llen = m$strlen(line)			!length of input text

info/keyw CBUF
csz = mid$info(2)			!size of character keyword CBUF
if mid$info(1) .eq. 0 then		!character buffer does not exist
  @p uves_max 500 {llen}
  clen = {Q1}
  write/key CBUF/c/1/{clen} " " ALL	!--> create
else					!character buffer does already exist.
  clen = m$strlen(CBUF)			!length of text already in the buffer.
  llen = llen + clen			!total length of changed buffer.
  clen = csz				!actual size of character buffer.
  if clen .lt. llen then		!we have to increase the buffer
    define/local TBUF/c/1/{clen} " " ALL	!--> create temporary buffer
    copy/kk CBUF TBUF
    delete/keyw CBUF
    clen = 2 * clen
    @p uves_max {clen} {llen}
    clen = {Q1}
    write/key CBUF/c/1/{clen} " " ALL		!--> create
    copy/kk TBUF/c/1/{clen} CBUF/c/1/{clen}	!restore old buffer
  endif
endif
llen = 100				!now: max.size of the input buffer

!write/out {LINE} {LLEN}
!write/out {CBUF} {CLEN}

run PIPE_EXE:uves_append.exe
return 0
