! $Id: uves_calmap.prg,v 1.8 2001/01/04 10:39:41 amodigli Exp $
!+++ UVES_CALMAP +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_calmap.prg
!
!.KEYWORDS      uves physical model, spectroscopy, echelle,
!
!.USAGE         @p (IDENT) wlc npline.tbl mapss
!
!.INPUT         wlc     	(P1): wavelength calibration frame
!		npline.tbl   	(P2): Predicted position table or projection map
!               mapss   	(P3): resulting session name
!		
!.OUTPUT        order.tbl, back.tbl, line.tbl.
!
!.RETURN        
!
!.DEFAULT	P1	{wlc}
!		P2	npline.tbl
!		P3	mapss
!
!.PURPOSE       This procedures makes the order definitions and wavelength 
!		calibration from the wavelength projection map, and saves the 
!		data in tables and global keywords in echelle context.  
!
!
!.ALGORITHM     Generates order.tbl, back.tbl, line.tbl, saves the first 
!		guess solution with SAVE/ECHELLE.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        P.Ballester, O.Boitquin, ESO/DMD
!
!.VERSION       1999/09/01  Created (OB).
!		1999/09/23-SW	PIXEL descriptor has to be saved as an real
!				array otherwise it will be interpreted as
!				double which arise problems in necident.prg!
!		1999/09/30-SW	Terminal output improved.
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.8 $
! $Log: uves_calmap.prg,v $
! Revision 1.8  2001/01/04 10:39:41  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
define/param P1 {WLC}  IMA	"Input frame:"
define/param P2 npline.tbl TAB  "Predicted position table:"
define/param P3 mapss  CHAR 	"Name of the resulting session"

! For messout
define/local pid/C/1/20 "uves_calmap"


mess/out 3 {pid} "Create the calibration tables ..."
mess/out 3 {pid} "Create the order table ..."

verify/echelle {P1}

define/local NMODTAB/C/1/30 {P2}
create/table {ORDTAB} 3 {{NMODTAB},TBLCONTR(4)}
copy/tt {NMODTAB} :ORDER {ORDTAB} :ORDER
copy/tt {NMODTAB} :XMOD  {ORDTAB} :X
copy/tt {NMODTAB} :YMOD  {ORDTAB} :Y
define/local  relord/I/1/1  {{ORDTAB},:ORDER,@1}
compute/table {ORDTAB} :ORDER = {relord} + 1 - :ORDER
copy/dd    {P1} START/D/1/2  {ORDTAB} defstart/D/1/2
copy/dd    {P1} START/D/1/2  {ORDTAB} defstep/D/1/2

sort/table {ORDTAB}  :ORDER,:X

statistic/table {ORDTAB} :ORDER {SESSOUTV}
echord(1) = outputr(2) - outputr(1) + 1
echord(4) = outputr(1)
echord(5) = outputr(2)

!@@ testregr !  ? ? 10.
regress/echelle  {SESSOUTV}

mess/out 3 {pid} "Create the background table ..."
prepare/background ? INI {SESSOUTV}
set/echelle BKGVISU=NO
load/echelle



mess/out 3 {pid} "Create the line table (first guess solution)..."

create/table {LINTAB} 3 {{NMODTAB},TBLCONTR(4)}
copy/tt {NMODTAB} :XMOD  {LINTAB} :X
copy/tt {NMODTAB} :ORDER {LINTAB} :Y
copy/tt {NMODTAB} :ORDER {LINTAB} :PEAK
copy/tt {NMODTAB} :IDENT {LINTAB} :IDENT
copy/tt {NMODTAB} :YMOD  {LINTAB} :YNEW
copy/tt {NMODTAB} :ORDER {LINTAB} :ORDER
copy/tt {NMODTAB} :IDENT {LINTAB} :WAVEC


compute/table {LINTAB} :AUX = :ORDER*:IDENT

define/local disprel/D/1/3 0,0,0
regress/poly {LINTAB} :AUX :X  3 {SESSOUTV}
copy/kk OUTPUTD/D/1/3  disprel/D/1/3
statistic/table {LINTAB} :ORDER >Null
write/descr  {LINTAB} ORDER/I/1/2  {outputr(2)},{outputr(1)}

set/format E25.16

compute/table {LINTAB} :PIXEL = ({disprel(2)} + 2*{disprel(3)}*:X)/:ORDER
statistic/table    {LINTAB} :PIXEL >Null
set/echelle   avdisp={outputr(3)}

write/descr   {LINTAB} PIXEL/R/1/2 {avdisp},{avdisp}

set/format
set/format F12.4
mess/out 3 {pid} "Average pixel size: {avdisp} wav. units\n"

regress/poly {LINTAB} :ORDER :X,:YNEW 0004,0004 KEYLONG {SESSOUTV}
save/regr    {LINTAB} RORD KEYLONG

save/echelle  {P3} >Null
set/echelle   wlcmtd=guess guess={P3} 

if FORCE_DRS(1:1) .eq. "Y" return 0		!That is the case for the
						!Online-Pipeline.
append/out "You can now extract a Th-Ar frame (EXTRACT/ECHELLE),\n"
append/out "search it for lines SEARCH/ECHELLE), and finally\n"
append/out "calibrate in guess mode (IDENT/ECHELLE) using session {P3}.\n"
append/out "(keywords already changed: WLCMTD={WLCMTD}, GUESS={GUESS}.\n"
mess/out 3 {pid}

return 0
