! $Id: uves_clean_trap.prg,v 1.1 2001/03/05 07:04:05 amodigli Exp $
!+++ UVES_CLEAN_TRAP +++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!
!.IDENT         uves_clean_trap.prg 
!
!.KEYWORDS      spectroscopy, echelle, cleaning
!
!.USAGE         @p (IDENT) ima binning 
!
!.INPUT         ima      (P1): input frame to be cleaned
!               binning  (P2): ima binning parameter (Xbin X Ybin: 1x2)
!.OUTPUT        cleaned image (corrected.bdf), bad pixel table (bp.tbl)
!
!.RETURN        Q1:  0: successful return
!                   -1: return failure
!
!.DEFAULT
!
!.PURPOSE       Clean images from trap columns artifacts
!
!.ALGORITHM     According to the bin defines Xs,Ys,Xe,Ye of the traps to be 
!               cleaned and clean them using the command modify/row
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        Andrea Modigliani, ESO-DMO
!
!.VERSION       1.0 2001/02/23  Create
!
!.COMMENT	
!		
!.TODO		
!
!------------------------------------------------------------------------------
!$Name:  $
!$Revision: 1.1 $
!$Log: uves_clean_trap.prg,v $
!Revision 1.1  2001/03/05 07:04:05  amodigli
!Cleaned, added crossres, adjusted trap
!location, simplified
!
!------------------------------------------------------------------------------

crossref in_ima binning
define/param p1 ? c "enter name of input image:"
define/param p2 ? c "enter name of binning:"


define/local br1/i/1/4 0 all +lower_levels
define/local br2/i/1/4 0 all +lower_levels
define/local br3/i/1/4 0 all +lower_levels
define/local br4/i/1/4 0 all +lower_levels
define/local br5/i/1/4 0 all +lower_levels
define/local br6/i/1/4 0 all +lower_levels
define/local br7/i/1/4 0 all +lower_levels

define/local bin/c/1/10 "{p2}"

!-----------------------------
! set the bad raw Xs,Ys,Xe,Ye
!-----------------------------
     if "{bin}" .eq. "1x1" then 
        write/key br1/i/1/4  1,4,2088,4
        write/key br2/i/1/4  1,63,2282,63
        write/key br3/i/1/4  1,108,1778,108
        write/key br4/i/1/4  1,176,2443,176
        write/key br5/i/1/4  1,196,2021,196
        write/key br6/i/1/4  1,285,1975,285
        write/key br7/i/1/4  1,352,1942,352
else if "{bin}" .eq. "1x2" then                 !x=2,y=1
        write/key br1/i/1/4  1,4,1044,4
        write/key br2/i/1/4  1,63,1141,63
        write/key br3/i/1/4  1,108,894,108
        write/key br4/i/1/4  1,176,1222,176
        write/key br5/i/1/4  1,196,1011,196
        write/key br6/i/1/4  1,285,988,285
        write/key br7/i/1/4  1,352,971,352
else if "{bin}" .eq. "2x2" then
        write/key br1/i/1/4  1,3,1044,3
        write/key br2/i/1/4  1,33,1141,33
        write/key br3/i/1/4  1,55,894,56
        write/key br4/i/1/4  1,89,1222,90
        write/key br5/i/1/4  1,99,1011,100
        write/key br6/i/1/4  1,144,988,145
        write/key br7/i/1/4  1,177,971,178
else if "{bin}" .eq. "2x3" then            !x=3,y=2
        write/key br1/i/1/4  1,3,696,4
        write/key br2/i/1/4  1,33,761,33
        write/key br3/i/1/4  1,55,592,56
        write/key br4/i/1/4  1,89,814,90
        write/key br5/i/1/4  1,99,674,100
        write/key br6/i/1/4  1,144,658,144
        write/key br7/i/1/4  1,177,647,178
endif

!------------------
! display the image
!------------------
load/ima {p1} scale=-4,2  cuts=f center=c,200

!----------------------------
! creates the bad pixel table
!----------------------------
@p uves_clean_trap,CREA_BAD_PIX_TAB bp.tbl 
!The following seems to not work properly, better use the other method
!this works only with a relative small area to be corrected
!@p uves_clean_trap,CLEAN_PIX {p1} bp.tbl corrected.bdf 
!@p uves_clean_trap,CLEAN_ROW {p1} corrected.bdf

load/ima corrected.bdf   scale=-4,8  cuts=f center=c,100

!=================================================================
!=================================================================

ENTRY CLEAN_PIX
!-----------------------------------------------------------------
! cleans all the bad pixel of a given frame from a given bad pixel 
! table creating a corrected frame
!-----------------------------------------------------------------
define/parameter p1 ? c "enter input frame to be corrected : "
define/parameter p2 ? c "enter input bad pixel table : "
!define/parameter p3 ? c "enter output corrected frame : "

modify/pixel {p1},{p2},P {p3} 4,4 3,3,1 ? n

return 0
!=================================================================
ENTRY CLEAN_ROW
!------------------------------------------------------
!clean all the bad raws (trap columns) of a given frame
!------------------------------------------------------
define/parameter p1 ? c "enter File Name: "
define/parameter p2 ? c "enter File Name: "


@p uves_clean_trap,MODIFY_ROW {p1} {p2} {br1(2)} {br1(4)}
@p uves_clean_trap,MODIFY_ROW {p2} {p2} {br2(2)} {br2(4)}
@p uves_clean_trap,MODIFY_ROW {p2} {p2} {br3(2)} {br3(4)}
@p uves_clean_trap,MODIFY_ROW {p2} {p2} {br4(2)} {br4(4)}
@p uves_clean_trap,MODIFY_ROW {p2} {p2} {br5(2)} {br5(4)}
@p uves_clean_trap,MODIFY_ROW {p2} {p2} {br6(2)} {br6(4)}
@p uves_clean_trap,MODIFY_ROW {p2} {p2} {br7(2)} {br7(4)}

return 0
!=================================================================
ENTRY MODIFY_ROW
!--------------------------------
!modify the row of a given frame
!--------------------------------
define/param p1 ? c "Enter Input Frm Name (to be corrected)"
define/param p2 ? c "Enter Output Frm Name (corrected) "
define/param p3 ? n "Enter Bad Raw Y Id "
define/param p4 ? n "Enter Bad Raw Y Id "

modify/row {p1} {p2} V @{p3},@{p4} >Null

return 0
!=================================================================
!=================================================================
ENTRY CREA_BAD_PIX_TAB
!-------------------------------------------------
!creates a given table with 4 columns and 7 raws
!-------------------------------------------------
define/parameter p1 ?       c "Enter Table Name: "

crea/tab {p1}
CREATE/COLU {p1} :XSTARTPIX
CREATE/COLU {p1} :YSTARTPIX
CREATE/COLU {p1} :XENDPIX
CREATE/COLU {p1} :YENDPIX

@p uves_clean_trap,ADD_ROW {p1} 1 {br1(1)} {br1(2)} {br1(3)} {br1(4)} 
@p uves_clean_trap,ADD_ROW {p1} 2 {br2(1)} {br2(2)} {br2(3)} {br2(4)} 
@p uves_clean_trap,ADD_ROW {p1} 3 {br3(1)} {br3(2)} {br3(3)} {br3(4)} 
@p uves_clean_trap,ADD_ROW {p1} 4 {br4(1)} {br4(2)} {br4(3)} {br4(4)} 
@p uves_clean_trap,ADD_ROW {p1} 5 {br5(1)} {br5(2)} {br5(3)} {br5(4)} 
@p uves_clean_trap,ADD_ROW {p1} 6 {br6(1)} {br6(2)} {br6(3)} {br6(4)} 
@p uves_clean_trap,ADD_ROW {p1} 7 {br7(1)} {br7(2)} {br7(3)} {br7(4)} 
return 0
!=================================================================
!=================================================================
ENTRY ADD_ROW
!--------------------------------------------------
! add a raw entry at a given index in a given table
!--------------------------------------------------
define/parameter p1 ? c "Enter Table Name: "
define/parameter p2 ? n "Enter bad raw index (1-7)"
define/parameter p3 ? n "Enter X start Pix: "
define/parameter p4 ? n "Enter Y start Pix: "
define/parameter p5 ? n "Enter X end Pix: "
definE/parameter p6 ? n "Enter Y end Pix: "

define/local x1/i/1/1 {p3}
define/local y1/i/1/1 {p4}
define/local x2/i/1/1 {p5}
define/local y2/i/1/1 {p6}


WRITE/TABLE {p1} :XSTARTPIX @{p2}   {x1} 
WRITE/TABLE {p1} :YSTARTPIX @{p2}   {y1}
WRITE/TABLE {p1} :XENDPIX   @{p2}   {x2}
WRITE/TABLE {p1} :YENDPIX   @{p2}   {y2}

return 0
!=================================================================



