! $Id: uves_confinstr.prg,v 1.2 2001/01/04 10:39:37 amodigli Exp $
!+++ UVES_CONFINSTR +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_confinstr.prg
!
!.KEYWORDS      keywords
!
!.USAGE         @p (IDENT) input
!
!.INPUT         input	(P1): echelle data file (already split).
!
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Set general global keys: GRATING, CROSS, CCD, CCDBIN ...
!
!.ALGORITHM     Get the size of the frame using command IMAGE/SECTION. This
!		returns the optinal binning of the data. SCAN/ECHELLE sets
!		the global keywords IMSIZE and SCAN. The call to GET/PATHID
!		determines the optical path and the CCD used. The global
!		keyword PATHID will be set and allows to read several other
!		keywords from the descriptors of {P1}.
!		Finally, the following global keywords will be set:
!		INSTR, CCD, CROSS, GRAT, CCDBIN, IMSIZE, SCAN, WCENTER
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/07/14  Created.
!		    1999/07/15	Set central wavelength keyword WCENTER,
!				UVES_ARM, and UVES_AIR
!		    1999/10/14	Call CONFIG/FILTER
!		    1999/12/18	Determine the wavelength range (WLRANGE)
!
!.COMMENT	(1) Does not work for bias and dark exposures! They do not have
!		instrument keywords (ESO.INS.*).
!		(2) Units of central wavelength in the FITS header is nm.
!		Global keyword WCENTER is in Angstroem as all calculations
!		are usually done in Angstroems (line table!).
!
!.TODO		Check how to get the water vapour pressure !??? (UVES_AIR(3))
!		Check conversion factor mbar -> mmHg!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_confinstr.prg,v $
! Revision 1.2  2001/01/04 10:39:37  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/param p1 i ? "Enter name image frame:"
!
! constants:
define/local mbar2mm/d/1/1 1.13		!mbar to mm conversion
!
! local variables:
define/local arm/c*4/1/3 " " ALL
define/local modext/c/1/5 " " ALL
define/local binx/i/1/1 1
define/local biny/i/1/1 1

arm(1) = "RED"
arm(2) = "RED"
arm(3) = "BLUE"

config/port {p1}					!set: uves_outi, _nout
config/filt {p1}					!set: uves_filti
set/format I1
image/section {p1} Y					!Set global keywords:
binx = outputi(9)					!CCDBIN
biny = outputi(10)					!and
scan/echelle {p1} {outputi(2)},{outputi(4)} >Null	!IMSIZE and SCAN.

get/pathid {p1}						!set global key PATHID
modext = q2						!save mode extension.


set/echelle INSTR="{{p1},{h_instrume}}" CCD={h_nameccd({PATHID})}
set/echelle CROSS="{{p1},{h_namecros({PATHID})}}" GRAT=UVES 
set/echelle CCDBIN={binx},{biny}

WCENTER = m$value({p1},{h_cwlen({PATHID})})		!Central wavelength
if WCENTER .lt. 1100.0 WCENTER = WCENTER * 10.		!should be in Angstroem
if {{p1},NAXIS} .eq. 1 then
  WLRANGE(1) = m$value({p1},START(1))
  WLRANGE(2) = m$value({p1},NPIX(1)) * m$value({p1},STEP(1)) + WLRANGE(1)
endif

UVES_ARM = arm({PATHID})				!UVES path: RED or BLUE

UVES_AIR(1) = m$value({p1},{h_pressure})		!air pressure [mbar]
UVES_AIR(1) = UVES_AIR(1) * mbar2mm			!air pressure [mmHg]
UVES_AIR(2) = m$value({p1},{h_tempcam({PATHID})})	!air temperatur [C]
UVES_AIR(3) = 3.0					!water pressure [mmHg]
							!^fixed (check)
set/format
return {PATHID} {modext}
