! $Id: uves_crea_trap_tab.prg,v 1.2 2001/03/12 06:46:34 amodigli Exp $
!+++ UVES_CREA_TRAP_TAB +++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!
!.IDENT         uves_crea_trap_tab.prg 
!
!.KEYWORDS      spectroscopy, echelle, cleaning
!
!.USAGE         @p (IDENT) in_ima out_tab 
!
!.INPUT         ima      (P1): input frame to be cleaned
!               out table (P2): table bad pix table
!.OUTPUT        bad pixel table (trap.tbl)
!
!.RETURN        Q1:  0: successful return
!                   -1: return failure
!
!.DEFAULT
!
!.PURPOSE       Get table to be used to clean images from trap columns artifacts
!
!.ALGORITHM     According to the bin defines Xs,Ys,Xe,Ye of the traps to be 
!               cleaned (for RED EEV chip). To be used as input of CORRECT/TPIX
!               context: FIND/TPIX, CORRECT/TPIX
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        Andrea Modigliani, ESO-DMO
!
!.VERSION       1.0 2001/02/23  Create
!
!.COMMENT	
!		
!.TODO		
!
!------------------------------------------------------------------------------
!$Name:  $
!$Revision: 1.2 $
!$Log: uves_crea_trap_tab.prg,v $
!Revision 1.2  2001/03/12 06:46:34  amodigli
!typos in header description
!
!Revision 1.1  2001/03/05 06:58:18  amodigli
!Simplified. Now binning treated inside proc.
!
!------------------------------------------------------------------------------
crossref in_ima out_tab
define/param p1 ?          ima "enter name of input image:"
define/param p2 "trap.tbl" tbl "enter name of out tab: "


define/local br1/i/1/4 0 all +lower_levels
define/local br2/i/1/4 0 all +lower_levels
define/local br3/i/1/4 0 all +lower_levels
define/local br4/i/1/4 0 all +lower_levels
define/local br5/i/1/4 0 all +lower_levels
define/local br6/i/1/4 0 all +lower_levels
define/local br7/i/1/4 0 all +lower_levels

define/local bin/c/1/10 " "
define/local binX/i/1/1 0
define/local binY/i/1/1 0
set/format i1
binX = {{p1},{h_xwinbin}}
binY = {{p1},{h_ywinbin}}

!w/o "binX={binX} binY={binY}"
bin = "{binX}x{binY}"
set/format

!-----------------------------
! set the bad raw Xs,Ys,Xe,Ye
!-----------------------------
     if "{bin}" .eq. "1x1" then 
        write/key br1/i/1/4  1,4,2088,4
        write/key br2/i/1/4  1,63,2282,63
        write/key br3/i/1/4  1,108,1778,108
        write/key br4/i/1/4  1,176,2443,176
        write/key br5/i/1/4  1,196,2021,196
        write/key br6/i/1/4  1,285,1975,285
        write/key br7/i/1/4  1,352,1942,352
else if "{bin}" .eq. "1x2" then                 !means sizeX=2,sizeY=1
                                                !X and Y are swapped
        write/key br1/i/1/4  1,4,1044,4
        write/key br2/i/1/4  1,63,1141,63
        write/key br3/i/1/4  1,108,894,108
        write/key br4/i/1/4  1,176,1222,176
        write/key br5/i/1/4  1,196,1011,196
        write/key br6/i/1/4  1,285,988,285
        write/key br7/i/1/4  1,352,971,352
else if "{bin}" .eq. "2x2" then
        write/key br1/i/1/4  1,3,1044,3
        write/key br2/i/1/4  1,33,1141,33
        write/key br3/i/1/4  1,55,894,56
        write/key br4/i/1/4  1,89,1222,90
        write/key br5/i/1/4  1,99,1011,100
        write/key br6/i/1/4  1,144,988,145
        write/key br7/i/1/4  1,177,971,178
else if "{bin}" .eq. "2x3" then            !means sizeX=3,sizeY=2 
                                           !X and Y are swapped
        write/key br1/i/1/4  1,3,696,4
        write/key br2/i/1/4  1,33,761,33
        write/key br3/i/1/4  1,55,592,56
        write/key br4/i/1/4  1,89,814,90
        write/key br5/i/1/4  1,99,674,100
        write/key br6/i/1/4  1,144,658,144
        write/key br7/i/1/4  1,177,647,178
endif
!----------------------------
! creates the bad pixel table
!----------------------------
@p uves_crea_trap_tab,CREA_BAD_PIX_TAB {p2} 

!=================================================================
!=================================================================
ENTRY CREA_BAD_PIX_TAB
!-------------------------------------------------
!creates a given table with 4 columns and 7 raws
!-------------------------------------------------
define/parameter p1 ?       c "Enter Table Name: "

crea/tab {p1}
CREATE/COLU {p1} :XSTARTPIX
CREATE/COLU {p1} :YSTARTPIX
CREATE/COLU {p1} :XENDPIX
CREATE/COLU {p1} :YENDPIX

@p uves_clean_trap,ADD_ROW {p1} 1 {br1(1)} {br1(2)} {br1(3)} {br1(4)} 
@p uves_clean_trap,ADD_ROW {p1} 2 {br2(1)} {br2(2)} {br2(3)} {br2(4)} 
@p uves_clean_trap,ADD_ROW {p1} 3 {br3(1)} {br3(2)} {br3(3)} {br3(4)} 
@p uves_clean_trap,ADD_ROW {p1} 4 {br4(1)} {br4(2)} {br4(3)} {br4(4)} 
@p uves_clean_trap,ADD_ROW {p1} 5 {br5(1)} {br5(2)} {br5(3)} {br5(4)} 
@p uves_clean_trap,ADD_ROW {p1} 6 {br6(1)} {br6(2)} {br6(3)} {br6(4)} 
@p uves_clean_trap,ADD_ROW {p1} 7 {br7(1)} {br7(2)} {br7(3)} {br7(4)} 
return 0
!=================================================================
!=================================================================
ENTRY ADD_ROW
!--------------------------------------------------
! add a raw entry at a given index in a given table
!--------------------------------------------------
define/parameter p1 ? c "Enter Table Name: "
define/parameter p2 ? n "Enter bad raw index (1-7)"
define/parameter p3 ? n "Enter X start Pix: "
define/parameter p4 ? n "Enter Y start Pix: "
define/parameter p5 ? n "Enter X end Pix: "
definE/parameter p6 ? n "Enter Y end Pix: "

define/local x1/i/1/1 {p3}
define/local y1/i/1/1 {p4}
define/local x2/i/1/1 {p5}
define/local y2/i/1/1 {p6}


WRITE/TABLE {p1} :XSTARTPIX @{p2}   {x1} 
WRITE/TABLE {p1} :YSTARTPIX @{p2}   {y1}
WRITE/TABLE {p1} :XENDPIX   @{p2}   {x2}
WRITE/TABLE {p1} :YENDPIX   @{p2}   {y2}

return 0
!=================================================================



