! $Id: uves_expfini.prg,v 1.2 2001/01/04 10:39:51 amodigli Exp $
!+++ UVES_EXPFINI +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT	(C) 1998  European Southern Observatory
!
!.IDENT		uves_expfini.prg
!
!.KEYWORDS	Determine initial parameter set for later use with uves_nonlfit
!
!.USAGE		@p (IDENT) image table partbl
!
!.INPUT		image	(P1): name of data image
!
!.OUTPUT	table	(P2): name of data table
!		partbl	(P3): name of parameter table
!
!		updated parameter and data table
!
!.RETURN	
!
!.DEFAULT
!
!.PURPOSE	Copy the ONE-DIMENSIONAL input data into a data table (P2).
!		Create all columns in P2 needed for the non-linear fit.
!		Create the parameter table and initialize them.
!
!.ALGORITHM	nonlfit performs a non-linear least squares fit.
!		The functions to be fitted are user-defined functions.
!
!		Here the initialization shall be done for a special case:
!                          a*exp(-b*x) + c
!
!		The parameters a, b, and c will be determined by the first
!		guesses:
!
!		c = MEDIAN(y), a = y[1] - c, b = -ln(D)/(x[N] - x[1]);
!		D = acc*y[N]
!
!		(x[],y[]) are the x, and y-coordintes of the data which have
!		to be fitted. acc is a desired accuracy for the first guess.
!		These parameters will be stored in a table which will be
!		created by this procedure. This parameter-table will be the
!		input to the call of nonlfit.
!
!.ENVIRON	UVES, CCDRED context
!   
!.LANGUAGE	MIDAS 98NOV
!
!.AUTHOR	Sebastian Wolf, ESO-DMO
!
!.VERSION	0.9 1998/02/04	Creation
!		1.0 1998/11/25	Add 1.0 to column :X (should start at 1.0)
!.COMMENT	Equidistant X axis is assumed (step size: 1.0).
!		This could be changed in future!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_expfini.prg,v $
! Revision 1.2  2001/01/04 10:39:51  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of data image: "
define/param p2 ? c "Enter name of data table (will be created): "
define/param p3 ? c "Enter name of parameter table: "
!
! local variable declaration
!
define/local num/i/1/1 0
define/local dd/r/1/1 0.0
define/local x1/r/1/1 0.0
define/local xn/r/1/1 0.0
define/local y1/r/1/1 0.0
define/local yn/r/1/1 0.0
define/local ym/r/1/1 0.0
define/local acc/r/1/1 0.001
!
! Create data table
!
copy/it {P1} {P2} :X
name/col {P2} #2 :Y
creat/col {P2} :SIG " "
creat/col {P2} :FIT " " 
creat/col {P2} :RESIDUAL " "
dd = {{P2},#1,1}
comp/tab {P2} :X = :X - {dd} + 1.0
comp/tab {P2} :SIG = 1.0
comp/tab {P2} :FIT = 0.0
comp/tab {P2} :RESIDUAL = 0.0

!
! Create parameter table
!
crea/tab {P3} 3,3
crea/col {P3} :PARAM " "
crea/col {P3} :FIXED " "
crea/col {P3} :RESULT " "

statistic/ima {P1} ? ? ? GN >Null
num = outputi(1)
ym = outputr(8)			!MEDIAN
x1 = m$value({P2},:X,1)
xn = m$value({P2},:X,{num})
y1 = m$value({P2},:Y,1)
yn = m$value({P2},:Y,{num})

{P3},:PARAM,3 = ym
{P3},:PARAM,1 = y1 - ym
{P3},:PARAM,2 = -m$ln( m$abs(yn - ym) / (y1 - ym) ) / (xn - x1)

compute/tab {P3} :FIXED = 0.0		!no parameter is fixed by default
compute/tab {P3} :RESULT = 0.0
read/tab {P3}

write/keyw INPUTI/i/1/8 1,2,3,4,5,1,2,3
return 0

