! $Id: uves_extrord.prg,v 1.2 2001/01/04 10:39:50 amodigli Exp $
!+++ UVES_EXTRORD +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_extrord.prg
!
!.KEYWORDS      keywords
!
!.USAGE         @p (IDENT) sof_in out sof_ref [optpar]
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!               sof_ref (P3): set of reference frames (ascii list)
!				ORDER_TABLE_x and DRS_SETUP_TABLE
!               optpar  (P4): optional parameter
!
!.OUTPUT        out     (P2): path + basename of output files:
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!		Q2: return the classification of the main product
!
!.DEFAULT
!
!.PURPOSE       X
!
!.ALGORITHM     X
!
!.ENVIRON       UVES, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/09  Created.
!                   1999/12/10	return the main product as Q2
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_extrord.prg,v $
! Revision 1.2  2001/01/04 10:39:50  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/param p1 ? c "Enter name of input catalog:"
define/param p2 ? c "Enter name of output catalog:"
define/param p3 ? c "Enter name of reference catalog:"

define/local pid/c/1/10 "extrord" ? +lower_levels

define/local ifrm/c/1/100 " " all
define/local modext/c/1/25 " " ALL
define/local do_clas/c/1/25 " " all
define/local dis/i/1/3 2,3,2
define/local catal/i/1/1 0

if NICE_CREA .eq. "Y" assign/disp d,2 >Null	!select display
						!Check output catalog option
if m$exist(p2) .eq. 0 .AND. p2(1:1) .ne. "+" -
  create/icat {p2} NULL {UVES_IDSTRNG} >Null

set/format I1
ILOOP:
  store/frame ifrm {p1} 1 FINI

  @p uves_extrord,CONFIG {ifrm} {p3}
  if {q1} .lt. 0 return -1
  modext = q2

  assign/disp {dis({pathid})} >Null
  set/eche bkgvisu=NO
  load/eche					!show the order trace

  mess/out 3 {pid} "Do the order extraction for {ifrm}..."
  if MESS_LEVEL .eq. 4 then
    extract/echelle {ifrm} x_{ifrm}
  else
    extract/echelle {ifrm} x_{ifrm} >Null
  endif

  if p2(1:1) .ne. "+" add/icat {p2} x_{ifrm} >Null
  do_clas = "{EXTR_PX}{{ifrm},{UVES_IDSTRNG}}{modext}"
  classify/product x_{ifrm} {do_clas} {d_pre} {d_pho} OK

  if NICE_CREA .eq. "Y" then
    load/ima x_{ifrm} sca=-8,10 cuts=f,2sig
    select/gra {pathid}
    plot/uves x_{ifrm} -2,-2
  endif

  mess/out 3 {pid} "Order extraction done, output: x_{ifrm}."
  goto ILOOP
FINI:
return 0 {do_clas}

!=== SUBROUTINES =============================================================

ENTRY CONFIG
define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of image catalog:"

define/local modext/c/1/25 " " ALL
config/instr {p1}
!get/pathid {p1}				!set global keyword PATHID

search/icat {p2} {DRSRC_ID({PATHID})} NB
if m$strlen(out_a) .eq. 0 then
  append/out "No appropriate DRS setup table found!\n"
  if FORCE_DRS(1:1) .eq. "Y" goto SKIP
  mess/out 2 {pid}
  mess/out 3 {pid} "Use global keyword settings (SHOW/ECHELLE)!"
else
  savini/eche {out_a} READ
endif

search/icat {p2} {ORDER_ID({PATHID})} NB
if m$strlen(out_a) .eq. 0 then
  mess/out 1 {pid} "No appropriate order table found!"
  goto SKIP
else
  set/echelle ORDTAB={out_a}
endif

set/echelle EXTMTD=AVERAGE			!order extraction method

return 0 {modext}

SKIP:
  append/out "--> skip input frame: `{p1}'"
  mess/out 1 {pid}
  return -1
