! $Id: uves_find_hotpix.prg,v 1.1 2001/03/05 06:55:35 amodigli Exp $
!+++ UVES_FIND_TRAP +++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!
!.IDENT         uves_find_hotpix.prg 
!
!.KEYWORDS      spectroscopy, echelle, find trap columns
!
!.USAGE         @p (IDENT) ima 
!
!.INPUT         ima      (P1): input frame to be cleaned
!               tbl      (P2): output table with hot pixels (hp.tbl)
!    
!.OUTPUT        hot pixel table (hp.tbl)
!
!.RETURN        Q1:  0: successful return
!                   -1: return failure
!
!.DEFAULT
!
!.PURPOSE       Identify hot pixels artifacts in a frame
!
!.ALGORITHM     Uses statist/ima and find/pix to identify hot pixels
!               These are saved in a table hp.tbl
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        Andrea Modigliani, ESO-DMO
!
!.VERSION       1.0 2001/02/23  Create
!
!.COMMENT	
!		
!.TODO		
!
!------------------------------------------------------------------------------
!$Name:  $
!$Revision: 1.1 $
!$Log: uves_find_hotpix.prg,v $
!Revision 1.1  2001/03/05 06:55:35  amodigli
!created
!
!Revision 1.1  2001/03/01 09:16:39  amodigli
!First creation TBD
!
!------------------------------------------------------------------------------
crossref ima_in tbl_out
define/param p1 ?          ima "enter name of input image:"
define/param p2 "hp.tbl" tbl "enter name of output table:"

define/local infrm/c/1/80 {p1}       !Input image frame
define/local outTab/c/1/80 {p2}      !output hotpix table

!======================
! calculates datamedia
!======================
define/local nx_pix/i/1/1 0          !No of X pix
define/local ny_pix/i/1/1 0          !No of Y pix
define/local sx_pix/i/1/1 0          !Size of X bin in pix
define/local sy_pix/i/1/1 0          !Size of Y bin in pix

define_local sample_x/i/1/1 100      !X size of sampling window in pix
define_local sample_y/i/1/1 100      !Y size of sampling window in pix
define/local x_cent_s/i/1/1 0        !X sampling window starting point
define/local x_cent_e/i/1/1 0        !X sampling window ending point
define/local y_cent_s/i/1/1 0        !Y sampling window starting point
define/local y_cent_e/i/1/1 0        !Y sampling window ending point

define/local upp_cut/r/1/1 10.0      !upper fct to define upp_threshold
define/local low_cut/r/1/1 3.0       !lower fct to define low_threshold
define/local upp_treshold/r/1/1 0.0  !upper threshold to define hotpix
define/local low_treshold/r/1/1 0.0  !lower threshold to define hotpix

nx_pix = m$value({infrm},NPIX(1))
ny_pix = m$value({infrm},NPIX(2))

sx_pix = m$value({infrm},{h_xwinbin})
sy_pix = m$value({infrm},{h_ywinbin})

x_cent_s = (nx_pix - sample_x)/2
x_cent_e = (nx_pix + sample_x)/2
y_cent_s = (ny_pix - sample_y)/2
y_cent_e = (ny_pix + sample_y)/2

stat/ima {infrm} [{x_cent_s},{y_cent_s}:{x_cent_e},{y_cent_e}] {SESSOUTV}
upp_treshold = outputr(3)+ upp_cut*outputr(4)
low_treshold = outputr(3)- low_cut*outputr(4)

find/pix {infrm} {low_treshold},{upp_treshold} out A {outTab} 100000 

return 0
!=============================================================================
