! $Id: uves_finish.prg,v 1.4 2001/06/20 14:21:15 amodigli Exp $
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       uves_finish.prg
!
!.KEYWORDS    Pipeline shutdown
!
!.USAGE       finish/pipeline  catalog frm_set ctx
!
!.INPUT       catalog     (P1)    Catalog of result frames
!   	      frm_set     (P2)    Set of output frames    
!   	      ctx         (P3)    Context identifier
!             main        (P4)    Main pipeline product identifier
!
!.OUTPUT      None
!
!.RETURNS     The procedure returns 0 on success, otherwise 1. The keyword
!             Q1 is updated with the exit status.
!
!.PURPOSE     The procedure disables the loaded context and converts the
!             output frames from MIDAS formats to FITS format. The names of
!             the result frames in the current working directory are taken
!             from 'catalog'. The names of the output FITS frames are
!             automatically generated. If the output set of frames is a comma
!             seperated list of MIDAS catalogs (i.e. the suffix '.cat' must be
!             present) and if each name of this list starts with the capital
!             letters 'XSF', the procedure assumes to run within a recipe
!             queue and copies (appends) the entries from the input catalog to
!             each catalog given in the list. In this case the environmet is
!             kept (i.e. no contexts can be deleted).        
!
!             If the name of the input catalog is a '+' any output is
!             suppressed, i.e. neither FITS files nor catalogs will be
!             produced or updated respectively. This is useful to just delete
!             some contexts. All contexts may be removed from the environment
!             by the context identifier '-all'.
!
!.ENVIRON     UVES context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa & S. Wolf, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!	      1.0  1999/03/11:	Copy from FORS context
!		   1999/07/22:	Save postscript files if NICE_HCOPY=YES
!		   1999/09/06:	uves_updtbl is obsolete using from now as
!				98NOVpl2.2 has fixed the bug in necbackprep.prg
!
!------------------------------------------------------------------------------
! $Log: uves_finish.prg,v $
! Revision 1.4  2001/06/20 14:21:15  amodigli
! Using QC1 LOG and PAF files
!
! Revision 1.3  2001/05/02 15:00:45  amodigli
! Added entry (still disactivated) to support
! logging of QC info on FITS and ops log files
! with a common procedure
!
! Revision 1.2  2001/01/04 10:39:50  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref catname frames context

define/param p1  ?      c    "Catalog of result frames:"
define/param p2  ?      c    "Basename of output frame(s):"
define/param p3  +      c    "Context identifier:"
define/param p4  +      c    "Main pipeline product:"

define/maxpar 4


! Procedure identification
define/local procID/c/1/80	"FinishPipe"


! Local constants
define/local listfile/c/1/11  "product.lst"


! Local variables
define/local pos/i/1/1     0
define/local i/i/1/1       0
define/local nctx/i/1/1    0
define/local nout/i/1/1    0
define/local errsts/i/1/1  0
define/local ctx/c/1/100   {p3}



ctx = m$lower(p3)

set/format i2
pos = m$index(p2,".cat")

if pos .ge. 1 then
  nout = m$parse(p2,"cout")

  do i = 1 nout 1
    pos = m$indexb(cout{i}, ".cat")

    if cout{i}(1:3) .ne. "XSF" .or. pos .le. 1 then
      mess/out 1 {procID} "Invalid file name {cout{i}}!"
      return 1
    else
      if p1(1:1) .ne. "+" then
        update/catalog {p1} {cout{i}}
        mess/out 4 {procID} "SUPER MODE: Adding output frames to {cout{i}}"
      endif
    endif
  enddo


! Check if context shall be disabled
  if p_ctxdel .ne. 0 then

    nctx = m$parse(ctx,"ctx")

    do i = 1 nctx 1
      clear/context {ctx{i}}
    enddo

    set/ctxdel reset
    mess/out 3 {procID} "Context(s) {ctx} removed from environment."
  endif  

! Close internal logging facility
  shortlog/out -1 {p4}

else  

! Close internal logging facility and write shortlog to HISTORY buffer
  shortlog/out 0 {p1}

!  @p uves_finish,UPDBAKTAB {p1}
  if p1(1:1) .ne. "+" then
    put/frames {p1} {p2} {p4} {listfile}
    errsts = {q1}
  endif

! save postscript files
  if nice_hcopy(1:1) .eq. "Y" then
    @p uves_finish,SAVEPS {p2}
    mess/out 4 {procID} "{q1} POSTSCRIPT files saved."
  endif

! Show short-log
  shortlog/out -2

!am 02/05/01 Added but temporally commented out (still supported old Logging system)
! Create QC1 Log
  qc1log/out 0 

  if ctx .eq. "-all" then

    mess/out 2 {procID} "All contexts will be disabled!"
    clear/context -all >Null

  elseif ctx(1:1) .ne. "+" then

    nctx = m$parse(ctx,"ctx")

    do i = 1 nctx 1
      clear/context {ctx{i}}
    enddo

  endif
endif

set/format

! Account for errors not yet handled
if errsts .ne. 0 then
  return 1
endif

return 0



!=== SUBROUTINES ==============================================================
ENTRY UPDBAKTAB

define/param p1 ? c "Enter input image catalog:"

define/local i1/i/1/1 1

do i1 = 1 3
  search/icat {p1} {BACK_ID({i1})} >Null
  if m$strlen(out_a) .gt. 0 @p uves_updtbl S {out_a}
enddo

!..............................................................................
ENTRY SAVEPS
define/param p1 ? c "Enter output path + basename:"

define/local psfile/c/1/100 " " ALL
define/local catal/i/1/1 0
define/local cnt/i/1/1 0

create/acat ps.cat *.ps >Null

psbeg:
  store/frame psfile ps.cat 1 psend
  -rename {psfile} {p1}_{cnt}.ps
  cnt = cnt + 1
  goto psbeg
psend:
-delete ps.cat
return {cnt}
