! $Id: uves_fmtstab.prg,v 1.21 2001/09/08 15:29:13 amodigli Exp $
!+++ UVES_FMTSTAB +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_fmtstab.prg
!
!.KEYWORDS      uves physical model, spectroscopy, echelle,
!
!.USAGE         @p (IDENT) wlc lincat
!
!.INPUT         wlc     (P1): ThAr format check frame
!				        These are ThAr exposures obtained with a small
!				        slit length.
!			            (P2): reference input catalog
!			            (P3): catalog to save product
!			            (P4): Dimension of measurement box (X,Y)
!			            (P5): Translation of the ccd (X,Y)
!			            (P6): Angle offsets (echelle, CD, CCD-rot)
!			            (P7): Measurement position offsets (X,Y) 
!				              from the predicted position.
!		        disp,fitreg (p8): display flag, fits regression, COMPUTE/REGR:
!				                  YES: Show result on graphics window
!				                  NO:  Creates no plot.
!				                  YES: Fits the residual differences and builds 
!				                       the order tables and line tables.
!				                  NO:  No fitting and produces no tables.
!				                  YES: Compute regression.
!				                  NO:  No Compute regression.
!
!.OUTPUT        line table [LINE_TABLE_x] which may be used as first guess
!		        DRS table  [DRS_SETUP_x], order table [ORDER_TABLE_x], and
!		        background table [BACKGR_TABLE_x] (x: BLUE, REDL, REDU).
!
!.RETURN        Q1:  0: successful return
! 		            -1: a successful return failed
!		        Q2: return the classification of the main product
!
!.DEFAULT	P1	{wlc}
!		P2	+
!		P3	+ 
!		P4	40,40
!		P5	0.,0.
!		P6	0.,0.,0.
!		P7	0.,0.
!		p8	Y,Y,Y
!
!.PURPOSE: 	Create a line table based on the physical model of UVES.
!
!.ALGORITHM	Define global echelle keywords before calling 
!		    the subroutines: "uves_phymod.prg", "uves_msrawxy.prg", 
!		    "uves_plotmod.prg", "uves_calmap.prg", 
!		    do classification of the products and store them in
!		    DB catalogue.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        O.Boitquin, ESO/DMD
!
!.VERSION       1999/09/01  Created (OB).
!		        1999/09/30-SW  Parameter P4 changed to P7, shift former
!				               P5..P7 to P4..P6.
!				               This is more consistent with the call to
!				               uves_physmod.prg (see also COMMENT).
!		        1999/10/01-OB  Header with angle offsets (echelle, CD, CCD rot)
!               1999/10/04-OB  Add usefull flags P8 to fit the physical mod. 
!		        1999/10/04-SW  Create the output catalog if it does not exist!
!		        1999/11/25-SW  procedure header made more infomative.
!		        1999/11/26-SW  save number of orders in DRS table.
!		        1999/11/29-SW  write DEFPOL, WIDTHI and THRESI to DRS table.
!		        1999/12/07-SW  product header: ESO descr. were missing!
!               1999/12/10-SW  return the main product as Q2
!               1999/12/16-SW  The binning has to be considered for MBOX !
!			                   Improved NBORDI determination - select central
!		        2000/02/22-OB  Compute regression 
!				               band.
!               2000/03/19-AK  Andreas changes: stability check + qc1 output added. 
!               2000/04/07-OB  renames the uves_predict to uves_fmtstab.
!                              Separated Andreas changes moved into the new file 
!                              uves_fmtstab.prg.
!
!.COMMENT   Parameters P4, P5, and P6 are the parameters used by the
!           physical model routine uves_physmod.prg.
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.21 $
! $Log: uves_fmtstab.prg,v $
! Revision 1.21  2001/09/08 15:29:13  amodigli
! PIPEFILE->h_pipefile
!
! Revision 1.20  2001/08/12 16:14:48  amodigli
! Passing p_version as string par
!
! Revision 1.19  2001/08/07 16:34:28  amodigli
! Used format f8.4 to log QC1 key.
! Error in DIFFXMED value fixed
! Added QC ORDMIN/ORDMAX QC WLENMIN/WLENMAX
!
! Revision 1.18  2001/07/16 07:57:13  amodigli
! Added prefix QC to DET CHIPi NAME
!
! Revision 1.17  2001/07/15 14:18:56  amodigli
! Eliminated unnecessary call to uves_getpversion.prg
!
! Revision 1.16  2001/07/04 07:51:52  amodigli
! New QC1 nam conventions. Some additions/corrections
!
! Revision 1.15  2001/06/20 14:21:16  amodigli
! Using QC1 LOG and PAF files
!
! Revision 1.14  2001/06/16 06:59:17  amodigli
! Changing QC-Naming conventions
!
! Revision 1.13  2001/05/23 08:47:07  amodigli
! Fixed portability bug to alpha:
! without selecting out undefined entry in
! LINTAB, when we convert it in an ima to
! apply stat/ima and get the median value,
! on alpha platforms stat/ima can crash
! due to -Inf generated by copy/ti
!
! Revision 1.12  2001/05/04 13:58:07  amodigli
! Clla orderpos with 2 par (passing DRSTAB
! and input raw frame to get the binning
! used for new bkg parameter setting)
! Added help to QC descriptors
!
! Revision 1.11  2001/02/09 17:42:27  amodigli
! Added PRO QC asked from DFO
!
! Revision 1.10  2001/01/04 10:39:55  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!

DEFINE/PARAM P1 ? 	    IMA    "Input frame:"
define/param P2 +           CHAR   "Enter name REFERENCE Catalogue"
define/param P3 +           CHAR   "Enter catalog name to save product"
define/param P4 40,40       NUMB   "Dimension of measurement box (X,Y): "
define/param P5 0,0         NUMB   "Translation of the ccd (X,Y): "
define/param P6 0,0,0       NUMB   "Angle offsets (echelle, CD, CCD-rot):"
define/param P7 0,0         NUMB   "Measurement position offsets (X,Y): "
define/param P8 Y,Y,Y       CHAR   "Displays results, fits regression, COMPUTE/REGR flags: Y or N ?"

define/local XYOFF/R/1/2  {P7}		!never used !!?? (SW)
define/local MBOX/R/1/2   {P4}
define/local trans/D/1/2  {p5}
!define/local angoff/D/1/3 {P6}

define/local stabflag/c/1/1 0		! stability test flag 
define/local wcen/R/1/1   0.0
define/local Modext/c/1/25 " " ALL +lower_levels
define/local mode/c/1/25 " " ALL

define/local K/I/1/1  0
define/local disp/C/1/3 " " ALL
define/local fitreg/C/1/3 " " ALL
define/local reg_opt/C/1/3 " " ALL
define/local stab_ref_file/c/1/80 " " ALL

! For messout
define/local pid/C/1/20 "uves_fmtstab" ? +lower_levels

! For stabcheck
define/local modtab/C/1/25 "mline"


						!Check if output catalog exist
if m$exist(p3) .eq. 0 .AND. p3(1:1) .ne. "+" -
  create/icat {p3} NULL {UVES_IDSTRNG} >Null

set/format F4.0

@p uves_fmtstab,CONFIG {p2} {p1}  ! set LINCAT and get master formatchecks
stabflag = q1	
if stabflag .eq. "1" stab_ref_file = "{q2}"


wcen = WCENTER/10.0
set/echelle wlc={P1} 
set/echelle ordtab=o{wcen}{modext(2:)}.tbl baktab=b{wcen}{modext(2:)}.tbl
set/echelle lintab=l{wcen}{modext(2:)}.tbl
mess/out 3 {pid} "Start prediction of {p1} ..."
mess/out 3 {pid} "Use line reference list: {LINCAT}"

! Passing from pix to bin

mbox(1) = mbox(1) / CCDBIN(1)                ! Dimension of measurement box (x)
mbox(2) = mbox(2) / CCDBIN(2)                ! Dimension of measurement box (y)

set/format F5.1
if NICE_CREA(1:1) .eq. "Y" then
  assign/disp 1
  load/image {wlc} scale=f cen=c cuts=f,ihap
endif

k = m$parse(p8,"flag")
if k .gt. 2 then
  disp = "{flag01}"
  fitreg = "{flag02}"
  reg_opt = "{flag03}"
else if k .gt. 1 then
  disp = "{flag01}"
  fitreg = "{flag02}"
  reg_opt = "YES"
else if k .eq. 1 then
  disp = "{flag01}"
  fitreg = "YES"
  reg_opt = "YES"
endif
disp = m$upper(disp)
fitreg = m$upper(fitreg)
reg_opt = m$upper(reg_opt)

 
! Runs the physical model and creates the line table
@p uves_phymod.prg -
   {wlc} {lincat} {mbox(1)},{mbox(2)} {trans(1)},{trans(2)} {P6} {reg_opt}

if Q1 .eq. "1" then
  mess/out 1 {pid} "Error occuring in the physical model. Abort..."
  return -1
endif

! Mesures the positions of lineswith and matches with the positions predicted 
@p uves_msrawxy.prg  {wlc}  p4={NICE_CREA(1:1)} p7={mbox(1)},{mbox(2)} p8={fitreg} !p5={XYOFF(1)},{XYOFF(2)}



! Displays the differences between projection and measurements
if disp(1:1) .eq. "Y" .AND. NICE_CREA(1:1) .eq. "Y" then 
  @p uves_plotmod.prg rline {wlc} {trans(1)},{trans(2)} {P6}
  modify/graph W
  copy/grap g,3 >Null
endif

if fitreg(1:1) .eq. "Y" then
  ! Builds the order and line tables
  @p uves_calmap.prg

!==========================================
!am added for reordering :XDIF :YDIF
!
! Save :XDIF :YDIF  into {LINCAT}

  define/local qc1pm/c/1/60 {LINTAB} ? +lower_levels
  define/local rlinetab/c/1/60 rline.tbl ? +lower_levels

  write/key qc1pm {LINTAB}
  write/key rlinetab rline.tbl
  run PIPE_EXE:uves_qc1pmtbl.exe
!==========================================

  copy/dd {WLC} *,3 {LINTAB}			!copy the descriptors
  copy/dd {WLC} *,3 {ORDTAB}			!of input frame
  copy/dd {WLC} *,3 {BAKTAB}			!to the products.

  DRSTAB = "drs_setup{modext}.tbl"		!name of setup table
  @p uves_ordpos,SETDRS {DRSTAB} {p1}	!create DRS setup table

  define/local xbox/I/1/2 0,0
  define/local ybox/I/1/2 0,0
  xbox(1) = imsize(1)/2 - imsize(1)*0.05
  xbox(2) = imsize(1)/2 + imsize(1)*0.05
  ybox(1) = 50.0
  ybox(2) = imsize(2) - 50.0
  select/table {ORDTAB} :X.GT.{xbox(1)}.AND.:X.LT.{xbox(2)}
  stat/table {ORDTAB} :ORDER >Null
  outputi(1) = outputr(2) - outputr(1) + 1
  write/out "Found {outputi(1)} orders"  
  write/desc {DRSTAB} NBORDI {outputi(1)}	!save number of detected orders
  select/tab {ORDTAB} all
    
  write/desc {DRSTAB} DEFPOL 2,3		!order defin. polynom. degrees
  write/desc {DRSTAB} WIDTHI 0			!order width (automatic if 0)
  write/desc {DRSTAB} THRESI 0			!threshold for following order
  classify/product {DRSTAB} {DRSRC_ID({PATHID})} {d_pre} {d_pho} OK
  classify/product {ORDTAB} {ORDER_ID({PATHID})} {d_pre} {d_pho} OK
  classify/product {BAKTAB}  {BACK_ID({PATHID})}  {d_pre} {d_pho} OK
  classify/product {LINTAB} {LINTAB_ID({PATHID})} {d_pre} {d_pho} OK

  if p3(1:1) .ne. "+" then
    add/icat {p3} {drstab} >Null
    add/icat {p3} {ordtab} >Null
    add/icat {p3} {baktab} >Null
    add/icat {p3} {lintab} >Null
  endif
endif


!============================================================================
! AMO: 25/07/00 Addition to monitor QC1 quality of physical model predictions
!============================================================================
define/local date/c/1/30 " " all +lower_levels
define/local fname/c/1/30 " " all +lower_levels
define/local qc1_nall/i/1/1 0
define/local qc1_nsel/i/1/1 0
define/local qc1_dx_s/d/1/1 0.0
define/local qc1_dy_s/d/1/1 0.0
define/local qc1_dx_m/d/1/1 0.0
define/local qc1_dy_m/d/1/1 0.0
define/local qc1_dx_v/d/1/1 0.0
define/local qc1_dy_v/d/1/1 0.0
define/local qc1_wlen_min/d/1/1 0.0
define/local qc1_wlen_max/d/1/1 0.0
define/local qc1_nord_min/i/1/1 0
define/local qc1_nord_max/i/1/1 0

if m$existd(p1,h_origname) .eq. 1 then
  fname = "{{p1},{h_origname}}"
else
  fname = "{{p1},{h_obsname}}_{{p1},{h_obstpno}}"
endif

date   = "{{P1},ESO.TPL.START}"

set/format i1 f8.4,f8.4
w/o "starting the selection..."
select/table {LINTAB} ALL  {SESSOUTV}
qc1_nall = {outputi(1)}


select/table {LINTAB} :SELPLOT.eq.1  {SESSOUTV}
qc1_nsel = {outputi(1)}

stat/tab {LINTAB} :XDIF  {SESSOUTV}
qc1_dx_s = {outputr(4)}
qc1_dx_m = {outputr(3)}

stat/tab {LINTAB} :YDIF  {SESSOUTV}
qc1_dy_s = {outputr(4)}
qc1_dy_m = {outputr(3)}

stat/tab {LINTAB} :WAVEC {SESSOUTV}
!NB we divide by ten as LINTAB contains values in A 
!   and DICB want QC LOG in nm
qc1_wlen_min = {outputr(1)}/10.  
qc1_wlen_max = {outputr(2)}/10.


stat/tab {LINTAB} :ORDER {SESSOUTV}
qc1_nord_min = {outputr(1)}
qc1_nord_max = {outputr(2)}

select/table {LINTAB} ALL  {SESSOUTV}

define/local qc1_wmin_det/d/1/1 0.0
define/local qc1_wmax_det/d/1/1 0.0
define/local qc1_nomin_det/i/1/1 0
define/local qc1_nomax_det/i/1/1 0

stat/tab {LINTAB} :WAVEC {SESSOUTV}
!NB we divide by ten as LINTAB contains values in A 
!   and DICB want QC LOG in nm
qc1_wmin_det = {outputr(1)}/10.  
qc1_wmax_det = {outputr(2)}/10.


stat/tab {ORDTAB} :ORDER {SESSOUTV}
qc1_nomin_det = {outputr(1)}
qc1_nomax_det = {outputr(2)}
set/format
!========================================================================
! AMO 26/07/00 To find out also the median I have to transform the table
!             in an ima and use statistic/ima
!========================================================================
! to remove undefined entries from the table before conversion in image
! which could lead to -INF or +INF ima points on alpha machines on which
! the following stat/ima would fail we do a preliminary selection of all 
! the defined points. The result of stat/ima for the median value will
! not change.

select/table {LINTAB} :SELPLOT.EQ.1 {SESSOUTV}
copy/ti {LINTAB} midtmp_lintab.bdf

set/refc {LINTAB} :XDIF
inputi = {{LINTAB},TBLCONTR(5)}
statistic/image midtmp_lintab.bdf [<,@{inputi}:>,@{inputi}] ? ? xn >Null
qc1_dx_v = {outputr(8)}

set/refc {LINTAB} :YDIF
inputi = {{LINTAB},TBLCONTR(5)}
statistic/image midtmp_lintab.bdf [<,@{inputi}:>,@{inputi}] ? ? xn >Null
qc1_dy_v = {outputr(8)}

set/refc {LINTAB} seq
-delete midtmp_lintab.bdf
!========================================================================
! AMO: 25/07/00 END
!========================================================================

!
! output to the logging system and FITS file
!
set/format F13.6
!

!am 02/05/01 QC1LOG/OUT command temporally disactivated
!            still using old stile of Logs.

define/local dpr_type/c/1/80 " " all
define/local tpl_id/c/1/80 " " all
define/local arcfile/c/1/80 " " all
define/local pro_catg/c/1/80 " " all

qc1log/out 2 
set/format i1 f8.4,f8.4
qc1log/out 1 {LINTAB} "{h_qc_did}"  "{d_qc_did}"                "Data Dictionary for QC"
qc1log/out 1 {LINTAB} "PRO.REC1.PIPE.ID" "{p_version}" "Pipeline (unique) identifier" PRO
qc1log/out 1 {LINTAB} "QC.TEST1.ID" "Physical-Model-Prediction-Results"  "Name of QC Test"
qc1log/out 1 {LINTAB} "QC.MODEL.ID" "UVES_phys_mod/1.1.0"       "Physical Model Id"
qc1log/out 1 {LINTAB} "QC.MODEL.DATE" "2000:03:18T00:00:00.000" "Physical Model Parameters Last Change"

if m$existd(p1,h_dprtype) .eq. 1 then
  dpr_type = "{{p1},{h_dprtype}}"
  qc1log/out 1 {LINTAB} {h_dprtype} "{dpr_type}" + DPR
endif
if m$existd(p1,h_tpid) .eq. 1 then
  tpl_id = "{{p1},{h_tpid}}"
  qc1log/out 1 {LINTAB} {h_tpid} "{tpl_id}" + TPL
endif
if m$existd(p1,h_arcfile) .eq. 1 then
  arcfile = "{{p1},{h_arcfile}}"
  qc1log/out 1 {LINTAB} {h_arcfile} {arcfile} + PRIMARY-FITS
endif
if m$existd(LINTAB,h_procatg) .eq. 1 then
  pro_catg = "{{LINTAB},{h_procatg}}"
  qc1log/out 1 {LINTAB} {h_procatg} {pro_catg} + PRO
endif
qc1log/out 1 {LINTAB} {h_pipefile} {LINTAB} + PRO



set/format i1 f8.4,f8.4
qc1log/out 1 {LINTAB} "QC.MODEL.NLINALL" {qc1_nall}     "Number of predicted lines"               
qc1log/out 1 {LINTAB} "QC.MODEL.NLINSEL" {qc1_nsel}     "Number of lines selected"              
qc1log/out 1 {LINTAB} "QC.MODEL.ORDMIN" {qc1_nord_min}  "minimum predicted order value"             
qc1log/out 1 {LINTAB} "QC.MODEL.ORDMAX" {qc1_nord_max}  "maximum predicted order value"              
qc1log/out 1 {LINTAB} "QC.MODEL.WLENMIN" {qc1_wlen_min} "minimum predicted wavelength value"             
qc1log/out 1 {LINTAB} "QC.MODEL.WLENMAX" {qc1_wlen_max} "maximum predicted wavelength value"               
qc1log/out 1 {LINTAB} "QC.MODEL.DIFFXRMS"  {qc1_dx_s}   "Std dev of X difference to physical model"             
qc1log/out 1 {LINTAB} "QC.MODEL.DIFFXAVG"  {qc1_dx_m}   "Average of X difference to physical model"              
qc1log/out 1 {LINTAB} "QC.MODEL.DIFFXMED"  {qc1_dx_v}   "Median of X difference to physical model"             
qc1log/out 1 {LINTAB} "QC.MODEL.DIFFYRMS"  {qc1_dy_s}   "Std dev of Y difference to physical model"            
qc1log/out 1 {LINTAB} "QC.MODEL.DIFFYAVG"  {qc1_dy_m}   "Average of Y difference to physical model"             
qc1log/out 1 {LINTAB} "QC.MODEL.DIFFYMED"  {qc1_dy_v}   "Median of Y difference to physical model"     
qc1log/out 1 {LINTAB} "QC.WLENMIN"         {qc1_wmin_det}   "minimum wavelength"     
qc1log/out 1 {LINTAB} "QC.WLENMAX"         {qc1_wmax_det}   "maximum wavelength"     
qc1log/out 1 {LINTAB} "QC.ORDMIN"          {qc1_nomin_det}   "minimum order number detected"     
qc1log/out 1 {LINTAB} "QC.ORDMAX"          {qc1_nomax_det}   "maximum order number detected"     
qc1log/out 3
set/format
!============================================================================
! AMO: 25/07/00  END
!============================================================================

if stabflag .eq. "1" then
 mess/out 3 {pid} "Stability check for {p1} ..."
! goto CONT
 !
 ! save the table with the measured positions of the input frame
 !
 -copy {modtab}.tbl {modtab}m.tbl
 !
 ! Mesures the positions of lineswith and matches with the positions predicted 
 ! now for the reference frame
 !
 @p uves_msrawxy.prg fmtMASTER  p4={NICE_CREA(1:1)} p7={mbox(1)},{mbox(2)} p8=N
 !
 ! save the table with the measured positions of the reference frame
 !
 -copy {modtab}.tbl {modtab}r.tbl
 !
 @p uves_fmtstab,STABCHECK {p1} {modtab}m {modtab}r "{stab_ref_file}"
 !
 CONT:
endif

return 0 {LINTAB_ID({PATHID})}


!=== SUBROUTINES ==============================================================

ENTRY CONFIG
define/param p1 ? c "Enter name of image catalog:"
define/param p2 ? c "Enter name of input image:"

define/local arcfile/c/1/80 " " ALL

config/instr {p2}				!Set global keyword PATHID, etc
Modext = q2					!Mode extension (REDU,REDL,..)
						!
search/icat {p1} {LINCAT_ID} NB			!get reference line catalog
if m$strlen(out_a) .eq. 0 then
  append/out "No line reference table found!\n"
  goto SKIP
else
  set/echelle LINCAT={out_a}
endif

search/icat {p1} {MFORM_ID({PATHID})} NB	!get first master formatcheck
 if m$strlen(out_a) .eq. 0 then		
   append/out "No appropriate master format check frame found!\n"
   append/out "-> no stability check will be performed!\n"
   mess/out 2 {pid}
   return 0 
 else
   -copy {out_a} fmtMASTER.bdf
   append/out "Master formatcheck found!\n"
 endif

mess/out 3 {pid}

    if m$existd(out_a,"ESO.PRO.QC.ARCFILE") .eq. 1 then 
       arcfile = "{{out_a},ESO.PRO.QC.ARCFILE}"    
elseif m$existd(out_a,"ARCFILE") .eq. 1 then
        arcfile = "{{out_a},ARCFILE}"
else  
        arcfile = "uknown"
endif

return 1 {arcfile}


SKIP:
  append/out "--> skip input frame: `{p1}'"
  mess/out 1 {pid}
  return -1



ENTRY STABCHECK
!
define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter name of measured position table:"
define/param p3 ? c "Enter name of reference position table:"
define/param p4 ? c "Enter name of reference frame:"


define/local pid/c/1/15 "uves_stabcheck"
define/local med/i/1/1 0

define/local wlen/r/1/1 0.0
define/local temp/r/1/1 0.0
define/local press/r/1/1 0.0

define/local MeanDX/r/1/1 0.0
define/local MeanDY/r/1/1 0.0
define/local MedDX/r/1/1 0.0
define/local MedDY/r/1/1 0.0
define/local stab_ref_frame/c/1/80 "{p4}"

set/format
!
comp/tab {p2} :X = :XMES
comp/tab {p2} :Y = :YMES
comp/tab {p2} :S = :STATUS
!
comp/tab {p3} :X = :XMES
comp/tab {p3} :Y = :YMES
comp/tab {p3} :S = :STATUS
!
! merge columns of the two tables
!
copy/tt  {p2} :X    {p3}  :Xm	
copy/tt  {p2} :Y    {p3}  :Ym	
copy/tt  {p2} :WAVE {p3}  :WAVEm	
copy/tt  {p2} :S    {p3}  :Sm	
!
! compute the shifts now
!
comp/tab {p3} :DX = :Xm-:X
comp/tab {p3} :DY = :Ym-:Y
comp/tab {p3} :DW = ABS(:WAVEm-:WAVE)  
sele/tab {p3} :DW.lt.0.001.AND.Sm.EQ.0.AND.S.EQ.0  {SESSOUTV}
copy/tab {p3} &REF
!    assign/gra G,0
!    set/gra 
!    plot/tab &REF :Y :DX
!    wait 1	
!    plot/tab &REF :X :DY
!      
! do the statistics and save results to {out}
stat/tab &REF :DX	{SESSOUTV}
med = outputi(2)/2
MeanDX = outputr(3)    
sort/tab &REF :DX     
MedDX = {&REF,:DX,@{med}}   
!
stat/tab &REF :DY   {SESSOUTV}
MeanDY = outputr(3)    
sort/tab &REF :DY     
MedDY = {&REF,:DY,@{med}}   
!   	
sele/tab {p3} all
!
get/pathid {p1}

press  = m$value({p1},{h_pressure})
wlen   = m$value({P1},{h_cwlen({PATHID})})	
temp   = m$value({P1},{h_tempcam({PATHID})})
! for simplicity use start time of template
! start time (date) set up in main program

! date needs a ':' instead of 'T' as ususal ...
! am: no it has to be with T to properly work for QC1LogWriter
!@p uves_fmtstab,REPL_CH T {date} :
!date = outputc
!w/o "date={date}"
!inquire/key outputc break
 
!
! Log data 
!
  set/format F7.3
  append/out  "SUMMARY STABILITY CHECK: {p1}   \n"
  append/out  "average shift in X: {MeanDX} pix\n"
  append/out  "median  shift in X: {MedDX} pix \n"
  append/out  "average shift in Y: {MeanDY} pix\n"
  append/out  "median  shift in Y: {MedDY} pix \n"
  mess/out 3 {pid}
!
! output to the logging system and FITS
!
set/format F13.6
!
define/local pn/i/1/1 0
set/format i1
pn = {uves_portid({PATHID})}
set/format 


define/local dpr_type/c/1/80 " " all
define/local tpl_id/c/1/80 " " all
define/local arcfile/c/1/80 " " all
define/local pro_catg/c/1/80 " " all

qc1log/out 2 
set/format i1 f8.4,f8.4
qc1log/out 1 {LINTAB} "{h_qc_did}"  "{d_qc_did}"     "Data Dictionary for QC" 
qc1log/out 1 {LINTAB} "PRO.REC1.PIPE.ID" "{p_version}" "Pipeline (unique) identifier" PRO
qc1log/out 1 {LINTAB} "QC.TEST2.ID" "Stability-Check-Results" "Name of QC Test"    
qc1log/out 1 {LINTAB} "QC.MODEL.ID" "UVES_phys_mod/1.1.0"       "Physical Model Id"
qc1log/out 1 {LINTAB} "QC.MODEL.DATE" "2000:03:18T00:00:00.000" "Physical Model Parameters Last Change"

if m$existd(p1,h_dprtype) .eq. 1 then
  dpr_type = "{{p1},{h_dprtype}}"
  qc1log/out 1 {LINTAB} {h_dprtype} "{dpr_type}" + DPR
endif
if m$existd(p1,h_tpid) .eq. 1 then
  tpl_id = "{{p1},{h_tpid}}"
  qc1log/out 1 {LINTAB} {h_tpid} "{tpl_id}" + TPL
endif
if m$existd(p1,h_arcfile) .eq. 1 then
  arcfile = "{{p1},{h_arcfile}}" 
  qc1log/out 1 {LINTAB} {h_arcfile} {arcfile} + PRIMARY-FITS
endif 
if m$existd(p1,h_procatg) .eq. 1 then
  pro_catg = "{{LINTAB},{h_procatg}}"
  qc1log/out 1 {LINTAB} {h_procatg} {pro_catg} + PRO
endif
qc1log/out 1 {LINTAB} {h_pipefile} {p1} + PRO
set/format i1 f8.4,f8.4
!qc1log/out 1 {LINTAB} "ESO.INS.MODE"       {MODEXT(2:5)}                 "Instrument mode used"      UVES_ICS
 
 
if m$existd(p1,"{h_inspath}") .eq. 1 then 
   qc1log/out 1 {LINTAB} {h_inspath} {{p1},{h_inspath}}                 "Optical path used."         UVES_ICS
else
   mess/out 2 {pid} "Missing descriptor {h_inspath}"
endif
if m$existd(p1,"{h_insmode}") .eq. 1 then 
   qc1log/out 1 {LINTAB} {h_insmode} {{p1},{h_insmode}}                 "Instrument mode used."      UVES_ICS  
else
   mess/out 2 {pid} "Missing descriptor {h_insmode}"
endif
              
qc1log/out 1 {LINTAB} "{h_namecros({PATHID})(5:18)}"   {CROSS}       "Grating common name"            UVES_ICS
qc1log/out 1 {LINTAB} "{h_cwlen({PATHID})(5:)}"         {wlen}           "Grating central wavelength [nm]" UVES_ICS 
qc1log/out 1 {LINTAB} "{h_fits_nameccd({PATHID})(5:)}" {CCD}         "Detector chip name"              CCDDCS   
qc1log/out 1 {LINTAB} "{h_tempcam({PATHID})(5:)}"       {temp}           "Average temperature"        UVES_ICS 
! We log value of INS SENS26 MEAN as TEL AMBI PRES:
qc1log/out 1 {LINTAB} "QC.AMBI.PRES"                    {press}          "Ambient pressure [mm] Hg."  
qc1log/out 1 {LINTAB} "QC.REF.PNAME"                    {stab_ref_frame} "Reference file name"
qc1log/out 1 {LINTAB} "QC.SHFTXAVG"                     {MeanDX}         "average shift in x"          
qc1log/out 1 {LINTAB} "QC.SHFTXMED"                     {MedDX}          "median shift in x"
qc1log/out 1 {LINTAB} "QC.SHFTYAVG"                     {MeanDY}         "mean shift in y"
qc1log/out 1 {LINTAB} "QC.SHFTYMED"                     {MedDY}          "median shift in y"
set/format

qc1log/out 3  

!
!if NICE_CREA .eq. "Y" then
!  delete/disp 0
!  delete/grap 0
!endif
return 0

ENTRY REPL_CH

define/param P1 ? c "Enter character to replace: "
define/param P2 ? c "Enter string where character should be replaced: "
define/param P3 ? c "Enter new character: "


define/local i1/i/1/1 0
define/local len/i/1/1 0
define/local ch/c/1/4 " " ALL

len = m$strlen(P2)			!get length of string

i1 = m$index(P1,"blank")
if i1 .le. 0 ch = P1(1:1)		!if P1 not " " get replace character
					!from P1, ch is preset to blanks!

outputc = P2
do i1 = 1 len
  if outputc({i1}:{i1}) .eq. ch(1:1) outputc({i1}:{i1}) = P3(1:1)
enddo

return {len}


