! $Id: uves_formatchk.prg,v 1.4 2001/01/04 10:39:55 amodigli Exp $
!+++ UVES_PREPCAL +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_prepcal.prg
!
!.KEYWORDS      spectroscopy, echelle, calibrations
!
!.USAGE         @p (IDENT) incat refcat [bkgmtd] [pause] [fits]
!
!.INPUT         incat   (P1): catalog with input frames (raw data)
!				Format check exposure,
!				Order definition flat, ThAr exposure.
!				Biases (-> master bias)
!				Flats (-> master flat)
!               refcat  (P2): image catalog of reference data:
!				ThAr reference list
!               bkgmtd  (P3): background measurement method:
!				MEDIAN or MINIMUM
!               pause   (P4): pause after each single calibration (Y/N)
!               fits    (P5): create FITS files at the end (Y/N)
!
!.OUTPUT        set of calibration tables stored in a reference catalog:
!			    drs setup table, order table, background table,
!			    line tables, and optionally a master bias and a
!			    master flat.
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT	P3	MEDIAN
!		    P4	N
!		    P5	Y
!
!.PURPOSE       Produce a complete set of calibration frames needed for
!		        a science reduction in a certain central wavelength.
!
!.ALGORITHM     Calls PREDICT/UVES, ORDERP/UVES, WAVECA/UVES, MASTER/UVES
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/10/13  Created.
!		    1999/12/17	Interface changed!
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.4 $
! $Log: uves_formatchk.prg,v $
! Revision 1.4  2001/01/04 10:39:55  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
define/param p1 ? c "Enter raw data catalog:"
define/param p2 ? c "Enter reference catalog:"
define/param p3 MEDIAN c "Enter BACKGROUND measurement method (MEDIAN,MINIMUM)"
define/param p4 N c "Enter Y or N if pause should be activated:"
define/param p5 Y c "Enter Y or N to create FITS files at the end:"

define/local pid/c/1/15 "prepcal" ? +lower

define/local incat/c/1/25 " " ALL
define/local in_cats/c*25/1/3 " " ALL +lower
define/local bs_cats/c*25/1/3 " " ALL +lower
define/local ff_cats/c*25/1/3 " " ALL +lower
define/local mode/c*15/1/3 " " ALL
define/local class/c/1/25 " " ALL +lower
define/local date/c/1/25 " " ALL
define/local i1/i/1/1 0
define/local time/i/1/1 0

time = m$secs();

							!Split input catalog
in_cats(1) = "sel{EXREDL}"				!into 3 catalogs:
in_cats(2) = "sel{EXREDU}"				!sel{EXREDL},
in_cats(3) = "sel{EXBLUE}"				!sel{EXREDU},
bs_cats(1) = "bs{EXREDL}"				!into 3 catalogs:
bs_cats(2) = "bs{EXREDU}"				!sel{EXREDL},
bs_cats(3) = "bs{EXBLUE}"				!sel{EXREDU},
ff_cats(1) = "ff{EXREDL}"				!into 3 catalogs:
ff_cats(2) = "ff{EXREDU}"				!sel{EXREDL},
ff_cats(3) = "ff{EXBLUE}"				!sel{EXREDU},
mode(1) = "RED ARM EEV"					!sel{EXBLUE}.
mode(2) = "RED ARM MIT"
mode(3) = "BLUE ARM"


CONFIG/DISPL p3=0.7
CLEAN/DISPL
-copy {p1} rawdata.cat
incat = "rawdata.cat"
show/icat {incat} >Null				!check if we have data
if outputi(1) .le. 0 return -1
show/icat {p2} >Null
if outputi(1) .gt. 0 add/icat {incat} {p2} >Null !merge the catalogs
SPLIT/UVES {incat}
show/icat split.cat >Null
if outputi(1) .le. 0 -copy {incat} split.cat
show/icat prodsplit.cat >Null
if outputi(1) .gt. 0 add/icat split.cat prodsplit.cat >Null

@p uves_formatchk,SPLITCAT split.cat

do i1 = 1 3 1
  PATHID = i1
  show/icat {in_cats({i1})} >Null	!check if INPUT catalog is empty
  if outputi(1) .gt. 0 then
    mess/out 3 {pid} "Running {mode({i1})} ..."
    add/icat {in_cats({i1})}.cat {p2} >Null
    read/icat {in_cats({i1})}.cat
    @p uves_formatchk,DO_CALIB -
    {in_cats({i1})}.cat {bs_cats({i1})}.cat {ff_cats({i1})}.cat {p3} {p4} {p5}
    if {q1} .eq. -1 mess/out 3 {pid} "-> Skip {mode({i1})} setting."
  endif
enddo

date = m$isodate()
-copy {incat} cat{class}_{date}.cat

time = m$secs() - time
mess/out 3 {pid} "TIME used: {time}"

return 0 cat{class}_{date}.cat




!=== SUBROUTINES ==============================================================

ENTRY DO_CALIB

define/param p1 ? c "Enter INPUT catalog:"
define/param p2 + c "Enter BIAS catalog:"
define/param p3 + c "Enter FLATFIELD catalog:"
define/param p4 MEDIAN c "Enter BACKGROUND measurement method (MEDIAN,MINIMUM)"
define/param p5 N c "Enter Y or N if pause should be activated:"
define/param p6 Y c "Enter Y or N to create FITS files at the end:"



define/local catal/i/1/1 0
define/local opathid/i/1/1 0
define/local NOBIAS/i/1/1 0
define/local modext/c/1/10 " " ALL
define/local query/c/1/100 " " ALL
define/local det/c/1/10 " " ALL
define/local dic/c/1/10 " " ALL
define/local wcen/c/1/10 " " ALL
define/local bin/c/1/3 " " ALL
define/local refcat/c/1/100 "refcat"
define/local Frm/c/1/100 " " ALL
define/local Ord/c/1/100 " " ALL
define/local Gue/c/1/100 " " ALL
define/local Ref/c/1/100 " " ALL
define/local Wav/c/1/100 " " ALL
define/local OFLAT_ID/c*25/1/3 " " ALL
OFLAT_ID(1) = "ORDER_FLAT_REDL"
OFLAT_ID(2) = "ORDER_FLAT_REDU"
OFLAT_ID(3) = "ORDER_FLAT_BLUE"
define/local ARC_ID/c*25/1/3 " " ALL
ARC_ID(1) = "ARC_LAMP_REDL"
ARC_ID(2) = "ARC_LAMP_REDU"
ARC_ID(3) = "ARC_LAMP_BLUE"
define/local FRM_ID/c*25/1/3 " " ALL
FRM_ID(1) = "ARC_LAMP_FORM_REDL"
FRM_ID(2) = "ARC_LAMP_FORM_REDU"
FRM_ID(3) = "ARC_LAMP_FORM_BLUE"

show/icat {p1} >Null			!check if INPUT catalog is empty
if outputi(1) .eq. 0 goto ERR1


CLEAN/DISP
!=====================!
! [0] Create displays !
!=====================!

loop:
store/frame Ref {p1} 1 fini
  if m$filtyp(Ref,".bdf") .eq. 2 goto loop
fini:
if m$filtyp(Ref,".bdf") .eq. 2 goto ERR1

CONFIG/INSTR {Ref}
modext = q2
det = "{q2(2:2)}{CCD}"
det = m$lower(det)
outputi = wcenter / 10
set/format I1
wcen = "{outputi}"
set/format F1.0

! The reversal of bin(2)xbin(1) serves for the 
! naming convention before data rotation
bin = "{CCDBIN(2)}x{CCDBIN(1)}"

!if m$existd(Ref,h_insmode) .gt. 0 then
  dic = "{{Ref},{h_insmode}}"
  outputi(1) = m$strlen(dic)
  outputc = dic({outputi(1)}:{outputi(1)})
  if m$tstno(outputc) .eq. 1 then	!it's a number
    dic = "d{outputc(1:1)}"
  else
    dic = "d0"
  endif
!endif
set/format
class = "{wcen}{dic(1:2)}{det(1:2)}{bin}"

refcat = "ref{wcen}_{bin}.cat"
-delete {refcat}

!goto LAB4


!=============================!
! [1] Get the FIRST GUESS     !
!=============================!

query = "{LINCAT_ID}"			!get the line reference table
!write/out "Search for {query} in {p1} ..."
SEARCH/ICAT {p1} {query} >Null
if m$strlen(out_a) .eq. 0 goto ERR2
Ref = out_a
query = "{FRM_ID({PATHID})}"		!get the format check frame
!write/out "Search for {query} in {p1} ..."
SEARCH/ICAT {p1} {query} >Null
if m$strlen(out_a) .eq. 0 goto ERR2
Frm = out_a

!PREDICT/UVES {Frm} {Ref} {refcat}
@p uves_fmtstab.prg {Frm} {p1} {refcat} 

@p uves_formatchk,save {p6} {refcat} {class}
@p uves_formatchk,pause {p5} "[pause]: Create ref.catalog."

!----------------------------------!
! 1.1 Create the REFERENCE CATALOG !
!----------------------------------!
LAB2_1:
					!DRS and first guess line table
					!are already in the catalog.
!wr/o "reference catalog:"
!READ/ICAT {refcat}
query = "{LINTAB_ID({PATHID})}"
!write/out "Search for {query} in {refcat} ..."
SEARCH/ICAT {refcat} {query} >Null
if m$strlen(out_a) .eq. 0 goto ERR2
Gue = out_a

!=============================!
! [2] Get the ORDER-POSITIONS !
!=============================!

query = "{DRSRC_ID({PATHID})}"
!write/out "Search for {query} in {refcat} ..."
SEARCH/ICAT {refcat} {query} >Null
if m$strlen(out_a) .eq. 0 goto ERR2
savini/echelle {out_a} READ
query = "{OFLAT_ID({PATHID})}"
!write/out "Search for {query} in {p1} ..."
SEARCH/ICAT {p1} {query} >Null
if m$strlen(out_a) .eq. 0 goto ERR2
Ord = out_a
!show/echelle o

-delete ord.cat
ORDERP/UVES {Ord} ord.cat {refcat}
modify/graph I

$cp ord.cat {refcat}
ADD/ICAT {refcat} {LINCAT} >Null
ADD/ICAT {refcat} {Gue} >Null
!read/icat {refcat}

@p uves_formatchk,save {p6} ord.cat {class}
@p uves_formatchk,pause {p5} "[pause]: Wavelength Calibration."
$mv pm*.ps  mod_{class}.ps

!=================================================!
! [3] Get the LINE TABLE (wavelength calibration) !
!=================================================!
LAB3:
query = "{ARC_ID({PATHID})}"
!write/out "Search for {query} in {p1} ..."
SEARCH/ICAT {p1} {query} >Null
if m$strlen(out_a) .eq. 0 goto ERR2
Wav = out_a
!show/echelle w

-delete wav{wcen}.cat
WAVECA/UVES {Wav} wav{wcen}.cat {refcat} AUTO YES YES

assign/graph POSTSCRIPT 
plot/residual 
copy/graph
$mv postscript.ps  residual_{class}.ps
$mv mod_{class}.ps  physmod_{class}.ps

modify/graph I
clear/disp
write/descr {WLC} IDENT   " {{WLC},ESO.OBS.NAME} {CLASS}"
load/image {wlc}
load/ident >Null
label/disp "{{wlc},ORIGFILE}" 10,20
label/disp "{{wlc},ARCFILE}"  285,20
copy/display postscript
$mv screen00.ps screen_{class}.ps

set/echelle wlcmtd=order
$rm -f spformat_{class}.txt
ident/echelle >spformat_{class}.txt

@p uves_formatchk,save {p6} wav{wcen}.cat {class}
@p uves_formatchk,pause {p5} "[pause]: Update ref.catalog."

!----------------------------------!
! 3.1 Update the REFERENCE CATALOG !
!----------------------------------!

ADD/ICAT {refcat} l{wcen}*.tbl >Null

LAB:

$rm -f col* deriv* dirf* dat* fmt* lsro* middumm*  orddet* stat*  wrk.tbl

create/acat ps.cat *ps >Null

mess/out 3 {pid} "Reference catalog created: `{refcat}`\n"
mess/out 3 {pid} "POSTSCRIPT catalog created: `ps.cat`\n"
mess/out 3 {pid} "Use reference catalog for science reduction (REDUCE/UVES)."
append/out "Save reference data and POSTSCRIPT files\n"
append/out "in your calibration database. Use `SAVETO/CALDB`."
mess/out 3 {pid}
mess/out 3 {pid} "Calibration finished."

return 0 {class}


ERR1:
  mess/out 1 {pid} "INPUT catalog `{p1}` is empty !"
  return -1
ERR2:
  mess/out 1 {pid} "No {query} found in INPUT catalog `{p1}` !"
  return -1




ENTRY pause
define/param p1 ? c "Enter Y or N if pause should be activated:"
if p1(1:1) .eq. "Y" INQUIRE/KEY INPUTC {p2}
return 0



ENTRY save
define/param p1 ? c "Enter Y or N on order to save as FITS:"
define/param p2 ? c "catalog name:"
define/param p3 ? c "classification:"

define/local ifrm/c/1/100 " " ALL
define/local ofrm/c/1/100 " " ALL
define/local catg/c/1/100 " " ALL
define/local ftyp/i/1/1 0
define/local catal/i/1/1 0
define/local i1/i/1/1 0
define/local slen/i/1/1 0
define/local win/c*3/1/3 " " ALL
win(1) = "low"
win(2) = "med"
win(3) = "upp"

if P1(1:1) .eq. "Y" then
write/out " "
loopsav:
  store/frame ifrm {p2} 1 ENDSAV

  catg = "{{ifrm},{h_procatg}}"
  branch catg(1:3) DRS,ORD,BAC,LIN,MAS DRS,ORD,BAC,LIN,MAS
  wr/o "Unknown product type, NO SAVE!"
  goto loopsav
DRS:
  ofrm = "drs{p3}"
  goto tablesav
ORD:
  ofrm = "ord{p3}"
  goto tablesav
BAC:
  ofrm = "bkg{p3}"
  goto tablesav
LIN:
  i1 = m$strlen(catg)
  ofrm = catg({i1}:{i1})
  if m$tstno(ofrm) .eq. 1 then
    i1 = {ofrm(1:1)}
    ofrm = "lin{p3}{win({i1})}"
  else
    ofrm = "gue{p3}"
  endif
  goto tablesav
MAS:
  if m$index(catg,"BIAS") .gt. 0 then
    ofrm = "mbs{p3(6:)}"
  else
    get/pathid {ifrm}
    set/format I1
    slen = {{ifrm},{h_sllen({PATHID})}}
    ofrm = "mff{p3}s{slen}"
  endif
  goto imagesav
endbranch:

tablesav:
    create/image empty 0 ? no			!Yes.
    copy/dd {ifrm} eso.* empty			!copy all ESO.xyz desc
    copy/dd {ifrm} instrume empty		!copy desc INSTRUME
    copy/dd {ifrm} exptime empty		!copy desc EXPTIME
    @p uves_parsepath {ifrm}
    outdisk/sfits empty.bdf,{ifrm} {ofrm}.tfits	!convert to single FITS
    mess/out 3 save "{ifrm} saved as FITS file:\n{ofrm}.tfits"
    goto loopsav
imagesav:
    outdisk/fits {ifrm} {ofrm}.fits
    mess/out 3 save "{ifrm} saved as FITS file:\n{ofrm}.fits"
    goto loopsav
endsav:
-delete empty.bdf
endif
write/out " "
return 0


ENTRY SPLITCAT
define/param p1 ? c "Enter name of input image catalog:"

define/local MBS_FOUND/i/1/1 0
define/local MFF_FOUND/i/1/1 0

define/local ifrm/c/1/100 " " ALL
define/local modext/c/1/25 " " ALL
define/local frmtyp/c/1/25 " " ALL
define/local catal/i/1/1 0
define/local i1/i/1/1 0

creat/icat {in_cats(1)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {in_cats(2)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {in_cats(3)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {bs_cats(1)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {bs_cats(2)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {bs_cats(3)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {ff_cats(1)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {ff_cats(2)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {ff_cats(3)}.cat NULL {UVES_IDSTRNG} >Null

ILOOP:						!Loop through input image
  store/frame ifrm {p1} 1 ELOOP			!catalog in order to distribute
  						!input frames to separate
						!image catalogs being all of
  frmtyp = "{{ifrm},{UVES_IDSTRNG}}"		!the same data part.
  get/idex {frmtyp}				!Get identifier extension
  modext = q3

  branch modext(1:5) {EXREDL},{EXREDU},{EXBLUE} REDL,REDU,BLUE
  goto endbranch

REDL:	
  add/icat {in_cats(1)} {ifrm} >Null
  goto endbranch
REDU:
  add/icat {in_cats(2)} {ifrm} >Null
  goto endbranch
BLUE:
  add/icat {in_cats(3)} {ifrm} >Null
  goto endbranch

endbranch:
goto ILOOP
ELOOP:
						!Now the main catalog is
						!split into sub catalogs which
						!only contain data from ONE
catal = 0					!UVES arm.
do i1 = 1 3					!In the next step we have to
IDIST:						!extract the flats and the
  store/frame ifrm {in_cats({i1})}.cat 1 EDIST	!biases from the sub catalogs.
  frmtyp = "{{ifrm},{UVES_IDSTRNG}}"		!
  branch frmtyp(1:4) BIAS,FLAT,MAST BIAS,FLAT,MAST
    goto IDIST
BIAS:
    subtract/icat {in_cats({i1})} {ifrm} >Null
    if MBS_FOUND .eq. 0 add/icat {bs_cats({i1})} {ifrm} >Null
    goto IDIST
FLAT:
    subtract/icat {in_cats({i1})} {ifrm} >Null
    if MFF_FOUND .eq. 0 add/icat {ff_cats({i1})} {ifrm} >Null
    goto IDIST
MAST:
    subtract/icat {in_cats({i1})} {ifrm}
    if frmtyp(1:11) .eq. "MASTER_BIAS" then
      -delete {bs_cats({i1})}
      bs_cats({i1}) = ifrm
      MBS_FOUND = 1
    else if frmtyp(1:11) .eq. "MASTER_FLAT" then
      -delete {ff_cats({i1})}
      ff_cats({i1}) = ifrm
      MFF_FOUND = 1
    endif
    goto IDIST
EDIST:

  if MBS_FOUND .eq. 0 then			!Check if there are
    show/icat {bs_cats({i1})} N			!some biases in the
    if outputi(1) .eq. 0 bs_cats({i1}) = "+ "	!catalog otherwise
  endif						!mark as "+".
  if MFF_FOUND .eq. 0 then			!The same for the flats
    show/icat {ff_cats({i1})} N			!Do only the check in
    if outputi(1) .eq. 0 ff_cats({i1}) = "+ "	!case NO master were
  endif						!found!

  MBS_FOUND = 0					!Reset these flags for
  MFF_FOUND = 0					!the next catalogs to
  catal = 0					!check.
enddo						!

return 0
