! $Id: uves_gausfit.prg,v 1.2 2001/01/04 10:39:49 amodigli Exp $
!+++ UVES_GAUSFIT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT	(C) 1998  European Southern Observatory
!
!.IDENT		uves_gausfit.prg
!
!.KEYWORDS	Determine initial parameter set for later use with uves_nonlfit
!
!.USAGE		@p (IDENT) image table partbl
!
!.INPUT		image	(P1): name of data image
!
!.OUTPUT	table	(P2): name of data table
!		partbl	(P3): name of parameter table
!
!		updated parameter and data table
!
!.RETURN	
!
!.DEFAULT
!
!.PURPOSE	Copy the ONE-DIMENSIONAL input data into a data table (P2).
!		Create all columns in P2 needed for the non-linear fit.
!		Create the parameter table and initialize them.
!
!.ALGORITHM	nonlfit performs a non-linear least squares fit.
!		The functions to be fitted are user-defined functions.
!
!		Here the initialization shall be done for a special case:
!                          a*exp( {-[x-b]/c}^2 ) + d
!
!		The parameters a, b, c and d will be determined by the first
!		guesses:
!
!		b = xmax, a = ymax-d,
!		c = (x-xmax)*[ln({ymax-d}/{y-d})]^-0.5
!
!		(x[],y[]) are the x, and y-coordintes of the data which have
!		to be fitted. acc is a desired accuracy for the first guess.
!		These parameters will be stored in a table which will be
!		created by this procedure. This parameter-table will be the
!		input to the call of nonlfit.
!
!.ENVIRON	UVES, CCDRED context
!   
!.LANGUAGE	MIDAS 98NOV
!
!.AUTHOR	Sebastian Wolf, ESO-DMO
!
!.VERSION	0.9 1998/02/04	Creation
!		1.0 1998/11/25	Add 1.0 to column :X (should start at 1.0)
!.COMMENT	Equidistant X axis is assumed (step size: 1.0).
!		This could be changed in future!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_gausfit.prg,v $
! Revision 1.2  2001/01/04 10:39:49  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? c "Enter name of data image: "
define/param p2 ? c "Enter name of data table (will be created): "
define/param p3 ? c "Enter name of parameter table: "
!
! local variable declaration
!
define/local num/i/1/1 0
define/local dd/r/1/1 0.0
define/local x1/r/1/1 0.0
define/local y1/r/1/1 0.0
define/local xx/r/1/1 0.0
define/local xm/r/1/1 0.0
define/local ym/r/1/1 0.0
define/local acc/r/1/1 0.001
!
! Create data table
!
copy/it {P1} {P2} :X
name/col {P2} #2 :Y
creat/col {P2} :SIG " "
creat/col {P2} :FIT " " 
creat/col {P2} :RESIDUAL " "
dd = {{P2},#1,1}
comp/tab {P2} :X = :X - {dd} + 1.0
comp/tab {P2} :SIG = SQRT({RON}*{RON}/{GAIN}/{GAIN} + ABS(:Y) / {GAIN})
!comp/tab {P2} :SIG = :SIG / 10.0
comp/tab {P2} :FIT = 0.0
comp/tab {P2} :RESIDUAL = 0.0

set/gra pmode=1 colo=1
plot/row {p1}

!
! Create parameter table
!
crea/tab {P3} 3 4
crea/col {P3} :PARAM " "
crea/col {P3} :FIXED " "
crea/col {P3} :RESULT " "

statistic/ima {P1} ? ? ? GN >Null
num = outputi(1)
ym = outputr(2)			!MAXIMUM
xm = outputi(3)			!position of maximum
outputr(20) = 0.01*(ym-outputr(1))
outputi(20) = xm - 3 / m$value({p1},STEP(1))
if outputi(20) .lt. 0 outputi(20) = xm + 3 / m$value({p1},STEP(1))
x1 = m$value({P2},:X,{outputi(20)})
y1 = m$value({P2},:Y,{outputi(20)})
xx = (ym-outputr(20))/(y1-outputr(20))
if xx .le. 1.0 goto ERR
{P3},:PARAM,4 = outputr(20)		!d a3
{P3},:PARAM,1 = ym - outputr(20)	!a a0
{P3},:PARAM,2 = xm			!b a1
{P3},:PARAM,3 = m$abs(x1 - xm) / m$sqrt( m$ln(xx) )

compute/tab {P3} :FIXED = 0.0		!no parameter is fixed by default
compute/tab {P3} :RESULT = 0.0
read/tab {P3}

write/keyw INPUTI/i/1/8 1,2,3,4,5,1,2,3

@p uves_nonlfit {P2} {P3} 0.01 2


compute/table {P2} :X = :X + {dd} - 1.0

set/gra colo=3
!overplot/table {p2} :x :residual
!set/gra colo=3 stype=5
!overplot/table {p2} :x :fit
outputr(1) = m$value({p3},:result,1)
outputr(2) = m$value({p3},:result,2)
outputr(3) = m$value({p3},:result,3) / m$sqrt(2.0)
outputr(4) = m$value({p3},:result,4)
outputr(5) = dd - 1.0
outputr(6) = outputr(5) + outputr(2)
outputr(7) = m$value({p1},NPIX(1)) * m$value({p1},STEP(1)) / 1000
outputr(8) = 1.0 / (m$sqrt(6.2831853) * {outputr(3)})
read/key outputr
CREATE/IMAG gaus -
  1,1000 {outputr(5)},{outputr(7)} GAUSS {outputr(6)},{outputr(3)}
COMPUT/IMAG gaus =  {outputr(4)} + {outputr(1)} * gaus / {outputr(8)}
read/key outputr
overplot/row gaus

return 0

ERR:
  mess/out 1 {pid} "No gaussfit possible !"
  return -1
