! $Id: uves_getfilter.prg,v 1.2 2001/01/04 10:39:48 amodigli Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       uves_getfilter.prg
!
!.KEYWORDS    Determine instrument filter setup
!
!.USAGE       config/filter  frame
!
!.INPUT       frame    (P1)   Input frame from which the instrument filter
!                             setup is extracted.
!
!.OUTPUT      None.
!
!.RETURNS     The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     Get the number and the names of all active filters.
!
!.ENVIRON     UVES context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/07/03:  Creation
!		   1999/03/10:	Copy from FORS context (SW)
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_getfilter.prg,v $
! Revision 1.2  2001/01/04 10:39:48  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
crossref  frame


define/param p1 ? c "Enter name of input frame:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/80      "UvesGetFilter"


! Local variables
define/local i/i/1/1         0
define/local nf/i/1/1        1
define/local lockno/i/1/2    0     all
define/local frame/c/1/100   {p1}
define/local dscname/c/1/80  " "   all



! Unlock keywords if necessary
info/keyw uves_nfilt
lockno(1) = mid$info(4)
if lockno(1) .ne. 0 then
  unlock/key uves_nfilt {lockno(1)}
endif

info/keyw uves_filti
lockno(2) = mid$info(4)
if lockno(2) .ne. 0 then
  unlock/key uves_filti {lockno(2)}
endif


! Reset filter names
do i = 1 UVES_MAXFILT 1
  uves_filti({i}) = " "
enddo


! Check for filter keyword without index
dscname = kbase_filti // ksuff_filti
if m$existd(frame,dscname) .eq. 1 then
  uves_filti({nf}) = "{{frame},{dscname}}"
  nf = nf + 1
else
! Check for filter keywords with index
  set/format i1
  do i = 1 UVES_MAXFILT 1
    dscname = kbase_filti // "{i}" // ksuff_filti
    if m$existd(frame,dscname) .eq. 1 then
      uves_filti({nf}) = "{{frame},{dscname}}"
      nf = nf + 1
    endif
  enddo
  set/format
endif

uves_nfilt = nf - 1


! Protect keywords
if lockno(1) .ne. 0 then
  lock/key uves_nfilt {lockno(1)}
endif

if lockno(2) .ne. 0 then
  lock/key uves_filti {lockno(2)}
endif

return 0
