! $Id: uves_getpathid.prg,v 1.2 2001/01/04 10:39:47 amodigli Exp $
!+++ UVES_GETPATHID ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_getpathid.prg
!
!.KEYWORDS      keywords
!
!.USAGE         @p (IDENT) frame
!
!.INPUT         frame   (P1): input frame (.bdf or .tbl)
!
!.OUTPUT        none
!
!.RETURN        Q1:  0: on successful return
!                   -1: a successful return failed
!		Q2:  indtrument extension found
!
!.DEFAULT
!
!.PURPOSE       Get the instrument path identifier in order to
!		easily access the ID-keywords (ORDER_ID, BACKGR_ID, ...)
!		which depend on the used instrument path ({h_inspath}).
!
!.ALGORITHM     Check the {UVES_IDSTRNG} header keyword and set PATHID to
!		the approriate index. The instrument mode extension is
!		an appendix of the {UVES_IDSTRNG}. This appendix is set
!		by SPLIT/UVES. Each input file has to go through
!		SPLIT/UVES where the frame transformation and the mode
!		identification takes place.
!		Known appendices are: _REDL, _REDU, and _BLUE
!		   The global key PATHID will be properly set due to the
!		identification. Thus, all mode-dependend ID-keywords may
!		be properly accessed through this index.
!
!		Example:
!		Having a SCIENCE_REDL frame then _REDL is the appendix.
!		Now, you want to perform an order extraction, then you
!		need an appropriate order table selected from a catalog:
!		@p uves_getpathid frame
!		search/icat catalog {ORDER_ID({PATHID})}
!		
!		... will automatically find the right order table.
!
!
!.ENVIRON       UVES, CCDRED context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/09  Created.
!		1.0 1999/07/01	Retrieval of mode extension keyword improved.
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_getpathid.prg,v $
! Revision 1.2  2001/01/04 10:39:47  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

define/param p1 ? c "Enter filename:"

define/local modext/c/1/25
define/local imode/i/1/1

if m$existd(p1,UVES_IDSTRNG) .gt. 0 then
  modext = "{{p1},{UVES_IDSTRNG}}"
else if m$existd(p1,h_procatg) .gt. 0 then
  modext = "{{p1},{h_procatg}}"
else
  mess/out 1 getpathid "Path ID could not be found!"
  PATHID = 0
  return -1
endif

get/idex {modext}
modext = q3

branch modext(1:5) {EXREDL},{EXREDU},{EXBLUE} REDL,REDU,BLUE
mess/out 1 getpathid "Path ID could not be found!"
return -1

REDL:
	PATHID = 1
	goto endbranch
REDU:
	PATHID = 2
	goto endbranch
BLUE:
	PATHID = 3
	goto endbranch

endbranch:

mess/out 4 getpathid "Instrument path ID: {PATHID}"
return 0 {modext}
