! $Id: uves_getwbin.prg,v 1.2 2001/01/04 10:39:47 amodigli Exp $
!+++ UVES_GETWBIN +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_getwbin.prg
!
!.KEYWORDS      spectroscopy, echelle, wavelength calibration, wavelength bin
!
!.USAGE         @p (IDENT) input
!
!.INPUT         input	(P1): input frame (UVES science frame)
!               wave	(P2): wavelength where to get the wavelength bin.
!				Wavelength units are assumed to be in A.
!				Set to 0.0 in order to get result at central
!				wavelength of the selected order.
!               absord	(P3): order number:
!				>0: absolute order number
!				<0: relative order number
!				    (for example: -1: 1st order at the bottom)
!
!.OUTPUT        OUTPUTD(1): wavelength binsize at specified wavelength and
!				echelle order in Angstroem.
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT	
!
!.PURPOSE       Determine the wavelength bin at a given wavelength.
!
!.ALGORITHM     Calls the executable uves_getwbin.exe which is based on the
!		physical model library for UVES.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/07/14  Created.
!                   1999/10/05	New version using PLOT/CALIB instead of
!                               physical model.
!
!.COMMENT	(1) The conversion of relative to absolute order numbers relys
!		on the correct value of ECHORD(6).
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_getwbin.prg,v $
! Revision 1.2  2001/01/04 10:39:47  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/param p1 {LINTAB} c "Enter line table (LINTAB):"

plot/calibrat ? N

return 0



define/param p1 ? c "Enter input frame (UVES header):"
define/param p2 ? c "Enter wavelength, wavelength table, or name of data array [A]:"

define/local absord/i/1/1 0

config/instr {p1}				!Get instrument configuration
						!for physical model initializ.


if m$tstno(p2) .eq. 1 then			!single wavelength specified

  define/param p3 ? n "Enter order number (>0: absolut, <0: relative):"
  absord = {p3}
  if absord .lt. 0 then				!relative order num. specified
    absord = echord(6) + absord
  endif
  IN_A = "{absord}"				!absolut order number
  IN_B = p2					!wavelength in [A]

else

  parse/path {p2}
  ext = m$upper(q3)
  if ext(1:4) .eq. ".TBL" then
    if m$exist(p2) .eq. 1 then			!work on the input table
      define/param p3 ? c "Enter name of input column (wavelength) [A]:"
      define/param p4 ? c "Enter name of output column (delta wavelength) [A]:"
      IN_A = p2					!input table
      IN_B = p3
      OUT_A = p4
      mess/out 2 {pid} "Table input not yet supported!"
      return -1
    else
      goto ERR1
    endif
  else
    define/param p3 ? c "Enter name of output array (delta wavelength) [A]:"
    IN_A = p2
    OUT_A = p3
    mess/out 2 {pid} "Array input not yet supported!"
    return -1
  endif
endif


run PIPE_EXE:uves_getwbin.exe

return {outputd(1)}				!wavelength binsize in A.


!Error handling
ERR1:
  mess/out 1 {pid} "Table `{p2}` does not exist!"
  return -1
