! $Id: uves_init.prg,v 1.5 2001/05/23 15:31:00 amodigli Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       uves_init.prg
!
!.KEYWORDS    Pipeline initialization
!
!.USAGE       init/pipeline  raw_set ref_set cat_accept ctx [session]
!                            [obsmode] [cat_reject] [cat_result]
!
!.INPUT       raw_set     (P1)    Set of raw input frames to be processed or
!                                 a catalog of input frames.
!             ref_set     (P2)    Set of reference frames used for processing
!                                 (bad pixel mask, bias, dark ...)
!             ctx         (P4)    Identifier of required context
!             session     (P5)    Keyword defaults for pipline
!             mode        (P6)    Indicator of current observing mode
!
!.OUTPUT      cat_accept  (P3)    Name of accepted frames output catalog
!             cat_reject  (P7)    Name of rejected frames output catalog
!             cat_result  (P8)    Name of result frames output catalog
!
!.RETURNS     The recipe returns 0 on success. If no frames are found in
!             the input set the procedure returns 2, on any other error 1.
!             Keyword Q1 is updated with the exit code.
!
!.PURPOSE     The procedure starts the context ECHELLE and initializes the
!             keywords with the pipeline defaults. The input frame set is
!             processed, i.e. the raw frames are copied to the local 
!             directory and converted from fits format to MIDAS formats.
!             If the input frame set is a MIDAS catalog (i.e. the suffix
!             '.cat') must be present, then the procedure assumes to run
!             within a recipe queue and skips the initialization and frames
!             conversion. Only the temporary catalog for the result frames is
!             created. 
!
!.ENVIRON     UVES context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      S. Wolf, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!		   1999/03/10:	Copy from FORS context (SW)
!		   1999/03/22:	use savini/echelle for context initial. (SW)
!		   1999/07/01:	Re-definition of ECHELLE context commands
!				are done in the context file uves.ctx, now.
!		    1999/09/06	uves_updtbl is obsolete using from now as
!				98NOVpl2.2 has fixed the bug in necbackprep.prg
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.5 $
! $Log: uves_init.prg,v $
! Revision 1.5  2001/05/23 15:31:00  amodigli
! Now p_version is defined only in setup
! which creates $PIPE_HOME/uves/UVES_VERSION
! holding the UVES pipe version. uves_get_pversion.prg
! will get such a value.
!
! Revision 1.4  2001/05/02 14:19:29  amodigli
! Added an entry to support prohead/init
! command
!
! Revision 1.3  2001/01/04 10:39:45  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------


define/param p1  ?      c    "Input frame set:"
define/param p2  ?      c    "Reference frame set:"
define/param p3  +      c    "Accepted frames catalog name:"
define/param p4  +      c    "Context identifier: "
define/param p5  +      c    "Startup file:"
define/param p6  +      c    "Observing mode:"
define/param p7  +      c    "Rejected frames catalog name:"
define/param p8  +      c    "Result frames catalog name:"


! Procedure identification
define/local procID/c/1/80	"InitPipeline"



! Local variables
define/local ctx/c/1/10      " "  all
define/local session/c/1/10  " "  all
define/local obsmode/c/1/3   " "  all
define/local ctxflg/i/1/1    0
define/local pos/i/1/1       0
define/local percent/r/1/1   1.0


if p4(1:1) .eq. "+" then
  ctx = "NONE"
else
  ctx = m$lower(p4)
endif

if p5(1:1) .eq. "+" then
  session = "NONE"
else
  session = p5
endif

if p6(1:1) .ne. "+" then
  obsmode = p6
endif


! Reset delete flag for auxiliary contexts
set/ctxdel reset

pos = m$indexb(p1,".cat")

if pos .gt. 0 then
  mess/out 4 {procID} "SUPER MODE: Restricted initialization!"
  -copy {p1} {p3}


! Create empty catalog for result frames
  if p8 .ne. "+" then
    create/icat {p8} NULL {UVES_IDSTRNG} >Null
  endif

! If a context is requested check if it is already enabled. If this is not
! the case it is loaded and initialized.
  if ctx .ne. "NONE" then
    show/context {ctx}
    ctxflg = outputi(5)

    if ctxflg .ne. 0 then
      mess/out 3 {procID} "Context {ctx} already enabled."
    else
      set/context {ctx} {p_ctxdir} >Null
      show/context {ctx}
      ctxflg = outputi(5)

      if ctxflg .ne. 0 then  
        mess/out 3 {procID} "Context {ctx} enabled."
        set/ctxdel
      else
        mess/out 2 {procID} "Context {ctx} cannot be enabled."
      endif

      if ctxflg .ne. 0 .and. session .ne. "NONE" then
        config/drs {p3}
        if m$strlen(outputc) .ne. 0 then
          copy/tab {outputc} {session}KEYS.tbl
  
          if ctx .eq. "echelle" then
            savini/echelle {session}KEYS.tbl READ
            mess/out 3 {procID} "Initialization file {session}KEYS.tbl used."
          endif
        else
          mess/out 2 {procID} "Session {session} cannot be initialized."
        endif 
      else
       if session .ne. "NONE" then
         mess/out 1 {procID} "Cannot load session without context."
          return 1
        endif
      endif
    endif
  endif

else

!gets uves pipeline actual version
@p uves_get_pversion
! Reset internal logging facility
  @p uves_init,RESET_SLOG

! Load context and create auxiliary keywords
  if ctx .ne. "NONE" then

! Enable context and check its availability
    set/context {ctx} {p_ctxdir} >Null
    show/context {ctx}
    ctxflg = outputi(5)

    if ctxflg .ne. 0 then  
      mess/out 3 {procID} "Context {ctx} enabled."
    else
      mess/out 2 {procID} "Context {ctx} cannot be enabled."
    endif
  endif


!am added to be used for pro key
! Set start date
! Unlock keyword if necessary
  define/local lockno/i/1/1    0
  info/keyw p_startdate
  lockno = mid$info(4)
  if lockno .ne. 0 then
    unlock/key p_startdate {lockno}
  endif

  p_startdate = m$isodate()//"            "  ! get date in ISO 8601 format

! Protect keyword
  if lockno .ne. 0 then
    lock/key p_startdate {lockno}
  endif
!am

! Configure pipeline displays
  percent = pipeline_aux(3) / 100.0
  
! For preventing conflicts with FORS pipeline
  write/key nice_crea/C/1/1 "N"
  reset/display 

  config/displays p3={percent}
  if {q1} .gt. 0 then
    mess/out 1 {procID} "Display configuration failed!"
    return 1
  endif


! Create empty catalog for result frames
  if p8 .ne. "+" then
    create/icat {p8} NULL {UVES_IDSTRNG} >Null
  endif


! Process set of input frames
  get/frames {p1} {p2} {p3} {p7}
  if {q1} .gt. 0 then
    return 2
  endif
!  @p uves_init,UPDBAKTAB {p3}

  if m$strlen(obsmode) .ne. 0 then
    UVES_INSMODE = m$upper(obsmode)
  endif


! Initialize global environment
!write/out "CONFIG/ENV disabled !"
!  config/env {UVES_INSMODE}

! Initialize working environment
  if ctxflg .ne. 0 .and. session .ne. "NONE" then


! Look for DRS configuration file in current frame set. If there is one,
! load it.
    config/drs {p3}
    if m$strlen(outputc) .ne. 0 then
      copy/tab {outputc} {session}KEYS.tbl
  
      if ctx .eq. "echelle" then
        savini/echelle {session}KEYS.tbl READ
        mess/out 3 {procID} "Initialization file {session}KEYS.tbl used."
      endif
    else
      mess/out 2 {procID} "Session {session} cannot be initialized."
    endif 
  else
    if session .ne. "NONE" then
      mess/out 1 {procID} "Cannot load session without context."
      return 1
    endif
  endif
endif

if MESS_LEVEL .lt. 4 then
  echo/sess SILENT
else 
  echo/sess VERBOSE
  sessoutv = "       "
endif

return 0


!=== SUBROUTINES ==============================================================
ENTRY UPDBAKTAB

define/param p1 ? c "Enter input image catalog:"

define/local i1/i/1/1 1

do i1 = 1 3
  search/icat {p1} {BACK_ID({i1})} >Null
  if m$strlen(out_a) .gt. 0 then
    @p uves_updtbl R {out_a}
  endif
enddo

!..............................................................................
ENTRY RESET_SLOG

define/local i/i/1/1         0
define/local lockno/i/1/1    0

! Unlock keywords if necessary
info/keyw slog_id
lockno = mid$info(4)
if lockno .ne. 0 then
  unlock/key slog_id {lockno}
endif


! Reset logging facility
slog_id(1) = 0


! Protect keywords
if lockno .ne. 0 then
  lock/key slog_id {lockno}
endif

return 0
