! $Id: uves_jd2date.prg,v 1.2 2001/01/04 10:39:45 amodigli Exp $
!+++ UVES_JD2DATE +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT	(C) 1998  European Southern Observatory
!
!.IDENT		uves_jd2date.prg       
!
!.KEYWORDS	Convert Julian date to civil date (YYYY-MM-DDTHH:MM:SS)
!
!.USAGE		@p (IDENT) jd
!
!.INPUT		jd     	(P1): julian date or modified julian date
!				or file name (see below)
!
!.OUTPUT	outputi(1) ... outputi(6): YEAR, MONTH, DAY, HOUR, MIN, SEC
!		outputr(1): UT as float
!
!.RETURN	Q1:  0: successful return
!		    -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE	Convert julian date to civil date. Negative input value (P1)
!		will be interpreted as modified julian date (MJD). Modified
!		julian date is: JD - 2400000.5 and is for example stored in
!		a FITS-headerkeyword MJD-OBS.
!		Set P1 to a BDF-file then the MJD will be read from the
!		standard descriptor O_TIME(4).
!
!		CAUTION: Works only for the period 01.01.1901 to 31.12.2099
!
!.ALGORITHM	
!
!.ENVIRON	UVES context
!   
!.LANGUAGE	MIDAS 98NOV
!
!.AUTHOR	Sebastian Wolf, ESO-DMO
!
!.VERSION	1.0 1998/11/09	Creation
!		    1999/05/12	Clear output buffer outputc in advance.
!		    1999/10/05	m$nint() argument should not be too big!!
!
!.COMMENT	CAUTION: Works only for the period 01.01.1901 to 31.12.2099
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_jd2date.prg,v $
! Revision 1.2  2001/01/04 10:39:45  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/param P1 ? c "Enter Julian date: "

define/local JUD/d/1/1 0.0
define/local UT/d/1/1 0.0
define/local dd/d/1/1 0.0
define/local YEAR/i/1/1 0
define/local MONTH/i/1/1 0
define/local DAY/i/1/1 0
define/local HOUR/i/1/1 0
define/local MIN/i/1/1 0
define/local SEC/i/1/1 0
define/local mon/i/1/1 0
define/local yea/i/1/1 0
define/local zzz/i/1/1 0
define/local iii/i/1/1 0

set/format I2 F24.8
if m$tstno(P1) .ne. 1 then			!file name
  JUD = m$value({P1},O_TIME(4)) + 2400000.5
else
  JUD = {P1}
  if JUD .lt. 0.0 JUD = -JUD + 2400000.5	!(see DICB definition)
endif

if JUD .lt. 2415386.0 .OR. JUD .gt. 2488069.0 then
  mess/out 1 jd2date -
     "Convert works only in the range\nbetween: (241536.0,2488069.0) (P1={P1})"
  return -1
endif


set/format F19.10
dd =	JUD-1720981.5D0
zzz = dd					!integer part
dd = dd - zzz					!decimal part
UT = 24. * dd					!UT always positive or 0.0

yea = (zzz-122.1) / 365.25
iii = 365.25 * yea
mon = (zzz-iii) / 30.6001
iii = iii + 30.6001 * mon
DAY = zzz-iii
if mon .GT. 13 then
   mon = mon-12
   yea = yea+1
endif
MONTH = mon-1
YEAR = yea

if ut .ne. 0.0 then
  HOUR = m$nint(UT-0.5)
  dd   = ( UT - HOUR ) * 60.
  if dd .gt. 0.0 then
    MIN  = m$nint(dd-0.5)
    SEC  = ( dd - MIN ) * 60.
  endif
endif
write/key outputc " " ALL
outputc = "{YEAR}-{MONTH}-{DAY}T{HOUR}:{MIN}:{SEC} ({UT})"
outputi(1) = YEAR
outputi(2) = MONTH
outputi(3) = DAY
outputi(4) = HOUR
outputi(5) = MIN
outputi(6) = SEC
outputr(1) = UT

mess/out 4 jd2date "{JUD} -> {outputc}"

