! $Id: uves_messages.prg,v 1.2 2001/01/04 10:39:44 amodigli Exp $
!+++ UVES_MESSAGES ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT	(C) 1998  European Southern Observatory
!
!.IDENT		uves_messages.prg
!
!.KEYWORDS	Put a message on the terminal in a standard format.
!
!.USAGE		@p (IDENT) id pname text
!
!.INPUT		id	(P1): message id (valid values see below)
!		pname	(P2): name of calling program
!		text	(P3): message text
!
!.OUTPUT	terminal output
!
!.RETURN	Q1:  0: successful return
!		    -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE	Put a message on the terminal in a standard format. All
!		outputs are logged in the MIDAS log file.
!
!.ALGORITHM	A message-id controls the way the message should be displayed
!		on the terminal. At the moment there exist 5 different id's:
!
!		     id   output                     bell
!		    ----+--------------------------+------
!		     1: | ERROR   [prgname]: text  | ON
!		     2: | WARNING [prgname]: text  | OFF
!		     3: | INFO    [prgname]: text  | OFF
!		     4: | INFO    [prgname]: text  | OFF
!
!		The negative counterpart writes no \n at the end of the message
!		text.
!
!		Your text may be formated using the usual C-syntax (characters
!		and \n,\r,\t,\b). Parameter specifications like %d are not
!		possible.
!
!		There are 2 ways to pass the message-text to the procedure.
!		The first, and usual way: as parameter P3, but here you are
!		restricted to 100 characters (length of MIDAS-token).
!		The second, is to write your text to a global or lower-level
!		keyword CBUF. MIDAS procedure UVES_APPEND allows to create
!		messages longer then 100 characters.
!
!		CR-LF (=\n) have to be controlled by the calling program!
!
!		The message-level may be set by a global variable MESS_LEVEL.
!		All messages will be send to the terminal having an ID less
!		equal MESS_LEVEL. The MIDAS procedure UVES_VERBOSE controls
!		this keywords.
!
!.ENVIRON	UVES context
!   
!.LANGUAGE	MIDAS 98NOV
!
!.AUTHOR	Sebastian Wolf, ESO-DMO
!
!.VERSION	0.5 1998/03/03	Creation
!
!.COMMENT	Uses global keywords MESS_LEVEL and CBUF
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_messages.prg,v $
! Revision 1.2  2001/01/04 10:39:44  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
! Check parameters
!
define/param p1 ? i "Type of message (message-id):"
define/param p2 ? c "Name of calling program: "
define/param p3 + c "Message text: "
define/param p4 0 i "Enter additional indent after \n:"
!
! local variable declaration
!
define/local ID/i/1/1 {P1}
define/local INDENT/i/1/1 {P4}
define/local PLEN/i/1/1 0
define/local CLEN/i/1/1 0
define/local PNAM/c/1/100 {P2}

info/keyw MESS_LEVEL		!check message level set by VERBOSE/OUT
if mid$info(1) .eq. 1 then
  if m$abs(ID) .gt. MESS_LEVEL then
    info/keyw CBUF
    clen = mid$info(2)
    write/key CBUF/c/1/{clen} " " ALL	!clear buffer
    return -1
  endif
endif


if {P1} .eq. 0 return -1	!do not support id==0
				!==> id==0 NO OUTPUT at all



if P3(1:1) .ne. "+" then	!message text is a parameter
  plen = m$strlen(P3)
  info/keyw CBUF
  clen = mid$info(2)
  if mid$info(1) .eq. 3 then	!character buffer already exists
    if plen .lt. clen then
      write/keyw CBUF/c/1/{clen} " " ALL
    else
      del/key CBUF
      write/keyw CBUF/c/1/{plen} " " ALL
    endif
  else				!character buffer does not exist
    write/key CBUF/c/1/500 " " ALL
  endif
  write/key CBUF {P3}
endif
plen = m$len(PNAM)		!length of program name
info/keyw CBUF
clen = mid$info(2)		!size of character keyword CBUF

!run LPIPE_EXE:uves_message.exe
run PIPE_EXE:uves_message.exe

write/key CBUF/c/1/{clen} " " ALL	!clear buffer
return 0
