! $Id: uves_mkmaster.prg,v 1.20 2001/09/08 15:28:45 amodigli Exp $
!+++ UVES_MKMASTER ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_mkmaster.prg
!
!.KEYWORDS      spectroscopy, echelle, master creation, background subtraction.
!
!.USAGE         @p (IDENT) input catalog refer bmode dmode bmeasur
!
!.INPUT         input   (P1): image catalog with calibration frames
!				bias, dark, flatfield, or arclamp exposures.
!               refer   (P3): reference catalog, should contain:
!				. master bias and/or master dark.
!				. DRS setup table  (for flats)
!				. background table (for flats)
!		bmode	(P4): bias subtraction mode:
!				M: take the master frame,
!				S: take a smoothed version of the master,
!				   append smoothing window: S,<x>,<y>
!				   (see FILTER/SMOOTH)
!				A: take the average of the master,
!				   subtract this constant.
!				<num>: Specify a number to be subtracted.
!		dmode	(P5): dark subtraction mode:
!				M: take the master frame,
!				S: take a smoothed version of the master,
!				   append smoothing window: S,<x>,<y>
!				   (see FILTER/SMOOTH)
!				A: take the average of the master,
!				   subtract this constant.
!				<num>: Specify a number to be subtracted.
!		bmeasur (P6):	background measuring method
!				MINIMUM: take the minimum within a box (BKGRAD)
!					 at the background positions defined by
!					 the background table of the reference
!					 catalog.
!				MEDIAN:  take the MEDIAN instead.
!
!
!.OUTPUT        catalog (P2): output catalog for products
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!		Q2: return the classification of the main product
!
!.DEFAULT	P2	+
!		P3	+
!		P4	M
!		P5	M
!		P6	MEDIAN
!
!.PURPOSE       Do an averaging of the calibration frames in pixel-pixel space.
!
!.ALGORITHM     Check existance of master frames, then reduce the input frames.
!		Bias and dark subtraction may be done. The dark frame will be
!		normalized in time before subtraction.
!		Having reduced every input frame determine MEDIAN stack average
!		as master frame.
!		For flatfields do a background subtraction at the end.
!		A REDFLAG array will be set accordingly to the reduction steps
!		made:	REDFLAG(1) = 1	bias subtraction.
!			REDFLAG(2) = 1	dark subtraction.
!			REDFLAG(3) = 1	interorder background subtraction.
!		which is saved as a descriptor in the product frame.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/24  Created.
!		1.0 1999/06/10	Get the correct pathid.
!		    1999/07/01	Output and reference catalog are optional, now.
!                   1999/07/13	Crossref command implemented.
!                   1999/07/21	Do the background subtraction for flatfields.
!                   1999/09/03	Name of master frame contains data type and
!                   		optionally the central wavelength.
!                   1999/09/16	Parameters for classify/product properly set.
!                   1999/09/22	FORCE_DRS implemented.
!                   1999/09/23	BIAS: skip reduction, goto rskip.
!                   1999/10/14	Name of master frame bug fixed!
!                   1999/11/17	The _l, _u, _b was missing in the product name.
!                   1999/12/10	return the main product as Q2
!
!.COMMENT	(1) No quality control at the moment!
!		(2) The NORMALIZATION of the flatfields may be done only within
!		science-reduction recipe because of the extraction weights in
!		the object window of the echelle orders.
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.20 $
! $Log: uves_mkmaster.prg,v $
! Revision 1.20  2001/09/08 15:28:45  amodigli
! PIPEFILE->h_pipefile
!
! Revision 1.19  2001/08/12 16:14:50  amodigli
! Passing p_version as string par
!
! Revision 1.18  2001/08/07 14:04:47  amodigli
! Added writing of ESO.PRO.DATAMED in f7.3
!
! Revision 1.17  2001/08/07 10:18:14  amodigli
! Fixed a MIDAS naming convention on master
! bias in case WLEN key is negative.
! Changed format of DATAMED to f7.3
! added a replace/ima in case of RED frames
! in structX/Y calculation as asked from RHA
!
! Revision 1.16  2001/07/15 14:18:58  amodigli
! Eliminated unnecessary call to uves_getpversion.prg
!
! Revision 1.15  2001/07/04 07:53:44  amodigli
! New QC1 key naming convention
!
! Revision 1.14  2001/06/20 14:21:14  amodigli
! Using QC1 LOG and PAF files
!
! Revision 1.13  2001/06/16 07:01:36  amodigli
! Changing QC-LOG naming conventions
! Added QC-LOG in addition to FITS log
!
! Revision 1.12  2001/05/22 09:31:35  amodigli
! Fixed small bug in passing frame to MIDAS
! proc (noticed only on alpha)
!
! Revision 1.11  2001/05/04 14:01:00  amodigli
! New bkg parameter setting.
! Added bkg image as data product
!
! Revision 1.10  2001/03/12 15:50:03  amodigli
! Eliminated some improper comments
!
! Revision 1.9  2001/03/12 07:28:04  amodigli
! Sobstitute
! CREATE/TRAP with FIND/TPIX
! CORRECT/TRAP with CORRECT/TPIX
!
! Revision 1.8  2001/03/05 17:42:18  amodigli
! Used real UVES commands
! CREATE/TRAP CORRECT/TRAP
! for trap column correction
!
! Revision 1.7  2001/03/05 15:30:37  amodigli
! Added automatic correction of trap column
!
! Revision 1.6  2001/02/09 17:26:08  amodigli
! Added First set of PRO QC from DFO.
!
! Revision 1.5  2001/01/10 08:43:13  amodigli
! Fixed a small bug in Master flat field
! evaluation as spotted by VH
!
! Revision 1.4  2001/01/04 10:39:43  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
crossref input catalog refer bmode dmode bmeasur
!
define/param p1 ? c "Enter name of input catalog:"
define/param p2 + c "Enter name of output catalog:"
define/param p3 + c "Enter name of reference catalog:"
define/param p4 M c "Enter bias subtraction mode ([M],S,A,<number>):"
define/param p5 M c "Enter dark subtraction mode ([M],S,A,<number>):"
define/param p6 MEDIAN c "Enter background measur. method ([MEDIAN],MINIMUM):"

define/local pid/c/1/10 "mkmaster" ? +lower

define/local ifrm/c/1/100 " " all
define/local modext/c/1/25 " " ALL
define/local main_prod/c/1/25 " " ALL
define/local incat/c/1/25 " " ALL
define/local catalogs/c*25/1/3 " " ALL +lower
define/local dtyp/c*16/1/3 " " ALL +lower
define/local frmtypid/i/1/1 0
define/local i1/i/1/1 0
define/local REDFLAG/i/1/3 0 all +lower
define/local date/c/1/30 " " all +lower_levels

dtyp(1) = "LOWER RED frames"
dtyp(2) = "UPPER RED frames"
dtyp(3) = "BLUE frames"

if NICE_CREA .eq. "Y" assign/disp d,1 >Null		!Select display
set/graph						!Reset graphic
set/graph pmode=1					!Set global outlook


							!Check output catalog
if m$exist(p2) .eq. 0 .AND. p2(1:1) .ne. "+" -
  create/icat {p2} NULL {UVES_IDSTRNG} >Null

set/format I1

							!Split input catalog
catalogs(1) = "mkm{EXREDL}"				!into 3 catalogs:
catalogs(2) = "mkm{EXREDU}"				!mkm{EXREDL},
catalogs(3) = "mkm{EXBLUE}"				!mkm{EXREDU},
@p uves_mkmaster,SPLIT_CATAL {p1}			!mkm{EXBLUE}.
frmtypid = {q1}						!Save the frame type id

do i1 = 1 3						!Loop through catalogs
  PATHID = i1
  incat = catalogs({i1})
  show/icat {incat} >Null				!Check if catalog is
  if outputi(1) .gt. 0 then				!not empty -> AVERAGE.
    write/out " "
    mess/out 3 {pid} "Start doing the master creation on {dtyp({PATHID})} ..."
    @p uves_mkmaster,AVERAGE -
       {incat}.cat {p3} {p2} {p4} {p5} {p6} {frmtypid}
    if {q1} .lt. 0 return -1
    main_prod = q2
  endif
enddo

return 0 {main_prod}


!=== SUBROUTINES =============================================================

ENTRY SPLIT_CATALOG
define/param p1 ? c "Enter name of input image catalog:"

define/local ifrm/c/1/100 " " ALL
define/local modext/c/1/25 " " ALL
define/local frmtyp/c/1/25 " " ALL
define/local catal/i/1/1 0
define/local i1/i/1/1 0

creat/icat {catalogs(1)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {catalogs(2)}.cat NULL {UVES_IDSTRNG} >Null
creat/icat {catalogs(3)}.cat NULL {UVES_IDSTRNG} >Null
ILOOP:						!Loop through input image
  store/frame ifrm {p1} 1 ELOOP			!catalog in order to distribute
  						!input frames to separate
						!image catalogs being all of
  frmtyp = "{{ifrm},{UVES_IDSTRNG}}"		!the same data part.
  get/idex {frmtyp}				!Get identifier extension
  modext = q3

  branch modext(1:5) {EXREDL},{EXREDU},{EXBLUE} REDL,REDU,BLUE
  goto endbranch

REDL:	
  add/icat mkm{EXREDL}.cat {ifrm} >Null
  goto endbranch
REDU:
  add/icat mkm{EXREDU}.cat {ifrm} >Null
  goto endbranch
BLUE:
  add/icat mkm{EXBLUE}.cat {ifrm} >Null
  goto endbranch

endbranch:
goto ILOOP
ELOOP:

if m$index(frmtyp,"BIAS") .eq. 1 then
  i1 = 1
else if m$index(frmtyp,"DARK") .eq. 1 then
  i1 = 2
else if m$index(frmtyp,"FLAT") .eq. 1 then
  i1 = 3
else
  i1 = 0
endif

return {i1} {frmtyp}



!..............................................................................
ENTRY AVERAGE
define/param p1 ? c "Enter name of input image catalog:"
define/param p2 ? c "Enter name of reference image catalog:"
define/param p3 ? c "Enter name of output catalog:"
define/param p4 ? c "Enter bias subtraction mode (M,S,A,<number>):"
define/param p5 ? c "Enter dark subtraction mode (M,S,A,<number>):"
define/param p6 MINIMUM c "Enter background measur. method (MEDIAN, MINIMUM):"
define/param p7 ? c "Enter datatyp (1:BIAS,2:DARK,3:FLAT):"

define/local ifrm/c/1/100 " " ALL
define/local mbias/c/1/100 " " ALL
define/local mdark/c/1/100 " " ALL
define/local refcat/c/1/30 " " ALL
define/local redcat/c/1/30 " " ALL
define/local do_class/c/1/25 " " all
define/local main_prod/c/1/25 " " all
define/local mnam/c/1/100 " " ALL
define/local modext/c*4/1/3 " " ALL
define/local catal/i/1/1 0
define/local dkexptime/r/1/1 1.0
define/local ffexptime/r/1/1 1.0

modext(1) = "REDL"
modext(2) = "REDU"
modext(3) = "BLUE"

mdark = "0.0"
mbias = "0.0"
refcat = "c_{p1}"
create/icat {refcat} NULL {h_procatg} >Null	!reference frame catalog
redcat = "r_{p1}"
create/icat {redcat} NULL {UVES_IDSTRNG} >Null	!reduced frame catalog


if {p7} .le. 1 then				!Bias or unclassified frames:
  -copy {p1} {redcat}				!reduced frames = input frames
  store/frame ifrm {p1} 1			!Get first input frame
  do_class = "{{ifrm},{UVES_IDSTRNG}}"		!Get DO_CLASSIFICATION.
  @p uves_mkmaster,MNAME {ifrm} {do_class}
  mnam = q1					!Name of master frame.
  goto rskip					!Skip search for reference.
endif
if {p7} .eq. 2 goto BIA				!Skip search for dark.
						!Fetch reference frames:
if p2(1:1) .ne. "+" then			!Get master dark
  search/icat {p2} {MDARK_ID({PATHID})} NB
else
  goto CDRK					!Check constant dark
endif
if m$strlen(out_a) .eq. 0 then
CDRK:
  mdark = "0.0"
  if m$tstno(p5) .eq. 1 then			!A constant dark has been
    mdark = "{p5}"				!passed to the recipe
    mess/out 4 {pid} "Use as dark: {mdark}" 	!use this for subtraction.
    REDFLAG(2) = 1
  else
    REDFLAG(2) = 0				!Nothing specified for dark
    mdark = "0.0"
  endif
else
  mdark = out_a
  if p5(1:1) .eq. "M" then			!Take it as a MASTER frame
    add/icat {refcat} {mdark} >Null
  else if p5(1:1) .eq. "S" then			!Take the SMOOTHED version
    filter/smooth {mdark} s_{mdark} {p5(3:)}
    -rename s_{mdark} {mdark}
    add/icat {refcat} {mdark} >Null
  else if p5(1:1) .eq. "A" then			!Take the AVERAGE of the frame
    statistic/image {mdark} ? ? ? SN
    mdark = "{outputr(3)}"
  endif
  mess/out 4 {pid} "Use as dark: {mdark}"
  REDFLAG(2) = 1
endif

if m$tstno(mdark) .ne. 1 then			!master dark frame specified
  dkexptime = m$value({mdark},O_TIME(7))	!Get exposure time.
else
  dkexptime = 1
endif


BIA:
if p2(1:1) .ne. "+" then
  search/icat {p2} {MBIAS_ID({PATHID})} NB	!Get master bias
else
  goto CBIA					!Check constant bias
endif
if m$strlen(out_a) .eq. 0 then
CBIA:
  mbias = "0.0"
  if m$tstno(p4) .eq. 1 then			!A constant bias has been
    mbias = "{p4}"				!passed to the recipe
    mess/out 4 {pid} "Use as bias: {mbias}"	!use this for subtraction.
    REDFLAG(1) = 1
  else
    REDFLAG(1) = 0				!Nothing specified for bias
    mbias = "0.0"
  endif
else
  mbias = out_a
  if p4(1:1) .eq. "M" then			!Take it as a MASTER frame
    add/icat {refcat} {mbias} >Null
  else if p4(1:1) .eq. "S" then			!Take the SMOOTHED version
    filter/smooth {mbias} s_{mbias}
    -rename s_{mbias} {mbias}
    add/icat {refcat} {mbias} >Null
  else if p4(1:1) .eq. "A" then			!Take the AVERAGE of the frame
    statistic/image {mbias} ? ? ? SN
    mbias = "{outputr(3)}"
  endif
  mess/out 4 {pid} "Use as bias: {mbias}"
  REDFLAG(1) = 1
endif


outputi(1) = REDFLAG(1) + REDFLAG(2)		!Check if reduction may be
if outputi(1) .eq. 0 then			!done.
  append/out "Neither bias nor dark as reference specified !\n"
  append/out "--> assume 0.0 for both !"
  mess/out 2 {pid}
!  goto SKIP
endif
  
						!Now do the checks for the
if {p7} .eq. 3 then				!flatfields:
  search/icat {p2} {DRSRC_ID({PATHID})} NB	!Get DRS_SETUP
  if m$strlen(out_a) .ne. 0 then
    savini/echelle {out_a} R
  else
    append/out "No appropriate DRS setup table found!"
    if FORCE_DRS(1:1) .eq. "Y" goto SKIP
    mess/out 2 {pid}
    mess/out 3 {pid} "Use global keyword settings (SHOW/ECHELLE)!"
  endif
  search/icat {p2} {BACK_ID({PATHID})} NB	!Get BACKGROUND TABLE
  if m$strlen(out_a) .ne. 0 then
    set/echelle BAKTAB={out_a}
  else
    append/out "background position table is missing!\n"
    goto skip
  endif
endif



						!Now loop through the input
rloop:						!frames and do the reduction
  store/frame ifrm {p1} 1 endrloop
  mess/out 3 {pid} "Reduce input frame: `{ifrm}' ..."
  if {P7} .eq. 3 then				!scale flatfields by exposuret.
    ffexptime = m$value({ifrm},O_TIME(7))
    if ffexptime .le. 0.0 then
      append/out "bad exposure time: {ffexptime}!\n"
      goto skip
    endif
  else
    ffexptime = 1.0
  endif
  compute/image r_{ifrm} = ({ifrm}- {mbias})/{ffexptime} - {mdark}/{dkexptime}
  add/icat {redcat} r_{ifrm} >Null
  do_class = "{{ifrm},{UVES_IDSTRNG}}"
  goto rloop
endrloop:

@p uves_mkmaster,MNAME {ifrm} {do_class}	!Get the name of the master
mnam = q1

rskip:
if NICE_CREA .eq. "Y" then
  clean/disp D					!Clear displays
  show/icat {redcat} >Null			!Display the reduced input data
  @p uves_mkmaster,PLOT {redcat} {outputi(1)} 1	!on image display 1
  show/icat {refcat} >Null			!Display the reference data
  @p uves_mkmaster,PLOT {refcat} {outputi(1)} 3	"Reference data used."
						!on image display 3
endif
mess/out 3 {pid} "Do the averaging ..."


						!Do the averaging of the input
AVERAGE/IMAGE {mnam} = {redcat} N ? MEDIAN,0,0	!data by taking the MEDIAN of
						!all input data at each pixel.


if {p7} .eq. 3 then				!Do the background subtraction
  mess/out 3 {pid} "Subtract background ..."
  flat = mnam
  flat(1:2) = "av"
  -rename {mnam}.bdf {flat}.bdf
  if MESS_LEVEL .eq. 4 show/eche b		!Show the ECHELLE settings
  select/grap {pathid}				!Select graphics window
  write/key   OLDMTD/C/1/12 {BKGMTD}
  

!am: new code 
  !show/echelle BKGSMO
  -copy {flat}.bdf pre_bg_sub_{flat}.bdf
  subtract/back {flat} tmp_bg_{flat}.bdf {mnam} P6={p6} >Null
  define/local sizeX/i/1/1  0
  define/local sizeY/i/1/1  0
  sizeX = m$value({flat}.bdf,NPIX(1))
  sizeY = m$value({flat}.bdf,NPIX(2))
  !w/o "sizeX={sizeX}"
  !w/o "sizeY={sizeY}"
  define/local radSmoX/i/1/1 25
  define/local radSmoY/i/1/1 10
  if "{do_class}" .eq. "FLAT_BLUE" then
     radSmoX = 25 * {sizeX}/4096
  else
     radSmoX = 50 * {sizeX}/4096
  endif
  define/local wlen/i/1/1 0
  wlen = m$value({flat}.bdf,{h_cwlen({PATHID})})
  if wlen .eq. 860 then
     radSmoY = 300 * {sizeY}/2048
  else
     radSmoY = 100 * {sizeY}/2048
  endif
  mess/out 3 {pid} "radSmoX={radSmoX} radSmoY={radSmoY}"
  filter/smooth tmp_bg_{flat}.bdf  bg_{flat(3:)} {radSmoX},{radSmoY} >Null
  compute/image {flat} = pre_bg_sub_{flat}.bdf - bg_{flat(3:)}
!am: end new code


  write/desc bg_{flat(3:)} IDENT "Background image"
  if BKGVISU(1:1) .eq. "Y" .AND. NICE_CREA .eq. "Y" then
    hardcopy/plot N,G bg_{flat(3:)}.ps
    assign/disp 3
    load/ima bg_{flat(3:)} sca=f
  endif
  REDFLAG(3) = 1
endif




copy/kd REDFLAG/i/1/3 {mnam} REDFLAG/i/1/3	!create REDFLAG descriptor
do_class = "MASTER_{do_class}"			!classify product
main_prod = "{do_class}"
classify/product {mnam} {do_class} {d_pre} {d_empty} OK y y
if p3(1:1) .ne. "+" add/icat {p3} {mnam} >Null	!Add to output catalog [option]
! am: adding Master FF bkg (if exist, usually yes)
if m$exist("bg_{flat(3:)}.bdf") .eq. 1 then
      redflag(1) = 2
      redflag(2) = 2
      redflag(3) = 2
      copy/kd REDFLAG/i/1/3 bg_{flat(3:)}  REDFLAG/i/1/3       !create REDFLAG descriptor
      do_class = "BKG_FLAT_{modext({PATHID})}"                           !put diff value from MFF
      classify/product bg_{flat(3:)}.bdf {do_class} {d_pre} {d_pho},{d_geo},{d_det} OK y n
      if p3(1:1) .ne. "+" add/icat {p3} bg_{flat(3:)}.bdf >Null	!Add to output catalog [option]
endif

!am 08/02/01: Adding QC key
   define/local dpr_type/c/1/80 " " all
   define/local tpl_id/c/1/80 " " all
   define/local arcfile/c/1/80 " " all
   define/local pro_catg/c/1/80 " " all  

   date   = "{{mnam},ESO.TPL.START}"
   qc1log/out 2
   qc1log/out 1 "{mnam}.bdf" "{h_qc_did}"  "{d_qc_did}"          "Data Dictionary for QC"
   qc1log/out 1 "{mnam}.bdf" "PRO.REC1.PIPE.ID" "{p_version}" "Pipeline (unique) identifier" PRO
   if {p7} .eq. 1 then
      qc1log/out 1 "{mnam}.bdf" "QC.TEST1.ID" "Test-on-Master-Bias"  "Name of QC Test"
   else if {p7} .eq. 3  then
      qc1log/out 1 "{mnam}.bdf" "QC.TEST1.ID" "Test-on-Master-Flat"  "Name of QC Test"
   endif
   define/local counter/i/1/1 0

!Loop over input frames catalog
   catal = 0
   store/frame ifrm {redcat} 1 LAB1
   if m$existd(ifrm,h_dprtype) .eq. 1 then
      dpr_type = "{{ifrm},{h_dprtype}}"
      qc1log/out 1 "{mnam}.bdf" {h_dprtype} "{dpr_type}" + DPR
    else
     w/o "no dprtype key found"
    endif
    if m$existd(ifrm,h_tpid) .eq. 1 then
       tpl_id = "{{ifrm},{h_tpid}}"
       qc1log/out 1 "{mnam}.bdf" {h_tpid} "{tpl_id}" + TPL
    else
     w/o "no tp id key found"
    endif
   catal = 0
LAB1:   
   set/format i1
   store/frame ifrm {redcat} 1 LAB2
   if m$existd(ifrm,h_arcfile) .eq. 1 then
      counter = counter +1
      arcfile = "{{ifrm},{h_arcfile}}"
      qc1log/out 1 "{mnam}.bdf" {h_arcfile} {arcfile} + PRIMARY-FITS
   endif
   goto LAB1
   set/format
LAB2:    
   catal = 0  	
   store/frame ifrm {redcat} 1 LAB3
   if m$existd(ifrm,h_procatg) .eq. 1 then
      pro_catg = "{{ifrm},{h_procatg}}"
      !NB here we use main_prod to write the mainprod as 
      !   mnam.bdf does not have yet h_procatg set properly
      qc1log/out 1 "{mnam}.bdf" {h_procatg} {main_prod} + PRO
   endif
   qc1log/out 1 "{mnam}.bdf" {h_pipefile} "{mnam}.bdf" + PRO
  @p uves_mkmaster.prg,QCLogComm "{redcat}" "{mnam}.bdf"
if {p7} .eq. 1 then

   if "{MBIAS_ID({PATHID})}" .eq. "MASTER_BIAS_REDL" then
      FIND/TPIX {mnam} bp.tbl
      CORRECT/TPIX {mnam} bp.tbl A
   endif
   @p uves_mkmaster.prg,QCLogBias "{redcat}" "{mnam}.bdf"
else if {p7} .eq. 3  then
   @p uves_mkmaster.prg,QCLogFlat "{mnam}"
endif
qc1log/out 3 
  
if NICE_CREA .eq. "Y" then
  assign/disp d,2
  load/ima {mnam} sca=f cuts=f,3sig
  if NICE_HCOPY(1:1) .eq. "Y" hardcopy/plot P {mnam}.ps {mnam}
endif

mess/out 3 {pid} "Master: {mnam}.bdf ({main_prod}) created.\n"
return 0 "{main_prod}"   !{do_class}

SKIP:
append/out "-> Skip master creation !"
mess/out 1 {pid}
return -1


!..............................................................................
ENTRY MNAME
define/param p1 ? c "Enter name of input frame:"
define/param p2 ? c "Enter DO classification:"

define/local mnam/c/1/100 " " ALL
define/local lub/c/1/3 lub

outputc = m$lower(p2)
mnam = "m{outputc(1:1)}"

if m$existd(p1,"{h_cwlen({PATHID})}") .gt. 0 then
  set/format F4.0					!add the wavelength
  if {{p1},{h_cwlen({PATHID})}} .ge. 0 then
     mnam = "{mnam}{{p1},{h_cwlen({PATHID})}}"
  else
     mess/out 2 {pid} "Negative wavelength setting. Probably a Bias Frm."
     mnam = "{mnam}"
  endif 
else
  outputi = m$indexb(p2,"_")
  if outputi .gt. 0 then
    outputi = outputi + 1
    mnam = "{mnam}{p2({outputi(1)}:)}"
  else
    mnam = "{mnam}_{h_nameccd({PATHID})}"
  endif
endif
							!add the binning
set/format F1.0
mnam = "{mnam}_{CCDBIN(1)}x{CCDBIN(2)}"
set/format
if m$existd(p1,"{h_slwidth({PATHID})}") .gt. 0 then
  set/format I2						!add the slit width
  outputi = {{p1},{h_slwidth({PATHID})}} * 10
  mnam = "{mnam}_s{outputi(1)}"
endif

mnam = "{mnam}_{lub({PATHID}:{PATHID})}"
set/format
set/format I1
return {mnam}


!..............................................................................
ENTRY NORM
define/parameter P1 ? c "Enter name of flatfield to be normalized:"
define/parameter P2 ? c "Enter name of reference catalog:"

set/echelle CORRECT="n{p1}" BLAZE="b{p1}" REBMTD=SKIP
FLAT/ECHELLE {p1}
return 0


!..............................................................................
ENTRY PLOT
define/parameter P1 ? c "image catalog which should be displayed"
define/parameter P2 ? n "number of images"
define/parameter P3 ? n "Display ID:"
define/parameter P4 + c "Enter Title:"

if {p2} .le. 0 return -1
outputi(1) = m$sqrt({P2})               !Determine num. of columns and rows
outputr(1) = 1.0*{P2} / outputi(1)      !takes only integer part  --> columns
outputi(2) = m$nint(outputr(1))         !nearest integer (3.5->4) --> rows
outputi(3) = outputi(1) * outputi(2)
if outputi(3) .lt. {P2} then
  if outputi(1) .lt. outputi(2) then
    outputi(1) = outputi(1) + 1
  else
    outputi(2) = outputi(2) + 1
  endif
endif

if p4(1:1) .eq. "+" then
  outputc = "REDUCED IMAGES will build MASTER FRAME"
else
  outputc = "{p4}"
endif
@p uves_multiplt {P1} {outputi(1)} {outputi(2)} -{P3} P7="{outputc}"

return 0
!..............................................................................
!============================
! Adding QC info to FITS file
!============================
!============================
! Common PRO QC FITS Keywords
!============================
ENTRY QCLogComm
define/parameter P1 ? c "Input image catalog : "
define/parameter P2 ? c "Output image frame : "
define/local incat/c/1/80 {P1}
define/local outfrm/c/1/80 {P2}

! calculates DATANCOM
show/icat {incat} >Null			!Display the reduced input data
define/local qc_datancom/i/1/1 0
qc_datancom = {outputi(1)}

set/format i1 f8.4,f8.4
date   = "{{outfrm},ESO.TPL.START}"
qc1log/out 1 {outfrm} "PRO.DATANCOM"  {qc_datancom} "Number of frames combined" PRO
write/descr {outfrm} ESO.PRO.DATANCOM/i/1/1 {qc_datancom}
write/dhelp {outfrm} ESO.PRO.DATANCOM               "Number of frames combined"
set/format

return 0
!..............................................................................
!=====================================================
! Master Bias Specific PRO QC FITS Keywords Generation
!=====================================================

ENTRY QCLogBias
define/parameter P1 ? c "Input image catalog : "
define/parameter P2 ? c "Output image frame : "

define/local incat/c/1/80 {P1}
define/local outfrm/c/1/80 {P2}

!======================
! calculates datamedia
!======================
define/local nx_pix/i/1/1 0  !No of X pix
define/local ny_pix/i/1/1 0  !No of Y pix
define/local sx_pix/i/1/1 0  !Size of X bin in pix
define/local sy_pix/i/1/1 0  !Size of Y bin in pix

define_local sample_x/i/1/1 100 !X size of sampling window in pix
define_local sample_y/i/1/1 100 !Y size of sampling window in pix
define/local x_cent_s/i/1/1 0   !X sampling window starting point
define/local x_cent_e/i/1/1 0   !X sampling window ending point
define/local y_cent_s/i/1/1 0   !Y sampling window starting point
define/local y_cent_e/i/1/1 0   !Y sampling window ending point
date   = "{{outfrm},ESO.TPL.START}"


nx_pix = m$value({outfrm},NPIX(1))
ny_pix = m$value({outfrm},NPIX(2))

sx_pix = m$value({outfrm},{h_xwinbin})
sy_pix = m$value({outfrm},{h_ywinbin})


x_cent_s = (nx_pix - sample_x)/2
x_cent_e = (nx_pix + sample_x)/2
y_cent_s = (ny_pix - sample_y)/2
y_cent_e = (ny_pix + sample_y)/2


define/local upp_treshold/r/1/1 0.0
define/local low_treshold/r/1/1 0.0
define/local extra/r/1/1 0.1
define/local qc_ron_master/r/1/1 0.0

stat/ima {outfrm} [{x_cent_s},{y_cent_s}:{x_cent_e},{y_cent_e}] {SESSOUTV}
upp_treshold = outputr(8)*(1 + extra)
low_treshold = outputr(8)*(1 - extra)
qc_ron_master = outputr(4)

define/local pn/i/1/1 0
set/format i1
pn = {uves_portid({PATHID})}
set/format 

set/format i1 f8.4,f8.4
qc1log/out 1 {outfrm} "QC.OUT{pn}.RON.MASTER"  {qc_ron_master} "Read noise frame in ADU"
set/format

@p uves_mkmaster.prg,QCRonRaw {incat} {outfrm} {x_cent_s} {x_cent_e} {y_cent_s} {y_cent_e}

stat/ima {outfrm} + bins=1 exc={low_treshold},{upp_treshold} {SESSOUTV}
define/local master_median/r/1/1 0.0
set/format i1 f7.3,f7.3
master_median = outputr(8)

qc1log/out 1 {outfrm} "PRO.DATAMED"  {master_median} "Median of pixel values" PRO
write/descr {outfrm} {h_promedi}/r/1/1 {master_median}
write/dhelp {outfrm} {h_promedi} "Median of pixel values"

set/format

!==========================
!Calculates Bias struct
!==========================
define/local min/r/1/1 0.0
define/local max/r/1/1 0.0
define/local struct_col/r/1/1 0.0
define/local struct_row/r/1/1 0.0
define/local tmpfrm/c/1/80 "rep_{outfrm}"
 
!in case of RED frame cuts out values greater than 300.
!as the frame can be affected by this local operation we 
!do the calculation on a copy of the original frame
if "{MBIAS_ID({PATHID})}" .ne. "MASTER_BIAS_BLUE" then
   replace/ima {outfrm} {tmpfrm} 300,>=300.
else
   -copy {outfrm}  {tmpfrm}
endif
average/col avg_col = {tmpfrm}
stat/ima avg_col  {SESSOUTV}
!restricts statistics to +/- 2 ADU around mean
min = outputr(3) - 2
max = outputr(3) + 2 
stat/ima avg_col + exc={min},{max} {SESSOUTV}
struct_col = outputr(4)

average/row avg_row = {tmpfrm}
stat/ima avg_row {SESSOUTV}
!restricts statistics to +/- 2 ADU around mean
min = outputr(3) - 2
max = outputr(3) + 2 
stat/ima avg_row + exc={min},{max} {SESSOUTV}
struct_row = outputr(4)

set/format i1 f8.4,f8.4
qc1log/out 1 {outfrm} "QC.OUT{pn}.STRUCTX"  {struct_row}  "structure in X (bias slope)"
qc1log/out 1 {outfrm} "QC.OUT{pn}.STRUCTY"  {struct_col}  "structure in Y (bias slope)"
set/format
 
return 0

!..............................................................................
!=====================================================
! Master Flat Specific PRO QC FITS Keywords Generation
!=====================================================
ENTRY QCLogFlat
define/parameter P1 ? c "Output image frame : "

define/local outfrm/c/1/80 {P1}

!TBD
return 0
!..............................................................................
ENTRY QCRonRaw
define/parameter P1 ? c "Input image catalog : "
define/parameter P2 ? c "Output image frame : "
define/parameter P3 ? N "X start: "
define/parameter P4 ? N "X end: "
define/parameter P5 ? N "Y start: "
define/parameter P6 ? N "Y end: "

define/local incat/c/1/80 {P1}
define/local outfrm/c/1/80 {P2}
define/local infrm/c/1/80 " " all
define/local catal/i/1/1 0
define/local x_cent_s/i/1/1 {P3}   !X sampling window starting point
define/local x_cent_e/i/1/1 {P4}   !X sampling window ending point
define/local y_cent_s/i/1/1 {P5}   !Y sampling window starting point
define/local y_cent_e/i/1/1 {P6}   !Y sampling window ending point
define/local qc_ron_raw/r/1/1 0.0
date   = "{{outfrm},ESO.TPL.START}"

store/frame infrm {incat} 1			!Get first input frame

stat/ima {infrm} [{x_cent_s},{y_cent_s}:{x_cent_e},{y_cent_e}] {SESSOUTV}
qc_ron_raw = outputr(4)
define/local pn/i/1/1 0
set/format i1
pn = {uves_portid({PATHID})}
set/format 

set/format i1 f8.4,f8.4
qc1log/out 1 {outfrm} "QC.OUT{pn}.RON.RAW"  {qc_ron_raw} "Read noise frame in ADU"
set/format

return 0
!..............................................................................
