! $Id: uves_mplot_bkg.prg,v 1.2 2001/05/18 08:29:05 amodigli Exp $
!+++ UVES_MPLOT_BKG +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_mplot_bkg.prg
!
!.KEYWORDS      keywords
!
!.USAGE         @p (IDENT) in_ima in_ima [switch] [switch] [switch]
!
!.INPUT         in_ima    (P1): input raw ima
!               in_ima    (P2): input bkg ima 
!               [switch]  (P4): switch to create hardcopy (postscript.ps)
!               [switch]  (P4): switch to do normalize ([N]/A)
!               [switch]  (P5): switch to choose abscissa ([Y]/X)
!.OUTPUT        out       plots and if appropriate poscript version of them
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       To do QC on background extraction
!
!.ALGORITHM     For a list of slice position do the plot of intensity vs selected 
!               abscissa in absolute or normalized coordinates displaying the col
!               number and the max of intensity in that slice as labels of each plot
!               In the plot is the Int vs abscissa of the raw ima and the oplot of
!               the same quantity for the bkg ima
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 01FEB
!
!.AUTHOR        Andrea Modigliani, ESO-DMO
!
!.VERSION       0.5 2001/04/06  Created.
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_mplot_bkg.prg,v $
! Revision 1.2  2001/05/18 08:29:05  amodigli
! substitute entry DISPL_TIT with
! call to uves_mplot_title
!
! Revision 1.1  2001/05/18 07:49:07  amodigli
! Renamed uves_backgr_qc.prg as
!  uves_mplot_bkg.prg
! Added uves_mplot_title.prg
!
! Revision 1.4  2001/05/04 14:05:50  amodigli
! Eliminated path to specify output dir
!
! Revision 1.3  2001/05/02 13:52:24  amodigli
! Done some functionality tests
! Disactivated log scale plot option
!
! Revision 1.2  2001/05/01 07:30:07  amodigli
! Saved 1st version with good setting
!
! Revision 1.1  2001/04/06 13:30:14  amodigli
! Added uves_mplot_bkg.prg
!
!------------------------------------------------------------------------------
crossref raw_ima bkg_ima hp_sw scale_sw abscissa_sw

define/par p1 ?   ima "Enter ima to plot"
define/par p2 ?   ima "Enter ima to oplot"
define/par p3 "H" c   "Hardcopy option? : [H]/n"
define/par p4 "N" c   "Normalized/absolute ?: [N]/A "
define/par p5 "Y" c   "Abscissa ?: [X]/Y "


define/local pid/c/1/15 "uves_mplot_bkg" ? +lower_levels

define/local rawima/c/1/80 "{p1}" ? +lower_levels
define/local bkgima/c/1/80 "{p2}" ? +lower_levels
define/local hardcopySw/c/1/30 "{p3}" 
define/local ordinateSw/c/1/30 "{p4}"
define/local abscissaSw/c/1/30 "{p5}"
hardcopySw = m$upper(hardcopySw)
abscissaSw = m$upper(abscissaSw)
ordinateSw = m$upper(ordinateSw)




set/format I1
show/disp >Null                       !check if graph.window already exists
if outputi(12) .lt. 0 then
    CONFIG/DISP P3=0.6
endif
ASSIGN/GRAP G,3
modify/grap W
clear/grap



     if "{abscissaSw(1:1)}" .eq. "X" then
else if "{abscissaSw(1:1)}" .eq. "Y" then
else
   mess/out 3 {pid} "Abscissa {abscissaSw} not supported"
   mess/out 3 {pid} "Allowed only X or Y values"
   mess/out 3 {pid} "As default we set abscissa = Y"
   abscissaSw = "Y"
endif 

     if "{ordinateSw(1:1)}" .eq. "N" then
else if "{ordinateSw(1:1)}" .eq. "A" then
else
   mess/out 3 {pid} "Scaling {ordinateSw} not yet supported"
   mess/out 3 {pid} "Supported only N(ormalized) and A(bsolute)"
   mess/out 3 {pid} "As default we set N scale"
   ordinateSw = "N"
endif 


 

define/local val/i/1/1 0 ? +lower_levels


define/local XLL/r/1/1 30.0 ? +lower_levels
define/local YLL/r/1/1 15.0 ? +lower_levels
define/local XSZ/r/1/1 0.0 ? +lower_levels
define/local YSZ/r/1/1 0.0 ? +lower_levels

@p uves_plt_set_size


define/local x_base/r/1/1 4   !Arbitrary value: 8 plots per row

define/local x_sc/r/1/1 -32 
define/local y_sc/r/1/1 -18
define/local x_res/r/1/1 40.
define/local y_res/r/1/1 24.


!readjusts sub-plot sizes to graphical window size in case one set
!CONFIG/DISPLAY window sizes different from the default
x_sc = x_sc*(XSZ/242)*(7/x_base)   !The ref value 242 is OK for a default 
                                   !X size. x_base, number of plots per 
                                   !raw can now be arbitrary
y_sc = y_sc*(YSZ/147)*(7/x_base)   !The ref value 147 is OK for a default 
                                   !Y size to fit 5 rows along Y 
x_res = x_res*(XSZ/242)*(7/x_base)
y_res = y_res*(YSZ/147)*(7/x_base)



define/local fct/r/1/1 1
define/local nloop0/i/1/1 0
define/local nloop1/i/1/1 0
define/local jloop/i/1/1 0
define/local i/i/1/1 0
define/local j/i/1/1 0
define/local no/i/1/1 16


define/local icol/i/1/{no} 0 ? +lower_levels

define/local binX/i/1/1 0
define/local binY/i/1/1 0
define/local wlen/i/1/1 0

binX = {{rawima},ESO.DET.WIN1.BINY}  !X and Y bins are swapped!
binY = {{rawima},ESO.DET.WIN1.BINX}

set/format i1
get/pathid {rawima}
wlen = m$value({rawima},{h_cwlen({PATHID})})


if "{abscissaSw}" .eq. "Y" then
    if wlen .eq. 346 .or. wlen .eq. 437 then
       icol(1) = 2/{binX}
       icol(2) = 50/{binX}
       icol(3) = 100/{binX}
       icol(4) = 200/{binX}
       icol(5) = 400/{binX}
       icol(6) = 600/{binX}
       icol(7) = 800/{binX}
       icol(8) = 1000/{binX}
       icol(9) = 1200/{binX}
       icol(10) = 1400/{binX}
       icol(11) = 1600/{binX}
       icol(12) = 1800/{binX}
       icol(13) = 2000/{binX}
       icol(14) = 2300/{binX}
       icol(15) = 2600/{binX}
       icol(16) = 3000/{binX}
    else 
       icol(1) = 6/{binX}
       icol(2) = 24/{binX}
       icol(3) = 48/{binX}
       icol(4) = 96/{binX}
       icol(5) = 192/{binX}
       icol(6) = 384/{binX}
       icol(7) = 450/{binX}
       icol(8) = 600/{binX}
       icol(9) = 768/{binX}
       icol(10) = 900/{binX}
       icol(11) = 1200/{binX}
       icol(12) = 1500/{binX}
       icol(13) = 1800/{binX}
       icol(14) = 2400/{binX}
       icol(15) = 3600/{binX}
       icol(16) = 4080/{binX}
    endif
else   ! abscissa is X
   icol(1) = 6/{binY}
   icol(2) = 12/{binY}
   icol(3) = 24/{binY}
   icol(4) = 48/{binY}
   icol(5) = 96/{binY}
   icol(6) = 192/{binY}
   icol(7) = 384/{binY}
   icol(8) = 450/{binY}
   icol(9) = 600/{binY}
   icol(10) = 768/{binY}
   icol(11) = 900/{binY}
   icol(12) = 1200/{binY}
   icol(13) = 1500/{binY}
   icol(14) = 1536/{binY}
   icol(15) = 1920/{binY}
   icol(16) = 2040/{binY}
endif

   nloop0 = no/x_base
   nloop1 = no - nloop0*x_base
set/format 




set/grap tsize=0.5
set/grap colour=1
set/grap pmode=-1
plot/axes
set/grap pmode=2

@p uves_mplot_title "{p1}" "QC on background"

jloop = x_base - 1
nloop0 = nloop0 - 1
do i = 0 nloop0 1
do j = 0 jloop 1
   @p uves_mplot_bkg,DO_PLOT {i} {j} {x_sc} {y_sc} {x_res} {y_res} {ordinateSw} {abscissaSw}
enddo
enddo



i = i+1
nloop1 = nloop1 - 1
if nloop1 .ge. 1 then
   do j = 0 nloop1 1
      @p uves_mplot_bkg,DO_PLOT {i} {j} {x_sc} {y_sc} {x_res} {y_res} {ordinateSw} {abscissaSw}
   enddo
endif

define/local prefix/c/1/10 ""

     if "{ordinateSw(1:1)}" .eq. "N" then
   prefix = "n"
else if "{ordinateSw(1:1)}" .eq. "L" then
   prefix = "l"
else
   prefix = "a"
endif

if "{abscissaSw(1:1)}" .eq. "Y" then
   prefix = "{prefix}y"
else
   prefix = "{prefix}x"
endif

if "{hardcopySw(1:1)}" .eq. "H" then
      copy/grap postscript
      set/format i1
      define/local filename/c/1/80 " " all
      if m$existd(p1,"ESO.PRO.QC.ARCFILE") .eq. 1 then 
          filename = "{{p1},ESO.PRO.QC.ARCFILE}"    
      elseif m$existd(p1,"ARCFILE") .eq. 1 then
          filename = "{{p1},ARCFILE}"
      else  
          filename = "uknown"
      endif
      !this line if you like to have copied the file somewere
      -copy postscript.ps {prefix}"_"{wlen}"_"{h_nameccd({PATHID})}"_"{binX}"x"{binY}"_"{filename}".ps"
endif
if "{hardcopySw(1:1)}" .eq. "H" then
   mess/out 3 {pid} "A Poscript file  {prefix}_{wlen}_{h_nameccd({PATHID})}_{binX}x{binY}_{filename}.ps will be produced"
endif

return 0

!======================================================================
ENTRY DO_PLOT
define/param p1 ? N "Enter X-plot index sequence coordinate:"
define/param p2 ? N "Enter Y-plot index sequence coordinate:"
define/param p3 ? N "Enter X scale :"
define/param p4 ? N "Enter Y scale :"
define/param p5 ? N "Enter X resolution :"
define/param p6 ? N "Enter Y resolution :"
define/param p7 ? c "Normalized/Absolute ?: [N]/A "
define/param p8 ? c "Abscissa? : [X]/Y "



define/local des/c/1/80 " "
define/local i/i/1/1 {p1}
define/local j/i/1/1 {p2}
define/local x_sc/r/1/1 {p3}
define/local y_sc/r/1/1 {p4}
define/local x_res/r/1/1 {p5}
define/local y_res/r/1/1 {p6}
define/local ordinateSw/c/1/1 "{p7}"
define/local abscissaSw/c/1/1 "{p8}"
abscissaSw = m$upper(abscissaSw)
ordinateSw = m$upper(ordinateSw)




define/local x_off/r/1/1 0
define/local y_off/r/1/1 0

define/local xLabOrd/r/1/1  0.0
define/local yLabOrd/r/1/1  0.0
define/local y_lab0/c/1/20 "Int"
define/local lab/c/1/40 0

define/local x_min/r/1/1  0
define/local x_max/r/1/1  4096

define/local y_min/r/1/1  0
define/local y_max/r/1/1  0

define/local tmpCol/i/1/1 0
define/local max/r/1/1 0



if "{abscissaSw(1:1)}" .eq. "X" then
   define/local x_lab0/c/1/20 "X"
   x_max = m$value({rawima},NPIX(1))
else
   define/local x_lab0/c/1/20 "Y"
   x_max = m$value({rawima},NPIX(2))
endif


set/format i1
val = val + 1
tmpCol = {icol({val})}
if "{abscissaSw(1:1)}" .eq. "X" then
   stat/ima {rawima} [<,@{tmpCol}:>,@{tmpCol}] >Null
else
   stat/ima {rawima} [@{tmpCol},<:@{tmpCol},>] >Null
endif
max = outputr(2)

define/local x_thick_big/r/1/1   0
define/local y_thick_big/r/1/1   0
define/local x_thick_small/r/1/1 0
define/local y_thick_small/r/1/1 0
x_thick_big   = 500
x_thick_small = 100
define/local binX/i/1/1 0
define/local binY/i/1/1 0

binX = {{rawima},ESO.DET.WIN1.BINY}
binY = {{rawima},ESO.DET.WIN1.BINX}

define/local treshX1/r/1/1 0
define/local treshX2/r/1/1 0
define/local treshY1/r/1/1 0
define/local treshY2/r/1/1 0


treshX1 = 200/{binX}
treshX2 = 3500/{binX}
treshY1 = 192/{binY}
treshY2 = 1920/{binY}

set/format

if "{abscissaSw(1:1)}" .eq. "X" then     !plot I vs X
if "{ordinateSw(1:1)}" .eq. "N" then     !normalized plot
    compute/ima ref = {rawima} / {max}
    compute/ima over = {bkgima} / {max}

    if max .le. 100 then
       y_thick_big   = 0.1
       y_thick_small = 0.05
       y_max =  1.0
       y_min = -0.1
    else if max .gt. 100 .and. max .lt. 500 then
       y_thick_big   = 0.05
       y_thick_small = 0.01
       y_max =  0.2
       y_min = -0.1
    else if max .gt. 500 .and. max .lt. 1000 then
       y_thick_big   = 0.01
       y_thick_small = 0.005
       y_max =  0.06
       y_min = -0.02
    else
       y_thick_big   = 0.01
       y_thick_small = 0.002
       y_max =  0.03
       y_min = -0.01
    endif
else if "{ordinateSw(1:1)}" .eq. "L" then  !logaritmic plot
       y_thick_big   = 1
       y_thick_small =  -1
    y_min = 1
    y_max = 5
else                       ! absolute plot
       y_thick_big   = 25
       y_thick_small =  5
    y_min = -10
    y_max = 250
endif
else                          !plot I vs Y
if "{ordinateSw(1:1)}" .eq. "N" then  !normalized plot
    compute/ima ref = {rawima} / {max}
    compute/ima over = {bkgima} / {max}

    if max .le. 100 then
       y_thick_big   = 0.1
       y_thick_small = 0.05
       y_max =  1.0
       y_min = -0.1
    else if max .gt. 100 .and. max .lt. 500 then
       y_thick_big   = 0.06
       y_thick_small = 0.01
       y_max =  0.2
       y_min = -0.1
    else if max .gt. 500 .and. max .lt. 1000 then
       y_thick_big   = 0.01
       y_thick_small = 0.005
       y_max =  0.05
       y_min = -0.02
    else
       y_thick_big   = 0.01
       y_thick_small = 0.002
       y_max =  0.03
       y_min = -0.01
    endif
else if "{ordinateSw(1:1)}" .eq. "L" then  !logaritmic plot
       y_thick_big   = 1
       y_thick_small =  -1
    y_min = 1
    y_max = 5
else
       y_thick_big   = 25
       y_thick_small =  5
    y_min = -10
    y_max = 250
endif

endif
    x_off = j*x_res+10 
    y_off = i*y_res+10
    xLabOrd = x_min+(x_max-x_min)*0.2
    yLabOrd = y_min+(y_max-y_min)*1.1

    set/grap colour=1
    overplot/axes {x_min},{x_max},{x_thick_big},{x_thick_small} -
                  {y_min},{y_max},{y_thick_big},{y_thick_small} -
                  {x_sc},{y_sc},{x_off},{y_off} {x_lab0} {y_lab0} 
    !mess/out 3 {pid} "Plot order No. {val}"
    set/format i1
    lab = "col_{icol({val})}"
    set/format f5.3
    lab = "{lab}_max={max}"
    label/graph {lab} {xLabOrd},{yLabOrd},wo 0 2
    set/format 

    @p uves_mplot_bkg,PLOT_DAT {icol({val})} {ordinateSw} {abscissaSw}
return 0

!==========================================================================
!==========================================================================
ENTRY PLOT_DAT
define/param p1 ? N     "Enter column number:"
define/param p2 "N" c   "Enter normalization switch:"
define/param p3 "X" c   "Abscissa?: [X]/Y "


set/grap stype=1
set/grap ssize=1
set/grap colour=1



define/local index/i/1/1 0
define/local iter/i/1/1 0
define/local tmpVal/i/1/1 0  
define/local switch/c/1/1 {p2}

set/format i1


set/grap stype=1
set/grap ssize=1
set/grap colour=1

   tmpVal = {p1}
   if "{switch}" .eq. "N" then
      !w/o "normalizing..."
      if "{p3}" .eq. "Y" then
         OVERPLOT/COL ref @{tmpVal}
         set/grap colour=2
         OVERPLOT/COL over @{tmpVal}
         set/grap colour=1
      else
         OVERPLOT/ROW ref @{tmpVal}
         set/grap colour=2
         OVERPLOT/ROW over @{tmpVal}
         set/grap colour=1
      endif
   else
      !w/o "plot the whole thing " {tmpVal}
      if "{p3}" .eq. "Y" then
         OVERPLOT/COL {rawima} @{tmpVal}
         set/grap colour=2
         OVERPLOT/COL {bkgima} @{tmpVal}
         set/grap colour=1
      else
         OVERPLOT/ROW {rawima} @{tmpVal}
         set/grap colour=2
         OVERPLOT/ROW {bkgima} @{tmpVal}
         set/grap colour=1
      endif
   endif
set/format

return 0
!==================================================================
