! $Id: uves_mplot_title.prg,v 1.3 2001/05/30 16:31:14 amodigli Exp $
!+++ UVES_MPLOT_TITLE ++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 2000  European Southern Observatory
!
!.IDENT         uves_mplot_title.prg
!
!.KEYWORDS      uves multi plotting
!
!.USAGE         @p (IDENT) inframe title 
!
!.INPUT 
!               Name        (p1): Input Frame name
!               Title       (p2): Basic title to display
!          
!.OUTPUT        None
!
!.RETURN        1 on error, 0 else
!
!.DEFAULT	   
!               none
!
!.PURPOSE: 	    Display title for QC check plots (optimal, background extraction) 
!
!.ALGORITHM     Displays a basic title with a few more info
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 01FEB
!
!.AUTHOR        A.Modigliani, ESO/DMD
!
!.VERSION       2001/05/17  Created (AM).
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_mplot_title.prg,v $
! Revision 1.3  2001/05/30 16:31:14  amodigli
! clean/doc
!
! Revision 1.2  2001/05/18 08:25:12  amodigli
! Improved info to display
!
! Revision 1.1  2001/05/18 07:49:07  amodigli
! Renamed uves_backgr_qc.prg as
!  uves_mplot_bkg.prg
! Added uves_mplot_title.prg
!
!------------------------------------------------------------------------------
!
!------------------------------------------------------------------------------
! Who    When        Why               Where
! AM     30/05/01    clean/doc         all routine
!------------------------------------------------------------------------------
!Short description:
! Get from the frame some basic info like arm path id, mode id, central wav,
! binx, biny, dpr type, filename and next plots all as a header title using
! also what provided as p2
!------------------------------------------------------------------------------
crossref inframe title
define/param p1 ? c "Enter input frame name:"
define/param p2 + c "Enter title:"


define/local i1/i/1/1 0
define/local i2/i/1/1 0
define/local x0/r/1/1 0.0
define/local y0/r/1/1 0.0
define/local ystp/r/1/1 0.0
define/local instrum/c/1/8 " " ALL
define/local title/c/1/200 " " ALL
define/local datobs/c/1/25 " " ALL
define/local filt/c/1/20 " " ALL
define/local fname/c/1/100 {p1}
define/local arcfile/c/1/80 " "

    if m$existd(p1,"ESO.PRO.QC.ARCFILE") .eq. 1 then 
       arcfile = "{{p1},ESO.PRO.QC.ARCFILE}"    
elseif m$existd(p1,"ARCFILE") .eq. 1 then
        arcfile = "{{p1},ARCFILE}"
else  
        arcfile = "uknown"
endif

define/local ins_mode/c/1/20 " " all
define/local ins_path/c/1/20 " " all
if m$existd(p1,h_insmode) .eq. 1 then		
   ins_mode = "{{p1},{h_insmode}}"
else
   ins_mode ="mode unknown"
endif
if m$existd(p1,h_inspath) .eq. 1 then		
   ins_path = "{{p1},{h_inspath}}"
else
   ins_path = "path unknown"
endif


define/local binX/i/1/1 0
binX = {{p1},ESO.DET.WIN1.BINY}
define/local binY/i/1/1 0
binY = {{p1},ESO.DET.WIN1.BINX}




define/local user/c/1/80 " "
define/local time/c/1/80 " "

filt = uves_filti(2)				!filter used

!Get date of observ. used for subtitle
@p uves_jd2date {p1}				
i1 = m$len(outputc)
datobs = outputc(1:{i1})


!Take the original filename for the title
if m$existd(p1,h_origname) .eq. 1 then		
  fname = "{{p1},{h_origname}}"			
endif

define/local dpr_type/c/1/80 " " all
if m$existd(p1,h_dprtype) .eq. 1 then		
  dpr_type = "{{p1},{h_dprtype}}"			
endif

if p2(1:1) .eq. "+" then
  instrum = "{{p1},{h_instrume}}"		!Name of instrument
  title = "Echelle data ({instrum}): {fname}"
else
  if p2(1:1) .eq. """ then			!Strip leading and pending
    i1 = 2					!quotes of title parameter
    i2 = m$strlen(p2) - 1
  else
    i1 = 1
    i2 = 100
  endif

  title(1:100) = p2({i1}:{i2})
endif
get/pathid {p1}

user = m$symbol("USER")
time = m$time()

!================================================================

if mid$sess(16:25) .eq. "99NOVpl1.1" then

    ystp = YLL / 6					!Display title
    x0 = XSZ / 2
    y0 = YSZ + 4 * ystp
    label/graph "{title}" {x0},{y0},mm 0 2 0

    y0 = YSZ + 2 * ystp				!Display sub-title (1)
    set/format F5.0	
    outputr(1) = {{p1},{h_cwlen({PATHID})}} * 10.0
    title = "Path: {ins_path}, Mode: {ins_mode}, CCD: {h_nameccd({PATHID})}, Cent.wav.: {outputr(1)} A"
    set/format
    title = "{title}, Filter: {filt}"
    label/graph "{title}" {x0},{y0},mm 0 1 0

    y0 = y0 - ystp					!Display sub-title (2)
    title = "Obs.date: {datobs} Arcfile: {arcfile}"
    label/graph "{title}" {x0},{y0},mm 0 1 0

else
    label/grap "{time}; USER: {user}" 5,5,mm 0 1 1
    !Display title
    label/graph "{title}" 0.5,0.98,no 0 2 0

    !Display sub-title (1)
    set/format F5.0	
    outputr(1) = {{p1},{h_cwlen({PATHID})}} * 10.0
    !outputc = "{XLOW},{XUPP} A "
    
    title = "Path: {ins_path}, Mode: {ins_mode}, CCD: {h_nameccd({PATHID})}"
    set/format i1
    title = "{title}, Cent. wav: {outputr(1)} A, Bin. (Spect x Spat): {binX}x{binY}"
    
    set/format
    label/graph "{title}" 0.5,0.95,no 0 1 0

    !Display sub-title (2)
    title = "DPR Type: {dpr_type} Arcfile: {arcfile}"
    label/graph "{title}" 0.5,0.92,no 0 1 0

endif

return 0



!=============================================================
