! $Id: uves_ordwid.prg,v 1.2 2001/01/04 10:39:38 amodigli Exp $
!+++ UVES_ORDWID ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_ordwid.prg
!
!.KEYWORDS      spectroscopy, echelle, order extraction
!
!.USAGE         @p (IDENT) inframe psname
!
!.INPUT         inframe (P1): echelle flatfield
!		psname	(P2): name of POSTSCRIPT file.
!				set to + in order to display nothing.
!
!.OUTPUT        An order detection table will be created: orddet.tbl
!
!.RETURN        Q1:  order width in pixel (integer)
!                   -1: a successful return failed
!		Q2:  number of orders found
!
!.DEFAULT	P2	+
!
!.PURPOSE       Determine the number of the echelle orders and their widths.
!
!.ALGORITHM     Calculate the derivative of the central column of the input
!		frame. The distance between succeeding +,- peaks defines the
!		the thickness of an order. The mean of all these thicknesses
!		will be returned as the mean width of the echelle orders.
!		The number of all orders on the frame is determined by a
!		gliding mean. The boxwidth is the width of the orders. All
!		datavalues greater than 3/2 std.dev. from the gliding mean
!		are stored in a table. Only the detections with a minimum
!		distance of order width are counted. This number will be
!		returned as the second value by this procedure.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/09  Created.
!		    1999/04/23	Order number detection implemented.
!
!.COMMENT	This routine is necessary as the automatic order detection
!		mode (NBORDI=0) of the DEFINE/HOUGH command does not always
!		detect every order! 
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.2 $
! $Log: uves_ordwid.prg,v $
! Revision 1.2  2001/01/04 10:39:38  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

define/param p1 ? c "Enter name of order flatfield or standard star:"
define/param p2 + c "Enter name of POSTSCRIPT file: "
define/param p3 1.5 n "Enter threshold in units of std.deviation:"
!
! define constants
!
define/local pid/c/1/25 uves_ordwid
define/local YLIM/r/1/1 1000.0
!
! define local variables
!
define/local i1/i/1/1 0
define/local i2/i/1/1 0
define/local ix/i/1/1 0
define/local nx/i/1/1 0
define/local ny/i/1/1 0
define/local num/i/1/1 0
define/local owid/i/1/1 0
define/local x1/r/1/1 0
define/local x2/r/1/1 0
define/local y1/r/1/1 0
define/local y2/r/1/1 0
define/local tim/i/1/1 0

tim = m$secs()

nx = m$value({p1},npix(1))
ny = m$value({p1},npix(2))
ix = nx / 2
i1 = ix - 1
i2 = ix + 1
average/column col1.bdf = {p1} [{i1},{i2}]
col1,START(1) = 1.0

-copy col1.bdf col2.bdf

col1,start(1) = 0.0					!shift 1 pixel

compute/image deriv = col1 - col2			!compute derivative
copy/it deriv.bdf deriv.tbl
name/colum deriv.tbl #1 :Y
!create/colum deriv.tbl :X "pixel" I5 I
!create/colum deriv.tbl :XDIFF "pixel" I5 I
compute/table deriv.tbl :X = SEQ
compute/table deriv.tbl :XDIFF = SEQ

select/table deriv.tbl abs(:Y).gt.{YLIM} >Null
num = outputi(1)					!number of select.rows.
copy/table deriv.tbl derivs.tbl				!extract the peaks

delete/row derivs.tbl {num}				!delete last row
num = num - 2						!as this order is
do i1 = 2 num						!usually incomplete.
  i2 = i1 + 1
  x1 = m$value(derivs.tbl,:X,{i1})
  x2 = m$value(derivs.tbl,:X,{i2})
  y1 = m$value(derivs.tbl,:Y,{i1})
  y2 = m$value(derivs.tbl,:Y,{i2})
  if y1 .gt. 0 .AND. y2 .lt. 0 then			!get only the peaks
    derivs.tbl,:XDIFF,{i1} = x2 - x1			!which determine the
  else							!beginning and the end
    derivs.tbl,:XDIFF,{i1} = 0				!of the orders in y-dir
  endif							!othewise set to 0.0
enddo
delete/row derivs.tbl @{i2}				!delete last row

select/table derivs.tbl :XDIFF.gt.4 >Null
statistic/table derivs.tbl :XDIFF >Null

owid = m$nint(outputr(3))

@p uves_ordwid,GET_ONUM deriv.bdf {owid} {p3} {p2}
num = {q1}

set/format I1
mess/out 4 {pid} -
  "Mean order width in pixel: {owid}\nNumber of orders found:    {num}"

!tim = m$secs() - tim 
!set/format I1
!write/out  "Time used: {tim} s"

return {owid} {num}




ENTRY GET_ONUM
define/param p1 ? i "Enter image name: "
define/param p2 ? n "Enter window width: "
define/param p3 ? n "Enter threshold in units of std.dev.: "
define/param p4 + c "Enter name of POSTSCRIPT file: "

define/local pfrm/c/1/100 " " ALL
define/local dif/r/1/1 0.0
define/local dat/r/1/1 0.0
define/local thresh/r/1/1 0.0
define/local mean/r/1/1 0.0
define/local nstp/i/1/1 {p2}
define/local npx/i/1/1 0
define/local num/i/1/1 0
define/local i1/i/1/1 0
define/local i2/i/1/1 0
define/local j1/i/1/1 0
define/local k1/i/1/1 0
define/local k2/i/1/1 0

nstp = nstp * 2.0
pfrm = "p{p1}"
find/pixel {p1} 0,> IN ALL {pfrm},image >Null
write/desc {pfrm} IDENT/c/1/72 " " ALL
write/desc {pfrm} IDENT/c/1/72 Derivative

npx = m$value({pfrm},NPIX(1))
cuts/imag {pfrm} {p1}

copy/it {pfrm} wrk.tbl				!create a working table
NAME/COLU wrk.tbl :LAB001 :Y			!on which the statistical
compute/table wrk :X = seq			!selection will be done
COPY/TABL wrk.tbl orddet.tbl			!Create empty copy
DELETE/ROW  orddet.tbl @1..{npx}		!of work table --> will finally
						!be the selected orders.
statistic/imag {pfrm} [{nstp}*1] ? ? SN stat.tbl >Null
show/table stat.tbl >Null
num = outputi(2)
k2 = 0
do i1 = 1 {num}
  mean = m$value(stat.tbl,:MEAN,{i1})
  thresh = mean + {p3} * m$value(stat.tbl,:STD,{i1})
  k1 = k2 + 1
  k2 = k2 + nstp
  if k2 .gt. {npx} then
    k1 = npx - nstp + 1				!last iteration:
    k2 = npx					!set window to same width
    num = i1					!force loop to stop at the end.
  endif
  compute/table wrk.tbl :DIF = ABS(:Y - {mean})
  outputc = ":DIF.gt.{thresh}.AND.:Y.gt.0.AND.seq.ge.{k1}.AND.seq.le.{k2}"
  select/table wrk.tbl {outputc} >Null
  merge/table wrk.tbl orddet.tbl w.tbl
  -rename w.tbl orddet.tbl
enddo
compute/table orddet.tbl :FLAG = -1
sort/table orddet.tbl :X
show/tab orddet >Null
num = outputi(2) - 1

do i1 = 2 {num}
  i2 = i1 - 1
  dif = m$value("orddet.tbl",:x,{i1}) - m$value("orddet.tbl",:x,{i2})
  if dif .gt. 1 then
    orddet,:FLAG,{i2} = 1
  else
    orddet,:FLAG,{i2} = -1
  endif
enddo
i1 = num + 1					!The last detected order has
orddet,:FLAG,{i1} = 1				!always to be selected

select/tab orddet :FLAG.gt.0 >Null
num = outputi(1)
if NICE_CREA(1:1) .eq. "Y" .AND. P4(1:1) .ne. "+" then
  statistic/table orddet :Y >Null
  mean = outputr(2)				!maximum in y
  assign/graph G,1
  set/graph default
  set/graph pmode=0
  plot/row {pfrm} >Null
  set/graph colour=2
  overplot/tab orddet :x :y
  set/format I1
  label/graph "{num} orders found" 1.0,{mean} ? ? 1
  copy/graph postscript
  -rename postscript.ps {p4}
endif

return {num}
