! $Id: uves_phymod.prg,v 1.3 2001/01/04 10:39:36 amodigli Exp $
!+++ UVES_PHYMOD +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_phymod.prg
!
!.KEYWORDS      uves physical model, spectroscopy, echelle,
!
!.USAGE         @p (IDENT) wlc lincat
!
!.INPUT         wlc     (P1): wavelength calibration frame
!		lincat	(P2): Line catalog
!			(P3): Measurement box (X,Y)
!			(P4): Translation ccd (X,Y)
!			(P5): Angle offsets (echelle, CD, CCD rot)
!			(P6): Compute regression flag
!
!.OUTPUT        pline.tbl: predicted positions table,
!		modfree_sp_rg.tbl, modfree_sp_rg.dat: free spectral range table 
!		and ascii file.
!
!.RETURN        1 on error, 0 else
!
!.DEFAULT	P1	{wlc}
!		P2	{lincat}
!		P3	40,40
!		P4	0.,0.
!		P5	0.,0.,0.
!		P6  Y
!
!.PURPOSE: 	Physical model of UVES dispersion relation. 
!		Creates the projection table containing the predicted positions 
!		(x,y) from ThAr arc lamp line catalogue.
!               Generates the free spectral range, prepares the configuration,
!		before CALLing "physmodtbl.c" routine.
!
!.ALGORITHM     Creates tables pline.tbl, modfree_sp_rg.tbl, prepares data and
!		configuration before running the physical model. 
!		Midas interface routine with the C routine "physmodtbl.c".
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        O.Boitquin, ESO/DMD
!
!.VERSION       1999/09/01  Created (OB).
!		1999/09/30-SW	. Retrieving of the physical model input
!				  parameter from the input frame improved.
!				. Unnecessary code (checking the configindex)
!				  removed as already done in physmodtbl.c
!				. Terminal output improved.
!				. Binning of phys.model implemented.
!		1999/10/06-OB  New columns added in the "modfree_sp_rg.tbl": 
!		"physmodtbl.exe" computes the pixel-scale taking into account
!		of the anamorphic magnification of CD, length of the line. 
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_phymod.prg,v $
! Revision 1.3  2001/01/04 10:39:36  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!

define/param  P1 {WLC}		IMA   	"Input frame:"
define/param  P2 {LINCAT}	TABL  	"Line catalog:" 
define/param  P3 40,40		NUMB	"Measurement box (X,Y): " 
define/param  P4 0.,0.    	NUMB	"Translation ccd (X,Y): "
define/param  P5 0.,0.,0.   NUMB 	"Angle offsets (echelle, CD, CCD rot):"
define/param  P6 Y		    CHAR 	"COMPUTE/REGR: Yes or No"


set/format F13.7
define/local MBOX/R/1/2 {P3}
define/local trans/D/1/2 {p4}
define/local angoff/D/1/3    {P5}
define/local reg_opt/C/1/1 {P6}
reg_opt = m$upper(reg_opt)

!*****************************************************************
!
! Initialize configuration keywords
!
!*****************************************************************
 !
!   Configuration tables
!
!Index:  1       2       3      4      5      6
!
!Arm:   Blue    Blue    Red    Red    Red    Red
!Chip:  EEV     EEV     EEV    EEV    MIT    MIT
!CDisp:  1       2       3      4      3      4
!

set/format F20.10  I1


!*****************************************************************
!
! Reading configuration information from FITS header
!
!*****************************************************************

define/local arm/C/1/4      	"BLUE"
define/local ArmId/C/1/1	    "b"
define/local chip/C/1/3     	"e"
define/local DO_CLASS/C/1/10 	{{P1},DO_CLASSIFICATION}
define/local modext/c/1/25 " "  ALL    
define/local wcent/D/1/1    	0.
define/local ECHID/I/1/1    	0
define/local CDID/I/1/1     	0
define/local cfgindx/I/1/1   	0
define/local FRSPRG/C/1/20	    "modfree_sp_rg.tbl"
define/local Xtrans/D/1/1   	0.
define/local Ytrans/D/1/1   	0.
define/local echoff/D/1/1   	0.
define/local CDoff/D/1/1   	    0.
define/local ccdrotoff/D/1/1   	0.

! For messout

define/local pid/C/1/20 "uves_phymod"
mess/out 3 {pid} "Determine ThAr line positions using physical model."



config/inst {p1}

! Arm data are read from FITS keyword ESO INS PATH, e.g. 'BLUE'
arm = q2(2:)
ArmId = m$lower(arm(1:1))

chip = m$lower(CCD(1:1))				!Chip identifier (CCD)
!mess/out 3 {pid} "Detector: {CCD} (type: {chip})"
CDID = {CROSS(4:4)}					!Cross-disperser id.
!mess/out 3 {pid} "Cross-disperser number: {CDID}"
wcent = {WCENTER}/10.0					!Central wavel. [nm]
!mess/out 3 {pid} "Central wavelength: {WCENT} nm"

set/format F5.1
define/local tempcam/d/1/1 0. ? +lower_levels		
tempcam = {{p1},{h_tempcam({PATHID})}}			!Temperature [Celsius]

define/local pressure/d/1/1 0. ? +lower_levels
pressure = {{p1},{h_pressure}}				!Air pressure [bar]

! Humidity (ignored for the moment)
define/local humid/d/1/1 0. ? +lower_levels
humid = 3						!humidity
							!=water pressure [mmHg]
define/local slitwidth/d/1/1 0. ? +lower_levels
slitwidth = {{p1},{h_slwidth({PATHID})}}		!slit width [arcsec]

define/local slitlength/d/1/1 0. ? +lower_levels
slitlength = {{p1},{h_sllen({PATHID})}}			!slit length [arcsec]


append/out "UVES-configuration:\n"
append/out "  Optical path: {arm} ({ArmId})\n"
append/out "  Detector: {CCD}, Binning: {CCDBIN(1)},{CCDBIN(2)}\n"
append/out "  Cross-disperser (number): {CROSS} ({CDID})\n"
append/out "  Central wavelength: {WCENT} nm\n"
append/out "  Slit width: {slitwidth} arcsec\n"
append/out "  Slit length: {slitlength} arcsec\n"
append/out "  Temperature: {tempcam} C\n"
append/out "  Air pressure: {pressure} bar\n"
append/out "  Humidity: {humid} mmHg\n" ! (IGNORED)\n"
mess/out 3 {pid}


define/local   MODTAB/C/1/30   pline.tbl 

write/keyword IN_A {P2}
write/keyword OUT_A {MODTAB}
write/keyword FRSPRG "modfree_sp_rg.tbl"
write/keyword Xtrans {trans(1)}
write/keyword Ytrans {trans(2)}
write/keyword echoff {angoff(1)}
write/keyword CDoff  {angoff(2)}
write/keyword ccdrotoff {angoff(3)}

run PIPE_EXE:physmodtbl.exe
if Q1 .eq. "1" return 1

set/format F4.0
assign/print FILE modfree_sp_rg.dat
print/table modfree_sp_rg.tbl
-rename modfree_sp_rg.dat modfree_sp_rg_{wcent(1:3)}_{CCD}.dat
set/format F5.1
 
create/column {MODTAB} :IDENT R*8
compute/table {MODTAB} :IDENT = :WAVE *10.

if MBOX(1) .ne. 0. .and. MBOX(2) .ne. 0. then 
  clear/chan over
  copy/table {MODTAB} &p
  compute/table &p :XSTART = :XMOD - {MBOX(1)}/2.
  compute/table &p :XEND   = :XMOD + {MBOX(1)}/2.
  compute/table &p :YSTART = :YMOD - {MBOX(2)}/2.
  compute/table &p :YEND   = :YMOD + {MBOX(2)}/2.
  draw/rectangle  middummp.tbl  F 
endif

!show/table modfree_sp_rg.tbl {SESSOUTV}
!echord(1) = {outputi(2)} +1
!set/echelle NBORDI = {echord(1)}

return 0

