! $Id: uves_plot_chun.prg,v 1.7 2001/08/08 13:14:45 amodigli Exp $
!+++ UVES_PHYMOD +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 2000  European Southern Observatory
!
!.IDENT         uves_plot_obj_pos_debug.prg
!
!.KEYWORDS      uves optimal extraction QC check
!
!.USAGE         @p (IDENT) OrdNo Xmin,Xmax Ymin,Ymax
!
!.INPUT         OrdNo         (P1): order No to select
!		        Xmin,Xmax     (P2): min and max X coordinate
!			    Ymin,Ymax     (P3): min and max Y coordinate
!
!.OUTPUT        None
!
!.RETURN        1 on error, 0 else
!
!.DEFAULT	    P1	10
!		        P2	0,1500
!		        P3	0,40
!
!.PURPOSE: 	    Optimal extraction QC check. 
!		        Plots chunk positions before fit, after first and final fit to verify
!               optimal extraction accuracy.
!
!.ALGORITHM     Does several multiple selection and correspondent plot/overplot
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        A.Modigliani, ESO/DMD
!
!.VERSION       2000/12/11  Created (AM).
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.7 $
! $Log: uves_plot_chun.prg,v $
! Revision 1.7  2001/08/08 13:14:45  amodigli
! Added again prefix ESO to QC keywords
!
! Revision 1.6  2001/07/04 07:56:10  amodigli
! Fixed small bug in case some comp in P2
! was missing
!
! Revision 1.5  2001/06/28 12:50:41  amodigli
! Eliminated prefix ESO from QC.OBJ...keys
!
! Revision 1.4  2001/06/16 07:04:20  amodigli
! Adapted to new QC naming conventions
! Added check on presence of order No.
!
! Revision 1.3  2001/03/12 06:44:36  amodigli
! Added pid
! Added check on P5 (switch parameter)
!
! Revision 1.2  2001/02/21 09:54:03  amodigli
! Added crossref
!
! Revision 1.1  2001/02/05 16:38:12  amodigli
! Redefined
! uves_plot_multi_debug.prg uves_mplot_chun.prg
! uves_plot_obj_pos_debug.prg uves_plot_chun.prg
!
! Revision 1.6  2001/01/18 13:59:14  amodigli
! Version improoved
!
! Revision 1.5  2001/01/11 14:38:07  amodigli
! Added parameter to switch between
! pos and fwhm plots
!
! Revision 1.4  2001/01/08 12:57:24  amodigli
! Renamed debug.tbl order_trace.tbl
! debug2.tbl xorder_trace.tbl
!
! Revision 1.3  2001/01/08 09:40:36  amodigli
! Added debug table as parameter p1. Shifted the others
!
! Revision 1.2  2001/01/04 10:39:53  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!
crossref intab ord xp_size yp_size switch
!
define/param p1 "order_trace.tbl" c "Enter table name:"
define/param p2 10          i "Enter order No:"
define/param p3 0,1500      c "Enter xMin,xMax:"
define/param p4 0,40        c "Enter yMin,yMax:"
define/param p5 "pos"       c "Enter info to display [pos]/fwhm"


define/local intab/c/1/80 {p1}
define/local ord/i/1/1  {p2}
define/local parXmin/c/1/10 " "
define/local parXmax/c/1/10 " "
define/local parYmin/c/1/10 " "
define/local parYmax/c/1/10 " "
define/local k/i/1/1 0
define/local l/i/1/1 0
define/local info/c/1/80 {p5}
define/local pid/c/1/10 "plot_chun" ? +lower_levels

info = m$upper(info)

k = m$parse(p3,"par")
  
    if k .eq. 2 then
  parXmin = "{par01}"
  parXmax = "{par02}"
else if k .eq. 1 then
  parXmin = "{par01}"
  parXmax = "1500"
else if k .eq. 0 then
  parXmin = "0"
  parXmax = "1500" 
else
 mess/out 3 {pid} "Too many sub-pars for p2, max 2: xMin, xMax"
 return -1
endif

parXmin = m$upper(parXmin)
parXmax = m$upper(parXmax)

define/local x_min/r/1/1  {parXmin}
define/local x_max/r/1/1  {parXmax}






l = m$parse(p4,"val")
  
    if l .eq. 2 then
  parYmin = "{val01}"
  parYmax = "{val02}"
else if l .eq. 1 then
  parYmin = "{val01}"
  parYmax = "0"
else if l .eq. 0 then
  parYmin = "0"
  parYmax = "40" 
else
 mess/out 3 {pid} "Too many sub-pars for p4, max 2: xMin, xMax"
 return -1
endif

parYmin = m$upper(parYmin)
parYmax = m$upper(parYmax)

define/local y_min/r/1/1  {parYmin}
define/local y_max/r/1/1  {parYmax}
define/local x_lab/r/1/1  0.0
define/local y_lab/r/1/1  0.0
define/local lab/i/1/1 0
lab = {ord}

x_lab = x_min+(x_max-x_min)*0.1
y_lab = y_min+(y_max-y_min)*0.9




set/grap xaxis={x_min},{x_max} yaxis={y_min},{y_max}
set/grap stype=1
set/grap ssize=1
set/grap colour=1
!set/grap pmode=-1
!plot/axes
set/grap pmode=2



     if "{info(1:3)}" .eq. "POS" then
else if "{info(1:3)}" .eq. "FWH" then
else
   mess/out 1 {pid} "P5={info} not allowed! Allowed only P5=pos or P5=fwhm"
   return -1
endif 

set/format i1
if "{info(1:3)}" .eq. "POS" then
   if m$existd(intab,"ESO.QC.OBJ{ord}.POS") .eq. 1 then
   else  !sometime an order may be skipped from optimal extraction
      mess/out 2 {pid} "Value of descriptor ESO.QC.OBJ{ord}.POS not present, we skip!"
      return 0
   endif
else
   if m$existd(intab,"ESO.QC.OBJ{ord}.FWHM") .eq. 1 then
   else  !sometime an order may be skipped from optimal extraction
      mess/out 2 {pid} "Value of descriptor ESO.QC.OBJ{ord}.FWHM not present, we skip!"
      return 0
   endif
endif
set/format

select/table {intab} :ORDER.EQ.{ord}
w/o "plot all points as black dots"
if "{info(1:3)}" .eq. "POS" then
   plot/tab {intab} :X_CH_0 :P_CH
else
   plot/tab {intab} :X_CH_0 :P_FWHM
endif 

set/format i1
label/graph {lab} {x_lab},{y_lab},wo 0 2
set/format 

select/table {intab} :ORDER.EQ.{ord}.AND.:M0.EQ.1
set/grap stype=2
set/grap colour=2
w/o "overplot as red exagons the ones used in the first LSFIT"
if "{info(1:3)}" .eq. "POS" then
   overpl/tab {intab} :X_CH_0 :P_CH
else
   overpl/tab {intab} :X_CH_0 :P_FWHM
endif

set/grap stype=3
set/grap colour=3
w/o "overplot as green squares the results of the first fit"
if "{info(1:3)}" .eq. "POS" then
   overpl/tab {intab} :X_CH_0 :P_F0
else
   overpl/tab {intab} :X_CH_0 :S_CH_0
endif

set/grap stype=4
set/grap colour=4
w/o "overplot as blue triangles the results of the last fit"
if "{info(1:3)}" .eq. "POS" then
   overpl/tab {intab} :X_CH_0 :P_F1
else
   overpl/tab {intab} :X_CH_0 :S_CH_1
endif

select/table {intab} :ORDER.EQ.{ord}.AND.:M1.EQ.1
set/grap stype=6
set/grap colour=6
w/o "overplot as magenta croxes the points used in the last fit"
if "{info(1:3)}" .eq. "POS" then
   overpl/tab {intab} :X_CH_0 :P_CH
else
   overpl/tab {intab} :X_CH_0 :P_FWHM
endif

return 0
!------------------------------------------------------------------------------
