! $Id: uves_plt_dispres.prg,v 1.18 2001/09/08 15:26:09 amodigli Exp $
!+++ UVES_PLT_DISPRES +++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_plt_dispres.prg
!
!.KEYWORDS      spectroscopy, echelle, resolution
!
!.USAGE         @p (IDENT) resol wlc yrange disp
!
!.INPUT         resol   (P1): name of resolution table
!				product of CALC/RESOL of echelle context
!               wlc     (P2): wavelength calibration frame
!               yrange  (P3): plot ranges in y (4 values)
!				min1,max1,min2,max2
!				for the deltaX and resolution, respectively
!               disp    (P4): display flag:
!				YES: show result on graphics window
!				NO:  create a postscript file only
!
!.OUTPUT        postscript.ps.
!
!.RETURN        
!
!.DEFAULT	p1	&resol
!		p2	{wlc}
!		p3	+
!		p4	Y
!
!.PURPOSE       MIDAS procedure to display Echelle Spectra line's FWHMs
!		as a function of pixel number in X and Y as well as the
!		resolution lambda/dlambda
!
!		Creates 6 graphs and the postscript file with these 6 plots :
!		. plots column :DX (FWHM) vs column :X_1 and :YNEW_1,
!		. plots column :RESOL vs column :X_1 and :YNEW_1,
!		. plots column :RESOL vs column :IDENT_1 and :X_1,
! 		. plots column :YNEW_1 vs column :X_1,
!		Finally, it creates an ASCII file (free_sp_rg.dat) with columns:
!		  :X_1,:YNEW_1,:IDENT_1,:ORDER_1,:DX,:DLAMBDA,:RESOL
!
!
!.ALGORITHM     Creates a postscript file with plots of the above mentioned
!		columns of the input resolution table (P1).
!		For the title it reads the appropriate header descriptors
!		of the WLC-frame (P2).
!		At the end the result may be displayed controled via the
!		parameter P4.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/05/21	Created from a copy of necdispres.prg
!		    1999/10/08	date,user plot label added (uves_plt_ident.prg)
!		    1999/11/17	set/format F15.10 before jd2date !!
!		    1999/12/18	table with resolution statistic created.
!
!.COMMENT	This command may only be used in connection with @s necdispres
!		which determines the resolution of the instrument on the entire
!		detector. It is a command of the ECHELLE context and has to be
!		called in advance.
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.18 $
! $Log: uves_plt_dispres.prg,v $
! Revision 1.18  2001/09/08 15:26:09  amodigli
! PIPEFILE->h_pipefile
! h_nameccd now uses UVES_ICS
!
! Revision 1.17  2001/08/12 16:14:49  amodigli
! Passing p_version as string par
!
! Revision 1.16  2001/08/09 08:01:21  amodigli
! Added QC.LINE.RESIDAVG QC.LINE.RESIDRMS
!
! Revision 1.15  2001/07/15 14:18:56  amodigli
! Eliminated unnecessary call to uves_getpversion.prg
!
! Revision 1.14  2001/07/10 09:45:36  amodigli
! Used QC.INS.GRATi.NAME to log CHIP name
!
! Revision 1.13  2001/07/04 07:57:16  amodigli
! New QC1 parameter naming conv
!
! Revision 1.12  2001/06/20 14:21:13  amodigli
! Using QC1 LOG and PAF files
!
! Revision 1.11  2001/06/16 07:05:26  amodigli
! Changing QC key naming conventions
!
! Revision 1.10  2001/02/09 18:28:56  amodigli
! Added PRO QC keywords for DFO.
! Table alignment of resol tbl to be checked.
!
! Revision 1.9  2001/01/04 10:39:36  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------


define/param p1 &resol c "Enter name of resolution table:"
define/param p2 {wlc} c "Enter name of wavelength calibration frame WLC:"
define/param p3 + c "Enter y-ranges for the plots:"
define/param p4 Y c "Display result on graphics window: Yes or No"

define/local K/I/1/1  0
define/local Ncolour/I/1/1 4
define/local colsel/I/1/4 6,4,3,2
define/local typsel/I/1/4 5,3,2,8
define/local NX/I/1/1 0
define/local NY/I/1/1 0
define/local UPY/I/1/1 0
define/local LWY/I/1/1 0
define/local MINident/R/1/1  0.
define/local MAXident/R/1/1  0.
!define/local CENident/R/1/1  0.
define/local MINDX/I/1/3 1,1,1
define/local MINRESOL/R/1/3 30000.,30000.,30000.,
define/local MAXDX/I/1/3 6,6,4
define/local MAXRESOL/R/1/3 150000.,150000.,120000.
define/local MNDX/I/1/1 0
define/local MNRESOL/R/1/1 0.0
define/local MXDX/I/1/1 0
define/local MXRESOL/R/1/1 0.0
!
! statistic
!
define/local MeanDX/R/1/1  0.
define/local StdDX/R/1/1  0.
define/local N2DX/I/1/1  0.
define/local MeanRES/R/1/1  0.
define/local StdRES/R/1/1  0.
define/local N2RES/I/1/1  0.
define/local MedDX/R/1/1  0.
define/local MedRES/R/1/1  0.
!
! title
!
define/local restab/c/1/100 " " ALL
define/local hfname/c/1/100 " " ALL
define/local fname/c/1/100 " " ALL
define/local cname/c/1/100 " " ALL
define/local datobs/c/1/100 " " ALL
define/local swid/c/1/10 " " ALL
define/local temp/c/1/10 " " ALL
define/local wlen/c/1/10 " " ALL
define/local modext/c/1/10 " " ALL
define/local i1/i/1/1 0
define/local i2/i/1/1 0

config/instr {p2}
modext = q2


get/pathid {p2}
set/format F5.1
wlen = "{{P2},{h_cwlen({PATHID})}}"	!get central wavelength

temp = "{{P2},{h_tempcam({PATHID})}}"
swid = "{{P2},{h_slwidth({PATHID})}}"
set/format
if m$existd(P2,h_origname) .eq. 1 then
  fname = "{{P2},{h_origname}}"
  hfname = "Filename :"
else
  fname = "{{P2},{h_obsname}}_{{P2},{h_obstpno}}"
  hfname = "OB name :"
endif

!define date to set the appropriate output filename of ops.log files for QC1LogWriter
define/local date/c/1/30 " " all
date   = "{{P2},ESO.TPL.START}"


cname = h_nameccd({PATHID})
restab = "resol_{cname}_{wlen(1:3)}_{swid}"
i1 = m$indexb(restab,".")
i2 = i1 + 1
i1 = i1 - 1
restab = "{restab(1:{i1})}{restab({i2}:)}.tbl"

set/format F15.10
outputd(1) = -m$value({p2},O_TIME(4))
@p uves_jd2date {outputd(1)}
datobs = outputc
set/format F5.1

MNDX = MINDX({PATHID})
MXDX = MAXDX({PATHID})
MNRESOL = MINRESOL({PATHID})
MXRESOL = MAXRESOL({PATHID})
if p3(1:1) .ne. "+" then
  define/local yr/r/1/4 {MNDX},{MXDX},{MNRESOL},{MXRESOL}	!set defaults
  write/out "Y-ranges [default]: [{yr(1)},{yr(2)}]; [{yr(3)},{yr(4)}]"
  write/key yr/r/1/4 {p3}					!overwrite by
  write/out "Y-ranges [user]:    [{yr(1)},{yr(2)}]; [{yr(3)},{yr(4)}]"
  MNDX = yr(1)							!the user sett.
  MXDX = yr(2)
  MNRESOL = yr(3)
  MXRESOL = yr(4)
endif
!
! get the dimension of the WLC-image
!
NX = m$value({p2},NPIX(1))
NY = m$value({p2},NPIX(2))
!
! Determine the statistic of the table and save them in a table as descriptors
!
select/table {P1} all >Null
statis/table {P1} :DX >Null
!NB: this is in [pix] units
set/format i1 f8.4,f8.4
MeanDX = {outputr(3)}
StdDX = {outputr(4)}
N2DX = {outputi(2)}/2
set/format

statis/table {P1} :RESOL >Null
MeanRES = {outputr(3)}
StdRES = {outputr(4)}
N2RES = {outputi(2)}/2
sort/tabl {P1} :DX

!NB: This is in [pix] units
MedDX = m$value({P1},:DX,@{N2DX})

sort/tabl {P1} :RESOL
MedRES = m$value({P1},:RESOL,@{N2RES})
!
! Output to FITS file
! AM: stored on LINTAB (2) as asked from RHA
! output to descriptors of table (SWO/AKA Dec 99)
!
!create/table {restab} 1 1 NULL
define/local qc_wlen_min/r/1/1 0
define/local qc_wlen_max/r/1/1 0
define/local qc_ord_min/i/1/1 0
define/local qc_ord_max/i/1/1 0
define/local qc_wav_res/i/1/1 0
define/local qc_nsel/i/1/1 0
define/local qc_nall/i/1/1 0

define/local qc_mean_dx/r/1/1 0.0
define/local qc_med_dx/r/1/1 0.0
define/local qc_std_dx/r/1/1 0.0
define/local qc_n2dx/r/1/1 0.0

define/local qc_mean_res/r/1/1 0.0
define/local qc_med_res/r/1/1 0.0
define/local qc_std_res/r/1/1 0.0
define/local qc_n2res/r/1/1 0.0

stat/tab {LINTAB} :WAVEC {SESSOUTV}
!NB: we divide by 10 as column :WAVEC of LINTAB is expressed in A
!    but DICB specify that QC Keys should be in nm units
qc_wlen_min = outputr(1)/10.
qc_wlen_max = outputr(2)/10.

stat/tab {LINTAB} :ORDER  {SESSOUTV}
qc_ord_min = outputr(1)
qc_ord_max = outputr(2)

 
select/table {LINTAB} :RESIDUAL.LT.1 {SESSOUTV}
qc_wav_res = outputi(1)

select/table {LINTAB} ALL {SESSOUTV}
qc_nall = outputi(1)

select/table {LINTAB} :SELPLOT.EQ.1 {SESSOUTV}
qc_nsel = outputi(1)


statis/table {LINTAB} :DX {SESSOUTV}
!NB: this is in [pix] units
qc_mean_dx = {outputr(3)}
qc_std_dx = {outputr(4)}
qc_n2dx = {outputi(2)}/2

statis/table {LINTAB} :RESOL {SESSOUTV}
qc_mean_res = {outputr(3)}
qc_std_res = {outputr(4)}
qc_n2res = {outputi(2)}/2

sort/tabl {LINTAB} :DX
!NB: this is in [pix] units
qc_med_dx = m$value({LINTAB},:DX,@{N2DX})

sort/tabl {LINTAB} :RESOL
qc_med_res = m$value({LINTAB},:RESOL,@{N2RES})


 
define/local qc_resid_avg/r/1/1 0.0
define/local qc_resid_rms/r/1/1  0.0
select/tab {LINTAB} all
select/tab {LINTAB} :IDENT.GT.0
stat/tab {LINTAB} :RESIDUAL  {SESSOUTV}
qc_resid_avg = outputr(3)*1000/10 !To have in picometer units
qc_resid_rms = outputr(4)*1000/10
 
!
!copy/dd {p2} *,3 {restab}
!
! output to the logging system (AKA 2000-03-18)

define/local pn/i/1/1 0
set/format i1
pn = {uves_portid({PATHID})}
set/format 


define/local dpr_type/c/1/80 " " all
define/local tpl_id/c/1/80 " " all
define/local arcfile/c/1/80 " " all
define/local pro_catg/c/1/80 " " all

qc1log/out 2 
set/format i1 f8.4,f8.4
qc1log/out 1 {LINTAB} "{h_qc_did}"  "{d_qc_did}"              "Data Dictionary for QC"

qc1log/out 1 {LINTAB} "QC.TEST1.ID" "Resolution-Test-Results"  "Name of QC Test"
qc1log/out 1 {LINTAB} "PRO.REC1.PIPE.ID" "{p_version}" "Pipeline (unique) identifier" PRO
if m$existd(p2,h_dprtype) .eq. 1 then
  dpr_type = "{{p2},{h_dprtype}}"
  qc1log/out 1 {LINTAB} {h_dprtype} "{dpr_type}" + DPR
endif
if m$existd(p2,h_tpid) .eq. 1 then
  tpl_id = "{{p2},{h_tpid}}"
  qc1log/out 1 {LINTAB} {h_tpid} "{tpl_id}" + TPL
endif
if m$existd(p2,h_arcfile) .eq. 1 then
  arcfile = "{{p2},{h_arcfile}}"
  qc1log/out 1 {LINTAB} {h_arcfile} {arcfile} + PRIMARY-FITS
endif
if m$existd(LINTAB,h_procatg) .eq. 1 then
  pro_catg = "{{LINTAB},{h_procatg}}"
  qc1log/out 1 {LINTAB} {h_procatg} {pro_catg} + PRO
endif
qc1log/out 1 {LINTAB} {h_pipefile} {LINTAB} + PRO 
if m$existd(LINTAB,"{h_inspath}") .eq. 1 then 
   qc1log/out 1 {LINTAB} {h_inspath} {{LINTAB},{h_inspath}}                 "Optical path used."         UVES_ICS
else
   mess/out 2 {pid} "Missing descriptor {h_inspath}"
endif
if m$existd(LINTAB,"{h_insmode}") .eq. 1 then 
   qc1log/out 1 {LINTAB} {h_insmode} {{LINTAB},{h_insmode}}                 "Instrument mode used."      UVES_ICS  
else
   mess/out 2 {pid} "Missing descriptor {h_insmode}"
endif

qc1log/out 1 {LINTAB} "{h_namecros({PATHID})(5:18)}" {CROSS} "Cross disperser ID"                        UVES_ICS
qc1log/out 1 {LINTAB} "{h_slwidth({PATHID})}"         {swid}     "Slit width (arcsec) [arcsec] (hs)."    UVES_ICS
qc1log/out 1 {LINTAB} "{h_cwlen({PATHID})(5:)}"       {wlen}     "Grating central wavelength [nm] (hs)." UVES_ICS
qc1log/out 1 {LINTAB} "{h_fits_nameccd({PATHID})(5:)}" {CCD} "Grating unique ID (hls)."                  CCDDCS
qc1log/out 1 {LINTAB} "{h_tempcam({PATHID})(5:)}"      {temp}    "Average temperature [c] (ho)."         UVES_ICS

qc1log/out 1 {LINTAB} "QC.FWHMAVG"                   {MeanDX}  "average FWHM in X of lines selected"
qc1log/out 1 {LINTAB} "QC.FWHMRMS"                    {StdDX}  "sdev of FWHM in X of lines selected"
qc1log/out 1 {LINTAB} "QC.FWHMMED"                    {MedDX}  "median FWHM on X of lines selected"
qc1log/out 1 {LINTAB} "QC.RESOLAVG"                 {MeanRES}  "average resolving power of lines selected"
qc1log/out 1 {LINTAB} "QC.RESOLRMS"                  {StdRES}  "sdev of resolving power of lines selected"
qc1log/out 1 {LINTAB} "QC.RESOLMED"                  {MedRES}  "median resolving power of lines selected"

qc1log/out 1 {LINTAB} "QC.WLENMIN"                  {qc_wlen_min} "minimum wavelength"
qc1log/out 1 {LINTAB} "QC.WLENMAX"                  {qc_wlen_max} "maximum wavelength"
qc1log/out 1 {LINTAB} "QC.ORDMIN"                   {qc_ord_min}  "minimum order number detected"
qc1log/out 1 {LINTAB} "QC.ORDMAX"                   {qc_ord_max}  "maximum order number detected"
qc1log/out 1 {LINTAB} "QC.NLINTOT"                  {qc_nall}     "No of lines found" 
qc1log/out 1 {LINTAB} "QC.NLINSEL"                  {qc_nsel}     "number of lines selected"
qc1log/out 1 {LINTAB} "QC.NLINRES1"                 {qc_wav_res}  "number of lines with residuals < 0.1 nm"
qc1log/out 1 {LINTAB} "QC.LINE.RESIDAVG"            {qc_resid_avg}  "mean resid of line pos to fit"
qc1log/out 1 {LINTAB} "QC.LINE.RESIDRMS"            {qc_resid_rms}  "sigma resid of line pos to fit"
 
 
set/format
qc1log/out 3

!
! Configure the plot
!
set/grap
assign/grap POSTSCRIPT.l
set/grap font=1 !yaxis=auto !pmode=0
!
! 1.plot
!
select/table {P1} all >Null !:Y_1.EQ.{ECHORD(4)}
statis/table {P1} :IDENT_1 >Null
MINident = {outputr(1)} - 50.
MAXident = {outputr(2)} + 50.
!CENident = {outputr(3)}
plot/axes  0,{NX} {MNDX},{MXDX} -65,-65,13,100 "\+ \+ X  (pixels)" "\+ \+ \Delta X (line FWHM in pixels)"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YNEW_1.GE.{LWY}.AND.:YNEW_1.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :X_1 :DX -65,-65,13,100 {typsel({K})}
enddo
!
! 2.plot
!
set/grap colour=1
select/table {P1} all >Null
overplot/axes  0,{NY} {MNDX},{MXDX} -65,-65,13,15 "\+ \+ Y  (pixels)" "\+ \+ \Delta X (line FWHM in pixels)"
UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YNEW_1.GE.{LWY}.AND.:YNEW_1.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :YNEW_1 :DX -65,-65,13,15 {typsel({K})}
enddo
set/grap colour=1
if mid$sess(16:25) .eq. "99NOVpl1.1" then
  @p uves_plt_ident 0 15 csize=0.9	!write the bottom label (date,user)
endif 
!
! 3.plot
!

set/grap colour=1
select/table {P1} all >Null

! here sometimes display a warning for table points illegal or NULL=> table points not plotted
overplot/axes  0,{NX} {MNRESOL},{MXRESOL} -65,-65,104,100 "\+ \+ X  (pixels)" "\+ \+ Resolution : \lambda / \Delta \lambda"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YNEW_1.GE.{LWY}.AND.:YNEW_1.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :X_1 :RESOL -65,-65,104,100 {typsel({K})}
enddo


!
! 4.plot
!

set/grap colour=1
select/table {P1} all >Null

! here sometimes display a warning for table points illegal or NULL=> table points not plotted
overplot/axes 0,{NY} {MNRESOL},{MXRESOL} -65,-65,104,15 "\+ \+ Y  (pixels)" "\+ \+ Resolution : \lambda / \Delta \lambda"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YNEW_1.GE.{LWY}.AND.:YNEW_1.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :YNEW_1 :RESOL -65,-65,104,15 {typsel({K})}
enddo
!
! 5.plot
!

set/grap colour=1
select/table {P1} all >Null

! here sometimes display a warning for table points illegal or NULL=> table points not plotted
overplot/axes {MINident},{MAXident} {MNRESOL},{MXRESOL} -65,-65,194,100 "\+ \+ \lambda  (\AA )" "\+ \+ Resolution : \lambda / \Delta \lambda"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YNEW_1.GE.{LWY}.AND.:YNEW_1.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :IDENT_1  :RESOL -65,-65,194,100 {typsel({K})}
enddo
!
! 6.plot
!

set/grap colour=1
select/table {P1} all >Null

! here sometimes display a warning for table points illegal or NULL=> table points not plotted
overplot/axes 0,{NX} 0,{NY} -65,-65,194,15 "\+ \+ X  (pixels)" "\+ \+ Y  (pixels)"

UPY = 0
do K = 1 {Ncolour} 1
  LWY = {UPY}
  UPY = {NY}/{Ncolour}*{K} 
  select/table {P1} :YNEW_1.GE.{LWY}.AND.:YNEW_1.LT.{UPY} >Null
  set/grap colour={colsel({K})}
  overpl/table {P1}   :X_1 :YNEW_1 -65,-65,194,15 {typsel({K})}
enddo
!
!
! Plot the title
!

set/gra pmode=-1 colour=1
overpl/axes >Null

if mid$sess(16:25) .eq. "99NOVpl1.1" then
    set/grap tsize=1.3					!main title
    label/grap "Central wavelength: {wlen} nm, slit: {swid} arcsec, CCD: {cname}, {temp} \circ C" 90,167,mm

    set/grap tsize=1					!sub title
    outputi(1) = m$len(datobs)
    label/grap "{hfname} {fname}, Observ. date: {datobs(1:{outputi(1)})}" 90,162,mm

    set/grap tsize=1.0					!sub title: statistic
    SET/FORM f5.3
    label/grap "Median(DX) = {MedDX} pix , Mean(DX) = {MeanDX} pix , \sigma (DX) = {StdDX} pix" 90,157,mm
    label/grap "Median(RES) = {MedRES} , Mean(RES) = {MeanRES} , \sigma (RES) = {StdRES}" 90,152,mm
else
    set/grap tsize=1.3					!main title
    label/grap "Central wavelength: {wlen} nm, slit: {swid} arcsec, CCD: {cname}, {temp} \circ C" 0.5,0.99,no

    set/grap tsize=1					!sub title
    outputi(1) = m$len(datobs)
    label/grap "{hfname} {fname}, Observ. date: {datobs(1:{outputi(1)})}" 0.5,0.96,no

    set/grap tsize=1.0					!sub title: statistic
    SET/FORM f5.3
    label/grap "Median(DX) = {MedDX} pix , Mean(DX) = {MeanDX} pix , \sigma (DX) = {StdDX} pix" 0.5,0.93,no
    label/grap "Median(RES) = {MedRES} , Mean(RES) = {MeanRES} , \sigma (RES) = {StdRES}" 0.5,0.90,no
    @p uves_plt_ident 0.02 0.02 csize=0.9	!write the bottom label (date,user)
endif    
!
!
! Check if plot has to be displayed on a graphics window:
!
if p4(1:1) .eq. "Y" then
  set/format I1
  show/disp >Null			!check if graph.window already exists
  if outputi(12) .lt. 0 create/grap 1 1000,707
  copy/grap  g,{outputi(12)} >Null
endif
return 0
