! $Id: uves_predict.prg,v 1.9 2001/05/02 14:13:07 amodigli Exp $
!+++ UVES_PREDICT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_predict.prg
!
!.KEYWORDS      uves physical model, spectroscopy, echelle,
!
!.USAGE         @p (IDENT) wlc lincat
!
!.INPUT         wlc     (P1): ThAr format check frame
!				        These are ThAr exposures obtained with a small
!				        slit length.
!			            (P2): reference input catalog
!			            (P3): catalog to save product
!			            (P4): Dimension of measurement box (X,Y)
!			            (P5): Translation of the ccd (X,Y)
!			            (P6): Angle offsets (echelle, CD, CCD-rot)
!			            (P7): Measurement position offsets (X,Y) 
!				              from the predicted position.
!		        disp,fitreg (p8): display flag, fits regression, COMPUTE/REGR:
!				                  YES: Show result on graphics window
!				                  NO:  Creates no plot.
!				                  YES: Fits the residual differences and builds 
!				                       the order tables and line tables.
!				                  NO:  No fitting and produces no tables.
!				                  YES: Compute regression.
!				                  NO:  No Compute regression.
!
!.OUTPUT        line table [LINE_TABLE_x] which may be used as first guess
!		        DRS table  [DRS_SETUP_x], order table [ORDER_TABLE_x], and
!		        background table [BACKGR_TABLE_x] (x: BLUE, REDL, REDU).
!
!.RETURN        Q1:  0: successful return
! 		            -1: a successful return failed
!		        Q2: return the classification of the main product
!
!.DEFAULT	P1	{wlc}
!		    P2	{lincat}
!		    P3	+
!		    P4	40,40
!		    P5	0.,0.
!		    P6	0.,0.,0.
!		    P7	0.,0.
!		    p8	Y,Y,Y
!
!.PURPOSE: 	Create a line table based on the physical model of UVES.
!
!.ALGORITHM	Define global echelle keywords before calling 
!		    the subroutines: "uves_phymod.prg", "uves_msrawxy.prg", 
!		    "uves_plotmod.prg", "uves_calmap.prg", 
!		    do classification of the products and store them in
!		    DB catalogue.
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        O.Boitquin, ESO/DMD
!
!.VERSION       1999/09/01  Created (OB).
!		        1999/09/30-SW  Parameter P4 changed to P7, shift former
!				               P5..P7 to P4..P6.
!				               This is more consistent with the call to
!				               uves_physmod.prg (see also COMMENT).
!		        1999/10/01-OB  Header with angle offsets (echelle, CD, CCD rot)
!               1999/10/04-OB  Add usefull flags P8 to fit the physical mod. 
!		        1999/10/04-SW  Create the output catalog if it does not exist!
!		        1999/11/25-SW  procedure header made more infomative.
!		        1999/11/26-SW  save number of orders in DRS table.
!		        1999/11/29-SW  write DEFPOL, WIDTHI and THRESI to DRS table.
!		        1999/12/07-SW  product header: ESO descr. were missing!
!               1999/12/10-SW  return the main product as Q2
!               1999/12/16-SW  The binning has to be considered for MBOX !
!			                   Improved NBORDI determination - select central
!		        2000/02/22-OB  Compute regression 
!				               band.
!.COMMENT   Parameters P4, P5, and P6 are the parameters used by the
!           physical model routine uves_physmod.prg.
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.9 $
! $Log: uves_predict.prg,v $
! Revision 1.9  2001/05/02 14:13:07  amodigli
! Added P2={rawframe} invocating
! uves_ordpos,SETDRS as now the bkg setting
! are binning (frame) dependent.
!
! Revision 1.8  2001/01/04 10:39:34  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!

DEFINE/PARAM P1 ? 	        IMA    "Input frame:"
define/param P2 {LINCAT}    CHAR   "Enter name REFERENCE Catalogue"
define/param P3 +           CHAR   "OrderO.cat to save product"
define/param P4 40,40       NUMB   "Dimension of measurement box (X,Y): "
define/param P5 0,0         NUMB   "Translation of the ccd (X,Y): "
define/param P6 0,0,0       NUMB   "Angle offsets (echelle, CD, CCD-rot):"
define/param P7 0,0         NUMB   "Measurement position offsets (X,Y): "
define/param P8 Y,Y,Y       CHAR   "Displays results, fits regression, COMPUTE/REGR flags: Y or N ?"

define/local XYOFF/R/1/2  {P7}		!never used !!?? (SW)
define/local MBOX/R/1/2   {P4}
define/local trans/D/1/2  {p5}
!define/local angoff/D/1/3 {P6}

define/local wcen/R/1/1   0.0
define/local modext/c/1/25 " " ALL

define/local K/I/1/1  0
define/local disp/C/1/3 " " ALL
define/local fitreg/C/1/3 " " ALL
define/local reg_opt/C/1/3 " " ALL

! For messout

define/local pid/C/1/20 "uves_predict"

						!Check if output catalog exist
if m$exist(p3) .eq. 0 .AND. p3(1:1) .ne. "+" -
  create/icat {p3} NULL {UVES_IDSTRNG} >Null


config/instr {p1}  				!to define PATHID and UVES_ARM
modext = q2

set/format F4.0

wcen = WCENTER/10.0
set/echelle wlc={P1} lincat={p2}
set/echelle ordtab=o{wcen}{modext(2:)}.tbl baktab=b{wcen}{modext(2:)}.tbl
set/echelle lintab=l{wcen}{modext(2:)}.tbl
mess/out 3 {pid} "Start prediction of {p1} ..."
mess/out 3 {pid} "Use line reference list: {LINCAT}"

 
mbox(1) = mbox(1) / CCDBIN(1)                ! Dimension of measurement box (x)
mbox(2) = mbox(2) / CCDBIN(2)                ! Dimension of measurement box (y) 

set/format F5.1
if NICE_CREA(1:1) .eq. "Y" then
  assign/disp 1
  load/image {wlc} scale=f cen=c cuts=f,ihap
endif

k = m$parse(p8,"flag")
if k .gt. 2 then
  disp = "{flag01}"
  fitreg = "{flag02}"
  reg_opt = "{flag03}"
else if k .gt. 1 then
  disp = "{flag01}"
  fitreg = "{flag02}"
  reg_opt = "YES"
else if k .eq. 1 then
  disp = "{flag01}"
  fitreg = "YES"
  reg_opt = "YES"
endif
disp = m$upper(disp)
fitreg = m$upper(fitreg)
reg_opt = m$upper(reg_opt)

! Runs the physical model and creates the line table
@p uves_phymod.prg -
   {wlc} {lincat} {mbox(1)},{mbox(2)} {trans(1)},{trans(2)} {P6} {reg_opt}

if Q1 .eq. "1" then
  mess/out 1 {pid} "Error occuring in the physical model. Abort..."
  return -1
endif

! Mesures the positions of lineswith and matches with the positions predicted 
@p uves_msrawxy.prg  {wlc}  p4={NICE_CREA(1:1)} p7={mbox(1)},{mbox(2)} p8={fitreg} !p5={XYOFF(1)},{XYOFF(2)}

! Displays the differences between projection and measurements
if disp(1:1) .eq. "Y" .AND. NICE_CREA(1:1) .eq. "Y" then 
  @p uves_plotmod.prg rline {wlc} {trans(1)},{trans(2)} {P6}
  modify/graph W
  copy/grap g,3 >Null
endif

if fitreg(1:1) .eq. "Y" then
  ! Builds the order and line tables
  @p uves_calmap.prg

  copy/dd {WLC} *,3 {LINTAB}			!copy the descriptors
  copy/dd {WLC} *,3 {ORDTAB}			!of input frame
  copy/dd {WLC} *,3 {BAKTAB}			!to the products.

  DRSTAB = "drs_setup{modext}.tbl"		!name of setup table
  @p uves_ordpos,SETDRS {DRSTAB} {p1}		!create DRS setup table

  define/local xbox/I/1/2 0,0
  define/local ybox/I/1/2 0,0
  xbox(1) = imsize(1)/2 - imsize(1)*0.05
  xbox(2) = imsize(1)/2 + imsize(1)*0.05
  ybox(1) = 50.0
  ybox(2) = imsize(2) - 50.0
  select/table {ORDTAB} :X.GT.{xbox(1)}.AND.:X.LT.{xbox(2)}
  stat/table {ORDTAB} :ORDER >Null
  outputi(1) = outputr(2) - outputr(1) + 1
  write/out "Found {outputi(1)} orders"  
  write/desc {DRSTAB} NBORDI {outputi(1)}	!save number of detected orders
  select/tab {ORDTAB} all
    
  write/desc {DRSTAB} DEFPOL 2,3		!order defin. polynom. degrees
  write/desc {DRSTAB} WIDTHI 0			!order width (automatic if 0)
  write/desc {DRSTAB} THRESI 0			!threshold for following order
  classify/product {DRSTAB} {DRSRC_ID({PATHID})} {d_pre} {d_pho} OK
  classify/product {ORDTAB} {ORDER_ID({PATHID})} {d_pre} {d_pho} OK
  classify/product {BAKTAB}  {BACK_ID({PATHID})}  {d_pre} {d_pho} OK
  classify/product {LINTAB} {LINTAB_ID({PATHID})} {d_pre} {d_pho} OK

  if p3(1:1) .ne. "+" then
    add/icat {p3} {drstab} >Null
    add/icat {p3} {ordtab} >Null
    add/icat {p3} {baktab} >Null
    add/icat {p3} {lintab} >Null
  endif
endif

return 0 {LINTAB_ID({PATHID})}

