! $Id: uves_proclass.prg,v 1.4 2001/07/04 16:08:06 amodigli Exp $
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       uves_proclass.prg
!
!.KEYWORDS    Update data product keywords in frame header.
!
!.USAGE       classify/product  frame pro_catg pro_type pro_level 
!                               [pro_status] [ident] [stat]
!
!.INPUT       frame       (P1)    Frame to be updated.
!             pro_catg    (P2)    Category of pipeline product frame.
!             pro_type    (P3)    Pipeline product type.
!             pro_level   (P4)    Reduction level of data product.
!             pro_status  (P5)    Pipeline status of product frame.
!             ident       (P6)    DO_CLASSIFICATION->IDENT ?  [y|default:n]
!             stat        (P7)    Mean/sigma/median->PRO.DATA* ?  [y|default:n]
!
!.OUTPUT      None
!
!.RETURNS     The recipe returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure updates the frame header keywords specifying the
!             pipeline product identification/classification. 
!
!.ENVIRON     UVES context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/23:  Creation
!             1.3  1999/04/14:  P6 and P7 added (NR)
!                  1999/09/16:  copied latest version from FORS-pipeline (SW)
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.4 $
! $Log: uves_proclass.prg,v $
! Revision 1.4  2001/07/04 16:08:06  amodigli
! Better issue an error instead of a warning
! in case of bad datamin/max/avg/rms
!
! Revision 1.3  2001/07/04 15:55:15  amodigli
! Added check on value of datamin/max/rms/med
! to issue warning and errors in case they are
! inf or NANs. This is properly catched from
! reduce_uves.prg
!
! Revision 1.2  2001/01/04 10:39:34  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref frame pro_catg pro_type pro_level pro_status ident stat

define/param p1 ?  c    "Enter result frame catalog:"
define/param p2 ?  c    "Enter product frame category:"
define/param p3 ?  c    "Enter product type:"
define/param p4 ?  c    "Enter reduction level:"
define/param p5 +  c    "Enter pipeline status of product:"
define/param p6 n  c    "Enter flag for DO_CLASS.->IDENT [y|n]:"
define/param p7 n  c    "Enter flag for Statistic->PRO.DATA* [y|n]:"
define/maxpar 7


! Procedure identification
define/local procID/c/1/80      "ProClass" ? +lower_levels


! Local variables
define/local frame/c/1/100       "{p1}"
define/local pro_catg/c/1/100    "{p2}"
define/local pro_type/c/1/100    "{p3}"
define/local pro_level/c/1/100   "{p4}"
define/local pro_status/c/1/100  "{p5}"
define/local idflag/c/1/1        "{p6}"
define/local statflag/c/1/1      "{p7}"

pro_catg = m$upper(pro_catg)
pro_type = m$upper(pro_type)
pro_level = m$upper(pro_level)
pro_status = m$upper(pro_status)



! Update pipeline product category
! If the target descriptor is not present it is created.
if m$existd(frame,H_PROCATG) .eq. 1 then
  delete/descr {frame} {H_PROCATG}
endif
write/descr {frame} {H_PROCATG}/c/1/30 "{pro_catg}"
write/dhelp {frame} {H_PROCATG} "{I_PROCATG}"


! Update product type
! If the target descriptor is not present it is created.
if m$existd(frame,H_PROTYPE) .eq. 1 then
  delete/descr {frame} {H_PROTYPE}
endif
write/descr {frame} {H_PROTYPE}/c/1/20 "{pro_type}"
write/dhelp {frame} {H_PROTYPE} "{I_PROTYPE}"
    

! Update reduction level
! If the target descriptor is not present it is created.
if m$existd(frame,H_PROLEVEL) .eq. 1 then
  delete/descr {frame} {H_PROLEVEL}
endif
if m$index(pro_level,d_empty) .gt. 0 then
  write/descr {frame} {H_PROLEVEL}/c/1/21 " "
else
  write/descr {frame} {H_PROLEVEL}/c/1/21 "{pro_level}"
endif
write/dhelp {frame} {H_PROLEVEL} "{I_PROLEVEL}"


! Optionally the pipeline status for the current frame may be altered
if pro_status .ne. "+" then
  if m$existd(frame,H_PROSTAT) .eq. 1 then
    delete/descr {frame} {H_PROSTAT}
  else
    mess/out 4 {procID} "Creating descriptor {H_PROSTAT}!"
  endif
  write/descr {frame} {H_PROSTAT}/c/1/6 "{pro_status}"
  write/dhelp {frame} {H_PROSTAT} "{I_PROSTAT}"
endif

! Update the internal pipeline classification descriptor, so that 
! frame may be handled by super recipes.
if m$existd(frame,UVES_IDSTRNG) .eq. 1 then
  delete/descr {frame} {UVES_IDSTRNG}
endif
write/descr {frame} {UVES_IDSTRNG}/c/1/25 "{pro_catg}"


! Optionally copy pipeline classification to identifier descriptor:
if idflag .ne. "n" then
  write/descr {frame} {h_ident}/c/1/72 " " all
  write/descr {frame} {h_ident}/c/1/72 {{frame},{UVES_IDSTRNG}}
endif


! Optionally calculate statistics and save in descriptors:
if statflag .ne. "n" then
  !
  statistics/image {frame} + ? ? xn >Null
  define/local check_num/i/1/1 0
  check_num = 0
  if m$tstno("{outputr(1)}") .ne. 1 then
     @p uves_proclass,WARNING {frame} {pro_catg} {h_datamin} "{outputr(1)}"
     check_num = check_num + 1
  endif     
  if m$tstno("{outputr(2)}") .ne. 1 then
     @p uves_proclass,WARNING {frame} {pro_catg} {h_datamax} "{outputr(2)}"
     check_num = check_num + 1
  endif
  if m$tstno("{outputr(3)}") .ne. 1 then
     @p uves_proclass,WARNING {frame} {pro_catg} {h_promean} "{outputr(3)}"
     check_num = check_num + 1
  endif
  if m$tstno("{outputr(4)}") .ne. 1 then
     @p uves_proclass,WARNING {frame} {pro_catg} {h_prosigm} "{outputr(4)}"
     check_num = check_num + 1
  endif     
  if m$tstno("{outputr(8)}") .ne. 1 then
     @p uves_proclass,WARNING {frame} {pro_catg} {h_promedi} "{outputr(8)}"
     check_num = check_num + 1
  endif
  if check_num .eq. 0 then
  !
  write/descr {frame} {h_datamin}/r/1/1 {outputr(1)}
  write/dhelp {frame} {h_datamin} "{i_datamin}"
  write/descr {frame} {h_datamax}/r/1/1 {outputr(2)}
  write/dhelp {frame} {h_datamax} "{i_datamax}"
  write/descr {frame} {h_promean}/r/1/1 {outputr(3)}
  write/dhelp {frame} {h_promean} "{i_promean}"
  write/descr {frame} {h_prosigm}/r/1/1 {outputr(4)}
  write/dhelp {frame} {h_prosigm} "{i_prosigm}"
  write/descr {frame} {h_promedi}/r/1/1 {outputr(8)}
  write/dhelp {frame} {h_promedi} "{i_promedi}"
  else
    return -1
  endif
endif

return 0
!===========================================================================
ENTRY WARNING
define/par p1 ? c "Enter Frame:"
define/par p2 ? c "Enter pro catg:"
define/par p3 ? c "Enter key name:"
define/par p4 ? c "Enter key value:"


mess/out 1 {procID} "Error! Numerical key: {p3} = {p4}, in frame: {p1}, pro catg: {p2}" 

return 0
!===========================================================================
