! $Id: uves_psf.prg,v 1.3 2001/01/04 10:39:33 amodigli Exp $
!+++ UVES_PSF +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_psf.prg
!
!.KEYWORDS      uves physical model, spectroscopy, echelle,
!
!.USAGE         @p (IDENT) wlc lincat
!
!.INPUT         wlc     (P1): wavelength calibration frame
!			(P2): wavelength
!			(P3): Translation ccd (X,Y)
!			(P4): Angle offsets (echelle, CD, CCD rot)
!			(P5): free spectral range flag:
!				YES: Generates the free spectral range table,
!				NO: Does NOT generate the table.
!
!.OUTPUT        outputd(1): wavelength in nm in the center of CCD
!		outputd(2): Line width in pixel
!		outputd(3): Line length in pixel
!		outputd(4): Line width in Angstroem
!		outputd(5): Binsize in mA
!		outputd(6): Echelle pixel-scale in arcsec
!		outputd(7): CD pixel-scale in arcsec
!		outputd(8): Resolution
!		outputd(9): Absolute Order
!		outputd(10): Predicted position X in pixel for the wavelength
!		outputd(11): Predicted position Y in pixel for the wavelength
!		outputd(12): Minimum Wavelength in nm
!		outputd(13): Maximum Wavelength in nm
!		outputd(14): Free spectral Order range : maximun
!		outputd(15): Free spectral Order range : minimum
!		outputd(16): Number of order on the free spectral range
!		outputd(17): Real order range on CCD : maximun
!		outputd(18): Real order range on CCD : minimum
!		outputd(19): Real number of order on CCD
!		outputd(20): Blaze function
!
!		modfree_sp_rg.tbl, modfree_sp_rg.dat: free spectral range table 
!		and ascii file.
!
!.RETURN        1 on error, 0 else
!
!.DEFAULT	P1	{wlc}
!		P2	WCENTER
!		P3	0.,0.
!		P4	0.,0.,0.
!		p5	N
!
!.PURPOSE: 	Physical model of UVES dispersion relation. 
!		Computes the PSF (line width and length), the wavelength ranges
!		on the CCD, the predicted position (x,y) for one wavelength. 
!               Generates the free spectral range, prepares the configuration,
!		before CALLing "uves_psf.c" routine.
!
!.ALGORITHM     Creates modfree_sp_rg.tbl, prepares data and
!		configuration before running the physical model. 
!		Midas interface routine with the C routine "uves_psf.c".
!
!.ENVIRON       UVES context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        O.Boitquin, ESO/DMD
!
!.VERSION       1999/10/04  Created (OB).
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_psf.prg,v $
! Revision 1.3  2001/01/04 10:39:33  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
!

define/param  P1 {WLC}		IMA   	"Input frame:"
define/param  P2 {WCENTER}	NUMB	"wavelength: " 
define/param  P3 0.,0.    	NUMB	"Translation ccd (X,Y): "
define/param  P4 0.,0.,0.    	NUMB 	"Angle offsets (echelle, CD, CCD rot):"
define/param  P5 N		CHAR 	"Generates the free spectral range table: Yes or No"

set/format F13.7
define/local trans/D/1/2 {p3}
define/local angoff/D/1/3 {P4}
define/local fsr_opt/C/1/1 {P5}
fsr_opt = m$upper(fsr_opt)

!*****************************************************************
!
! Initialize configuration keywords
!
!*****************************************************************
!
!   Configuration tables
!
!Index:  1       2       3      4      5      6
!
!Arm:   Blue    Blue    Red    Red    Red    Red
!Chip:  EEV     EEV     EEV    EEV    MIT    MIT
!CDisp:  1       2       3      4      3      4
!

set/format F20.10  I1


!*****************************************************************
!
! Reading configuration information from FITS header
!
!*****************************************************************

define/local arm/C/1/4      	"BLUE"
define/local ArmId/C/1/1	"b"
define/local chip/C/1/3     	"e"
define/local DO_CLASS/C/1/10 	{{P1},DO_CLASSIFICATION}
define/local modext/c/1/25 " "  ALL    
define/local wcent/D/1/1    	0.
define/local ECHID/I/1/1    	0
define/local CDID/I/1/1     	0
define/local cfgindx/I/1/1   	0
define/local FRSPRG/C/1/20	"modfree_sp_rg.tbl"
define/local Xtrans/D/1/1   	0.
define/local Ytrans/D/1/1   	0.
define/local echoff/D/1/1   	0.
define/local CDoff/D/1/1   	0.
define/local ccdrotoff/D/1/1   	0.
define/local Blaze/D/1/1   	0.

! For messout

define/local pid/C/1/20 "uves_psf"
mess/out 3 {pid} "Determine the PSF, position,... for one wavelength"

config/inst {p1}

! Arm data are read from FITS keyword ESO INS PATH, e.g. 'BLUE'
arm = q2(2:)
ArmId = m$lower(arm(1:1))

chip = m$lower(CCD(1:1))				!Chip identifier (CCD)
!mess/out 3 {pid} "Detector: {CCD} (type: {chip})"
CDID = {CROSS(4:4)}					!Cross-disperser id.
!mess/out 3 {pid} "Cross-disperser number: {CDID}"
wcent = {WCENTER}/10.0					!Central wavel. [nm]
!mess/out 3 {pid} "Central wavelength: {WCENT} nm"

set/format F5.1
define/local tempcam/d/1/1 0. ? +lower_levels		
tempcam = {{p1},{h_tempcam({PATHID})}}			!Temperature [Celsius]

define/local pressure/d/1/1 0. ? +lower_levels
pressure = {{p1},{h_pressure}}				!Air pressure [bar]

! Humidity (ignored for the moment)
define/local humid/d/1/1 0. ? +lower_levels
humid = 3						!humidity
							!=water pressure [mmHg]
define/local slitwidth/d/1/1 0. ? +lower_levels
slitwidth = {{p1},{h_slwidth({PATHID})}}		!slit width [arcsec]

define/local slitlength/d/1/1 0. ? +lower_levels
slitlength = {{p1},{h_sllen({PATHID})}}			!slit length [arcsec]

define/local wavelength/d/1/1 0. ? +lower_levels
wavelength = {p2}					!wavelength [Angstroem]


append/out "UVES-configuration:\n"
append/out "  Optical path: {arm} ({ArmId})\n"
append/out "  Detector: {CCD}, Binning: {CCDBIN(1)},{CCDBIN(2)}\n"
append/out "  Cross-disperser (number): {CROSS} ({CDID})\n"
append/out "  Central wavelength: {WCENT} nm\n"
append/out "  Slit width: {slitwidth} arcsec\n"
append/out "  Slit length: {slitlength} arcsec\n"
append/out "  Temperature: {tempcam} C\n"
append/out "  Air pressure: {pressure} bar\n"
append/out "  Humidity: {humid} mmHg\n" ! (IGNORED)\n"
append/out "  Wavelength :  {wavelength} Angstroem\n"
mess/out 3 {pid}

write/keyword FRSPRG "modfree_sp_rg.tbl"
write/keyword Xtrans {trans(1)}
write/keyword Ytrans {trans(2)}
write/keyword echoff {angoff(1)}
write/keyword CDoff  {angoff(2)}
write/keyword ccdrotoff {angoff(3)}
 
run PIPE_EXE:uves_psf.exe
if Q1 .eq. "1" return 1
append/out "PSF for {wavelength} (A)ngstroem\n"
set/format F5.3
append/out "Line width :  {outputd(2)} pixel\n"
append/out "Line length : {outputd(3)} pixel\n"
set/format F5.6
append/out "Line width :  {outputd(4)} Angstroem\n"
append/out "Binsize :     {outputd(5)} mA\n"
append/out "Echelle pixel-scale : {outputd(6)} arcsec/pixel\n"
append/out "CD pixel-scale : {outputd(7)} arcsec/pixel\n"
Blaze = m$abs({outputd(20)})
append/out "Blase function : {Blaze} \n"
set/format F5.2
append/out "Resolution : {outputd(8)}\n"
set/format F5.0
append/out "Absolute Order : {outputd(9)}\n"
append/out "Predicted position (X,Y) for {wavelength} A: ({outputd(10)},{outputd(11)})\n"
set/format F5.2
append/out "Wavelength range : {outputd(12)} - {outputd(13)}\n"
append/out "Central wavelength of the CCD : {outputd(1)} nm\n"
set/format F5.0
append/out "Free spectral Order range : {outputd(15)} - {outputd(14)}\n"
append/out "Number of order on the free spectral range : {outputd(16)}\n"
append/out "Real order range on CCD : {outputd(18)} - {outputd(17)}\n"
append/out "Real number of order on CCD : {outputd(19)}\n"
mess/out 3 {pid}

if fsr_opt(1:1) .eq. "Y" then
   set/format F4.0
   assign/print FILE modfree_sp_rg.dat
   print/table modfree_sp_rg.tbl
   -rename modfree_sp_rg.dat modfree_sp_rg_{wcent(1:3)}_{CCD}.dat
   !set/format F5.1
endif

if nice_crea .eq. "Y" then
   define/local xstart/I/1/1 0
   define/local ystart/I/1/1 0
   define/local xend/I/1/1 0
   define/local yend/I/1/1 0
   define/local mbox/I/1/2 60,60
   define/local limits/I/1/4 0,0,0,0
   limits(1) = imsize(1) - mbox(1)/2
   limits(2) = imsize(2) - mbox(2)/2
   limits(3) = mbox(1)/2
   limits(4) = mbox(2)/2
   xstart = outputd(10) - mbox(1)/2
   ystart = outputd(11) - mbox(2)/2
   xend = outputd(10) + mbox(1)/2
   yend = outputd(11) + mbox(2)/2
   
   outputi = 0
   if xstart .gt. limits(3) .and. xstart .lt. limits(1) then
     if ystart .gt. limits(4) .and. ystart .lt. limits(2) then
       draw/rectangle {xstart},{ystart},{xend},{yend} F ? green
       outputi = 1
     endif
   endif
   if outputi(1) .eq. 0 then
      set/format F5.0
      w/o "Position (x,y) for {wavelength} A :({outputd(10)},{outputd(11)}) is outside of CCD field"
   endif
endif

return 0

