! $Id: uves_putframes.prg,v 1.5 2001/07/04 08:02:15 amodigli Exp $
!+++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT   (C) 1998  European Southern Observatory (ESO)
!
!.IDENT       uves_putframes.prg
!
!.KEYWORDS    Write result frames to master catalog/disk file.
!
!.USAGE       put/frames  catname froot [main]
!
!.INPUT       catname     (P1)    Catalog containing result frames.
!             prolist     (P4)    List file name for detailed list of
!                                 output frames
!
!.OUTPUT      froot       (P2)    Basename of all output files.
!             main        (P3)    Main pipeline product identifier
!
!.RETURNS     The recipe returns 0 on success or 1 on error. Keyword Q1 is
!             updated with the exit status. 
!
!.PURPOSE     The procedure converts the frames stored in the catalog
!             'catname' from MIDAS internal data formats to FITS. The
!             descriptor given by the keyword UVES_IDSTRNG is removed from the
!             result frames before they are converted to FITS format. The
!             MIDAS file name of the main pipeline data product may be given,
!             so that it is the first product frame that is converted to FITS
!             format. 
!
!.ENVIRON     UVES context
!
!.LANGUAGE    MIDAS command language 98NOV.
!
!.AUTHOR      R. Palsa, ESO-DMO
!
!.VERSION     0.5  1998/04/01:  Creation
!		   1999/03/11: Copy from FORS context
!		   1999/09/26: Check number of entries in result catalog!
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.5 $
! $Log: uves_putframes.prg,v $
! Revision 1.5  2001/07/04 08:02:15  amodigli
! Suppressed automatic writing of FITSMD5 key,
! now done at MIDAS level
!
! Revision 1.4  2001/05/28 12:48:21  amodigli
! We now chmod only on first pipe product
!
! Revision 1.3  2001/05/18 05:58:00  amodigli
! Added checksum info to pipe data products
!
! Revision 1.2  2001/01/04 10:39:33  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref  results froot main list
define/param p1 ?  c    "Enter result frame catalog:"
define/param p2 ?  c    "Enter output file root:"
define/param p3 +  c    "Enter main data product name:"
define/param p4 +  c    "Enter detailed list name:"

define/maxpar 4


! Procedure identification
define/local procID/c/1/80      "PutFrames"


! Local constants
define/local maxchar/i/1/1   256
define/local failmsg/c/1/20  "NOT converted..."


! Local variables
define/local i/i/1/1                 0
define/local pos1/i/1/1              0
define/local pos2/i/1/1              0
define/local catal/i/1/1             0
define/local n_frames/i/1/1          0
define/local n_ok/i/1/1              0
define/local n_failed/i/1/1          0
define/local n_deliver/i/1/1         0
define/local fctrl/i/1/2             0     all
define/local cat_result/c/1/100      {p1}
define/local froot/c/1/100           {p2}
define/local mdpr/c/1/100            {p3}
define/local prolist/c/1/100         {p4}
define/local cat_entry/c/1/80        " "   all
define/local class/c/1/30            " "   all
define/local errmsg/c/1/80           " "   all
define/local proinfo/c/1/{maxchar}   " "   all
define/local pipefile/c/1/{maxchar}  " "   all


 
! Get catalog information
show/icat {cat_result} N
n_frames = outputi(1)
if n_frames .eq. 0 return 1

! Update file header of output frames
do i = 1 n_frames 1
  store/frame cat_entry {cat_result} 1

  if m$existd(cat_entry,UVES_IDSTRNG) .eq. 1 then
    delete/descr {cat_entry} {UVES_IDSTRNG}
  endif

  if m$existd(cat_entry,H_FILENAME) .eq. 1 then
    delete/descr {cat_entry} {H_FILENAME}
  endif
  
  write/descr {cat_entry} {H_PRODMD5}/c/1/32 " " all
  write/dhelp {cat_entry} {H_PRODMD5} "{I_PRODMD5}"
enddo

! Write output to disk
pos1 = m$indexb(froot,"/") - 1

if mdpr(1:1) .eq. "+" then
  @d set_of_frames OUT {cat_result} {froot} + {prolist} >Null
else
  @d set_of_frames OUT {cat_result} {froot} {mdpr} {prolist} >Null
endif

errmsg = mid$errmess
n_ok = outputi(1)
froot = froot(1:{pos1})

if n_ok .eq. 0 then
  mess/out 1 {procID} "Cannot write product frames!"
  return 1
else if n_ok .lt. 0 then
  n_ok = m$abs(n_ok) - 1
  n_failed = n_frames - n_ok
endif

mess/out 3 {procID} " "
mess/out 3 {procID} "*** Processing Log ***"
mess/out 3 {procID} " "
mess/out 3 {procID} "Product directory: {froot}"
mess/out 3 {procID} " "


! Optionally write out list of converted files
if prolist(1:1) .ne. "+" .and. m$exist(prolist) .eq. 1 then
  open/file {prolist} READ fctrl
  i = 0

  pro_loop:

  read/file {fctrl(1)} proinfo {maxchar}
  if fctrl(2) .ge. 0 then
    i = i + 1
    pos1 = m$index(proinfo," ") - 1
    pipefile = proinfo(1:{pos1})
    pos1 = pos1 + 2
    proinfo = proinfo({pos1}:)
      
    if m$indexb(proinfo,failmsg) .le. 0 then
      pos2 = m$indexb(proinfo,"/")
      if pos2 .gt. 0 then  
        pos2 = pos2 + 1
      endif

      if m$existd(pipefile,h_procatg) .eq. 1 then
        class = m$value({pipefile},{h_procatg})
        mess/out 3 {procID} "Frame {pipefile} ({class})"
        mess/out 3 {procID} "  saved as {proinfo({pos2}:)}"
      else
        mess/out 3 {procID} "Frame {pipefile}"
        mess/out 3 {procID} "  saved as {proinfo({pos2}:)}"
      endif

      if p_mainonly .gt. 0 .and. i .eq. 1 then
        mess/out 3 {procID} "  and delivered to DHS as main product"
        n_deliver = 1
      else if p_mainonly .eq. 0 then
        mess/out 3 {procID} "  and delivered to DHS"
        n_deliver = n_deliver + 1
      endif
    else
      if m$existd(pipefile,h_procatg) .eq. 1 then
        class = m$value({pipefile},{h_procatg})
        mess/out 3 {procID} -
                 "Frame {pipefile} ({class}) {proinfo}"
      else
        mess/out 3 {procID} -
                 "Frame {pipefile} ({class}) {proinfo}"
      endif
    endif
       
    goto pro_loop
  else
    close/file {fctrl(1)}
    if n_failed .gt. 0 then
      errmsg(1:1) = m$upper(errmsg(1:1))
      mess/out 1 {procID} "{errmsg}"
    endif
    mess/out 3 {procID} " "
  endif
else
  if n_ok .gt. 0 .and. p_mainonly .eq. 1 then
    n_deliver = 1
  else
    n_deliver = n_ok
  endif

  if n_failed .gt. 0 then
    errmsg(1:1) = m$upper(errmsg(1:1))
    mess/out 1 {procID} "{errmsg}"
    mess/out 3 {procID} " "
  endif
endif

set/format i1
mess/out 3 {procID} "{n_ok} out of {n_frames} frames successfully written!"


! If pipeline delivery information is available show number
! of frames delivered to DHS
if p_mainonly .ge. 0 then
  mess/out 3 {procID} "{n_deliver} frame(s) delivered to DHS!"
endif
set/format

if n_failed .gt. 0 then
  set/format i1
  mess/out 1 {procID} "{n_failed} frame(s) could not be saved!"
  set/format
  return 1
endif

return 0
