! $Id: uves_reduce_spat.prg,v 1.6 2001/05/07 08:44:48 amodigli Exp $
!+++ UVES_REDUCE ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_reduce_2d.prg 
!
!.KEYWORDS      spectroscopy, echelle, science reduction
!
!.USAGE         @p (IDENT) incat [outcat] refcat 
!
!.INPUT         incat   (P1): set of input frames (ascii list)
!               refcat  (P3): set of reference frames (ascii list)
!				. DRS setup table
!				. order table
!				. background table
!				. line table for object slit window (o): {LINTAB_ID({PATHID})}2
!				. master flatfield
!				. [master bias]
!               BKGMTD  (p4): background subtraction method: MINIMUM, [median]
!
!.OUTPUT        outcat	(P2): path + basename of output files:
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!		Q2:  the classification of the main product
!
!.DEFAULT	P2	+
!
!
!.PURPOSE       Perform the science data reduction of UVES observations
!               keeping spatial resolution
!
!.ALGORITHM     The data reduction is based on commands from the ECHELLE
!		context: EXTRACT/ECHELLE, REBIN/ECHELLE, MERGE/ECHELLE ...
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 99NOV
!
!.AUTHOR        Andrea Modigliani, ESO-DMO
!
!.VERSION       0.5 2000/07/06  Create
!
!.COMMENT	
!		
!.TODO		
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.6 $
! $Log: uves_reduce_spat.prg,v $
! Revision 1.6  2001/05/07 08:44:48  amodigli
! Taken out monitor of bkground (@p uves_bkg_qc)
!
! Revision 1.5  2001/05/04 14:17:03  amodigli
! Typos corrected
!
! Revision 1.4  2001/05/04 14:03:14  amodigli
! New bkg parameter extraction
!
! Revision 1.3  2001/01/04 10:39:54  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref input catalog refer
define/param p1 ? c "enter name of input catalog:"
define/param p2 ? c "Enter name of output catalog:"
define/param p3 ? c "Enter name of reference catalog:"
define/param p4 MED,E,AV c "Enter extr. pars.: bkg[median]/minimum,FF[E]/P/N,Merge[A]/O/N:"
!
! Global variables
!
define/local pid/c/1/10 "reduce_spatial" ? +lower_levels
define/local LINTAB_2/c/1/100 " " ALL +lower_levels
define/local modext/c/1/25 " " ALL +lower_levels
define/local FFMTD/c/1/10 " " ALL +lower_levels


!
! Local variables
!
define/local ifrm/c/1/100 " " all
define/local ext/c/1/4 "" all
define/local num/i/1/1 0
define/local catal/i/1/1 0
define/local main_prod/c/1/25 " " all

define/local parBkg/c/1/10 " " all
define/local parFF/c/1/10 " " all
define/local parMer/c/1/10 " " all
define/local k/i/1/1 0
  
$date
k = m$parse(p4,"par")
     if k .eq. 3 then
  parBkg = "{par01}"
  parFF  = "{par02}"
  parMer = "{par03}"
else if k .eq. 2 then
  parBkg = "{par01}"
  parFF  = "{par02}"
  parMer = "Average"
else if k .eq. 1 then
  parBkg = "{par01}"
  parFF  = "E"
  parMer = "Average"
else if k .eq. 0 then
  parBkg = "Minimum"
  parFF = "E"
  parMer = "Average" 
else
 mess/out 3 {pid} "Too many sub-pars for p3, max 3: BkgMtd, FFMTD, MERGMETD"
 return -1
endif

parBkg = m$upper(parBkg)
parFF  = m$upper(parFF)
parMer = m$upper(parMer)

!========================================
!Check on reduction parameters values
!========================================
     if "{parBkg(1:2)}" .eq. "MI" then !OK
else if "{parBkg(1:2)}" .eq. "ME" then !OK
else 
   mess/out 3 {pid} "Bkg parameter value {parBkg} not allowed!MINIMUM/[MEDIAN]"
endif
     if "{parFF(1:1)}" .eq. "E" then !OK
else if "{parFF(1:1)}" .eq. "P" then !OK
else if "{parFF(1:1)}" .eq. "N" then !OK
else 
   mess/out 3 {pid} "Flat Fielding parameter value {parFF} not allowed![E/P]"
endif
     if "{parMer(1:1)}" .eq. "A" then  !OK
else if "{parMer(1:1)}" .eq. "O" then  !OK
else if "{parMer(1:1)}" .eq. "N" then  !OK
else 
   mess/out 3 {pid} "Merging parameter value {parMer} not allowed![AVERAGE/OPTIMAL/NOAPPEND]"
endif



if m$exist(p2) .eq. 0 .and. p2(1:1) .ne. "+" -
   create/icat {p2} NULL {UVES_IDSTRNG} >Null

parse/path {p1}
ext = m$upper(q3)

if ext(1:4) .eq. ".CAT" then
   show/icat {p1} >Null
   num = outputi(1)
   if num .lt. 1 goto ERR1
   goto CATALOG
else if ext(1:4) .eq. ".BDF" then
   ifrm = p1
   goto SINGLE
else
   goto ERR2
endif


set/format I1
CATALOG:
   store/frame ifrm {p1} 1 FINI
 
SINGLE:
   write/out " "
   @p uves_reduce_spat,CONFIG {ifrm} {p3} {parBkg} {parFF} {parMer}
   if {q1} .lt. 0 goto CHECK

   @p uves_reduce_spat,REDUCE {ifrm} {parMer}
   if {q1} .lt. 0 goto CHECK
   if parMer(1:1) .ne. "N" then
      mess/out 3 {pid} "DO classification..."
      @p uves_reduce_spat,DO_CLASS {ifrm} {p2}
   else
      mess/out 2 {pid} "If MERMTD=NOAPPEND no classification!"
   endif
   main_prod = q1

CHECK:
   if ext(1:4) .eq. ".BDF" goto FINI
   goto CATALOG

FINI:
   return 0 {main_prod}

ERR1:
   write/out "empty input catalog"
   return -1

ERR2:
   write/out "You have to specify full name of input frame (.cat, .bdf)!"
   return -1

ERR3:
   write/out "Input file `{ifrm}` does not exist !"
   return -1


!========SUBROUTINES======================================

!=========================================================
! ENTRY CONFIG
!
! Description: Configures the echelle data reduction
!              environment variables.
!              Search for all the necessary reference
!              frames, subtracts the bias,
!              subtracts background from object
!              and Flat Field, Flat Field the object
!              using pixel-to-pixel method
!
!=========================================================
ENTRY CONFIG
define/param p1 ?    c "Enter input frame"
define/param p2 ?    c "Enter reference frame"
define/param p3 MINIMUM    c "Enter background method (median,minimum)"
define/param p4 ?    c "Enter flat fielding method (P,E)"
define/param P5 AVERAGE c "Enter merging method (Average/Optimal)"

!===========================
! Local variables definition
!===========================
define/local rsample/r/1/1 0.0
define/local n_order/i/1/1 1
define/local ron/r/1/1 0.0
define/local gain/r/1/1 0.0

config/instr {p1} !This set up the right instrument configuration
                  !and so also the right PATHID
modext = q2

!=======================
! Search for Order table
!=======================
search/icat {p2} {ORDER_ID({PATHID})} NB
if m$strlen(out_a) .eq. 0 then
   w/o "No appropriate order table (obj) found!
   goto SKIP
else
   SET/ECHELLE ORDTAB = {out_a}
endif

!=======================
! Search for Bkg table
!=======================
search/icat {p2} {BACK_ID({PATHID})} NB
if m$strlen(out_a) .eq. 0 then
   append/out "No appropriate BKG table found!
   goto SKIP
else
   SET/ECHELLE BAKTAB = {out_a}
endif

!======================
! Search for Line Table
!======================
search/icat {p2} {LINTAB_ID({PATHID})}2 NB
if m$strlen(out_a) .eq. 0 then
   append/out "No appropriate line table (obj) found!
   goto SKIP
else
   LINTAB_2 = out_a
endif

!==========================================
! Search for BIAS Ima if found subtracts it
!==========================================
search/icat {p2} {MBIAS_ID({PATHID})}2 NB
if m$strlen(out_a) .eq. 0 then
   append/out "No appropriate master bias image found!\n No bias subtraction"
else
   if m$exist(p1,"BIAS_SUBTR") .ne. 0 then
     mess/out 3 {pid} "Bias already subtracted !"
   else
      mess/out 3 {pid} "subtracting Bias...."
      compute/image &b = 1.0*{p1} -{out_a}
      -rename middummb.bdf {p1}
      write/descriptor {p1} BIAS_SUBTR/c/1/3 YES  !To indicate bias subtraction
   endif
endif

!=====================
! Subtracts Background
!=====================
if m$existd(p1,"BACKGROUND") .eq. 0 then
   mess/out 3 {pid} "Subtract background from {p1}  ..."
!am: old code   
!   subtract/back {p1} bg_{p1} &b P5={p3}  >Null !subtracts background from science
!   -rename middummb.bdf {p1}
!am: new code 06/04/01
   -copy {p1} pre_bg_sub_{p1}
   subtract/back {p1} tmp_bg_{p1} &b >Null
   define/local sizeX/i/1/1  0
   define/local sizeY/i/1/1  0
   sizeX = m$value({p1},NPIX(1))
   sizeY = m$value({p1},NPIX(2))
   define/local radSmoX/i/1/1 25
   define/local radSmoY/i/1/1 10
   radSmoX = 300 * {sizeX}/4096   
   define/local wlen/i/1/1 0
   wlen = m$value({p1},{h_cwlen({PATHID})})
   if wlen .eq. 860 then
     radSmoY = 500 * {sizeY}/2048  
   else
     radSmoY = 200 * {sizeY}/2048
   endif
   w/o "radSmoX={radSmoX} radSmoY={radSmoY}"
   filter/smooth tmp_bg_{p1}  bg_{p1} {radSmoX},{radSmoY} >Null
   compute/image {p1} = pre_bg_sub_{p1} - bg_{p1}
!am: end new code  

   load/ima bg_{p1}

else
   mess/out 3 {pid} "Background from {p1} already subtracted!"
endif


!==========================================
! Search for Flat Ima if found subtracts it
!==========================================
search/icat {p2} {MFLAT_ID({PATHID})} NB
if m$strlen(out_a) .eq. 0 .or. p4(1:1) .eq. "N" then
   if p4(1:1) .eq. "N" then
       mess/out 2 {pid} "Flatfield disabled!"
   else
       mess/out 2 {pid} "No appropriate master flat image found!\n No flat correction"
   endif
   set/echelle FFOPT=NO CORRECT="no_ff"
   FFMTD = "NO"
else
   set/echelle flat = {out_a}
   if m$existd(flat,"BACKGROUND") .gt. 0 then
      mess/out 3 {pid} "Background from FF already subtracted!"
      set/echelle correct = {flat}
    else
      mess/out 3 {pid} "Subtract background from FF..."
      set/echelle correct = b_{flat} BLAZE = z_{flat}

      !subtracts background from FF
!am: old code      
!      subtract/back {flat} bg{modext} {correct} P5={p3} >Null 
!am: new code 06/04/01
      -copy {flat} pre_bg_sub_{flat}
      subtract/back {flat} tmp_bg_{flat} &b >Null
      define/local sizeXff/i/1/1  0
      define/local sizeYff/i/1/1  0
      sizeXff = m$value({flat},NPIX(1))
      sizeYff = m$value({flat},NPIX(2))
      define/local radSmoXff/i/1/1 25
      define/local radSmoYff/i/1/1 10
      do_class = "{{flat},{UVES_IDSTRNG}}"
      if "{do_class}" .eq. "FLAT_BLUE" then
         radSmoX = 25 * {sizeX}/4096
      else
         radSmoX = 50 * {sizeX}/4096
      endif
      if wlen .eq. 860 then
         radSmoYff = 300 * {sizeY}/2048
      else
         radSmoYff = 100 * {sizeY}/2048
      endif
      w/o "radSmoXff={radSmoXff} radSmoYff={radSmoYff}"
      filter/smooth tmp_bg_{p1}  bg{modext} {radSmoXff},{radSmoYff} >Null
      compute/image {correct} = pre_bg_sub_{flat} - bg{modext}
!am: end new code  
      delete/desc {correct} {h_origname} N
   endif


    !======================
    ! Flat Field Correction
    !======================
    if p4(1:1) .eq. "P" then
       if m$existd(p1,"FLATMETHOD") .ne. 0 then
          mess/out 3 {pid} "Flat Fielding already done"
       else
          mess/out 3 {pid} "Do the pixel-to-pixel Flat Fielding ..."
          statistic/ima {correct} ? ? ? SN >Null   !Do the normalization
          compute/ima {p1} = {p1} / {correct} * {outputr(3)}
          write/desc {p1} CUNIT/c/1/16 "RELATIVE FLUX"
          write/desc {p1} FLATMETHOD/c/1/11 PIXEL
          set/echelle FFOPT=DONE
          FFMTD = "PIXEL"
       endif
       set/echelle FFOPT=DONE
       FFMTD = "PIXEL"
    else
       mess/out 3 {pid} "Do the flatfielding during extraction."
       write/desc {flat} FLATMETHOD/c/1/11 EXTRACT
       set/echelle FFOPT=YES
       FFMTD = "EXTRACT"
    endif
endif

!====================
! Environment setting
!====================
SET/ECHELLE LINTAB = {LINTAB_2}
mess/out 3 {pid} "Used LINE Table: {LINTAB}"

echord(2) = m$value({LINTAB},ORDER(2))
echord(3) = m$value({LINTAB},ORDER(1))
n_order = echord(3) - echord(2) +1
rsample = 2. *{{LINTAB},PIXEL(1)}/3.


SET/ECHELLE SAMPLE = {RSAMPLE}
SET/ECHELLE EXTMTD=Linear
SET/ECHELLE SLIT=1.0
SET/ECHELLE MRGMTD={p5}
SET/ECHELLE MRGORD=1,{n_order}

!w/o "first order: {echord(2)}"
!w/o "last  order: {echord(3)}"
!w/o "Number of detected orders: {n_order}"



mess/out 3 {pid} "INFO: sampling step set to {SAMPLE}"

!=============================
! Set Read Out Noise
!=============================
ron = m$value({p1},{h_ron(1)})
if ron .le. 0.0 then
   ron = 5.0
   mess/out 3 {pid} "Bad RON value detected, take default: {ron} !"
endif

!=============================
! Set gain
!=============================
gain = m$value({p1},{h_confac(1)})
if gain .le. 0.0 then
   gain = 2.1
   mess/out 3 {pid} "Bad GAIN=CONAD value detected, take default: {gain} !"
endif

return 0

SKIP:
  w/o "--> Skip input frame: `{p1}'"
  return -1
!=========================================================
! END ENTRY CONFIG
!=========================================================
!*********************************************************
!=========================================================
! BEGIN ENTRY REDUCE
!
! Description: Extracts, rebins and merges the spectrum
!              Using a variable offset and a constant
!              extraction slit size of 1 pix width to scan
!              the whole observing slit size. In the end
!              reconstruct a 2D image as a composite of
!              each extracted,rebinned,merged spectra
!
!=========================================================
ENTRY REDUCE
define/param p1 ?    c "Enter input frame"
define/param P2 AVERAGE c "Enter merging method (Average/Optimal)"

define/local N/i/1/1

define_local slit_size_arcs/d/1/1 0.0
define/local slit_size/d/1/1 2.0
define/local pix_scale/d/1/1 1.0
define/local ccd_bin/d/1/1 {CCDBIN(2)}
define/local slit_size_pix/i/1/1 2.0
define/local slit_size_2_pix/i/1/1 1.0
define/local off/i/1/1 1

! to not make the key as global after using write/key
define/local dsample/r/1/1 0.0      
          
pix_scale = m$value({p1},{h_pixscl})
slit_size_arcs = m$value({p1},{h_sllen({PATHID})})
slit_size = m$nint({slit_size_arcs}/{pix_scale}/{ccd_bin}/2)*2 -2 !before here I was subtracting 2 pix
!slit_size = m$nint({slit_size_arcs}/{pix_scale}/{ccd_bin})
slit_size_pix = slit_size
slit_size_2_pix = {slit_size_pix}/2

mess/out 3 {pid} "slit size: {slit_size_pix} pix "
mess/out 3 {pid} "1/2 slit size: {slit_size_2_pix} pix "

!$date

!w/o "SAMPLE set to {SAMPLE}"
write/key dsample {SAMPLE}
!w/o "DSAMPLE set to {dsample}"

!Spectra extraction
mess/out 3 {pid} "Starting extraction..."
if FFMTD(1:1) .eq. "N" then
    mess/out 3 {pid} "extraction without Flat Fielding"
    EXTRACT/EC2D {p1} xb2d_{p1} {slit_size_pix},0 S
    -copy xb2d_{p1} fxb2d_{p1}
    mess/out 3 {pid} "rebin extracted NO FF object"
    REBIN/EC2D xb2d_{p1} wfxb2d_{p1} {dsample} >Null

    if  p2(1:1) .eq. "O" then
       mess/out 3 {pid} "calculate variance"
       @p uves_reduce_spat,CALC_VARY wfxb2d_{p1} var_wfxb2d_{p1}

       mess/out 3 {pid} "calculate error"
       compute/image err_wfxb2d_{p1} = sqrt ( var_wfxb2d_{p1} )
       copy/dd xb2d_{p1} *,3 var_wfxb2d_{p1}
       copy/dd xb2d_{p1} *,3 err_wfxb2d_{p1}

       !mess/out 3 {pid} "rebin error and variance frames"
       !rebin/ec2d err_wfxb2d_{p1} werr_wfxb2d_{p1} {dsample} NONL {LINTAB} >Null
       !rebin/ec2d var_wfxb2d_{p1} wvar_wfxb2d_{p1} {dsample} NONL {LINTAB} >Null
       $cp err_wfxb2d_{p1} werr_wfxb2d_{p1}
       $cp var_wfxb2d_{p1} wvar_wfxb2d_{p1}
       
       mess/out 3 {pid} "calculate weights"
       compute/image wght_wfxb2d_{p1} = 1.0 / wvar_wfxb2d_{p1} >Null
       compute/image wght_wfxb2d_{p1} = wght_wfxb2d_{p1} + 1.0e-4
       mess/out 3 {pid} "order merging"
       MERGE/EC2D wfxb2d_{p1} mwfxb2d_{p1} ? OPTIMAL wght_wfxb2d_{p1} var_mwfxb2d_{p1}
    else
       MERGE/EC2D wfxb2d_{p1} mwfxb2d_{p1} {p3}
    endif
else if FFMTD(1:1) .eq. "P" then
    mess/out 3 {pid} "Pixel-to-Pixel extraction"
    EXTRACT/EC2D {p1} xb2d_{p1} {slit_size_pix},0 S
    -copy xb2d_{p1} fxb2d_{p1}
    mess/out 3 {pid} "rebin extracted FF object"
    REBIN/EC2D fxb2d_{p1} wfxb2d_{p1} {dsample} 

    if  p2(1:1) .eq. "O" then
       mess/out 3 {pid} "calculate variance"
       @p uves_reduce_spat,CALC_VARY wfxb2d_{p1} var_wfxb2d_{p1}

       mess/out 3 {pid} "calculate error"
       compute/image err_wfxb2d_{p1} = sqrt ( var_wfxb2d_{p1} )
       copy/dd fxb2d_{p1} *,3 var_wfxb2d_{p1}
       copy/dd fxb2d_{p1} *,3 err_wfxb2d_{p1}

       !mess/out 3 {pid} "rebin error and variance frames"
       !skip this step (anyway the image are with same sampling steps)
       !compute/ima corrupts the format of a 2d rebbinned spectra => 
       !REBIN/ECH2D after compute/ima gives strange results
       !rebin/ec2d err_wfxb2d_{p1} werr_wfxb2d_{p1} {dsample} !NONL {LINTAB} 
       !rebin/ec2d var_wfxb2d_{p1} wvar_wfxb2d_{p1} {dsample} !NONL {LINTAB} 
       
       $cp err_wfxb2d_{p1} werr_wfxb2d_{p1}
       $cp var_wfxb2d_{p1} wvar_wfxb2d_{p1}
       mess/out 3 {pid} "calculate weights"
       compute/image wght_wfxb2d_{p1} = 1.0 / wvar_wfxb2d_{p1} >Null
       set/format f13.8
       compute/image wght_wfxb2d_{p1} = wght_wfxb2d_{p1} + 0.1
       set/format
       mess/out 3 {pid} "order merging"
       MERGE/EC2D wfxb2d_{p1} mwfxb2d_{p1} ? OPTIMAL wght_wfxb2d_{p1} var_mwfxb2d_{p1}
    else
       MERGE/EC2D wfxb2d_{p1} mwfxb2d_{p1} P4={p2}
    endif
else
    mess/out 3 {pid} "Extract the object"
    EXTRACT/EC2D {p1} xb2d_{p1} {slit_size_pix},0 S
    copy/dd {p1} *,3  xb2d_{p1}
    
    mess/out 3 {pid} "Extract the FF"
    EXTRACT/EC2D {correct} x{correct}        !Extracts the FF
    
    if  p2(1:1) .eq. "O" then  
        mess/out 3 {pid} "Compute VARIANCES of extracted flatfield ..."
        @p uves_reduce_2d,CALC_VARI x{correct} var_x{correct} p4=1
        copy/dd {correct} *,3 x{correct}		!copy the ESO-header.
        delete/desc x{correct} {h_origname} N
    endif
    
    mess/out 3 {pid} "Rescale extracted FF..."
    statistic/imag x{correct} ? ? ? SN
    compute/imag x{correct} = x{correct} / {outputr(3)}
    
    if  p2(1:1) .eq. "O" then      
        compute/imag var_x{correct} = var_x{correct} / {outputr(3)}**2
    endif
    
    mess/out 3 {pid} "FLATFIELD and RESAMPLE the extracted object ..."
    compute/ima fxb2d_{p1} = xb2d_{p1} / x{correct} >Null
   
    write/desc fxb2d_{p1} CUNIT/c/1/16 "RELATIVE FLUX"    

    SET/FORMAT F9.7,  
    REBIN/EC2D fxb2d_{p1} wfxb2d_{p1} {dsample}  >Null
    SET/FORMAT  
    write/desc wfxb2d_{p1} CUNIT/c/1/16 "Relative Flux"    

    if  p2(1:1) .eq. "O" then   
        mess/out 3 {pid} "Compute VARIANCE of flatfielded object ..."
        @p uves_reduce_spat,CALC_VARY xb2d_{p1} var_xb2d_{p1}
        @p uves_reduce_spat,CALC_V1V2 -
              xb2d_{p1} x{correct} var_xb2d_{p1} -
              var_x{correct} tot_var_xb2d_{p1}

        mess/out 3 {pid} "calculate error"
        compute/image err_xb2d_{p1} = sqrt ( tot_var_xb2d_{p1} )
        copy/dd xb2d_{p1} *,3 tot_var_xb2d_{p1}
        copy/dd xb2d_{p1} *,3 err_xb2d_{p1}
        delete/des tot_var_xb2d_{p1} {h_origname} N
        delete/des err_xb2d_{p1} {h_origname} N

        mess/out 3 {pid} "rebin"
        rebin/ec2d err_xb2d_{p1} werr_wfxb2d_{p1} {dsample} NONL {LINTAB} >Null
        rebin/ec2d tot_var_xb2d_{p1} wvar_xb2d_{p1} {dsample} NONL {LINTAB} >Null

        mess/out 3 {pid} "calculate weights"
        compute/image wght_wfxb2d_{p1} = 1.0 / wvar_xb2d_{p1} >Null
        compute/image wght_wfxb2d_{p1} = wght_wfxb2d_{p1} + 1.0e-4

        mess/out 3 {pid} "MERGE the orders of the object window ...\n"

        MERGE/EC2D wfxb2d_{p1} mwfxb2d_{p1} ? OPTIMAL wght_wfxb2d_{p1} var_mwfxb2d_{p1}   
   else        
        MERGE/EC2D wfxb2d_{p1} mwfxb2d_{p1} P4={p2}
        
   endif        
endif    


mess/out 3 {pid} "Line-by-line extraction, rebin, merging done."
if p2(1:1) .eq. "N" then
  @p uves_reduce_spat,DISPLAY_2D mwfx0001      -20 4
else
  @p uves_reduce_spat,DISPLAY_2D mwfxb2d_{p1} -200 4  
endif

!$date
return 0
!================================================================
! END ENTRY REDUCE
!================================================================


!================================================================
! BEGIN ENTRY DISPLAY_2D
!
! Description: Display composite image
!
!================================================================
ENTRY DISPLAY_2D
define/param p1 ? c "enter name of input catalog:"
define/param p2 ? i "enter x image scale:"
define/param p3 ? i "enter y image scale:"

define/local Imax/r/1/1 1.0
define/local Imin/r/1/1 1.0

find/minm {p1} >Null
Imin = 0.001 * outputr(1)
Imax = 0.001 * outputr(2)

clear/disp
load {p1} scale={p2},{p3} cuts={Imin},{Imax}


return 0
!================================================================
! END ENTRY DISPLAY_2D
!================================================================
!
!================================================================
! BEGIN ENTRY DO_CLASS
!
! Description: Classify the pipeline data products
!
!================================================================
ENTRY DO_CLASS
define/param p1 ? c "enter name of input catalog:"
define/param p2 ? c "Enter name of output catalog:"

define/local main_prod/c/1/25 " " all
define/local do_clas/c/1/25 " " all
define/local do_clas_in/c/1/25 " " all



do_clas_in = "{{p1},{UVES_IDSTRNG}}"
do_clas = "EXT_2D_{do_clas_in}"
!main_prod = "{REDU_PX}{do_clas_in}"
classify/product xb2d_{p1} {do_clas} {d_pre} {d_pho},{d_geo},{d_det} OK y y
@p uves_reduce,ADD_CAT {p2} xb2d_{p1}            !product: Extracted 

do_clas = "{FF_2D}{do_clas_in}"
classify/product fxb2d_{p1} {do_clas} {d_red} {d_pho},{d_geo},{d_det} OK y y
@p uves_reduce,ADD_CAT {p2} fxb2d_{p1}		     !product: Flat Fielded


do_clas = "{WCAL_2D}{do_clas_in}"
classify/product wfxb2d_{p1} {do_clas} {d_red} {d_pho},{d_geo},{d_det} OK y y
@p uves_reduce,ADD_CAT {p2} wfxb2d_{p1}		    !product: WCAL


do_clas = "{MER_2D}{do_clas_in}"
classify/product mwfxb2d_{p1} {do_clas} {d_red} {d_pho},{d_geo},{d_det} OK y y
@p uves_reduce,ADD_CAT {p2} mwfxb2d_{p1}		!product: Merged


return {main_prod}
!================================================================
! END ENTRY DO_CLASS
!================================================================
!****************************************************************
!================================================================
! BEGIN ENTRY ADD_CAT
!
! Description: Add a frame to an image catalog
!
!================================================================
define/param p1 ? c "Enter catalog name:"
define/param p2 ? c "Enter filename to add to catalog:"

if p1(1:1) .eq. "+" return -1
add/icat {p1} {p2} >Null      !>Null to suppress output on screen
return 0
!================================================================
! BEGIN ENTRY ADD_CAT
!================================================================
!****************************************************************
!================================================================
! BEGIN ENTRY CALC_VARY
!
! Description: Calculates frame's varaiance
!
!================================================================
ENTRY CALC_VARY
define/param p1 ? c "Enter input frame name:"
define/param p2 ? c "Enter output frame name:"
define/param p3 {ron},{gain} c "Enter detector parameters (RON,CONAD):"

write/key inputr/r/1/2 {p3}
define/local ron/r/1/1 0.0
define/local ron2/r/1/1 0.0

!================================================================
! Computes the variance frame.
! The input frame is assumed to be only background subtracted 
! (and de-biased) ??
! The variance in in units of [ADU^2]
!
! Vij = (RON/CONAD)^2 + |IN_ij|/CONAD
!
! [RON] = e-
! [CONAD] = e-/ADU
! [GAIN] = ADU/e-
!
!================================================================

if inputr(2) .gt. 0.0 then
   ron = inputr(1)/inputr(2)     !RON in ADU
   ron2 = ron * ron              !RON2 in ADU
   compute/image {p2} = ({ron2} + abs({p1}) )/ ({inputr(2)})
else
   w/o "bad CONAD to determine varaiance frame!"
endif

return 0
!================================================================
! BEGIN ENTRY ADD_CAT
!================================================================



!================================================================
! BEGIN ENTRY CALC_V1V2
!================================================================


ENTRY CALC_V1V2
define/param p1 ? c "Enter input 1 frame:"
define/param p2 ? c "Enter input 2 frame:"
define/param p3 ? c "Enter variance 1 frame:"
define/param p4 ? c "Enter variance 2 frame:"
define/param p5 ? c "Enter output variance frame:"
!
! Determine the variance of F = S1/S2 which is:
! dF^2 = F^2*[ (dS1/S1)^2 + (dS2/S2)^2 ] = [V1/S2^2 + V2*S1^2/S2^4]
! V1 = dS1^2, V2 = dS2^2
!
compute/ima {p5} = {p3} / ({p2}*{p2}) >Null
compute/ima {p5} = {p5} + {p4} * {p1} * {p1} / {p2}**4 >Null

return 0

!================================================================
! END ENTRY CALC_V1V2
!================================================================






