! $Id: uves_resp.prg,v 1.4 2001/07/04 15:57:26 amodigli Exp $
!+++ UVES_REDUCE ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_resp.prg 
!
!.KEYWORDS      response, spectroscopy, echelle, science reduction
!
!.USAGE         @p (IDENT) incat refcat
!
!.INPUT         incat   (P1): Reduced standard star (resampled but not merged!)
!				in a catalog or as a single file.
!		outcat	(P2): Output catalogue for the response frame.
!               refcat  (P3): Flux table (3D) or
!			      MIDAS image catalog with a flux table (3D).
!
!.OUTPUT        
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!		Q2: return the classification of the main product
!
!.DEFAULT	P1
!		P2	
!		P3	flxstd.tbl
!
!.PURPOSE       Compute the UVES response by comparing the extracted orders 
!		of the standard star to a table of fluxes. 
!	        This function has to be used after the reduction of the 
!		standard star by REDUCE/UVES with option rebinning 
!		without merging the orders.
!
!.ALGORITHM     The response calculation is based on the command from the
!		ECHELLE context: RESPONSE/ECHELLE. This part may be found
!		in the MIDAS ENTRY DET_RESP.
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Olivier BOITQUIN, Sebastian WOLF, Andrea Modigliani, ESO-DMO
!
!.VERSION       1.0 1999/09/15  Created: based on a copy of RESPONSE/ECHELLE.
!		    1999/11/08	Check product category of input file.
!				Save products in output catalog if specified.
!		    1999/11/26  New subroutine added: entry average.
!                   1999/12/10	return the main product as Q2
!		    1999/12/18	catal keyword was missing again!
!		    2000/01/08  upgrade of comm. 2 and some improvements.
!           2000/11/08  added Reduced Frame as main data product
!
!.COMMENT	Global ECHELLE keywords are used: FILTMED.
!               (default are 10 pixels for the both).
!		Global ECHELLE keywords are set:  FLUXTAB, RESPONSE
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.4 $
! $Log: uves_resp.prg,v $
! Revision 1.4  2001/07/04 15:57:26  amodigli
! Added a preliminary check to see if the
! standard star is present in table list.
! Before extraction. If not exit with error.
! This to save computation time
!
! Revision 1.3  2001/01/04 10:39:32  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

define/param  p1  ? c  "Enter name/catalog of reduced standard star:"
define/param  p2  + c  "Enter name of output catalog:"
define/param  p3  ? c  "Enter name of flux table (3D) or reference catalog:"
define/param  p4  + c  "Check only if STD star is in the provided flux [+]/check:"

define/local pid/c/1/16 resp ? +lower_levels
define/local modext/c/1/25 " " ALL +lower_levels
!
define/local main_prod/c/1/25 " " all
define/local num/I/1/1    0
define/local catal/I/1/1    0
define/local ext/c/1/4 " " ALL
define/local ifrm/c/1/100 " " ALL
define/local refc/c/1/100 " " ALL
define/local wc_id/c/1/25  " " ALL
define/local r_id/c/1/25  " " ALL
define/local procatg/c/1/25  " " ALL
define/local redfrm/c/1/25  " " ALL
define/local p_check/c/1/10 "{p4}"
p_check = m$upper(p_check) 

if m$exist(p2) .eq. 0 .AND. p2(1:1) .ne. "+" -
  create/icat {p2} NULL {UVES_IDSTRNG} >Null

set/echelle RESPONSE=" "

wc_id = WCAL_PX                                 !Get the input frame
if FFOPT(1:1) .eq. "Y" wc_id = WCFF_PX          !identifier = ESO.PRO.CATG
wc_id = "{wc_id}{STD_ID}"                       !entry prefix

						!Check input type of P3
parse/path {p3}					!Get file extension.
ext = m$upper(q3)
if ext(1:4) .eq. ".CAT" then			!Check if we got a catalog.
  search/icat {p3} {STDFLX_ID}
  if m$strlen(out_a) .eq. 0 goto ERR0		!... does not exist
  refc = out_a
else
  refc = p3
endif

						!Check input type of P1
parse/path {p1}					!Get file extension.
ext = m$upper(q3)
if ext(1:4) .eq. ".CAT" then			!Check if we got a catalog.
  show/icat {p1} >Null
  num = outputi(1)				!Check number of entries.
  if num .lt. 1 goto ERR1			!No entries in the catalog.
  goto CATALOG
else if ext(1:4) .eq. ".BDF" then
  ifrm = p1
  goto SINGLE
else
  goto ERR2					!Bad P1 specification
endif



CATALOG:					!Loop through the input catalog
  store/frame ifrm {p1} 1 FINI

SINGLE:
  if m$exist(ifrm) .eq. 0 goto ERR3
  procatg = "{{ifrm},{h_procatg}}"		!Check the type of the input
  if "{p_check}" .ne. "CHECK" then
     if m$index(procatg,wc_id) .le. 0 goto SKIP	!frame (RESAMPLED DATA)
  endif
  r_id = "{REDU_STD_ID({PATHID})}"
						!Check input type of P3
  parse/path {p1}					!Get file extension.
  ext = m$upper(q3)
 w/o "{REDU_STD_ID({PATHID})}"
 
  if ext(1:4) .eq. ".CAT" then			!Check if we got a catalog.
     search/icat {p1} {r_id}
     if m$strlen(out_a) .eq. 0 then
        mess/out 3 {pid} "No reduced frame give. Suppress its classification!"
        redfrm = "missing"
     else
        redfrm = out_a
     endif
  else
     mess/out 3 {pid} "No reduced frame give. Suppress its classification!"
     redfrm = "missing"
  endif
  
  
  mess/out 3 {pid} "Do the response calculation for\n`{ifrm}` ..."

  CONFIG/INSTR {ifrm}
  modext = q2

  mess/out 3 {pid} "Get reference from catalog `{refc}` ..."
  mess/out 3 {pid} "First check if STD star is present in reference flux table ..."
 
  ALIGN/STD {ifrm} {refc}			!Extracts the reference spect.
                            		!from reference catalog, its
                                    !name is returned in OUT_A
  !We first check if the STD star is present
  !If not previous line issue an error else 
  !If we are just in check mode we go out silently
  if {q1} .ge. 0 then
      if "{p_check}" .eq. "CHECK" then
         mess/out 3 {pid} "Only check mode: found STD star {out_a}. We can go on."
         mess/out 3 {pid} "Response determination after extraction."
         return 0
      endif  
  else
     if "{p_check}" .eq. "CHECK" then
         mess/out 3 {pid} "STD star {out_a} Not found."
         goto ERR4
      endif  
      goto ERR4	
  endif     
  set/echelle FLUXTAB="{out_a}.tbl"
  @p uves_resp,DET_RESP {ifrm} {p2} {redfrm}
  main_prod = q2


SKIP:
  if ext(1:4) .eq. ".BDF" goto FINI		!single frame -> finished.
  goto CATALOG
FINI:
if RESPONSE(1:1) .eq. " " goto ERR5

return 0 {main_prod}

						!Error handling
ERR0:
  mess/out 1 {pid} "Reference catalog of std. stars missing!"
  return -1
ERR1:
  mess/out 1 {pid} "Empty input catalog !"
  return -1
ERR2:
  mess/out 1 {pid} "You have to specify full name of input frame (.cat, .bdf)!"
  return -1
ERR3:
  mess/out 1 {pid} "Input file `{ifrm}` does not exist !"
  return -1
ERR4:
  mess/out 1 {pid} "No coincidence for `{ifrm}` found!\n--> Skip!"
  goto SKIP
ERR5:
  mess/out 1 {pid} "No {wc_id} frame found!\n--> No response determined!"
  mess/out 3 {pid} "FFOPT keyword controls the type of selection."
  return -1




!=== SUBROUTINES ==============================================================

ENTRY DET_RESP
define/param p1 ? c "Enter name of reduced input frame:"
define/param p2 ? c "Enter name of output catalog:"
define/param p3 ? c "Enter name of extracted-ff-wcal-merged-sky-sub input frame:"

define/local i1/i/1/1 0
define/local nrow/i/1/1 0
define/local ntot/i/1/1 0
define/local pixpos/i/1/1 0
define/local mpixnul/i/1/1 -10

define/local main_prod/c/1/25 " " all
define/local do_clas/c/1/25 " " all
define/local do_clas_in/c/1/25 " " all
 
 
 
filter/median {P1} &p1 {FILTMED(1)},{FILTMED(2)},{FILTMED(3)} {SESSOUTV}
filter/smooth &p1 &n {FILTSMO(1)},{FILTSMO(2)},{FILTSMO(3)} {SESSOUTV}

mess/out 3 {pid} "Compute flux calibration using reference:\n{FLUXTAB}"

CONVERT/TABLE   &o = {FLUXTAB} :{c_fxwave} :{c_fxflux} &n SPECTR
verify/echelle  &o AGAINST &n
compute/image   &p = &o/&n {SESSOUTV}

! Masking the data to eliminate boundary effect caused bye the smoothing
REPLACE/IMAGE &p1 &fb <,0.=0.,1. {SESSOUTV}
shift/image &fb &fd {PIXNUL(1)},0
mpixnul = - PIXNUL(1)
shift/image &fb &fl {mpixnul},0
compute/image   &ff = &fl*&fd
!REPLACE/IMAGE &p &z &ff/<,0.=0. {SESSOUTV}
compute/image  &z = &p*&ff 

write/image     &z  [<,<:@{PIXNUL(1)},>] 0.  ALL  {SESSOUTV}

nrow = m$value(middummz.bdf,NPIX(2))
do i1 = 1  {nrow}
   ntot = m$value({P1},NPTOT({i1}))
   pixpos = ntot - PIXNUL(2)
   write/image &z [@{pixpos},@{i1}:>,@{i1}]  0.  ALL  {SESSOUTV}   
enddo

set/echelle RESPONSE="resp_{INSTRUM}_{CCD}_{CROSS}{MODEXT}.bdf"
@p uves_split,REPL_CH # {RESPONSE} _
RESPONSE = outputc

!am: 3/11/00 Added product
do_clas_in = "{MODEXT}"
main_prod = "{RESP_ID}{do_clas_in}"


 

-rename middummz.bdf  {RESPONSE}
copy/dd {P1} *,3 {RESPONSE}
!am: 09/11/00 Added merged response


if NICE_CREA(1:1) .eq. "Y" then
  assign/disp 1
  !load/image {RESPONSE} scale=f cent=c,c
  load/image {RESPONSE} scale=-12,10 cent=c,c cut=f
endif

merge/eche {RESPONSE} m_{RESPONSE} ? AVERAGE
classify/product m_{RESPONSE} {main_prod} {d_pre} {d_geo},{d_det} OK y y
if p2(1:1) .ne. "+" add/icat {p2} m_{RESPONSE} {SESSOUTV}


do_clas = "{WCFF_STD}{do_clas_in}"
classify/product {RESPONSE} {do_clas} {d_pre} {d_geo},{d_det} OK y y

if p2(1:1) .ne. "+" add/icat {p2} {response} {SESSOUTV}
if p3 .ne. "+" then
   if p3 .ne. "missing" then
      do_clas = "{PROD_STD}{do_clas_in}"
      classify/product {p3} {do_clas} {d_red} {d_pho},{d_geo},{d_det} OK y y
      add/icat {p2} {p3}		!product: merged
   else
      mess/out 3 {pid} "Reduced Frame Not given. Not classifiable!"
   endif
endif

append/out "Response of {INSTRUM}/{CCD}/{CROSS}/{MODEXT} determined.\n"
append/out "Output: {RESPONSE}\n"
mess/out 3 {pid}
return 0 {main_prod}
