! $Id: uves_sel_cdb.prg,v 1.3 2001/01/04 10:39:30 amodigli Exp $
!**********************************************************************************
!
! Procedure:      uvesprepcdb.prg
! Author:         P.Ballester, ESO/DMD
! Date:           17 Dec., 1999
!
!**********************************************************************************
!
! Purpose:        Prepares configuration selection files ready 
!                 to use with the uves_popul.sh script.
!
!
! Syntax:         @@ uvesprepcdb ost_table  
!
! Precondition:   An ost_table must have been previously prepared with
!                 the data organizer, e.g. by starting the DO with sockets
!                 and delivering frames to it with deliFrame.
!
!**********************************************************************************
!
! Description: 
!                 The procedure for generation of new calibration database
!                 files is the following.
!
!                 - Log as ~pipeline, start the DO in socket mode, and deliver
!                   frames to it. 
!
!                 - Prepare an ost_table containing all files to be analysed.
!                   The ost_table may span several nights.
!
!                 - Log as ~uvesp, and copy the DO ost table found under
!                   /data/reduced to a local directory.
! 
!                 - Run this procedure (uvesprepcdb.prg) to generate 
!                   the .cfs files and a listing of all calibration
!                   groups (@@ uvesprepcdb ost_uves.tbl >list.txt )
!
!                 - Analyse the listing (list.txt) and verify the groups
!                   possibly removing unnecessary or bad quality calibration
!                   frames.
!
!                 - Edit the .cfs files to remove the corresponding 
!                   sequence numbers. If a file requires no editing, , 
!                   use awk '{print $2}' or any similar command
!                   to drop the sequence number. 
!
!                 - All configuration selection files are now ready. The mbsXXX.cfs
!                   list the files entering into the preparation of average
!                   biases (2 blue, 2 red). The other files WWWdXbYnxn.cfs 
!                   describe each of the 22 UVES standard configurations. There 
!                   should be in total 26 .cfs files.
!
!                 - Copy the ThAr table from  the calibration database into the
!                   local directory (thargood_2.tfits)
!
!                 - Prepare a configuration with the command
!                   uves_popul.sh `cat WWWdXbYnxn.cfs  mbsXXX.cfs` thargood_2.tfits
!                   (Beware the **inverted quotes**) 
!
!                 - uves_popul.sh automatically start a Midas session and does
!                   all the processing to generate the files for a configuration
!                   (may last about 30 min. for each configuration, binned blue
!                   are faster, unbinned red take the longest time).
!
!                 - The result files are found under ~/midwork/tmpwrk directory, and
!                   are called *.tfits for all calibration tables, m*.fits for
!                   master biases and flat-fields, cat*.cat for the catalogs. 
!
!                 - Copy the created calibration files to the UVES calibDB check
!                   directory: 
!                    (UVESCAL=~quality/calibDB/check/ins/ut2/uves/ech/cal)
!                   Remove the tables or master biases that may be already in the
!                   database and do not need to be refreshed (thargood_2.tfits, 
!                   mbsXXX.fits).
!
!                 - Run the command "updateCalibDb *.*" to ingest all new files.
!                   If the file has been previously ingested, it will simply be 
!                   overwritten. If the file is new, confirmation will be asked 
!                   before ingesting it. The updateCalibDb command must be runned
!                   with the ~quality account, while uvesprepcdb and uves_popul
!                   are usually runned with the ~uvesp account. 
!
!                 - Prepare new files with uves_popul.sh. It is possible to 
!                   skip the master bias correction by refering to the files already 
!                   in the calibDb: 
!                    uves_popul.sh `cat WWWdXbYnxn.cfs` $UVESCAL/mbsXXX.fits 
!                      $UVESCAL/thargood_2.tfits
!
!**********************************************************************************
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_sel_cdb.prg,v $
! Revision 1.3  2001/01/04 10:39:30  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
! 
define/param P1 "ost_uves_comm2.tbl" CHAR 
!
! Could be improved with INS.OPT.PATH, INS.MODE keywords
!
name/column {P1}  :FILENAME A60
name/column {P1}  :DATE     T20.6
name/column {P1}  :CONAD1   F6.3

define/local thisproc/C/1/30  "uves_sel_cdb"

write/out "<mbsbe1x1.fits> Master bias, Blue Arm, 1x1 "
SELECT/TABL {P1}  :DPR_TYPE.EQ."*BIAS*".AND.:BINX.EQ.1.AND.:BINY.EQ.1 
SELECT/TABL {P1}  SELECT.AND.:NCHIP.EQ.1.AND.:GRAT1_WLEN.EQ.NULL 
@@ {thisproc},display {P1} mbsbe1x1 :CONAD1

write/out "Selection <mbsbe2x2.fits> Master bias, Blue Arm, 2x2 "
SELECT/TABL {P1}  :DPR_TYPE.EQ."*BIAS*".AND.:BINX.EQ.2.AND.:BINY.EQ.2 
SELECT/TABL {P1}  SELECT.AND.:NCHIP.EQ.1.AND.:GRAT1_WLEN.EQ.NULL 
@@ {thisproc},display {P1} mbsbe2x2 :CONAD1

write/out "Selection <mbsre1x1.fits, mbsrm1x1.fits> Master bias, Red Arm, 1x1 "
SELECT/TABL {P1}  :DPR_TYPE.EQ."*BIAS*".AND.:BINX.EQ.1.AND.:BINY.EQ.1 
SELECT/TABL {P1}  SELECT.AND.:NCHIP.EQ.2.AND.:GRAT2_WLEN.EQ.NULL 
@@ {thisproc},display {P1} mbsr-1x1 :CONAD1

write/out "Selection <mbsre2x2.fits, mbsrm2x2.fits> Master bias, Red Arm, 2x2 "
SELECT/TABL {P1}  :DPR_TYPE.EQ."*BIAS*".AND.:BINX.EQ.2.AND.:BINY.EQ.2 
SELECT/TABL {P1}  SELECT.AND.:NCHIP.EQ.2.AND.:GRAT2_WLEN.EQ.NULL 
@@ {thisproc},display {P1} mbsr-2x2 :CONAD1

! 1x2 and 2x3

write/out "<mbsbe1x2.fits> Master bias, Blue Arm, 1x2 "
SELECT/TABL {P1}  :DPR_TYPE.EQ."*BIAS*".AND.:BINX.EQ.1.AND.:BINY.EQ.2 
SELECT/TABL {P1}  SELECT.AND.:NCHIP.EQ.1.AND.:GRAT1_WLEN.EQ.NULL 
@@ {thisproc},display {P1} mbsbe1x2 :CONAD1

write/out "Selection <mbsbe2x3.fits> Master bias, Blue Arm, 2x3 "
SELECT/TABL {P1}  :DPR_TYPE.EQ."*BIAS*".AND.:BINX.EQ.2.AND.:BINY.EQ.3 
SELECT/TABL {P1}  SELECT.AND.:NCHIP.EQ.1.AND.:GRAT1_WLEN.EQ.NULL 
@@ {thisproc},display {P1} mbsbe2x3 :CONAD1

write/out "Selection <mbsre1x2.fits, mbsrm1x2.fits> Master bias, Red Arm, 1x2 "
SELECT/TABL {P1}  :DPR_TYPE.EQ."*BIAS*".AND.:BINX.EQ.1.AND.:BINY.EQ.2
SELECT/TABL {P1}  SELECT.AND.:NCHIP.EQ.2.AND.:GRAT2_WLEN.EQ.NULL
@@ {thisproc},display {P1} mbsr-1x2 :CONAD1

write/out "Selection <mbsre2x3.fits, mbsrm2x3.fits> Master bias, Red Arm, 2x3 "
SELECT/TABL {P1}  :DPR_TYPE.EQ."*BIAS*".AND.:BINX.EQ.2.AND.:BINY.EQ.3
SELECT/TABL {P1}  SELECT.AND.:NCHIP.EQ.2.AND.:GRAT2_WLEN.EQ.NULL
@@ {thisproc},display {P1} mbsr-2x3 :CONAD1

!**********************************************************************************
!**********************************************************************************
!  End of biases
!**********************************************************************************
!**********************************************************************************

!**********************************************************************************
!**********************************************************************************
!  Start configurations in 1x1, 2x2
!**********************************************************************************
!**********************************************************************************


!******************** Blue Arm, 346 nm, No Dichroic *******************************

write/out "Selection <___346d0be1x1___.fits> Blue Arm, No Dichroic, 346 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-346.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_blue_*"
@@ {thisproc},display {P1} 346d0be1x1

write/out "Selection <___346d0be2x2___.fits> Blue Arm, No Dichroic, 346 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-346.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_blue_*"
@@ {thisproc},display {P1} 346d0be2x2

!******************** Blue Arm, 346 nm, Dichroic *******************************

write/out "Selection <___346d1be1x1___.fits> Blue Arm, Dichroic #1, 346 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-346.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 346d1be1x1

write/out "Selection <___346d1be2x2___.fits> Blue Arm, Dichroic #1, 346 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-346.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 346d1be2x2

!******************** Blue Arm, 390 nm, Dichroic #1 ****************************************

write/out "Selection <___390d1be1x1___.fits> Blue Arm, Dichroic #1, 390 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-390.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 390d1be1x1

write/out "Selection <___390d1be2x2___.fits> Blue Arm, Dichroic #1, 390 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-390.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 390d1be2x2


!******************** Blue Arm, 437 nm, No Dichroic *****************************
! 
write/out "Selection <___437d0be1x1___.fits> Blue Arm, No Dichroic, 437 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-437.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_blue_*"
@@ {thisproc},display {P1} 437d0be1x1

write/out "Selection <___437d0be2x2___.fits> Blue Arm, No Dichroic, 437 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-437.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_blue_*"
@@ {thisproc},display {P1} 437d0be2x2

!******************** Blue Arm, 437 nm, No Dichroic *****************************
! 

write/out "Selection <___437d2be1x1___.fits> Blue Arm, Dichroic #2, 437 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-437.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic2_*"
@@ {thisproc},display {P1} 437d2be1x1

write/out "Selection <___437d2be2x2___.fits> Blue Arm, Dichroic #2, 437 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-437.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic2_*"
@@ {thisproc},display {P1} 437d2be2x2

!******************** Red Arm, 520 nm, No Dichroic ****************************************

write/out "Selection <___520d0r_1x1___.fits> Red Arm, Dichroic #1, 520 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-520.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 520d0r-1x1

write/out "Selection <___520d0r_2x2___.fits> Red Arm, Dichroic #1, 520 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-520.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 520d0r-2x2

!******************** Red Arm, 564 nm, Dichroic #1 ****************************************
!
write/out "Selection <___564d1r_1x1___.fits> Red Arm, Dichroic #1, 564 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-564.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 564d1r-1x1

write/out "Selection <___564d1r_2x2___.fits> Red Arm, Dichroic #1, 564 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-564.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 564d1r-2x2

!******************** Red Arm, 580 nm, No Dichroic ****************************************

write/out "Selection <___580d0r_1x1___.fits> Red Arm, No Dichroic, 580 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-580.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 580d0r-1x1

write/out "Selection <___580d0r_2x2___.fits> Red Arm, No Dichroic, 580 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-580.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 580d0r-2x2


!******************** Red Arm, 580 nm, Dichroic #1 ****************************************

write/out "Selection <___580d1r_1x1___.fits> Red Arm, Dichroic #1, 580 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-580.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 580d1r-1x1

write/out "Selection <___580d1r_2x2___.fits> Red Arm, Dichroic #1, 580 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-580.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 580d1r-2x2

!******************** Red Arm, 860 nm, No Dichroic ****************************************

write/out "Selection <___860d0r_1x1___.fits> Red Arm, No Dichroic, 860 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-860.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 860d0r-1x1

write/out "Selection <___860d0r_2x2___.fits> Red Arm, No Dichroic, 860 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-860.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 860d0r-2x2


!******************** Red Arm, 860 nm, Dichroic #2 ****************************************

write/out "Selection <___860d2r_1x1___.fits> Red Arm, Dichroic #1, 860 nm, 1x1"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.1.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-860.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic2_*"
@@ {thisproc},display {P1} 860d2r-1x1

write/out "Selection <___860d2r_2x2___.fits> Red Arm, Dichroic #1, 860 nm, 2x2"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-860.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic2_*"
@@ {thisproc},display {P1} 860d2r-2x2

!*******************************************************************************************
!*******************************************************************************************
! End configurations 1x1, 2x2
!*******************************************************************************************
!*******************************************************************************************

!*******************************************************************************************
!*******************************************************************************************
! Start configurations 1x2, 2x3
!*******************************************************************************************
!*******************************************************************************************

!******************** Blue Arm, 346 nm, No Dichroic *******************************

write/out "Selection <___346d0be1x2___.fits> Blue Arm, No Dichroic, 346 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-346.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_blue_*"
@@ {thisproc},display {P1} 346d0be1x2

write/out "Selection <___346d0be2x3___.fits> Blue Arm, No Dichroic, 346 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-346.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_blue_*"
@@ {thisproc},display {P1} 346d0be2x3

!******************** Blue Arm, 346 nm, Dichroic *******************************

write/out "Selection <___346d1be1x2___.fits> Blue Arm, Dichroic #1, 346 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-346.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 346d1be1x2

write/out "Selection <___346d1be2x3___.fits> Blue Arm, Dichroic #1, 346 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-346.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 346d1be2x3

!******************** Blue Arm, 390 nm, Dichroic #1 ****************************************

write/out "Selection <___390d1be1x2___.fits> Blue Arm, Dichroic #1, 390 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-390.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 390d1be1x2

write/out "Selection <___390d1be2x3___.fits> Blue Arm, Dichroic #1, 390 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-390.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 390d1be2x3


!******************** Blue Arm, 437 nm, No Dichroic *****************************
! 
write/out "Selection <___437d0be1x2___.fits> Blue Arm, No Dichroic, 437 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-437.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_blue_*"
@@ {thisproc},display {P1} 437d0be1x2

write/out "Selection <___437d0be2x3___.fits> Blue Arm, No Dichroic, 437 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-437.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_blue_*"
@@ {thisproc},display {P1} 437d0be2x3

!******************** Blue Arm, 437 nm, No Dichroic *****************************
! 

write/out "Selection <___437d2be1x2___.fits> Blue Arm, Dichroic #2, 437 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-437.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic2_*"
@@ {thisproc},display {P1} 437d2be1x2

write/out "Selection <___437d2be2x3___.fits> Blue Arm, Dichroic #2, 437 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.1.AND.ABS(:GRAT1_WLEN-437.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic2_*"
@@ {thisproc},display {P1} 437d2be2x3

!******************** Red Arm, 520 nm, No Dichroic ****************************************

write/out "Selection <___520d0r_1x2___.fits> Red Arm, Dichroic #1, 520 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-520.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 520d0r-1x2

write/out "Selection <___520d0r_2x3___.fits> Red Arm, Dichroic #1, 520 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-520.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 520d0r-2x3

!******************** Red Arm, 564 nm, Dichroic #1 ****************************************
!
write/out "Selection <___564d1r_1x2___.fits> Red Arm, Dichroic #1, 564 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-564.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 564d1r-1x2

write/out "Selection <___564d1r_2x3___.fits> Red Arm, Dichroic #1, 564 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-564.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 564d1r-2x3

!******************** Red Arm, 580 nm, No Dichroic ****************************************

write/out "Selection <___580d0r_1x2___.fits> Red Arm, No Dichroic, 580 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-580.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 580d0r-1x2

write/out "Selection <___580d0r_2x3___.fits> Red Arm, No Dichroic, 580 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-580.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 580d0r-2x3


!******************** Red Arm, 580 nm, Dichroic #1 ****************************************

write/out "Selection <___580d1r_1x2___.fits> Red Arm, Dichroic #1, 580 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-580.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 580d1r-1x2

write/out "Selection <___580d1r_2x3___.fits> Red Arm, Dichroic #1, 580 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-580.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic1_*"
@@ {thisproc},display {P1} 580d1r-2x3

!******************** Red Arm, 860 nm, No Dichroic ****************************************

write/out "Selection <___860d0r_1x2___.fits> Red Arm, No Dichroic, 860 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-860.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 860d0r-1x2

write/out "Selection <___860d0r_2x3___.fits> Red Arm, No Dichroic, 860 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-860.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_red_*"
@@ {thisproc},display {P1} 860d0r-2x3


!******************** Red Arm, 860 nm, Dichroic #2 ****************************************

write/out "Selection <___860d2r_1x2___.fits> Red Arm, Dichroic #1, 860 nm, 1x2"
SELECT/TABL {P1}  :BINX.EQ.1.AND.:BINY.EQ.2.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-860.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic2_*"
@@ {thisproc},display {P1} 860d2r-1x2

write/out "Selection <___860d2r_2x3___.fits> Red Arm, Dichroic #1, 860 nm, 2x3"
SELECT/TABL {P1}  :BINX.EQ.2.AND.:BINY.EQ.3.AND.:NCHIP.EQ.2.AND.ABS(:GRAT2_WLEN-860.0).LT.0.1
SELECT/TABL {P1}  SELECT.AND.:DPR_TYPE.NE."*OBJECT*".AND.:DPR_TYPE.NE."*BIAS*".AND.:DPR_TYPE.NE."*STD*"
SELECT/TABL {P1}  SELECT.AND.:TP_ID.EQ."UVES_dic2_*"
@@ {thisproc},display {P1} 860d2r-2x3

!*******************************************************************************************
!*******************************************************************************************
! End configurations 1x2, 2x3
!*******************************************************************************************
!*******************************************************************************************

ENTRY DISPLAY
DEFINE/PARAMETER P1        ?      CHAR  "Input table:"
DEFINE/PARAMETER P2 default-conf  CHAR  "Conf. name"
!DEFINE/PARAMETER P3        " "    CHAR  "Additional columns"

if outputi(1) .eq. 0 then
   write/out "No configuration file was produced." 
   return
endif

copy/table {P1}  &s
name/colum &s :OBS_NAME  A30
name/colum &s :DO_CATG   A20

assign/print FILE filnam.txt
$$rm -f filnam.txt snam.txt snam.fmt uvcdbhhh.cfg
print/table  &s :FILENAME N
$$ awk -F"." '{print $2"."$3}' >snam.txt <filnam.txt
$$ echo 'DEFINE/FIELD 1 80 C*80 A25 :ARCFILE' >snam.fmt
create/table &t 1 * snam.txt snam.fmt
copy/tt &t :ARCFILE &s :ARCFILE

read/table &s  :ARCFILE :DPR_TYPE :DO_CATG :OBS_NAME  {P3}

assign/print FILE uvcdbhhh.cfg
print/table  &s :FILENAME N
$ awk '{print $1 " "  $2}' <uvcdbhhh.cfg >{p2}.cfs
$$rm -f filnam.txt snam.txt snam.fmt uvcdbhhh.cfg
write/out  "Written configuration selection file {p2}.cfs"



