! $Id: uves_wavecal.prg,v 1.11 2001/01/04 10:39:56 amodigli Exp $
!+++ UVES_WAVECAL +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_wavecal.prg
!
!.KEYWORDS      spectroscopy, echelle, order, wavelength calibration
!
!.USAGE         @p (IDENT) input catalog refer [mode] [FITS] [resol] [cent]
!
!.INPUT         input	(P1): image catalog with arc lamp exposures (.cat) or
!				a single arc lamp exposure (.bdf)
!               refer   (P3): reference catalog, should contain:
!				order-, drs-setup-table,
!				line reference table, and optionally
!				line table (previous solution = first guess)
!               mode    (P4): auto or interactive:
!				. auto: selects the WLCMTD=GUESS and looks for
!				  previously created line table (LINTAB_ID)
!				. interactive: selects the WLCMTD=PAIR; needs
!				  no previous solution.
!		resol	(P5): set to YES in order to get spectral resolution
!				plots -- 6 panel plot produced by PLOT/RESOL.
!		cent	(P6): set to YES in order to performe the wavelength
!				calibration only at the centre of the orders.
!				(purpose: object calibration, resolution plots)
!		FITS	(P7): set to YES in order to get output files in FITS
!				format at the end. This option makes only sense
!				outside the pipeline process.
!				
!
!.OUTPUT        catalog	(P2): output catalog for products
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!		Q2:  classification of the main product
!
!.DEFAULT	P2:	+		(no output catalog)
!		P3:	+		(no reference catalog)
!		P4:	Auto
!		P5:	NO
!		P6:	NO
!		P7:	NO
!
!		WLCOPT=2D DC=5 WLCNITER=5,30
!		
!
!.PURPOSE       Determine the dispersion relation solution from arc lamp
!		exposures.
!
!.ALGORITHM     Extract the echelle orders by means of the order table.
!		The transformation of the UVES arc lamp input files which
!		includes an optional splitting (red arm), rotation and
!		flipping of the frame should have already been done.
!		The line identification will be done with a first guess
!		solution -- a previously created line table either
!		from a physical model or through an interactive run.
!		In case there is no first guess solution you have to use
!		the interactive mode (P4). This solution (the line table)
!		may be stored as a first guess in the calibration database.
!		Set P5 to YES in order to produce FITS-format tables which
!		may be ingested into the calibration database.
!		Set P6 to YES in order to get spectral resolution plot.
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/09  Created.
!		    1999/05/20	Parameter P5 added: FITS-table output switch.
!		    1999/06/01	Parameter P6 added: spectral resolution switch.
!		    1999/07/05	Error handling of P1 improved.
!           1999/07/13	Crossref command implemented.
!           1999/07/15	DRS table controlls ECHELLE global key settings
!                       OR the user before calling this procedure.
!           1999/07/19	Switch to perform calibration only on central
!				        trace of the orders implemented.
!           1999/09/16	Parameters for classify/product 'properly' set.
!           1999/09/22	FORCE_DRS implemented.
!           1999/09/26	plot/resid uses set/gra! use plot/table.
!           1999/09/29	Use uves_plt_resid.prg for residual plots!
!           1999/10/12	Name of output tables now includes cent. wavel.
!				        and binning.
!           1999/11/03	Use SLIT keyword to get offsets to skywindows
!           1999/11/15	Save offset of line table in descriptor OFFSET.
!           1999/12/07	Parameter sequence changed: P5,P6,P7->P7,P5,P6
!           1999/12/10	return the main product as Q2
!
!.COMMENT	(1) WLC, WLCMTD, GUESS, THRES2 and OFFSET are set by this 
!		procedure, may not be changed by the user or DRS setup table.
!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.11 $
! $Log: uves_wavecal.prg,v $
! Revision 1.11  2001/01/04 10:39:56  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------

crossref input catalog refer mode FITS resol cent
!
define/param p1 ? c "Enter name of input catalog:"
define/param p2 + c "Enter name of output catalog:"
define/param p3 + c "Enter name of reference catalog:"
define/param p4 AUTO c "Enter wavelength calibration mode (auto or interactive):"
define/param p5 Y c "Enter YES to get spectral resolution, otherwise NO:"
define/param p6 N c "Enter YES to get wavelength calibration only at the centre, otherwise NO:"
define/param p7 N c "Enter YES to produce FITS output tables, otherwise NO:"

define/local pid/c/1/10 "wavecal" ? +lower_levels
!
define/local Modext/c/1/25 " " ALL +lower_levels
define/local Cwlen/c/1/10 " " ALL +lower_levels
define/local Bin/c/1/4 " " ALL +lower_levels
!
define/local ext/c/1/4 " " ALL
define/local do_clas/c/1/25 " " ALL
define/local winsta/i/1/1 1
define/local winend/i/1/1 3
define/local i1/i/1/1 0
define/local ok/i/1/1 0
define/local num/i/1/1 0
define/local catal/i/1/1 0
define/local auxlin/c/1/30 " " ALL

if NICE_CREA .eq. "Y" assign/disp d,1 >Null		!select display
set/format I1
if MESS_LEVEL .eq. 4 write/key sessoutv " " ALL

ext = m$upper(p6)
if ext(1:1) .eq. "Y" then				!Perform wavelength
  winsta = 2						!calibration only on
  winend = 2						!central trace of each
endif							!order.

							!Check if output ...
if m$exist(p2) .eq. 0 .AND. p2(1:1) .ne. "+" -
  create/icat {p2} NULL {UVES_IDSTRNG} >Null		!... catalog exists!

							!Check input type.
parse/path {p1}						!get file extension
ext = m$upper(q3)
if ext(1:4) .eq. ".CAT" then				!Check if we got a cat.
  show/icat {p1} >Null
  num = outputi(1)					!Check num. of entries.
  if num .lt. 1 goto ERR1				!No entries in the cat.
  goto CATALOG
else if ext(1:4) .eq. ".BDF" then
  WLC = p1
  goto SINGLE
else
  goto ERR2						!bad P1 specification
endif


CATALOG:						!loop through input frm
  q1 = "0"
  store/frame WLC {p1} 1 FINI

SINGLE:
  write/out " "
  mess/out 3 {pid} "Do the wavelength calibration using `{WLC}`..."

  @p uves_wavecal,CONFIG {p3} {p4}			!Configure the wavelen.
  if {q1} .lt. 0 then					!calibration:
    if ext(1:4) .ne. ".BDF" then			!. check existance of
      goto CATALOG					!  auxiliary tables
    else						!. set THRES2.
      goto FINI
    endif
  endif
  ok = 1


  do i1 = winsta winend					!loop: order windows
    @p uves_wavecal,WCAL {i1}
							!Do the wavel.calibr.
							!call: EXTRACT/ECHELLE
							!SEARCH/ECHELLE,
							!IDENTIFY/ECHELLE.
     							
    copy/dd {WLC} *,3 {LINTAB}				!Copy input header
    do_clas = "{LINTAB_ID({PATHID})}{i1}"		!DO-classification
    classi/product {LINTAB} {do_clas} {d_pre} {d_geo} OK
    if p2(1:1) .ne. "+" add/icat {p2} {LINTAB} >Null	!Add line-table to cat.
    
    @p uves_wavecal,RESOL_PLOT {WLC} {i1} {p5}  ! before saving line tables
    !if i1 .eq. 2 return/exit
    
    if p7(1:1) .eq. "Y" then
      mess/out 3 {pid} "Save line table {LINTAB} as FITS file ..."
      
      save/drs {WLC} {LINTAB} FITS {do_clas}
    endif

    ! at the next line sometimes displays a warning for order extraction of 2.window  
    ! cannot access ....no such file or directory  
    
    if i1 .eq. 2 save/echelle {Modext(2:)}{Cwlen} >Null	!save session for each
							!input frame once

    mess/out 3 {pid} "Wavelength calibration done, output: {LINTAB}.\n"
    ! old position of RESOL_PLOT entry
    if i1 .eq. 2 auxlin = "{LINTAB}"        !To store name of Obj Line table 
                                            !used below to create a new DRSTAB
  enddo

  set/eche OFFSET=0 SAMPLE=+++
  DRSTAB = "d{Cwlen}{Bin}.tbl"				!Name of setup table
  
  
  
  !Initially was used as reference ORDTAB: but if one does NOT use uves_popul with
  !A coherent (taken at the same time) set of data to generate new DRSTAB the MJDOBS
  !And other ESO HIERARCH keys reflect the ORDTAB values (es. old table from calibDB)
  !And not the correspondent values from the new raw file.
  !save/drs {DRSTAB} {ORDTAB}				!Save global keys (old way: until 2/10/00)
  
  !am 06/10/00: to fix bug on MJD_OBS key
  save/drs {DRSTAB} {auxlin}				!Save global keys (new way: we refer to auxlin
                                            !so to LINE2 to have FITS keys as in raw input
                                            !data in output DRS SETUP table. This was true 
                                            !before only if uves_popul was used to generate 
                                            !DRS SETUP
                                            
  copy/dd {WLC} *,3 {DRSTAB}                !To be sure that the raw keys are in the DRSTAB
  copy/dd {ORDTAB} *HISTORY* {DRSTAB}       !To add the HISTORY from ordtab (??)
 
  
  classi/product {DRSTAB} {DRSRC_ID({PATHID})} {d_pre} {d_empty} OK
  if p2(1:1) .ne. "+" add/icat {p2} {DRSTAB} >Null	!Add setup to catalog

  if p7(1:1) .eq. "Y" then
    mess/out 3 {pid} "Save line and drs-setup table as FITS file ..."
    save/drs {WLC} {DRSTAB} FITS {DRSRC_ID({PATHID})}
  endif
  if ext(1:4) .eq. ".BDF" goto FINI			!single f. -> finished.
  goto CATALOG
FINI:
if {q1} .lt. 0 .AND. ok .eq. 0 goto ERR3
return 0 {LINTAB_ID({PATHID})}2



!Error handling:

ERR1:
  mess/out 1 {pid} "Empty input catalog !"
  return -1
ERR2:
  mess/out 1 {pid} "You have to specify full name of input frame (.cat, .bdf)!"
  return -1
ERR3:
  mess/out 1 {pid} "No successful calibration done!"
  return -1

!=== SUBROUTINES =============================================================

ENTRY WCAL
define/param p1 ? n "Enter slit window indx (0: total;1:sky1, 2:obj., 3:sky2):"

define/local i1/i/1/1 {p1}
define/local owd/i/1/1 0
define/local s_off/i/1/4 0,-{SLIT},0,{SLIT}
!define/local s_off/i/1/4 0,{SKYOFF(1)},0,{SKYOFF(2)}
define/local ofrm/c/1/100 x_{WLC}
define/local mrms/c/1/20 " " ALL

i1 = {p1} + 1
if i1 .gt. 1 then
  set/echelle OFFSET={s_off({i1})} SLIT={SLIT}
  set/echelle LINTAB=l{Cwlen}{Bin}_{p1}.tbl		!name of line table
  ofrm = "x{p1}_{WLC}"					!name of order extract.
  mess/out 3 {pid} "Do the order extraction of {p1}. slit window ..."
else
  mess/out 3 {pid} "Do the order extraction ..."
  set/echelle offset=0 
  set/echelle LINTAB=l{Cwlen}{Bin}_{p1}.tbl		!name of line table
  ofrm = "x{p1}_{WLC}"					!name of order extract.
endif

extract/echelle {WLC} {ofrm}				!Do the order extract.

stat/ima {ofrm} ? ? ? FN
outputr(8) = 0.1 * outputr(8)
set/echelle THRES2={outputr(8)}				!Take the mean as
							!the threshold
mess/out 3 {pid} "Search for emission lines ..."
sloop:
search/echelle {ofrm} {SESSOUTV}
show/table {LINTAB} >Null
if outputi(2) .lt. 1000 then
  outputr(8) = {THRES2}/2.0
  if outputr(8) .gt. 1.0 then
    set/echelle THRES2={outputr(8)}
    mess/out 4 {pid} -
      "... {outputi(2)} lines found.\nTry to improve with THRES2={THRES2}"
    goto sloop
  endif
else if outputi(2) .gt. 1400 then
  outputr(8) = {THRES2}*1.5
  if outputr(8) .gt. 1.0 then
    set/echelle THRES2={outputr(8)}
    mess/out 4 {pid} -
      "... {outputi(2)} lines found.\nTry to improve with THRES2={THRES2}"
    goto sloop
  endif
endif

mess/out 3 {pid} "... {outputi(2)} lines found."

mess/out 3 {pid} "Identify emission lines ..."
define/local otol/r/1/1 {TOL}
define/local odc/i/1/1 {DC}
define/local owlcopt/c/1/15 {WLCOPT}

set/echelle TOL=0.6 DC=5 WLCOPT=1D
ident/echelle {SESSOUTV}

-copy {LINTAB} wcal2LINE.tbl
! TOL={otol} 
set/echelle DC={odc} WLCOPT={owlcopt} GUESS=wcal2
!show/echelle w
ident/echelle {SESSOUTV}
write/desc {LINTAB} OFFSET/r/1/1 {OFFSET}		!Save for REDUCE/UVES!
offset = 0						!Reset OFFSET keyword
if m$existd(LINTAB,"AVRMS") .eq. 1 then
  set/format F9.5
  mrms = "{{LINTAB},AVRMS(1)}"		!get mean RMS of wavel.cal.
  mess/out 3 {pid} -
    "Mean RMS of the {WLCOPT} fit: {mrms}\n Pixelsize: {SAMPLE}"
endif

set/format
!..............................................................................
ENTRY CONFIG
define/param p1 ? c "Enter name of image catalog:"
define/param p2 ? c "Enter wavelength calibr. method (auto or interactive):"

define/local mode/c/1/100 {p2}
define/local ifrm/c/1/100 " " ALL

ifrm = WLC					!save input frame as may be 
						!overwritten by DRS table!!
mode = m$upper(p2)				!mode of wavel.calibration
config/instr {WLC}				!Set global keyword PATHID, etc
Modext = q2					!Mode extension (REDU,REDL,..)
						!
search/icat {p1} {DRSRC_ID({PATHID})} NB
if m$strlen(out_a) .eq. 0 then			!The setup table has to be
  append/out "No appropriate setup table found!\n"
  if FORCE_DRS(1:1) .eq. "Y" goto SKIP		!selected first as it holds
  mess/out 2 {pid}				!the global keywords of the
  mess/out 3 {pid} "Use global keyword settings (SHOW/ECHELLE)!"
else						!previous ECHELLE-session.
  savini/echelle {out_a} READ
endif						!In the following steps
WLC = ifrm					!overwrite the operational tab.
						!keywords: ORDTAB, BAKTAB, ...
						!
search/icat {p1} {ORDER_ID({PATHID})} NB	!Get order table
if m$strlen(out_a) .eq. 0 then
  append/out "No appropriate order table found!\n"
  goto SKIP
else
  set/echelle ORDTAB={out_a}
endif

set/echelle BAKTAB=+++ BKGVISU=NO EXTMTD=AVERAGE
						!order extraction method and
						!disable background table.


search/icat {p1} {LINCAT_ID} NB			!get reference line catalog
if m$strlen(out_a) .eq. 0 then
  append/out "No line reference table found!\n"
  goto SKIP
else
  set/echelle LINCAT={out_a}
endif

if mode(1:4) .eq. "AUTO" then
  search/icat {p1} {LINTAB_ID({PATHID})} NB	!get first-guess line table
  if m$strlen(out_a) .eq. 0 then		!a previous line table solut.
    append/out "No appropriate line first-guess table found!\n"
    goto SKIP
  else
    -copy {out_a} wcalLINE.tbl			!IDENTIFY/ECHE will look for:
    set/echelle WLCMTD=GUESS GUESS=wcal		!{GUESS}LINE.tbl as first guess
  endif						!use a first guess.
else						!Interactive mode: bivariate
  set/echelle WLCMTD=PAIR			!polynomial fit.
endif

write/keyw echord 0 ALL				!Reset echelle order keyword
						!will be set by IDENTI/ECHE
						!
outputi = wcenter/10				!Get central wavelength [nm]
set/format I1
Cwlen = "{outputi}"
set/format F1.0
if PATHID .lt. 3 then
  Bin = "{modext(5:5)}{CCDBIN(1)}x{CCDBIN(2)}"
else
  Bin = "_{CCDBIN(1)}x{CCDBIN(2)}"
endif
set/format I1

return 0

SKIP:
  append/out "--> skip input frame: `{WLC}'"
  mess/out 1 {pid}
  return -1


!..............................................................................
ENTRY RESOL_PLOT
define/param p1 ? c "Enter input name:"
define/param p2 ? c "Enter slit number:"
define/param p3 ? c "Enter YES to get resolution plot:"

define/local i1/i/1/1

set/format I1
i1 = pathid - 1 
@p uves_mod {i1} 2
i1 = {q1} + 1
assign/grap g,{i1}
@p uves_plt_resid {LINTAB} {LINCAT}
load/identification >Null

if NICE_HCOPY(1:1) .eq. "Y" then
  copy/graph postscript
  -rename postscript.ps resid_{LINTAB}.ps
!
!  copy/display LASER NOSTOP N ? NOPRINT
!  -rename screen{mid$sess(11:12)}.ps {p1}.ps
endif

set/format

if {P2} .eq. 2 .AND. p3(1:1) .eq. "Y" then
  mess/out 3 wavecal "Determine spectral resolution ..."
  

  assign/graph g,3
  if nice_crea .eq. "y" then
     modify/graph W
  endif 

! at the following line displays a warning at the extraction of 2. window
! cannot access ....no such file or directory
  calcul/reso p6=N,Y >Null  !Change option to get the free spectral range
 

  if m$exist("free_sp_rg.dat") .eq. 1 then
   -rename free_sp_rg.dat free_sp_rg_{LINTAB}.dat
   -rename resolution.dat resolution_{LINTAB}.dat
  endif
  
  if nice_crea .eq. "y" then
     mess/out 3 wavecal "Display spectral resolution ..."
     plot/resol p4=Y
     copy/graph g,3     
  else
     mess/out 3 wavecal "Creates spectral resolution postscript file only..."
     plot/resol p4=N
  endif 
  -rename postscript.ps resol_{LINTAB}.ps
endif
