/* $Id: uves_annot.c,v 1.2 2001/01/04 13:36:36 amodigli Exp $ */
/*+++ FORS_ANNOT ++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		fors_annot.prg
 
 .KEYWORDS	utility, POSTSCRIPT, label
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT		IN_A    annotation line
                IN_B    annotation file
                INPUTC  text font (c: Courier, t: Times, h: Helvetica)
		INPUTR  text style parameter:
		        (1): X-position [cm]
		        (2): Y-position [cm]
		        (3): Rotation angle [degree]
		        (4): Font size [pt]
		        (5): Baselineskip factor
		        (6): Red fraction
		        (7): Green fraction
		        (8): Blue fraction

 .OUTPUT	OUT_A	name of output file (POSTSCRIPT-file)
 
 .RETURN	Q1:  0: successful return
 		    -1: a successful return failed
 
 .PURPOSE	Create a bad pixel mask from a set of flatfield measurements
 
 .ALGORITHM	X
 
 .ENVIRON	MIDAS
    
 .LANGUAGE	C
 
 .AUTHOR	Sebastian Wolf, ESO-DMO
 
 .VERSION	1.0 1998/08/28	Creation
 
 .COMMENT       (1) 
-----------------------------------------------------------------------------*/
/* ===========================================================================
 * $Name:  $
 * $Revision: 1.2 $
 * $Log: uves_annot.c,v $
 * Revision 1.2  2001/01/04 13:36:36  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */
#include <midas_def.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <strings.h>
#include <ctype.h>

#define BELL  0x7
#define MIN(A,B) ((A)<(B)?(A):(B))      /* minimum function */
#define MAX(A,B) ((A)>(B)?(A):(B))      /* maximum function */
#define CM 72./2.54

#define MLEN 255
#define SLEN 80

char  Version[]="1999.07.22-SW";
char  Key[]="%%Page: 1 1";
char  NameI[SLEN+1];            /* input filename */
char  NameO[SLEN+1];            /* output filename */
char  Annotat[SLEN+1];          /* annotation to ps-file */
char  AnnotF[SLEN+1];           /* annotation to ps-file */
char  Ftyp[20]="Courier";       /* Font type */
int   kOff=0;                   /* key-check limit offset */
float XCM=-1e33;                /* x-coordinate */
float YCM=-1e33;                /* y-coordinate */
float Fscl=14.;                 /* font scale */  
float Rang=0.0;                 /* rotation angle in degrees */
float Bstretch=1.0;             /* rotation angle in degrees */
float Rcol=0.0;                 /* RGB colours */
float Gcol=0.0;
float Bcol=0.0;
FILE *FpI;
FILE *FpO;
FILE *FpA;

char Mes[MLEN];                 /* message buffer */

main()
{
  int  i, err, eof, ch, ch2, lKey;
  char *str;
  float x, y, DY;

  SCSPRO("annot");

  err = cmdline();
  if (err<0){
    SCSEPI();  
    return(err);
  }

  i=0;
  lKey = strlen(Key)-kOff;
  /*  fprintf(stderr,"%d\n",lKey);
  ch2=getchar();  */
  DY = Bstretch*Fscl;

  fclose(FpA);
  FpI = (FILE*) fopen(NameI,"r");
  FpO = (FILE*) fopen(NameO,"w");
  if (FpI != NULL)
    {
      do
	{
	  ch = fgetc(FpI);

	  if ( ch == Key[i] )
	    {
	      i++;
	      /* fprintf(stderr,"%d %c\n",i,ch);
	      ch2=getchar(); */
	      putc(ch,FpO);
	      if (i==lKey)           /* keyword found */
		{                    /* now insert */
		  do { ch = fgetc(FpI);  putc(ch,FpO); }while (ch!='\n');
		  fprintf(FpO,"gsave\n",Ftyp);
		  fprintf(FpO,"/cm {72. 2.54 div mul} def\n");
		  fprintf(FpO,"/%s findfont\n",Ftyp);
		  fprintf(FpO,"%f scalefont setfont\n",Fscl);
		  fprintf(FpO,"%f %f %f setrgbcolor\n",Rcol,Gcol,Bcol);
		  fprintf(FpO,"%f cm %f cm translate %f rotate\n",XCM,YCM,Rang);
		  x=0.0;
		  y=0.0;
		  FpA = (FILE*) fopen(AnnotF,"r");
		  if (FpA != NULL){
		    do{
		      y=y-DY;
		      str=(char*)fgets(Annotat,255,FpA);
		      if (str!=NULL){
			Annotat[strlen(Annotat)-1] = '\0';
			fprintf(FpO,"%f %f moveto (%s) show\n",x,y,Annotat);
		      }
		    }while (feof(FpA)==0);
		    fclose(FpA);
		  }
		  else
		    fprintf(FpO,"%f %f moveto (%s) show\n",x,y,Annotat);
		  fprintf(FpO,"grestore\n",Ftyp);
		}
	    }
	  else
	    {
	      i=0;
	      if (feof(FpI)==0) putc(ch,FpO);
	    }
	}while (feof(FpI)==0);
      fclose(FpI);
    }
  else
    {
      sprintf(Mes,"Cannot open file: '%s'\n",NameI);
      SCTPUT(Mes);
    }
  SCSEPI();  

  return(0);
}


int cmdline()
{ char      font[2];
  int       err, num, unit, null;
  float     tstyle[8];

  FpA=0x0;
  err = SCKGETC("OUT_A",1,SLEN,&num,NameI);            /* input file */
  err = SCKGETC("OUT_B",1,SLEN,&num,NameO);            /* output file */
  SCKGETC("IN_A",1,SLEN,&num,Annotat);           /* annotation string */
  SCKGETC("IN_B",1,SLEN,&num,AnnotF);            /* annotation string */
  SCKGETC("INPUTC",1,1,&num,font)   ;           /* font indicator */
  SCKRDR("INPUTR",1,8,&num,tstyle,&unit,&null); /* text style */

  /* x-pos, y-pos, rotation angle, font size, baselinskip , R, G, B */
  XCM      = tstyle[0];
  YCM      = tstyle[1];
  Rang     = tstyle[2];
  Fscl     = tstyle[3];
  Bstretch = tstyle[4];
  Rcol     = tstyle[5];
  Gcol     = tstyle[6];
  Bcol     = tstyle[7];
  if (font[0]=='c')            /* font type */
    strcpy(Ftyp,"Courier");
  else if (font[0]=='t')
    strcpy(Ftyp,"Times");
  else if (font[0]=='h')
    strcpy(Ftyp,"Helvetica");
  else
    {
      sprintf(Mes,"Undefined font type '%c' use default: 'c'\n",font[0]);
      SCTPUT(Mes);
      strcpy(Ftyp,"Courier");
    }

  if (AnnotF[0] != '+')   /* open annotation file */
    {
      FpA = (FILE*) fopen(AnnotF,"r");
    }

  if (FpA == NULL && Annotat[0] == '+')
    {
      sprintf(Mes,"Annotation file: '%s' NOT found!\n",AnnotF);
      return(-1);
    }
  else if (Annotat[0] != '+')
    {
      if (XCM<=-1e32) XCM=0.7;            /* default position   */
      if (YCM<=-1e32) YCM=0.7;            /* bottom of the page */
    }
  else
    {
      if (XCM<=-1e32) XCM=0.7;              /* default position */
      if (YCM<=-1e32) YCM=27.9;             /* top of the page  */
    }
  return(0);
}


int strpos(source,srch)
char *source, *srch;
{
  int  i, j, len1, len2, dlen, found;

  len1 = strlen(source);
  len2 = strlen(srch);
  dlen = len1 - len2;

  if (len2>len1) return(-1);     /* srch cannot fullly be a part of source! */

  j=-1;
  do
    {
      j++;
      for (i=0;(i<len2)&&(j<dlen)&&(source[j]==srch[i]);i++,j++);
/*	printf("%c %c %d %d\n",source[j],srch[i],(source[j]==srch[i]),i); */

      found=(i>=len2);           /* otherwise source[j]!=srch[i] !! */
    }while ((!found)&&(j<dlen));

  if ((j>dlen)&&(!found))
    return(-1);                  /* not found */
  else
    return(j-len2);              /* returns index, startin at 0 */
}
