/* $Id: uves_qc1logtbl.c,v 1.5 2001/05/30 16:32:06 amodigli Exp $ */
/*+++ uves_qc1logtbl.c +++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		uves_qc1logtbl.exe
 
 .KEYWORDS	uves, spectroscopy, echelle, quality control
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT	   LINTAB:	Line catalog name,
           RESOTAB:	line table name,
		
 		 
 .OUTPUT	LINTAB 
 
 .RETURN	Q1:  0: successful return
 		    -1: a successful return failed
 
 .PURPOSE	Keep track of :DX :DLAMBDA :RESOL into {LINCAT}. 
 
 .ALGORITHM	Adding 4 new columns to {LINTAB}: :DX :DLAMBDA :RESOL and :SELPLOT = 1
            if data are relevant.

 .ENVIRON	MIDAS, UVES context
    
 .LANGUAGE	C
 
 .AUTHOR	Olivier BOITQUIN, ESO-DMD
 
 .VERSION	1.

 .COMMENT   Dedicted for Quatlity Control with QC1 logging procedure.
----------------------------------------------------------------------------*/
/* ===========================================================================
 * $Name:  $
 * $Revision: 1.5 $
 * $Log: uves_qc1logtbl.c,v $
 * Revision 1.5  2001/05/30 16:32:06  amodigli
 * code cleaning
 *
 * Revision 1.4  2001/01/04 13:36:31  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */
/* Who  When        why                   where
   AM   30/05/01    code cleaning         main
 */


/* includes */
#include <tbldef.h>
#include <midas_def.h>
#include <stdlib.h>

/* defines */
#define  PROID "uves_qc1logtbl"
#define CHARMAX                60
#define FILESIZE              200

/* function prototypes */

#ifdef __STDC__  /* ANSI */
  static void  error(void);
#else  /* K&R */
  static void  error();
#endif


#ifdef __STDC__  /* ANSI */
  static void check_status(int   col, 
                          char* info);
#else  /* K&R */
  static void check_status();
  int col;
  char* info;
#endif


/* for messout function: no "static" (otherwise no messages from other files) */
int messlevel;

#ifdef __STDC__  /* ANSI */
  void main(int argc, char *argv[])
#else  /* K&R */
  void main(argc, argv)
  int  argc;
  char *argv[];
#endif
{
/* As usual */
  
  int nVal=0;                    /* anctual no. of elements returned */
  int unit=0;                    /* address of unit-pointer */
  int lineTid  = 0;              /* table identifier */
  int resoTid = 0 ;              /* table identifier */
  int nCol   = 0;                /* No of columns in tbl */
  int nSort  = 0;                /* No of sorted columns in tbl */
  int allCol = 0;                /* No of allocated words per record */
  int allRow = 0;                /* No of allocated rows */
  int actVals =0;                /* Act No of chars returned */
  
  int resoRowNumber=0;           /* No. of rows */
  int lineRowNumber=0;           /* No. of rows */
 
  int null=0;                    /* No of null values in keyword */
  int i=0;                       /* loop variable */
  int l=0;                       /* loop variable */

  char   lineTbl[FILESIZE];      /* buffer for data values */
  char   resoTbl[FILESIZE];      /* buffer for data values */
  char   aHeading[FILESIZE];     /* column reference */
  char  tmps[FILESIZE];          /* for messout */
  
  int    seq1Col=0;              /* Column Id for sequence No.*/
  int    seqCol=0;               /* Column Id for sequence No. */
  int    dxCol=0;                /* Column Id for DX */
  int    dLambdaCol=0;           /* Column Id for DLAMBDA */
  int    resolCol=0;             /* Column Id for resolution */
  int    selCol=0;               /* Column Id */
  int    dxColR=0;               /* Column Id for DX */
  int    dLambdaColR=0;          /* Column Id for DLAMBDA */
  int    resolColR=0;            /* Column Id for resolution */
  
  int    selIndex=0;             /* value to indicate a line is on the CCD */
  
  float  dx=0.0;                 /* holder for DX */
  float  dLambda=0.0;            /* holder for DLAMBDA */
  float  seq1=0.0;               /* holder for sequence */
  float    seq=0.0;              /* holder for sequence */
  float    diff=0.0;             /* value of seq1-seq */
  double resol=0.0;              /* holder for resolution */
  

/* initializes chars */
  memset(lineTbl,  0, FILESIZE);       
  memset(aHeading, 0, FILESIZE);    
  memset(tmps,     0, FILESIZE);        
          
/*
 **************************
 * Starts to do something *
 **************************
 */      
  SCSPRO("uves_plt_dispres");
  SCKRDI("MESS_LEVEL",1,1,&nVal,&messlevel,&unit,&null);
  
/* Get value from keyword */

  SCKGETC("QC1TAB", 1, CHARMAX, &actVals, lineTbl);
  SCKGETC("RESOTAB", 1, CHARMAX, &actVals, resoTbl);
    
/* Open the resol table */

  TCTOPN(resoTbl, F_I_MODE, &resoTid);
  TCIGET (resoTid, &nCol, &resoRowNumber, &nSort, &allCol, &allRow); 
  
  sprintf(&tmps[0],"Opened table %s nCol=%d nrow=%d",resoTbl ,nCol,resoRowNumber);
  messout(messlevel,3,PROID,tmps);  

  strcpy(aHeading,":SEQU_1");
  TCCSER(resoTid, aHeading, &seq1Col);
  check_status(seq1Col,aHeading);

  strcpy(aHeading,":DX");  
  TCCSER(resoTid, aHeading, &dxColR);
  /*check_status(dxColR,aHeading); */
   
  strcpy(aHeading,":DLAMBDA"); 
  TCCSER(resoTid, aHeading, &dLambdaColR);
  check_status(dLambdaColR,aHeading);

  strcpy(aHeading,":RESOL");  
  TCCSER(resoTid, aHeading, &resolColR);  
  check_status(resolColR,aHeading);
  
/* Open and Update the LINTAB table */

  TCTOPN(lineTbl, F_IO_MODE, &lineTid);
  TCIGET (lineTid, &nCol, &lineRowNumber, &nSort, &allCol, &allRow); 
  
  sprintf(&tmps[0],"Opened table %s nCol=%d nrow=%d",lineTbl ,nCol,lineRowNumber);
  messout(messlevel,3,PROID,tmps);  

  strcpy(aHeading,":SEQU");
  TCCSER(lineTid, aHeading, &seqCol);
  check_status(seqCol,aHeading);
 
 
  /* if columns are present clean up first */ 
  strcpy(aHeading,":DX");  
  TCCSER(lineTid, aHeading, &dxCol);
  if (dxCol >= 0) {
    TCCDEL(lineTid, dxCol, &nCol);
  }
 
  strcpy(aHeading,":DLAMBDA"); 
  TCCSER(lineTid, aHeading, &dLambdaCol);
  if (dLambdaCol >= 0) {
    TCCDEL(lineTid, dLambdaCol, &nCol);
  }
 
  strcpy(aHeading,":RESOL");  
  TCCSER(lineTid, aHeading, &resolCol);
  if (resolCol >= 0) {
    TCCDEL(lineTid, resolCol, &nCol);
  }
  
  strcpy(aHeading,":SELPLOT");
  TCCSER(lineTid, aHeading, &selCol);
  if (selCol >= 0) {
    TCCDEL(lineTid, selCol, &nCol);
  }
 
  TCCINI(lineTid, D_R4_FORMAT, 1, "E12.6", " ", "DX", &dxCol);  
  TCCINI(lineTid, D_R4_FORMAT, 1, "E12.6", " ", "DLAMBDA", &dLambdaCol);
  TCCINI(lineTid, D_R8_FORMAT, 1, "D24.17", " ", "RESOL", &resolCol);
  TCCINI(lineTid, D_I4_FORMAT, 1, "I6", " ", "SELPLOT", &selCol);
  /* here writes dxCol, dLambdaCol, resolCol, selCol in the resolution table
     at the proper place
   */ 
  for (l=0; l<resoRowNumber; l++) {

    TCERDR(resoTid, (l+1), seq1Col, &seq1, &null);
    TCERDR(resoTid, (l+1), dxColR, &dx, &null);
    TCERDR(resoTid, (l+1), dLambdaColR, &dLambda, &null);
    TCERDD(resoTid, (l+1), resolColR, &resol, &null);   
    selIndex = 1;

    for (i=0; i<lineRowNumber; i++) {

        TCERDR(lineTid, (i+1), seqCol, &seq, &null);
        
        diff=fabs(seq-seq1);
        if (diff <= 0.001 ) {
           TCEWRR(lineTid, (i+1), dxCol, &dx);
           TCEWRR(lineTid, (i+1), dLambdaCol, &dLambda);
           TCEWRD(lineTid, (i+1), resolCol, &resol);
           TCEWRI(lineTid, (i+1), selCol, &selIndex);

           break;   
        }
    }
  }

/* close everything */

  TCTCLO(lineTid); 
  TCTCLO(resoTid);

  SCKWRC("Q1",1,"0",1,CHARMAX,&unit);
  SCSEPI();  
}

/*---------------------------------------------------------------------*/

#ifdef __STDC__  /* ANSI */
  static void error(void)
#else  /* K&R */
  static void error()
#endif
{
  int  r, unit;

  r = SCKWRC("Q1",1,"1",1,CHARMAX,&unit);
  r = SCSEPI();
}

/*---------------------------------------------------------------------*/
#ifdef __STDC__  /* ANSI */
  static void check_status(int   col, 
                          char* info)
#else  /* K&R */

  static void check_status()
  int col;
  char* info;

#endif
/*---------------------------------------------------------------------
 * issues a message in case of missing column
 *---------------------------------------------------------------------
 */
 
{
  char   tmps[FILESIZE];     /* for messout */
 
  if (col <= 0) {
    sprintf(&tmps[0],"Could not find column: %s",info);
    messout(messlevel,1,PROID,tmps);
    error();
  }
}
