/* $Id: uves_qc1pmtbl.c,v 1.6 2001/05/30 16:32:07 amodigli Exp $ */
/*+++ uves_qc1logtbl.c +++++++++++++++++++++++++++++++++++++++++++++++++++++++
 
 .COPYRIGHT	(C) 1998  European Southern Observatory
 
 .IDENT		uves_qc1pmtbl.exe
 
 .KEYWORDS	uves, spectroscopy, echelle, quality control
 
 .USAGE		run PIPE_EXE:(IDENT)
 
 .INPUT	   LINTAB:	Line catalog name,
           RLINTAB:	line table name,
		
 		 
 .OUTPUT	LINTAB 
 
 .RETURN	Q1:  0: successful return
 		    -1: a successful return failed
 
 .PURPOSE	It puts :XDIF,:YDIF,:XPRED,:YPRED.:SELPLOT with properly 
            aligned x with xModReg and y with yModReg
 
 .ALGORITHM	Line table elements alignment

 .ENVIRON	MIDAS, UVES context
    
 .LANGUAGE	C
 
 .AUTHOR	Andrea Modigliani, Olivier Boitquin, ESO-DMD
 
 .VERSION	1.

 .COMMENT   Dedicted for Quatlity Control with QC1 logging procedure.
----------------------------------------------------------------------------*/
/* Who  When        why                   where
   AM   29/05/01    code cleaning         main
 */
/* ===========================================================================
 * $Name:  $
 * $Revision: 1.6 $
 * $Log: uves_qc1pmtbl.c,v $
 * Revision 1.6  2001/05/30 16:32:07  amodigli
 * code cleaning
 *
 * Revision 1.5  2001/01/04 13:36:31  amodigli
 * Added Id, Name, Revision, Log conf. control info
 *
 * ===========================================================================
 */



/* includes */
#include <tbldef.h>
#include <midas_def.h>
#include <stdlib.h>
#define  PROID "uves_qc1pmtbl"
#define CHARMAX                60
#define FILESIZE              200

/* function prototypes */

#ifdef __STDC__  /* ANSI */
  static void  error(void);
#else  /* K&R */
  static void  error();
#endif


#ifdef __STDC__  /* ANSI */
  static void check_status(int   col, 
                          char* info);
#else  /* K&R */

  static void check_status();
  int col;
  char* info;
#endif

/* for messout function: no "static" (otherwise no messages from other files) */
int messlevel;           /* buffer for data values */

#ifdef __STDC__  /* ANSI */
  void main(int argc, char *argv[])
#else  /* K&R */
  void main(argc, argv)
  int  argc;
  char *argv[];
#endif
{

/*
 ***************************************
 * variables definition-initialization *
 ***************************************
 */
 
/* As usual */
 
  int nVal=0;                /* anctual no. of elements returned */
  int unit=0;                /* address of unit-pointer */
  int lineTid  = 0;          /* table identifier */
  int rLineTid = 0;          /* table identifier */
  int nCol   = 0;            /* No of columns in tbl */
  int nSort  = 0;            /* No of sorted columns in tbl */
  int allCol = 0;            /* No of allocated words per record */
  int allRow = 0;            /* No of allocated rows */
  int actVals=0;             /* Act No of chars returned */
  int rlineRowNumber=0;      /* number of rows in rline table */
  int lineRowNo=0;           /* number of Rows in line table */
  int null=0;                /* number of null values in keyword */
  int i=0;                   /* loop variable */
  int l=0;                   /* loop variable */


  
  int    xModCol=0;          /* column number */
  int    yModCol=0;          /* column number */
  int    xModRegCol=0;       /* column number */
  int    yModRegCol=0;       /* column number */
  int    xCol=0;             /* column number */
  int    yCol=0;             /* column number */
  int    xDifCol=0;          /* column number */
  int    yDifCol=0;          /* column number */
  int    xPredCol=0;         /* column number */
  int    yPredCol=0;         /* column number */
  int    selCol=0;           /* column number */
  int    xDifColR=0;         /* column number */
  int    yDifColR=0;         /* column number */
  int    selIdx=0;           /* actual value */
  
  char   lineTbl[FILESIZE];  /* buffer for data values */
  char   rLineTbl[FILESIZE]; /* buffer for data values */
  char   aHeading[FILESIZE]; /* column reference */
  char   tmps[FILESIZE];     /* for messout */
   
  
  
  double  xDif=0.0;          /* actual value */
  double  yDif=0.0;          /* actual value */
  double  xModReg=0.0;       /* actual value */
  double  yModReg=0.0;       /* actual value */
  double  xPred=0.0;         /* actual value */
  double  yPred=0.0;         /* actual value */
  double  x=0.0;             /* actual value */
  double  yNew=0.0;          /* actual value */
  double  diffX=0.0;         /* difference between x and y */
  double  diffY=0.0;         /* difference between x and y */
                             /* (rounding error threshold) */
  
/* initializes chars */
  memset(lineTbl,  0, FILESIZE);    
  memset(rLineTbl, 0, FILESIZE);    
  memset(aHeading, 0, FILESIZE);    
  memset(tmps,     0, FILESIZE);        
   
/*
 **************************
 * Starts to do something *
 **************************
 */ 
  SCSPRO("uves_qc1pmtbl.exe"); 
  SCKRDI("MESS_LEVEL",1,1,&nVal,&messlevel,&unit,&null);
  SCKGETC("QC1PM", 1, CHARMAX, &actVals, lineTbl);
  SCKGETC("RLINETAB", 1, CHARMAX, &actVals, rLineTbl);


  TCTOPN(rLineTbl, F_I_MODE, &rLineTid);
  TCIGET (rLineTid, &nCol, &rlineRowNumber, &nSort, &allCol, &allRow); 

  sprintf(&tmps[0],"Opened table %s nCol=%d nrow=%d",rLineTbl ,nCol,rlineRowNumber);
  messout(messlevel,3,PROID,tmps);  


  strcpy(aHeading,":XMODREG");
  TCCSER(rLineTid, aHeading, &xModRegCol);
  check_status(xModRegCol,aHeading);
  
  strcpy(aHeading,":YMODREG");  
  TCCSER(rLineTid, aHeading, &yModRegCol);
  check_status(yModRegCol,aHeading);
    
  strcpy(aHeading,":XMOD");
  TCCSER(rLineTid, aHeading, &xModCol);
  check_status(xModCol,aHeading);
  
  strcpy(aHeading,":YMOD");  
  TCCSER(rLineTid, aHeading, &yModCol);
  check_status(yModCol,aHeading);



  strcpy(aHeading,":XDIF");  
  TCCSER(rLineTid, aHeading, &xDifColR);
  check_status(xDifColR,aHeading);
  
  strcpy(aHeading,":YDIF"); 
  TCCSER(rLineTid, aHeading, &yDifColR);
  check_status(yDifColR,aHeading);
   

/*
 ************************************ 
 * Open and Update the LINTAB table *
 ************************************
 */
 
  TCTOPN(lineTbl, F_IO_MODE, &lineTid);
  TCIGET (lineTid, &nCol, &lineRowNo, &nSort, &allCol, &allRow); 
  
  sprintf(&tmps[0],"Opened table %s nCol=%d nrow=%d",lineTbl ,nCol,lineRowNo);
  messout(messlevel,3,PROID,tmps);  

/*
 *********************
 * Search the column *
 *********************
 */ 
  strcpy(aHeading,":X");
  TCCSER(lineTid, aHeading, &xCol);
  check_status(xCol,aHeading);
  
  strcpy(aHeading,":YNEW");
  TCCSER(lineTid, aHeading, &yCol);
  check_status(yCol,aHeading);
  
  
/* clean up columns listed below */  
  strcpy(aHeading,":XDIF");  
  TCCSER(lineTid, aHeading, &xDifCol);
  if (xDifCol >= 0) {
    TCCDEL(lineTid, xDifCol, &nCol);
  }
  
  strcpy(aHeading,":YDIF"); 
  TCCSER(lineTid, aHeading, &yDifCol);
  if (yDifCol >= 0) {
    TCCDEL(lineTid, yDifCol, &nCol);
  }
  
  strcpy(aHeading,":SELPLOT");
  TCCSER(lineTid, aHeading, &selCol);
  if (selCol >= 0) {
    TCCDEL(lineTid, selCol, &nCol);
  }
  
  strcpy(aHeading,":XPRED");
  TCCSER(lineTid, aHeading, &xPredCol);
  if (xPredCol >= 0) {
    TCCDEL(lineTid, xPredCol, &nCol);
  }
  
  strcpy(aHeading,":YPRED");
  TCCSER(lineTid, aHeading, &yPredCol);
  if (yPredCol >= 0) {
    TCCDEL(lineTid, yPredCol, &nCol);
  }

   
  TCCINI(lineTid, D_R8_FORMAT, 1, "D24.17", " ", "XPRED", &xPredCol);  
  TCCINI(lineTid, D_R8_FORMAT, 1, "D24.17", " ", "YPRED", &yPredCol);
  TCCINI(lineTid, D_R8_FORMAT, 1, "D24.17", " ", "XDIF", &xDifCol);  
  TCCINI(lineTid, D_R8_FORMAT, 1, "D24.17", " ", "YDIF", &yDifCol);
  TCCINI(lineTid, D_I4_FORMAT, 1, "I6", " ", "SELPLOT", &selCol);
  


/*
 ***************************************************************************
 * Now does the actual ordering of the elements writing in the right place *
 ***************************************************************************
 */
 
  for (l=0; l< rlineRowNumber; l++) {


 
    TCERDD(rLineTid, (l+1), xModRegCol, &xModReg, &null);
    TCERDD(rLineTid, (l+1), yModRegCol, &yModReg, &null);
    TCERDD(rLineTid, (l+1), xDifColR, &xDif, &null);
    TCERDD(rLineTid, (l+1), yDifColR, &yDif, &null);
    TCERDD(rLineTid, (l+1), xModCol, &xPred, &null);
    TCERDD(rLineTid, (l+1), yModCol, &yPred, &null);
      
    selIdx = 1;
    for (i=0; i< lineRowNo; i++) {
     
  
        TCERDD(lineTid, (i+1), xCol, &x, &null);
        TCERDD(lineTid, (i+1), yCol, &yNew, &null);

   
        diffX=fabs(x-xModReg);
        diffY=fabs(yNew-yModReg);
        if ((diffX <= 0.001) && (diffY <= 0.001)) {

           TCEWRD(lineTid, (i+1), xPredCol, &xPred);
           TCEWRD(lineTid, (i+1), yPredCol, &yPred);
           TCEWRD(lineTid, (i+1), xDifCol, &xDif);
           TCEWRD(lineTid, (i+1), yDifCol, &yDif);
           TCEWRI(lineTid, (i+1), selCol, &selIdx);
           
           break;   
        }
    }
  }

/* close everything */

  TCTCLO(lineTid); 
  TCTCLO(rLineTid);
  SCKWRC("Q1",1,"0",1,CHARMAX,&unit); 
  SCSEPI();  
}

/*---------------------------------------------------------------------*/

#ifdef __STDC__  /* ANSI */
  static void error(void)
#else  /* K&R */
  static void error()
#endif
{
/*
 ******************
 * Error function *
 ******************
 */
  int  r;    /* result */
  int  unit; /* unit-pointer */

  r = SCKWRC("Q1",1,"1",1,CHARMAX,&unit);
  r = SCSEPI();
}

/*---------------------------------------------------------------------*/
#ifdef __STDC__  /* ANSI */
  static void check_status(int   col, 
                          char* info)
#else  /* K&R */

  static void check_status()
  int col;
  char* info;

#endif
/*---------------------------------------------------------------------
 * issues a message in case of missing column
 *---------------------------------------------------------------------
 */
 
{
  char   tmps[FILESIZE];     /* for messout */
 
  if (col <= 0) {
    sprintf(&tmps[0],"Could not find column: %s",info);
    messout(messlevel,1,PROID,tmps);
    error();
  }
}
