! $Id: uves_cal_orderpos.prg,v 1.3 2001/05/04 14:10:07 amodigli Exp $
!+++ UVES_CAL_ORDERPOS ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_cal_orderpos
!
!.KEYWORDS      spectroscopy, echelle, order positions
!
!.USAGE         @p (IDENT) sof_in out sof_ref [optpar]
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!
!.OUTPUT        out     (P2): path + basename of output files:
!				order table (ORDER_ID),
!				background table (BACK_ID),
!				setup table (DRSRC_ID).
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT	P3	+
!
!.PURPOSE       Analyses any echelle exposure in order to determine the order
!		positions. Standard stars, flatfields or arc-lamp exposures
!		as input frames are recommended.
!
!.ALGORITHM     Does a transformation of each echelle frame of the set of
!		of input frame (rotate, split if necessary). Perform a
!		Hough-transform for each exposure using DEFINE/HOUGH which
!		returns the order table as well as the background table.
!
!		The products are classified as:
!		{ORDER_ID(i)}, {BACK_ID(i)} , where i depends on the instrument
!		path. Furthermore, all global keywords belonging to the ECHELLE
!		context are stored in a setup table {DRSRC_ID}.
!
!.ENVIRON       UVES, ECHELLE context
!   
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       0.5 1999/03/09  Created.
!		    1999/03/26	@p uves_updtbl implemented by INIT/PIPE
!				and FINISH/PIPE
!		    1999/04/23	Call of ORDER/POSITION implemnted.
!		    1999/06/21	ORDER/POSITION changed to ORDERP/UVES.
!		    1999/06/25	An input catalog is passed to ORDERP/UVES now.
!		    1999/07/14	Return statement was missing at the end.
!		    1999/09/26	Check return value of FINISH/PIPE
!		    1999/12/06	Set as default P3 to +
!		    1999/12/10	main product was not sent to FINISH/PIPELINE!
!
!.COMMENT	We need uves_updtbl as the background table uses a descriptor
!		STEP which will be deleted as soon as the table is saved as
!		a FITS file! The procedure uves_updtbl.prg creates a copy
!		of STEP which may be read after the FITS-file version of the
!		background table has been loaded in order to restore STEP !
!		   SO, EVERYWHERE where the background table is in use this
!		procedure should be used. To ensure this, the call will be
!		made in INITIALIZE/PIPELINE and FINISH/PIPELINE -- both
!		routines are obligatory procedures for recipes!
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.3 $
! $Log: uves_cal_orderpos.prg,v $
! Revision 1.3  2001/05/04 14:10:07  amodigli
! Added RB info to FITS header
!
! Revision 1.2  2001/01/04 13:44:16  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/local line/c/1/79 "-" ALL
write/out {line}
LOG/REC uves_cal_orderpos IN
!
! Check parameters
!
define/param p1 ? c "Enter name of ASCII filelist with INPUT frames:"
define/param p2 ? c "Enter name of OUTPUT frame:"
define/param p3 + c "Enter name of REFERENCE frame:"
!
! local constant declaration
!
define/local pid/c/1/16 cal_orderpos
!
! local variable declaration
!
define/local main_prod/c/1/25 " " all
define/local ifrm/c/1/100 " " ALL
define/local otbl/c/1/100 " " ALL
define/local btbl/c/1/100 " " ALL
define/local num/i/1/1 0

mess/out 3 {pid} -
  "Recipe UVES_CAL_ORDERPOS will determine\nechelle order postions."

@p uves_paragraph 1

INITIALIZE/PIPELINE {P1} {P3} orderI.cat + + + reject.cat orderO.cat
if {Q1} .gt. 0 return {Q1}


PROHEAD/INIT {P1} {P3} 1,"uves_cal_orderpos" 

@p uves_paragraph 2
clean/disp A					!clear displays and graph.win.


split/uves orderI.cat				!transpose and flip frames


orderp/uves split.cat orderO.cat orderI.cat	!Do the order definition.
if {q1} .ne. 0 goto ABORT			!An error had occured.
main_prod = q2

shortlog/out 1 "Order positions defined."
shortlog/out 1 "Background positions defined."

prohead/finish  orderO.cat 1

@p uves_paragraph 3
FINISH/PIPELINE orderO.cat {P2} + {main_prod}
if {q1} .eq. 1 goto ABORT

LOG/REC uves_cal_orderpos OUT
@p uves_paragraph 4
return 0

ABORT:
mess/out 1 {pid} "Error in uves_ordpos occured !"
write/error 1,APP,NO "Error in uves_ordpos occured !"
