! $Id: uves_cal_response.prg,v 1.5 2001/07/04 15:57:25 amodigli Exp $
!+++ UVES_CAL_RESPONSE ++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT     (C) 1998  European Southern Observatory
!
!.IDENT         uves_cal_response.prg
!
!.KEYWORDS      spectroscopy, echelle, order, wavelength calibration
!
!.USAGE         @p (IDENT) sof_in out sof_ref ffmeth exmeth
!
!.INPUT         sof_in  (P1): set of input frames (ascii list)
!               sof_ref (P3): set of reference frames (ascii list)
!				order-, line-table, line reference table,
!				drs-setup-table, and
!				catalog of flux standard spectra (3D table)
!               ffmeth  (P4): flatfielding method
!				. P: pixel-to-pixel flatfielding
!				. E: flatfielding during extraction
!               exmeth  (P5): extraction method
!				. AVERAGED
!				. OPTIMAL
!
!.OUTPUT        out     (P2): path + basename of output files:
!				response frame
!
!.RETURN        Q1:  0: successful return
!                   -1: a successful return failed
!
!.DEFAULT
!
!.PURPOSE       Do a echelle data reduction of standard star frame and
!		determine the response and the efficiency.
!
!.ALGORITHM     Calls REDUCE/UVES, RESPONSE/UVES and EFFICIENCY/UVES which
!		do the whole job.
!
!.ENVIRON       UVES, ECHELLE context
!
!.LANGUAGE      MIDAS 98NOV
!
!.AUTHOR        Sebastian Wolf, ESO-DMO
!
!.VERSION       1.0 1999/11/08  Created.
!		    1999/12/10	main product was not sent to FINISH/PIPELINE!
!		    1999/12/19	EFFICIENCY/UVES added.
!
!.COMMENT
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.5 $
! $Log: uves_cal_response.prg,v $
! Revision 1.5  2001/07/04 15:57:25  amodigli
! Added a preliminary check to see if the
! standard star is present in table list.
! Before extraction. If not exit with error.
! This to save computation time
!
! Revision 1.4  2001/05/14 13:18:59  amodigli
! Set default bkg method to median
!
! Revision 1.3  2001/05/04 14:10:53  amodigli
! Added RB info to FITS header
!
! Revision 1.2  2001/01/04 13:44:15  amodigli
! Added Id, Name, Revision, Log conf. control info
!
!------------------------------------------------------------------------------
define/local line/c/1/79 "-" ALL
write/out {line}
LOG/REC uves_cal_response IN
!
! Check parameters
!
define/param p1 ? c "Enter name of ASCII filelist with INPUT frames:"
define/param p2 ? c "Enter name of ASCII filelist with REFERENCE frames:"
define/param p3 ? c "Enter name of OUTPUT frame:"
define/param p4 ? c "Enter flatfielding method (P,E,...):"
define/param p5 ? c "Enter extraction method (AVERAGED,OPTIMAL):"
define/param p6 MEDIAN c "Enter background measur. method (MEDIAN, MINIMUM):"
!
! local constant declaration
!
define/local pid/c/1/16 cal_response
!
! local variable declaration
!
define/local ifrm/c/1/100 " " ALL
define/local do_clas/c/1/25 " " ALL
define/local main_prod/c/1/25 " " all
define/local i1/i/1/1 1
define/local num/i/1/1 0

mess/out 3 {pid} -
  "Recipe UVES_CAL_RESPONSE will determine\nthe response and efficieny"

@p uves_paragraph 1

INITIALIZE/PIPELINE {P1} {P3} respI.cat + + + reject.cat respO.cat
if {Q1} .gt. 0 return {Q1}

PROHEAD/INIT {P1} {P3} 1,"uves_cal_response" -
"FFMTD,{P4}" "EXMTD,{P5}" "BGMTD,{P6}"


@p uves_paragraph 2
init/echelle
clean/disp A					!clear displays and graph.win.


split/uves respI.cat				!transpose and flip frames
show/icat split.cat >Null			!and split for RED arm
num = outputi(1)				!number of input frames
						!split.cat hold ONLY RAW frames
!we 1st check the STD star is present in the catalog
!to save time: no reason to extract the spectra if in the end
!no response can be determined.						
RESPONSE/UVES split.cat respP.cat respI.cat check
if {q1} .lt. 0 goto ABORT4						
						!Do the std.star reduction:
REDUCE/UVES split.cat respO.cat respI.cat {p4} {p5} {p6}
if {q1} .lt. 0 goto ABORT1

						!Determine the response of the
RESPONSE/UVES respO.cat respP.cat respI.cat	!instrument by comparing the
if {q1} .lt. 0 goto ABORT2			!wavelength calibrated frame
main_prod = q2					!with the corresponding
						!refer. spectrum of a catalog


EFFICIENCY/UVES split.cat respP.cat respI.cat	!Compute the efficiency
if {q1} .lt. 0 goto ABORT3			!result is stored in a table


shortlog/out 1 "Standard star reduction done."
shortlog/out 1 "Response calculation done."
shortlog/out 1 "Efficiency computation done."

prohead/finish  respP.cat 1

@p uves_paragraph 3

FINISH/PIPELINE respP.cat {P2} + {main_prod}
if {q1} .eq. 1 goto ABORT

LOG/REC uves_cal_response OUT
@p uves_paragraph 4
return 0


ABORT1:
mess/out 1 {pid} "Error in uves_reduce occured !"
write/error 1,APP,NO "Error in uves_reduce occured !"
return -1

ABORT2:
mess/out 1 {pid} "Error in uves_response occured !"
write/error 1,APP,NO "Error in uves_response occured !"
return -1

ABORT3:
mess/out 1 {pid} "Error in uves_efficiency occured !"
write/error 1,APP,NO "Error in uves_efficiency occured !"
return -1

ABORT4:
mess/out 1 {pid} "Error in uves_response occured. No STD star found in preliminary check. !"
write/error 1,APP,NO "Error in uves_response occured. No STD star found in preliminary check !"
return -1
