#!/bin/sh
# $Id: uves_effic.sh,v 1.6 2001/02/12 14:53:16 amodigli Exp $
#+++ UVES_EFFIC +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         effic.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       standard star exposure [STANDARD_y]
#                       DRS table     [DRS_SETUP_x]
#                       order table   [ORDER_TABLE_x]
#                       background table [BACKGR_TABLE_x]
#                       line tables   [LINE_TABLE_xn]
#                       standard star flux table [FLUX_STD_TABLE]
#                       atmospheric extinction table [EXTCOEFF_TABLE]
#                       (y: RED, BLUE x:BLUE, REDL, REDU; n:1,2,3)
#
#
#
#.OUTPUT        dispersion relation solution [LINE_TABLE_y]
#		resolutin plot (POSTSCRIPT)
#		(y: REDL, REDU, BLUE)
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Creates the dispersion relation solution saved as a
#		line table and produces a resolution plot as a POSTSCRIPT
#		file.
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command WAVECAL/UVES.
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/17  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.6 $
# $Log: uves_effic.sh,v $
# Revision 1.6  2001/02/12 14:53:16  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:38  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------


cd /tmp
rm -rf gasg_${USER}
mkdir gasg_${USER}
wrk=/tmp/gasg_${USER}
echo $wrk
cd $HOME/midwork
rm -rf tmpwrk
mkdir tmpwrk
cd tmpwrk
if [ -s ${wrk}/frames.ascii ]
then 
      rm ${wrk}/frames.ascii
fi
if [ -s ${wrk}/effic.prg ]
then 
      rm ${wrk}/effic.prg
fi
list=`echo $*`
rm -f tmpdat
mkdir tmpdat
cd tmpdat
for i in $list
do
  printf "%s\n" `basename $i` >> ${wrk}/frames.ascii
  ln -s $i .
done

if [ ! -s ${wrk}/frames.ascii ]
then
  xdialog -bg white  -fg red -m '
ERROR: Empty set of input files.
You have to select the following list of input files:
 ThAr exposure [ARC_LAMP_x]
 DRS table [DRS_SETUP_x]
 order table [ORDER_TABLE_x]
 guess table [LINE_TABLE_x]
 line reference catalog [LINE_REFER_TABLE]
 (x: may be RED, or BLUE)'
 exit
fi

if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

printf '@d pipeline.start\n \
    @d pipeline.control d\n \
    write/key pipe_home/c/1/100 " " ALL \n \
    pipe_home = m$symbol("PIPE_HOME") \n \
    set/context uves {pipe_home}/uves/context \n  \    
    write/key wrk/c/1/100 " " ALL \n \
    wrk = m$symbol("USER") \n \
    wrk = "/tmp/gasg_{wrk}" \n \
    create/icat in.cat {wrk}/frames.ascii >Null\n \
    write/out "INFO    [effic]:            Transfering the data ..."\n \
    indisk/fits in.cat ROOT=ref >Null\n \
    create/icat effic.cat ref* >Null\n \
    @p uves_extrref effic.cat \n \
    CONFIG/DISP p3=0.8 \n \
    SPLIT/UVES data.cat \n \
    EFFICIEN/UVES split.cat out.cat refer.cat \n \
    ASSIGN/GRA g,3 \n \
    MODIF/GRA W \n \
    write/out " " \n \
    write/out "To exit MIDAS session type  bye" \n \
' > ${wrk}/effic.prg



xterm -geom 132x24+0-0 -fn fixed -T 'gasgano: effic.sh' -e inmidas -P -j '@@ /tmp/gasg_'${USER}'/effic.prg' &

exit
