#!/bin/sh
# $Id: uves_format.sh,v 1.6 2001/02/12 14:53:07 amodigli Exp $
#+++ UVES_POPUL +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         popul.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       Formatcheck frame [ARC_LAMP_FORM_x]
#                       ThAr line table   [LINE_REFER_TABLE]
#
#.OUTPUT        reference frames necessary for science reduction:
#			drs setup table, order table, background table,
#			line tables, and optionally a master bias and a
#			master flat.
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Creates all the reference frames necessary for the science
#		reduction.
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally call UVES context commands:
#		PREDICT/UVES, ORDERP/UVES, WAVECAL/UVES and MASTER/UVES
#		The master and dispersion relation creation will be skipped
#		as soon as no bias, flats or ThAr exposures specified.
#		Instead of biases you may specify a master bias for the
#		master flatfield creation.
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/17  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.6 $
# $Log: uves_format.sh,v $
# Revision 1.6  2001/02/12 14:53:07  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:35  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------



# Create a temprary working area

localdir=`pwd`

cd /tmp
rm -rf gasg2_${USER}
mkdir gasg2_${USER}
wrk=/tmp/gasg2_${USER}
echo $wrk

if [ -z "$MIDWORK" ] 
then 
   cd $HOME/midwork
else
   cd $MIDWORK
fi

rm -rf tmpwrk
mkdir tmpwrk
mkdir tmpwrk/data
cd tmpwrk

if [ -s ${wrk}/frames.ascii ]
then 
      rm ${wrk}/frames.ascii
fi
if [ -s ${wrk}/popul.prg ]
then 
      rm ${wrk}/popul.prg
fi



# Get the set of input files

sof=`echo $*`


#
# Create symbolic links to the input data as original path may be too long!
#

for frm in $sof
do
  printf "%s\n" `basename $frm` >> ${wrk}/frames.ascii
  if [ ! -s $frm ] 
  then
     frm=$localdir/$frm
  fi

  if [ ! -s $frm ] 
  then 
       echo 'File not found: ' $frm 
       exit 111
  else
       ln -s $frm .
  fi
done
#
#
#
# Do an error handling if set of input frames are empty
#
#
if [ ! -s ${wrk}/frames.ascii ]
then
  xdialog -bg white  -fg red -m '
ERROR: Empty set of input files.
You have to select the following list of input files:
  ThAr exposure [ARC_LAMP_x]
  ThAr exposure with short slit [ARC_LAMP_FORM_x]
  order definition flat [ORDER_FLAT_x]
  line reference catalog [LINE_REFER_TABLE]
  (x: may be RED, or BLUE)'
 exit
fi


if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 
printf '@d pipeline.start >Null \n \
    @d pipeline.control d >Null \n \
    write/key pipe_home/c/1/100 " " ALL \n \
    pipe_home = m$symbol("PIPE_HOME") \n \
    set/context uves {pipe_home}/uves/context \n  \
    write/key wrk/c/1/100 " " ALL \n \
    wrk = m$symbol("USER") \n \
    wrk = "/tmp/gasg2_{wrk}" \n \
    create/icat in.cat {wrk}/frames.ascii >Null\n \
    write/out "INFO    [popul]:            Transfering the data ..."\n \
    indisk/fits in.cat ROOT=ref >Null\n \
    create/icat popul.cat ref* >Null\n \
    @p uves_extrref popul.cat \n \
    FORMAT/UVES data.cat refer.cat P5=NO \n \
    -copy in.cat {q2} \n \
    -rename *fits ./data \n \
    write/out " " \n \
!    write/out "To exit MIDAS session type  bye" \n \
    bye  \n \
' > ${wrk}/format.prg

#
# Start the MIDAS session
#
xterm -geom 132x24+0-0 -fn fixed -T 'gasgano: uves_format.sh '$1 -e inmidas -P -j '@@ /tmp/gasg2_'${USER}'/format.prg' 

exit
