#!/bin/sh
# $Id: uves_plot.sh,v 1.6 2001/02/12 14:53:15 amodigli Exp $
#+++ UVES_PLOT +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         plot.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       Almost any of the result frames
#
#.OUTPUT        dispersion relation solution [LINE_TABLE_y]
#		resolutin plot (POSTSCRIPT)
#		(y: REDL, REDU, BLUE)
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Creates the dispersion relation solution saved as a
#		line table and produces a resolution plot as a POSTSCRIPT
#		file.
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command WAVECAL/UVES.
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/17  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.6 $
# $Log: uves_plot.sh,v $
# Revision 1.6  2001/02/12 14:53:15  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:37  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------


cd /tmp
rm -rf gasg_${USER}
mkdir gasg_${USER}
wrk=/tmp/gasg_${USER}
echo $wrk
cd $HOME/midwork
rm -rf tmpwrk
mkdir tmpwrk
cd tmpwrk
if [ -s ${wrk}/frames.ascii ]
then 
      rm ${wrk}/frames.ascii
fi
if [ -s ${wrk}/plot.prg ]
then 
      rm ${wrk}/plot.prg
fi
rm -f tmpdat
mkdir tmpdat
cd tmpdat

printf "%s\n" `basename $1` >> ${wrk}/frames.ascii


if [ ! -s ${wrk}/frames.ascii ]
then
  xdialog -bg white  -fg red -m '
ERROR: Empty set of input files.
You have to select the following list of input files:
 ThAr exposure [ARC_LAMP_x]
 DRS table [DRS_SETUP_x]
 order table [ORDER_TABLE_x]
 guess table [LINE_TABLE_x]
 line reference catalog [LINE_REFER_TABLE]
 (x: may be RED, or BLUE)'
 exit
fi
ln -s $1 .


if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

printf '@d pipeline.start >Null \n \
    @d pipeline.control d >Null \n \
    write/key pipe_home/c/1/100 " " ALL \n \
    pipe_home = m$symbol("PIPE_HOME") \n \
    set/context uves {pipe_home}/uves/context >Null\n  \    
    UVES_GXSZ(1) = 1000 \n \
    UVES_GYSZ(1) = 710 \n \
    write/key ifrm/c/1/100 " " ALL \n \
    write/key catal/i/1/1 0 \n \
    write/key wrk/c/1/100 " " ALL \n \
    wrk = m$symbol("USER") \n \
    wrk = "/tmp/gasg_{wrk}" \n \
    create/icat in.cat {wrk}/frames.ascii >Null\n \
    indisk/fits in.cat ? OC >Null\n \
    create/icat plot.cat *.bdf >Null\n \
    create/grap 1 {UVES_GXSZ(1)},{UVES_GYSZ(1)},200,400 \n \
    hardcop/plot ON \n \
    catal = 0 \n \
    store/frame ifrm plot.cat \n \
    config/instr {ifrm} \n \
    show/uves \n \
    if {{ifrm},NAXIS} .eq. 2 outputc = "1,{{ifrm},NPIX(2)}"
    if {{ifrm},NAXIS} .eq. 1 write/key outputc " " ALL
    plot/uves {ifrm} {outputc} \n \
    write/out " " \n \
    write/out " "\n \
    write/out "(1) To exit MIDAS session type  `bye`" \n \
    write/out "(2) Use `hardcop/plot` n,g PSfile to create" \n \
    write/out "    POSTSCRIPT hardcopies (see `help hardcop/plot`)" \n \
    write/out "(3) Use  `help plot/uves`  to get the options" \n \
    write/out "    redoing the plot" \n \
' > ${wrk}/plot.prg



xterm -geom 132x24+0-0 -fn fixed -T 'gasgano: plot.sh' -e inmidas -P -j '@@ /tmp/gasg_'${USER}'/plot.prg' &

exit
