#!/bin/sh
# $Id: uves_reduce.sh,v 1.6 2001/02/12 14:53:14 amodigli Exp $
#+++ UVES_REDUC +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         reduc.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#.INPUT         list    list of selected input files:
#                       science exposure [SCIENCE_y]
#                       DRS table     [DRS_SETUP_x]
#                       order table   [ORDER_TABLE_x]
#                       background table [BACKGR_TABLE_x]
#                       line tables   [LINE_TABLE_xn]
#                       master flat   [MASTER_FLAT_x]
#                       master bias   [MASTER_BIAS_x]
#                       (y: RED, BLUE, x:BLUE, REDL, REDU; n:1,2,3)
#
#.OUTPUT        reduced science spectra using average extraction
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       Do a science reduction using average extraction
#
#.ALGORITHM     Create an image catalog from the set of input frames and
#		finally calls UVES context command REDUCE/UVES
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Sebastian Wolf, ESO-DMO
#
#.VERSION       1.0 1999/12/20  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.6 $
# $Log: uves_reduce.sh,v $
# Revision 1.6  2001/02/12 14:53:14  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:37  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------


cd /tmp
rm -rf gasg_${USER}
mkdir gasg_${USER}
wrk=/tmp/gasg_${USER}
echo $wrk
cd $HOME/midwork
rm -rf tmpwrk
mkdir tmpwrk
cd tmpwrk
if [ -s ${wrk}/frames.ascii ]
then 
      rm ${wrk}/frames.ascii
fi
if [ -s ${wrk}/reduc.prg ]
then 
      rm ${wrk}/reduc.prg
fi
list=`echo $*`
rm -f tmpdat
mkdir tmpdat
cd tmpdat
for i in $list
do
  printf "%s\n" `basename $i` >> ${wrk}/frames.ascii
  ln -s $i .
done

if [ ! -s ${wrk}/frames.ascii ]
then
  xdialog -bg white  -fg red -m '
ERROR: Empty set of input files.
You have to select the following list of input files:
 ThAr exposure [ARC_LAMP_x]
 DRS table [DRS_SETUP_x]
 order table [ORDER_TABLE_x]
 guess table [LINE_TABLE_x]
 line reference catalog [LINE_REFER_TABLE]
 (x: may be RED, or BLUE)'
 exit
fi


if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

printf '@d pipeline.start\n \
    @d pipeline.control d\n \
    write/key pipe_home/c/1/100 " " ALL \n \
    pipe_home = m$symbol("PIPE_HOME") \n \
    set/context uves {pipe_home}/uves/context \n  \    
    write/key wrk/c/1/100 " " ALL \n \
    wrk = m$symbol("USER") \n \
    wrk = "/tmp/gasg_{wrk}" \n \
    create/icat in.cat {wrk}/frames.ascii >Null\n \
    write/out "INFO    [reduc]:            Transfering the data ..."\n \
    indisk/fits in.cat ROOT=ref >Null\n \
    create/icat reduc.cat ref* >Null\n \
    write/out "INFO    [reduc]:            ... stored in catalog reduc.cat"\n \
    @p uves_extrref reduc.cat \n \
    CONFIG/DISP p3=0.8 \n \
    SPLIT/UVES data.cat \n \
    REDUCE/UVES split.cat out.cat refer.cat E AVERAGE MEDIAN \n \
    ASSIGN/GRA g,3 \n \
    MODIF/GRA W \n \
    write/out " " \n \
    write/out "To exit MIDAS session type  bye" \n \
' > ${wrk}/reduc.prg



xterm -geom 132x24+0-0 -fn fixed -T 'gasgano: reduc.sh' -e inmidas -P -j '@@ /tmp/gasg_'${USER}'/reduc.prg' &

exit

