#!/bin/sh
# $Id: uves_stability.sh,v 1.6 2001/02/12 14:53:09 amodigli Exp $
#+++ UVES_CCDTEST +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
#
#.COPYRIGHT     (C) 1998  European Southern Observatory
#
#.IDENT         stability.sh
#
#.KEYWORDS      spectroscopy, echelle, calibrations
#
#.USAGE         call from GASGANO or from UNIX prompt
#
#                       Reference Formatcheck frame [MASTER_FORM_x]
#                       (must be the first one)
#                       Set of Formatcheck frames [ARC_LAMP_FORM_x]
#                       ThAr line table   [LINE_REFER_TABLE]
#
#.OUTPUT        
#
#.RETURN        
#
#.DEFAULT	no
#
#.PURPOSE       
#
#.ALGORITHM     
#
#.ENVIRON       UVES context
#   
#.LANGUAGE      shell script and MIDAS 99NOV
#
#.AUTHOR        Andreas Kaufer, ESO-PSO
#
#.VERSION       1.0 2000/01/12  Created.
#
#.COMMENT
#------------------------------------------------------------------------------
# $Name:  $
# $Revision: 1.6 $
# $Log: uves_stability.sh,v $
# Revision 1.6  2001/02/12 14:53:09  amodigli
# Made changes to pass to new
# pipeline structure for 01FEB
#
# Revision 1.5  2001/01/04 14:16:36  amodigli
# Added Id, Name, Revision, Log conf. control info
#
#------------------------------------------------------------------------------

cd /tmp
rm -rf gasg_${USER}
mkdir gasg_${USER}
wrk=/tmp/gasg_${USER}
echo $wrk
cd $HOME/midwork
rm -rf tmpwrk
mkdir tmpwrk
cd tmpwrk
if [ -s ${wrk}/frames.ascii ]
then 
      rm ${wrk}/frames.ascii
fi
if [ -s ${wrk}/stability.prg ]
then 
      rm ${wrk}/stability.prg
fi
list=`echo $*`
rm -f tmpdat
mkdir tmpdat
cd tmpdat
for i in $list
do
  printf "%s\n" `basename $i` >> ${wrk}/frames.ascii
  ln -s $i .
done

if [ ! -s ${wrk}/frames.ascii ]
then
  xdialog -bg white  -fg red -m '
ERROR: Empty set of input files.
You have to select the following list of input files:
 Formatcheck exposure [ARC_LAMP_FORM_x]
 line reference table [LINE_REFER_TABLE]'
 exit
fi


if [ -z "$PIPE_HOME" ]; then
    if [ -z "$MIDASHOME" ]; then
       echo 'Environment variable MIDASHOME is not defined. Abort!'
       exit
    fi
    if [ -z "$MIDVERS" ]; then
       echo 'Environment variable MIDVERS is not defined. Abort!'
       exit
    fi
    echo 'Environment variable PIPE_HOME is not defined. '
    echo 'Setting it to '${MIDASHOME}'/'${MIDVERS}'/pipeline.'
    PIPE_HOME=${MIDASHOME}/${MIDVERS}/pipeline
    export PIPE_HOME
    echo ${PIPE_HOME}
fi 

printf '@d pipeline.start\n \
    @d pipeline.control d\n \
    write/key pipe_home/c/1/100 " " ALL \n \
    pipe_home = m$symbol("PIPE_HOME") \n \
    set/context uves {pipe_home}/uves/context \n  \    
    write/key wrk/c/1/100 " " ALL \n \
    wrk = m$symbol("USER") \n \
    wrk = "/tmp/gasg_{wrk}" \n \
    create/icat in.cat {wrk}/frames.ascii >Null\n \
    write/out "INFO    [uves_stability]:       Transfering the data ..."\n \
    indisk/fits in.cat ROOT=ref >Null\n \
    create/icat stability.cat ref* >Null\n \
    CONFIG/DISP p3=0.8 \n \
    SPLIT/UVES stability.cat \n \
    @p uves_stability split.cat \n \
    write/out " " \n \
    write/out "To exit MIDAS session type  bye" \n \
' > ${wrk}/stability.prg



xterm -geom 132x24+0-0 -fn fixed -T 'gasgano: stability.sh' -e inmidas -P -j '@@ /tmp/gasg_'${USER}'/stability.prg' &

exit
