! $Id: uvesinit.prg,v 1.29 2001/09/17 10:06:48 amodigli Exp $
!++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
!
!.COPYRIGHT  (C) 1998  European Southern Observatory (ESO)
!
!.IDENT      uvesinit.prg
!
!.KEYWORDS   UVES pipeline, environment setup
!
!.USAGE      init/uves  [session]
!
!.INPUT      session      (P1)    Session name (file name prefix of keyword
!                                 file of a previous session)
!
!.OUTPUT     Session table uvestab.tbl
!
!.RETURNS    The recipe returns 0 on success, otherwise 1. Keyword Q1 is
!            updated with the exit status. 
!
!.PURPOSE    This procedure creates and initializes the UVES pipeline
!            environment (MIDAS UVES context). Note that apart from a few 
!            exceptions all keywords are read-only to inhibit any accidential
!            change of their values. Some of theses keywords may be modified
!            via dedicated commands.
!
!.ENVIRON    None.
!
!.LANGUAGE   MIDAS command language 98NOV.
!
!.AUTHOR     S. Wolf (ESO-DMD)
!
!.VERSION    0.5  1998/04/01:  Creation
!
!.INFO       The Description of the keywords may be 80 charcters long. In cases
!            where this has to be longer start every new line with the ~ char.
!------------------------------------------------------------------------------
! $Name:  $
! $Revision: 1.29 $
! $Id: uvesinit.prg,v 1.29 2001/09/17 10:06:48 amodigli Exp $
!------------------------------------------------------------------------------
crossref session

define/param p1  +   c   "Enter session name:"
define/maxpar 1


! Procedure identification
define/local procID/c/1/80      "UvesInit"


! Local variables
define/local msgflg/i/1/1     0
define/local msglev/i/1/1     0
define/local hcopy/c/1/3      " "  all  +lower
define/local skplck/c/1/200   " "  all  +lower



! Setup session
if p1(1:1) .ne. "+" then
! Remove keyword protection
  protect/uves uves 0
  savinit/uves {p1}KEYS.tbl read
  goto SKIPINIT
endif

write/out " "
write/out "Initializing UVES environment"
write/out " "
register/session uves {PIPE_PROC} uvesinit.prg uvestab.tbl

! Remove keyword protection
protect/uves uves 0


! Status of message keyword
info/keyword mess_level
if mid$info(1) .ge. 1 then
  if mid$info(1) .eq. 1 then
    msglev = {mess_level}
    msgflg = 1
  endif
  delete/keyw mess_level
endif


! Status of slog_id keyword
info/keyword slog_id
if mid$info(1) .ge. 1 then
  if mid$info(1) .eq. 1 then
    write/key slog_id/i/1/2 0 all
  endif
  delete/keyw slog_id
endif

! Status of message keyword
info/keyword nice_hcopy
if mid$info(1) .ge. 1 then
  if mid$info(1) .eq. 3 then			    !character keyword
    hcopy = m$upper(nice_hcopy)
  endif
  delete/keyw nice_hcopy
endif

!Begin Session List
!
! Instrument name
write/key UVES_NAME/c/1/10   "UVES"   	   !instrument name of UVES

! Data REduction Keywords
write/key objset/R/1/1       0.            ! Offset of the object in opt extr.
! Recipe index. May be determined elsewhere in a later version
  write/key reci/c/1/4          "REC1"

! Pipeline control keywords
write/key p_version_file/c/1/100  " " all !current pipe version file
if m$existk(rbinstrument) .eq. 0 then
   write/key rbinstrument/c/1/8 " " all
   rbinstrument = "UVES"
endif
p_version_file = m$symbol("PIPE_HOME") // "/" // m$lower(rbinstrument) // "/UVES_VERSION"
write/key p_version/c/1/15        " " all   !Pipeline version number

write/key p_ctxdir/c/1/100       " "  all  !Pipeline context directory. All contexts are loaded from here.
write/key p_ctxdel/i/1/1         0      l  !Delete flag for auxiliary contexts in super mode
write/key p_mainonly/i/1/1       0         !Pipeline product delivery flag
write/key p_nmodes/i/1/1         1         !Total number of supported instrument modes
write/key p_supported/c*15/1/{p_nmodes}   " "  all  !List of supported instrument modes
write/key p_modes/c*3/1/4        " "  all  !Internal names of instrument modes
write/key p_maxfrm/i/1/2         30,30     !Maximum number of frames allowed in input and reference sets.
write/key p_dspold/i/1/1         0         !Display status from previous recipe execution

write/key p_startdate/c/1/80     " "  all    ! For start date of recipe execution

write/key p_bindir/c/1/100       " "  all    ! Executables are found here.
p_bindir = m$symbol("PIPE_HOME") // "/" // m$lower(rbinstrument) // "/exec"


!Message Keyword
write/key MESS_LEVEL/i/1/1     3                    !Defines chattiness of the pipeline (set by VERBOSE/OUT)

! DRS setup table use
write/key FORCE_DRS/c/1/3     YES         !Force the use of DRS tables (YES)
!$ May be switched off (NO) for interactive use

! Pipeline internal logging facility keywords
write/key SLOG_ID/i/1/2        0   all   !File status of logfile
write/key SLOG_NAME/c/1/100    " " all   !Logfile name
write/key PROHEAD_FILE/c/1/20 "prohead.txt"         ! product header temp. file name

! Display Keywords
write/key NICE_CREA/c/1/1      "n"       !Display flag
write/key NICE_DPOS/c/1/30     " " all   !Position of win.title
write/key NICE_DLAB/c/1/30     " " all   !Display window title
write/key NICE_GLAB/c/1/30     " " all   !Graphics window title (mm)
write/key NICE_GLABW/c/1/40    " " all   !Graphics window title (world coords.)
write/key NICE_DSCR/i/1/2      0   all   !Screen center
write/key NICE_DDIM/i/1/2      0   all   !Screen size (display)
write/key NICE_GDIM/i/1/2      0   all   !Screen size (graph.)
write/key NICE_DIDS/i/1/10     0   all   !Available display windows
write/key NICE_GIDS/i/1/10     0   all   !Available graphic windows
write/key NICE_HCOPY/c/1/3     NO        !Do hardcopies YES or NO

write/key UVES_DXSZ/i/1/10 512.0 ALL	 !XSize of the image displays
write/key UVES_DYSZ/i/1/10 512.0 ALL	 !YSize of the image displays
write/key UVES_GXSZ/i/1/10 625.0 ALL	 !XSize of the graphic displays
write/key UVES_GYSZ/i/1/10 425.0 ALL	 !YSize of the graphic displays


write/key RBINS_OLD/c/1/20     "new"     !Instrument name

! Environment Keywords
write/key UVES_LOCK/i/1/1      13012119  !Lock number used to protect keywords
write/key UVES_IDSTRNG/c/1/20  "DO_CLASSIFICATION"  !Descriptor name that holds the frame identifier string
write/key UVES_NOCLASS/c/1/20  "No classification"  !UVES_IDSTRNG entry for frames that cannot be classified.
write/key UVES_INSMODE/c/1/10  " "   all !Instrument mode identification, i.e. IMG, LSS, or MOS
write/key UVES_ARM/c/1/4  " "   all  !Instrument arm: RED or BLUE
write/key UVES_AIR/d/1/3  0.0   all  !Instrument air character (p, T, p_H2O)

write/key UVES_MAXFILT/i/1/1      3  !Maximum index of filters
write/key UVES_MAXOUT/i/1/1       4  !Maximum number of readout ports
write/key UVES_MAXLAMP/i/1/1      6  !Maximum number of lamps
write/key UVES_DETNAME/c*15/1/2  " " ALL   !Detector names of UVES
UVES_DETNAME(1) = "ccdUvB - uvesb"	   !BLUE detector
UVES_DETNAME(2) = "ccdUvR - ccdUvr"	   !RED detector


! Instrument setup keywords
write/key kbase_outi/c/1/20      "ESO.DET.OUT"   !Base name of readout port header keyords
write/key ksuff_outi/c/1/20      ".NAME"         !Name suffix of readout port header keywords
write/key kbase_filti/c/1/20     "ESO.INS.FILT"  !Base name of the filter subsystem keywords
write/key ksuff_filti/c/1/20     ".NAME"         !Name suffix of the filter subsystem keywords


write/key uves_nout/i/1/1        0               !Total number of active readout ports
write/key uves_outi/i/1/4        0    all        !Identification numbers of active readout ports
write/key uves_nfilt/i/1/1       0               !Total number of filters used
write/key uves_filti/c*10/1/4    " "  all        !Names of used filters (pre-slit, 'normal'
write/key uves_nlamp/i/1/1       0               !Total number of active lamps
write/key uves_maxlin/r/1/1      65534.          !Linearity limit of detector (ADU)


! Column names for calibration tables
write/key c_rect/c/1/20     "RA_DEG"          !Standard star rectascension
write/key c_decl/c/1/20     "DEC_DEG"         !Standard star declination
write/key c_fxoname/c/1/20  "OBJECT"      !Object name column label in standard flux database
write/key c_fxndata/c/1/20  "NDATA"       !Label for column storing number of data items in flux database
write/key c_fxwave/c/1/20   "LAMBDA"      !Column label of wavelength array column in object spectrum table
write/key c_fxflux/c/1/20   "F_LAMBDA"    !Column label of flux array column in object spectrum table
write/key c_fxbwidth/c/1/20 "BIN_WIDTH"   !Column label of wavelength bin width array column in object spectrum table

! Pipeline processing levels and product types
write/key d_empty/c/1/25    "EMPTY"              !Dummy string for empty reduction levels
write/key d_tmp/c/1/25      "TEMPORARY"          !Type string of temporary products
write/key d_pre/c/1/25      "PREPROCESSED"       !Type string of preliminary products
write/key d_red/c/1/25      "RED"                !Type string of fully reduced products
write/key d_qcp/c/1/25      "QCPARAM"            !Type string for quality control parameters
write/key d_det/c/1/25      "DETECTOR"           !Product level for (partly) removed detector effects
write/key d_geo/c/1/25      "GEOMETRIC"          !Product level for removed instrumental effects
write/key d_pho/c/1/25      "PHOTOMETRIC"        !Product level for calibrated products
write/key d_uchk/c/1/25     "UNCHECKED"          !Initial certification level of raw frames


! Map frame descriptors names to keywords
! General
write/key h_airmass/c/1/25  "O_AIRM"             !Airmass descriptor
write/key h_instrume/c/1/25 "INSTRUME"           !Name of instrument in use
write/key h_ident/c/1/25    "IDENT"              !Original name of file
write/key h_origname/c/1/25 "ORIGFILE"           !Original name of file
write/key h_arcfile/c/1/25  "ARCFILE"            !Archive name of file
write/key h_pipefile/c/1/60 "PIPEFILE"           ! Pipeline name of file
write/key h_filename/c/1/25 "DO_FILENAME"        !Actual name of file

! Data product and data processing keywords
write/key h_dprcatg/c/1/25   "ESO.DPR.CATG"      !Data product category
write/key h_dprtype/c/1/25   "ESO.DPR.TYPE"	 !Data product type
write/key h_dprtech/c/1/25   "ESO.DPR.TECH"	 !Data product technique 
write/key h_datancom/c/1/25  "DATANCOM"          !Number of frames used for master frame
write/key h_datamin/c/1/25   "DATAMIN"	         !Minimum of data values
write/key h_datamax/c/1/25   "DATAMAX"	         !Maximum of data values
write/key h_promean/c/1/25   "ESO.PRO.DATAAVG"   !Mean of data values
write/key h_prosigm/c/1/25   "ESO.PRO.DATARMS"   !Std.deviation of data
write/key h_promedi/c/1/25   "ESO.PRO.DATAMED"   !Median of data values
write/key h_procatg/c/1/25   "ESO.PRO.CATG"	 !Category of pipeline product frame
write/key h_protype/c/1/25   "ESO.PRO.TYPE"	 !Pipeline product type
write/key h_prolevel/c/1/25  "ESO.PRO.REDLEVEL"  !Reduction level of data product
write/key h_prostat/c/1/25   "ESO.PRO.REC1.STATUS"  !Pipeline status of product frame
write/key h_prodmd5/c/1/60   "DATAMD5"              ! Pipeline status of product frame
write/key h_proqclev/c/1/25  "ESO.PRO.QC.LEVEL"     !Applied quality control level (0 or 1)
write/key h_proqcsts/c/1/25  "ESO.PRO.QC.STATUS"    !Quality control level of product frame
write/key h_proqccert/c/1/25 "ESO.PRO.QC.CERTIF"    !Certification level of frame
write/key h_proqcovex/c/1/25 "ESO.PRO.QC.OVEREXPO"  !Percentage of overexposed pixels
write/key h_proqcsmea/c/1/25 "ESO.PRO.QC.SKYMEAN"   !Mean of sky background (mag/arcs^2)
write/key h_proqcssig/c/1/25 "ESO.PRO.QC.SKYSIG"    !Standard deviation of sky background (mag/arcs^2)
write/key h_proqcsmed/c/1/25 "ESO.PRO.QC.SKYMEDI"   !Median of sky background (mag/arcs^2)
write/key h_proqcfwhm/c/1/25 "ESO.PRO.QC.IMGQUAL"   !Seeing (FWHM) determined from image (arcs)
write/key h_proqclimg/c/1/25 "ESO.PRO.QC.LIMAG"     !Limiting magnitude (for S/N=5)
write/key h_ovexmask/c/1/25  "OVEREXPO_MASK"        !Name of overexposure mask
write/key h_uit/c/1/25       "ESO.DET.WIN1.UIT1"    !User defined sub integration time
write/key h_tilt_pos/c*25/1/3 " " all               !Science camera tilt (pixels)
h_tilt_pos(1) = "ESO.INS.TILT2.POS" 
h_tilt_pos(2) = "ESO.INS.TILT2.POS"
h_tilt_pos(3) = "ESO.INS.TILT1.POS"
 
!QC keywords
write/key h_qc_did/c/1/25    "QC.DID"               !QC Dictionary ID
write/key d_qc_did/c/1/25    "UVES-1.14"             !QC Dictionary ID

! Observation block  keywords			      
write/key h_obsname/c/1/25  "ESO.OBS.NAME"	    !Observation block name
write/key h_obsid/c/1/25    "ESO.OBS.ID"	    !Observation block ID
write/key h_obstpno/c/1/25  "ESO.OBS.TPLNO"	    !Number of templates within OB           


! Template keywords				      
write/key h_tpid/c/1/25     "ESO.TPL.ID"	    !Template ID            
write/key h_tpnexp/c/1/25   "ESO.TPL.NEXP"	    !Number of exposures within template     
write/key h_tpexpno/c/1/25  "ESO.TPL.EXPNO"	    !Exposure sequence number within template


! Telescope keywords
write/key h_geolon/c/1/25   "ESO.TEL.GEOLON"        !Telescope geographic longitude (+ = North)
write/key h_geolat/c/1/25   "ESO.TEL.GEOLAT"        !Telescope geographic latitude (+ = East)


! Instrument keywords				      
write/key h_inspath/c/1/25  "ESO.INS.PATH"          !Instrument path
write/key h_insmode/c/1/25  "ESO.INS.MODE"          !Instrument mode

write/key h_slicer_name/c/1/25  "ESO.INS.SLIT1.NAME" !Image slicer
write/key h_slicer_no/c/1/25    "ESO.INS.SLIT1.NO"   !Number of slices

write/key uves_portid/i/1/3 4,1,1
uves_portid(1) = 4     !EEV
uves_portid(2) = 1     !MIT
uves_portid(3) = 1     !EEV

 
write/key h_slwidth/c*25/1/3                        !slit width
h_slwidth(1) = "ESO.INS.SLIT3.WID"
h_slwidth(2) = "ESO.INS.SLIT3.WID"
h_slwidth(3) = "ESO.INS.SLIT2.WID"

write/key h_sllen/c*25/1/3	                    !slit length
h_sllen(1) = "ESO.INS.SLIT3.LEN"
h_sllen(2) = "ESO.INS.SLIT3.LEN"
h_sllen(3) = "ESO.INS.SLIT2.LEN"

write/key h_cwlen/c*25/1/3  " " ALL		    !central wavelength
h_cwlen(1) = "ESO.INS.GRAT2.WLEN"    
h_cwlen(2) = "ESO.INS.GRAT2.WLEN"    
h_cwlen(3) = "ESO.INS.GRAT1.WLEN"    

write/key h_nameccd/c*25/1/3  " " ALL		    !name of the chip
h_nameccd(1) = "EEV"				    !RED
h_nameccd(2) = "MIT"				    !RED
h_nameccd(3) = "EEV"				    !BLUE


write/key h_fits_nameccd/c*25/1/3  " " ALL		    !name of the chip
h_fits_nameccd(1) = "ESO.DET.CHIP2.NAME"				    !RED
h_fits_nameccd(2) = "ESO.DET.CHIP2.NAME"				    !RED
h_fits_nameccd(3) = "ESO.DET.CHIP1.NAME"				    !BLUE

write/key h_namecros/c*25/1/3  " " ALL		    !name of the cross disp.
h_namecros(1) = "ESO.INS.GRAT2.NAME"		    !RED
h_namecros(2) = "ESO.INS.GRAT2.NAME"		    !RED
h_namecros(3) = "ESO.INS.GRAT1.NAME"		    !BLUE

write/key h_pressure/c/1/25 "ESO.INS.SENS26.MEAN"   !mean barometric pressure [mbar]

write/key h_tempcam/c*25/1/3  " " ALL		    !mean camera temperature [C]
h_tempcam(1) = "ESO.INS.TEMP2.MEAN"		    !RED
h_tempcam(2) = "ESO.INS.TEMP2.MEAN"                 !RED
h_tempcam(3) = "ESO.INS.TEMP1.MEAN"		    !BLUE

write/key h_pixscl/c/1/25 "ESO.INS.PIXSCALE"	    !pixel scale [pixel/arcsec]

! Detector keywords
write/key h_detector/c/1/25 "ESO.DET.NAME"          !Detector name
write/key h_xpixsize/c/1/25 "ESO.DET.CHIP1.PSZX"    !Size of pixels along x-axis
write/key h_nchips/c/1/25   "ESO.DET.CHIPS"	    !Total number of chips
write/key h_nout/c/1/25     "ESO.DET.OUTPUTS"	    !Total number of active readout ports
write/key h_confac/c*25/1/4 " "  all                !Conversion factor electrons to ADU
write/key h_ron/c*25/1/4    " "  all                !Detector readout noise
write/key h_pscan/c*25/1/4  " "  all		    !Prescan along x-axis       
write/key h_oscan/c*25/1/4  " "  all		    !Overscan along y-axis      
write/key h_nwin/c/1/25     "ESO.DET.WINDOWS"	    !Number of readout windows
write/key h_xwinstr/c/1/25  "ESO.DET.WIN1.STRX"     !start pixel on x-axis
write/key h_ywinstr/c/1/25  "ESO.DET.WIN1.STRY"     !start pixel on y-axis
write/key h_xwinn/c/1/25    "ESO.DET.WIN1.NX"       !number of pixels on x.
write/key h_ywinn/c/1/25    "ESO.DET.WIN1.NY"       !number of pixels on y.
write/key h_xwinbin/c/1/25  "ESO.DET.WIN1.BINX"     !Binning size along x-axis
write/key h_ywinbin/c/1/25  "ESO.DET.WIN1.BINY"     !Binning size along y-axis
write/key h_dit1/c/1/25     "ESO.DET.WIN1.DIT1"     !Actual integration time
write/key h_chipnx/c/1/25   "ESO.DET.CHIP1.NX"      !Number of pixels along X axis of chip
write/key h_chipny/c/1/25   "ESO.DET.CHIP1.NY"      !Number of pixels along Y axis of chip
write/key h_speed/c/1/25    "ESO.DET.READ.SPEED"    !Readout characteristic
write/key h_clock/c/1/25    "ESO.DET.READ.CLOCK"    !Readout clock pattern used

  
! Comments for header keywords (Note that session manager limit
! for keyword defaults is 50 characters.
write/key i_procatg/c/1/48  "Category of pipeline product frame"
write/key i_protype/c/1/48  "Product type"
write/key i_prolevel/c/1/48 "Reduction level of data product"
write/key i_prostat/c/1/48  "Pipeline status of frame"
write/key i_prodmd5/c/1/48  "MD5 data signature"                   ! Comment for keyword h_prodmd5
write/key i_datamin/c/1/60  "Minimum pixel value"
write/key i_datamax/c/1/60  "Maximum pixel value"
write/key i_promean/c/1/60  "Mean of pixel values"
write/key i_promedi/c/1/60  "Median of pixel values"
write/key i_prosigm/c/1/60  "Standard deviation of pixel values"
write/key i_proncom/c/1/60  "Number of frames combined"


write/key skyoff/i/1/2  -15,15 		            !offset to sky windows


write/key qc1log_table/c/1/20 "qc1log_tmp_table.tbl"
 
! Column names for calibration tables
! *none*


! Map DO classifications to pipeline internal variables.

write/key PATHID/i/1/1 1			    !instrument path id
						    !$ 1: red-arm lower part
						    !$ 2: red-arm upper part
						    !$ 3: blue-arm
write/key EXREDL/c/1/5         "_REDL"	            !UVES-arm extension RED/low
write/key EXREDU/c/1/5         "_REDU"		    !UVES-arm extension RED/up
write/key EXBLUE/c/1/5         "_BLUE"		    !UVES-arm extension BLUE

write/key DRSTAB/c/1/100       " " ALL		    !data reduction setup table
write/key DRSRC_ID/c*25/1/3    " " ALL		    !DRS setup file
DRSRC_ID(1) = "DRS_SETUP{EXREDL}"
DRSRC_ID(2) = "DRS_SETUP{EXREDU}"
DRSRC_ID(3) = "DRS_SETUP{EXBLUE}"
write/key ORDER_ID/c*25/1/3    " " ALL		    !Order definition table
ORDER_ID(1) = "ORDER_TABLE{EXREDL}"
ORDER_ID(2) = "ORDER_TABLE{EXREDU}"
ORDER_ID(3) = "ORDER_TABLE{EXBLUE}"
write/key BACK_ID/c*25/1/3     " " ALL		    !Background definition table
BACK_ID(1) = "BACKGR_TABLE{EXREDL}"
BACK_ID(2) = "BACKGR_TABLE{EXREDU}"
BACK_ID(3) = "BACKGR_TABLE{EXBLUE}"
write/key LINTAB_ID/c*25/1/3   " " ALL		    !Line tables
LINTAB_ID(1) = "LINE_TABLE{EXREDL}"
LINTAB_ID(2) = "LINE_TABLE{EXREDU}"
LINTAB_ID(3) = "LINE_TABLE{EXBLUE}"
write/key LINCAT_ID/c/1/25 "LINE_REFER_TABLE"	    !Line reference table

write/key BIAS_ID/c*25/1/3     " " ALL		    !Bias frame
BIAS_ID(1) = "BIAS{EXREDL}"
BIAS_ID(2) = "BIAS{EXREDU}"
BIAS_ID(3) = "BIAS{EXBLUE}"
write/key MBIAS_ID/c*25/1/3    " " ALL		    !Master bias frame
MBIAS_ID(1) = "MASTER_BIAS{EXREDL}"
MBIAS_ID(2) = "MASTER_BIAS{EXREDU}"
MBIAS_ID(3) = "MASTER_BIAS{EXBLUE}"
write/key DARK_ID/c*25/1/3     " " ALL		    !Dark frame
DARK_ID(1) = "DARK{EXREDL}"
DARK_ID(2) = "DARK{EXREDU}"
DARK_ID(3) = "DARK{EXBLUE}"
write/key MDARK_ID/c*25/1/3    " " ALL		    !Master dark frame
MDARK_ID(1) = "MASTER_DARK{EXREDL}"
MDARK_ID(2) = "MASTER_DARK{EXREDU}"
MDARK_ID(3) = "MASTER_DARK{EXBLUE}"
write/key FLAT_ID/c*25/1/3     " " ALL		    !Flat frame
FLAT_ID(1) = "FLAT{EXREDL}"
FLAT_ID(2) = "FLAT{EXREDU}"
FLAT_ID(3) = "FLAT{EXBLUE}"
write/key MFLAT_ID/c*25/1/3    " " ALL		    !Master flat frame
MFLAT_ID(1) = "MASTER_FLAT{EXREDL}"
MFLAT_ID(2) = "MASTER_FLAT{EXREDU}"
MFLAT_ID(3) = "MASTER_FLAT{EXBLUE}"
write/key MFORM_ID/c*25/1/3    " " ALL		    !Master format check frame
MFORM_ID(1) = "MASTER_FORM{EXREDL}"
MFORM_ID(2) = "MASTER_FORM{EXREDU}"
MFORM_ID(3) = "MASTER_FORM{EXBLUE}"


write/key SCIENCE_ID/c/1/25   "SCIENCE"         !Science frame
!
write/key STD_ID/c/1/25       "STANDARD"        !Standard star frame
!
write/key ESTD_ID/c/1/25      "STANDARD_EXTRACTED" !List of extracted standard star spectra
write/key ASTD_ID/c/1/25      "STANDARD_ALIGNED" !List of identified standard star spectra
write/key STDFLX_ID/c/1/25    "FLUX_STD_TABLE"  !Catalog of standard star fluxes/spectra
write/key ATMEXT_ID/c/1/25    "EXTCOEFF_TABLE"  !Atmospheric extinction coefficients
write/key RESP_ID/c/1/25      "INSTR_RESPONSE"  !Wavelength dependent instrument response
write/key EFFI_ID/c/1/25      "EFFICIENCY_TABLE" !Wavelength dependent instrument efficiency

write/key OVEXPT_ID/c/1/25    "OVEREXPO_TABLE"  !Statistics of (over)exposures
write/key OVEXPM_ID/c/1/25    "OVEREXPO_MASK"   !Mask with overexposed pixels
!
! Classification Prefixes
write/key EXTR_PX/c/1/8       "EXTRACT_"	    !order extraction prefix
write/key WCAL_PX/c/1/15      "WCALIB_"		!Wavelength calibrated frame
write/key WCFF_PX/c/1/15      "WCALIB_FF_"	!Wavelength calibrated and
						!$flatfielded frame
write/key MRGD_PX/c/1/15      "MERGED_"		!Merged frame
write/key REDU_PX/c/1/15      "RED_"		!Merged and sky subtracted.
 
write/key EXT_2D/c/1/15      "EXT2D_"		!Merged and sky subtracted.
write/key FF_2D/c/1/15       "FF2D_"		!Merged and sky subtracted.
write/key WCAL_2D/c/1/15     "WCAL2D_"		!Merged and sky subtracted.
write/key MER_2D/c/1/15      "MER2D_"		!Merged and sky subtracted.
 
write/key REDU_STD_ID/c*25/1/3    " " ALL    !Reduced standard star frame
REDU_STD_ID(1) = "{REDU_PX}{STD_ID}{EXREDL}"
REDU_STD_ID(2) = "{REDU_PX}{STD_ID}{EXREDU}"
REDU_STD_ID(3) = "{REDU_PX}{STD_ID}{EXBLUE}"

write/key PROD_STD/c/1/15    "RED_STD"     !Reduced standard star frame
write/key WCFF_STD/c/1/25      "WCALIB_FF_RESPONSE"		!Wavelength calib. Inst. Resp.


!End Session List

 
write/key inputi/i/1/10 0  ALL  ! keyword for integer input
write/key inputr/r/1/10 0. ALL  ! keyword for real input
write/key inputd/d/1/10 0. ALL  ! keyword for double input


! Pipeline context directory
p_ctxdir = m$symbol("MIDASHOME") // "/" // m$symbol("MIDVERS") // -
	  "/pipeline/context"


! Set pipeline delivery flag
if m$existk("PIPELINE") .eq. 1 then
  p_mainonly = pipeline(9)
else
  p_mainonly = -1
endif


! Default values for array keywords
p_supported(1) = "ECHELLE"		!p_modes have to be set
					!in the same way as p_supported !!!
p_modes(1) = "ECH"

h_pscan(1) = "ESO.DET.OUT1.PRSCX"
h_pscan(2) = "ESO.DET.OUT2.PRSCX"
h_pscan(3) = "ESO.DET.OUT3.PRSCX"
h_pscan(4) = "ESO.DET.OUT4.PRSCX"

h_oscan(1) = "ESO.DET.OUT1.OVSCX"
h_oscan(2) = "ESO.DET.OUT2.OVSCX"
h_oscan(3) = "ESO.DET.OUT3.OVSCX"
h_oscan(4) = "ESO.DET.OUT4.OVSCX"

h_confac(1) = "ESO.DET.OUT1.CONAD"
h_confac(2) = "ESO.DET.OUT2.CONAD"
h_confac(3) = "ESO.DET.OUT3.CONAD"
h_confac(4) = "ESO.DET.OUT4.CONAD"

h_ron(1) = "ESO.DET.OUT1.RON"
h_ron(2) = "ESO.DET.OUT2.RON"
h_ron(3) = "ESO.DET.OUT3.RON"
h_ron(4) = "ESO.DET.OUT4.RON"


SKIPINIT:
! Reset message level
if msgflg .eq. 1 then
  mess_level = msglev
endif

! Reset hardcopy option
if hcopy(1:1) .eq. "Y" then
  nice_hcopy = "YES"
endif


! Protect keywords
!skplck = "MESS_LEVEL,PATHID,UVES_INSMODE,uves_id,DRSTAB" // -
!         "nice_crea,nice_dpos,nice_dlab,nice_glab,nice_dscr,nice_ddim,"
!skplck = skplck // -
!         "nice_gdim,nice_dids,nice_gids,nice_glabw,nice_hcopy,rbins_old,"
!skplck = skplck // "p_dspold,slog_id,slog_name"
!
!protect/uves uves {UVES_LOCK}

return 0
